Package: eulerr
Title: Area-Proportional Euler and Venn Diagrams with Ellipses
Version: 7.0.2
Authors@R: c(person("Johan", "Larsson", email = "johanlarsson@outlook.com",
                    role = c("aut", "cre"),
                    comment = c(ORCID = "0000-0002-4029-5945")),
             person("A. Jonathan R.", "Godfrey", role = "ctb"),
             person("Peter", "Gustafsson", role = "ctb"),
             person("David H.", "Eberly", role = "ctb",
                    comment = "geometric algorithms"),
             person("Emanuel", "Huber", role = "ctb",
                    comment = "root solver code"),
             person("Florian", "Privé", role = "ctb"))
Description: Generate area-proportional Euler diagrams
    using numerical optimization. An Euler diagram is a generalization of a Venn
    diagram, relaxing the criterion that all interactions need to be
    represented. Diagrams may be fit with ellipses and circles via
    a wide range of inputs and can be visualized in numerous ways.
Depends: R (>= 3.3.0)
Imports: GenSA, graphics, grDevices, grid, polyclip, polylabelr, Rcpp,
        stats, utils
Suggests: covr, knitr, lattice, pBrackets, RConics, rmarkdown,
        testthat, spelling
LinkingTo: Rcpp (>= 0.12.12), RcppArmadillo (>= 0.7.600.1.0)
License: GPL-3
Encoding: UTF-8
LazyData: true
VignetteBuilder: knitr
URL: https://github.com/jolars/eulerr, https://jolars.github.io/eulerr/
BugReports: https://github.com/jolars/eulerr/issues
RoxygenNote: 7.2.3
Language: en-US
NeedsCompilation: yes
Packaged: 2024-03-28 10:02:12 UTC; gerd-jln
Author: Johan Larsson [aut, cre] (<https://orcid.org/0000-0002-4029-5945>),
  A. Jonathan R. Godfrey [ctb],
  Peter Gustafsson [ctb],
  David H. Eberly [ctb] (geometric algorithms),
  Emanuel Huber [ctb] (root solver code),
  Florian Privé [ctb]
Maintainer: Johan Larsson <johanlarsson@outlook.com>
Repository: CRAN
Date/Publication: 2024-03-28 15:40:09 UTC
Built: R 4.3.3; aarch64-apple-darwin20; 2025-01-24 15:44:10 UTC; unix
Archs: eulerr.so.dSYM

Package: eulerr
Title: Area-Proportional Euler and Venn Diagrams
Version: 8.1.0
Authors@R: c(person("Johan", "Larsson", email = "johan@jolars.co",
                    role = c("aut", "cre", "cph"),
                    comment = c(ORCID = "0000-0002-4029-5945")),
             person("A. Jonathan R.", "Godfrey", role = "ctb"),
             person("Peter", "Gustafsson", role = "ctb"),
             person("David H.", "Eberly", role = "ctb",
                    comment = "geometric algorithms"),
             person("Emanuel", "Huber", role = "ctb",
                    comment = "root solver code"),
             person("Florian", "Privé", role = "ctb"))
Description: Generate area-proportional Euler diagrams
    using numerical optimization. A Euler diagram is a generalization of a Venn
    diagram, relaxing the criterion that all interactions need to be
    represented. Diagrams may be fit with circles, ellipses, squares, and
    rectangles via a wide range of inputs and can be visualized in numerous ways.
Depends: R (>= 4.2)
Imports: graphics, grDevices, grid, parallel, stats, utils
Suggests: covr, knitr, lattice, pBrackets, RConics, rmarkdown,
        testthat, spelling
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
VignetteBuilder: knitr
URL: https://github.com/jolars/eulerr,
        https://jolars.github.io/eulerr/, https://eunoia.bz
BugReports: https://github.com/jolars/eulerr/issues
Language: en-US
Config/rextendr/version: 0.4.2
SystemRequirements: Cargo (Rust's package manager), rustc >= 1.88.0
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-06-30 11:49:31 UTC; jola
Author: Johan Larsson [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-4029-5945>),
  A. Jonathan R. Godfrey [ctb],
  Peter Gustafsson [ctb],
  David H. Eberly [ctb] (geometric algorithms),
  Emanuel Huber [ctb] (root solver code),
  Florian Privé [ctb]
Maintainer: Johan Larsson <johan@jolars.co>
Repository: CRAN
Date/Publication: 2026-06-30 13:40:09 UTC
Built: R 4.6.1; aarch64-apple-darwin23; 2026-06-30 16:41:42 UTC; unix
Archs: eulerr.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method("/",eulergram)
S3method("|",eulergram)
S3method(coef,euler)
S3method(euler,data.frame)
S3method(euler,default)
S3method(euler,list)
S3method(euler,matrix)
S3method(euler,table)
S3method(fitted,euler)
S3method(makeContent,EulerTags)
S3method(makeContext,EulerPanel)
S3method(plot,euler)
S3method(plot,eulergram)
S3method(plot,eulerr_venn)
S3method(plot,venn)
S3method(print,euler)
S3method(print,eulergram)
S3method(print,eulerr_venn)
S3method(print,venn)
S3method(residuals,euler)
S3method(venn,data.frame)
S3method(venn,default)
S3method(venn,list)
S3method(venn,matrix)
S3method(venn,table)
export(error_plot)
export(euler)
export(eulerr_options)
export(venn)
import(grid)
useDynLib(eulerr, .registration = TRUE)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# eulerr <a href="https://jolars.github.io/eulerr/"><img src='man/figures/logo.svg' align="right" height="139" /></a>

<!-- badges: start -->

[![R-CMD-check](https://github.com/jolars/eulerr/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/jolars/eulerr/actions/workflows/R-CMD-check.yaml)
[![Coverage
Status](https://codecov.io/gh/jolars/eulerr/branch/master/graph/badge.svg)](https://app.codecov.io/gh/jolars/eulerr)
[![CRAN
Badge](http://www.r-pkg.org/badges/version/eulerr)](https://cran.r-project.org/package=eulerr)
<!-- badges: end -->

**eulerr** generates area-proportional Euler diagrams that display set
relationships (intersections, unions, and disjoints) with circles or
ellipses. [Euler diagrams](https://en.wikipedia.org/wiki/Euler_diagram)
are Venn diagrams without the requirement that all set interactions be
present (whether they are empty or not), which means that, depending on
input, **eulerr** sometimes produces Venn diagrams and sometimes not.

With three or more sets intersecting, exact Euler diagrams are often
impossible. For such cases **eulerr** attempts to provide a good
approximation by numerically tuning the parameters of the ellipses or
circles to minimize the error in the resulting diagram. Residuals and
goodness of fit statistics are provided to assess whether the resulting
diagram can be trusted.

## Installation

Building **eulerr** from source requires a Rust toolchain. The minimum
supported `rustc` version is listed in the `SystemRequirements` field of
`DESCRIPTION` (currently `rustc >= 1.88.0`). If your system Rust is
older than that, install or update via [rustup](https://rustup.rs)
before installing the package from source. CRAN binary builds do not
need a local Rust toolchain.

### CRAN version

``` r
install.packages("eulerr")
```

### Development Version

``` r
devtools::install_github("jolars/eulerr")
```

### Web App

eulerr is also available as a web app at <https://eunoia.bz/app/>.

## Usage

``` r
library(eulerr)

set.seed(26)

# From Wilkinson 2012
fit <- euler(
  c(
    "A" = 4,
    "B" = 6,
    "C" = 3,
    "D" = 2,
    "E" = 7,
    "F" = 3,
    "A&B" = 2,
    "A&F" = 2,
    "B&C" = 2,
    "B&D" = 1,
    "B&F" = 2,
    "C&D" = 1,
    "D&E" = 1,
    "E&F" = 1,
    "A&B&F" = 1,
    "B&C&D" = 1
  ),
  shape = "ellipse"
)
```

We can inspect the goodness-of-fit metrics *diagError* and *stress* for
the solution,

``` r
fit$stress
#> [1] 1.280319e-19
fit$diagError
#> [1] 6.774157e-11
```

and plot it

``` r
plot(fit)
```

![](man/figures/README-plot_method-1.png)<!-- -->

Please see [the introductory
vignette](https://CRAN.R-project.org/package=eulerr/vignettes/introduction.html)
for a brief introduction or [*eulerr under the
hood*](https://CRAN.R-project.org/package=eulerr/vignettes/under-the-hood.html)
for details.

## Eunoia and Its Ecosystem

eulerr is based on [Eunoia](https://eunoia.bz), which is a Rust library
for fitting and visualizing Euler and Venn diagrams. The pure-Rust core
powers bindings in several languages, all backed by the same fitting
engine:

| Language | Package | Install |
|----|----|----|
| Python | [`eunoia`](https://pypi.org/project/eunoia/) ([repo](https://github.com/jolars/eunoia-py)) | `pip install eunoia` |
| Julia | [`Eunoia.jl`](https://platform.juliahub.com/ui/Packages/General/Eunoia) ([repo](https://github.com/jolars/Eunoia.jl)) | `] add Eunoia` |
| JavaScript | [`@jolars/eunoia`](https://www.npmjs.com/package/@jolars/eunoia) ([repo](https://github.com/jolars/eunoia)) | `npm install @jolars/eunoia` |
| Rust | `eunoia` on crates.io ([repo](https://github.com/jolars/eunoia)) | `cargo add eunoia` |

## License

eulerr is open source software, licensed under the
[MIT](https://github.com/jolars/eulerr/blob/master/LICENSE.md) license.

## Versioning

eulerr uses [semantic versioning](https://semver.org).

## Code of Conduct

Please note that this project is released with a [Contributor Code of
Conduct](https://github.com/jolars/eulerr/blob/master/CONDUCT.md). By
participating in this project you agree to abide by its terms.

## Acknowledgements

eulerr would not be possible without Ben Frederickson’s work on
[venn.js](http://www.benfrederickson.com) or Leland Wilkinson’s
[venneuler](https://cran.r-project.org/package=venneuler).

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----colorexamle, fig.cap = "The eight first colors of the default color palette"----
set.seed(2)
library(eulerr)
con <- c(A = 1, B = 1, C = 1, D = 1, E = 1, F = 1, G = 1, H = 1,
         "A&B" = 0.2, "B&C" = 0.2, "C&D" = 0.2, "D&E" = 0.2, "E&F" = 0.2,
         "F&G" = 0.2, "G&H" = 0.2)
plot(euler(con), labels = as.character(1:8))

## -----------------------------------------------------------------------------
knitr::opts_chunk$set(
  echo = FALSE,
  warning = FALSE,
  message = FALSE,
  fig.height = 3,
  fig.width = 3,
  collapse = TRUE,
  comment = "#>"
)

library(grid)
library(RConics)
library(eulerr)
library(lattice)

pal <- c(
  "#999999",
  "#E69F00",
  "#56B4E9",
  "#009E73",
  "#F0E442",
  "#0072B2",
  "#D55E00",
  "#CC79A7"
)

ellipse_arc <- function(
  saxes = c(1, 1),
  loc = c(0, 0),
  theta = 0,
  n = 200,
  rng = c(0, 2 * pi)
) {
  b <- min(saxes[1], saxes[2])
  a <- max(saxes[1], saxes[2])
  d2 <- (a - b) * (a + b)
  if (length(rng) == 1) {
    phi <- rng - theta
  } else {
    phi <- seq(rng[1], rng[2], len = n) - theta
  }
  sp <- sin(phi)
  cp <- cos(phi)
  r <- a * b / sqrt((saxes[2] * cp)^2 + (saxes[1] * sp)^2)
  P <- matrix(nrow = n, ncol = 2)
  P[, 1] <- r * cp
  P[, 2] <- r * sp

  if (theta != 0) {
    P <- P %*%
      matrix(
        c(cos(theta), sin(theta), -sin(theta), cos(theta)),
        byrow = TRUE,
        nrow = 2,
        ncol = 2
      )
  }
  P <- P + matrix(loc[1:2], nrow = nrow(P), ncol = 2, byrow = TRUE)
  P
}

set.seed(1)

## -----------------------------------------------------------------------------
c0 <- ellipseToConicMatrix(c(1, 1), c(0, 0), 0)
c1 <- ellipseToConicMatrix(c(0.7, 0.7), c(1.2, 0), 0)
pp <- intersectConicConic(c0, c1)

theta0 <- atan2(pp[2, 1], pp[1, 1])
theta1 <- atan2(pp[2, 2], pp[1, 2])

phi0 <- atan2(pp[2, 1], pp[1, 1] - 1.2)
phi1 <- atan2(pp[2, 2], pp[1, 2] - 1.2)

seg <- rbind(
  ellipse_arc(c(0.7, 0.7), c(1.2, 0), 0, n = 50, c(-pi, phi0)),
  ellipse_arc(c(1, 1), c(0, 0), 0, n = 100, c(theta0, theta1)),
  ellipse_arc(c(0.7, 0.7), c(1.2, 0), 0, n = 50, c(phi1, pi))
)

ospot <- c(mean(seg[, 1]), mean(seg[, 2]))

xyplot(
  1 ~ 1,
  xlim = c(-1.1, 2),
  ylim = c(-1.5, 1.1),
  asp = "iso",
  xlab = NULL,
  ylab = NULL,
  scales = list(draw = FALSE),
  par.settings = list(axis.line = list(col = "transparent")),
  panel = function() {
    panel.polygon(seg, col = "steelblue1", alpha = 0.5, border = "transparent")
    grid::grid.circle(
      0,
      0,
      r = 1,
      default.units = "native",
      gp = gpar(fill = "transparent")
    )
    grid::grid.circle(
      1.2,
      0,
      r = 0.7,
      default.units = "native",
      gp = gpar(fill = "transparent")
    )
    pBrackets::grid.brackets(1.2, -1.2, 0, -1.2, h = 0.05)
    pBrackets::grid.brackets(-1, 0, 0, 0, h = 0.05)
    pBrackets::grid.brackets(1.2, 0, 1.9, 0, h = 0.05)
    panel.lines(c(0, 0), c(0, -1.2), lty = 2, col = "grey65")
    panel.lines(c(1.2, 1.2), c(0, -1.2), lty = 2, col = "grey65")
    panel.text(
      0.6,
      unit(-1.3, "native"),
      labels = expression(italic(d[ij])),
      pos = 1
    )
    panel.text(ospot[1], ospot[2], labels = expression(italic(O[ij])))
    panel.text(
      -0.5,
      0.1,
      labels = expression(italic(r[i])),
      pos = 3,
      default.units = "native"
    )
    panel.text(
      1.55,
      0.1,
      labels = expression(italic(r[j])),
      pos = 3,
      default.units = "native"
    )
    panel.points(c(0, 1.2), c(0, 0), pch = 21, col = 1, cex = 1, fill = "white")
  }
)

## -----------------------------------------------------------------------------
h <- 0.7
k <- 0.5
a <- 1
b <- 0.6
phi <- pi / 5
n <- 200
ellipse <- RConics::ellipse(c(a, b), c(h, k), phi, n = n)

xyplot(
  1 ~ 1,
  xlim = c(-0.3, 1.7),
  ylim = c(-0.3, 1.3),
  asp = "iso",
  xlab = NULL,
  ylab = NULL,
  par.settings = list(axis.line = list(col = "transparent")),
  scales = list(draw = FALSE),
  panel = function(x, y, ...) {
    # grid
    panel.refline(h = 0)
    panel.refline(v = 0)

    # rotation
    arc <- ellipse_arc(c(0.2, 0.2), c(h, k), rng = c(0, phi))
    theta <- ellipse_arc(c(a, b), c(h, k), phi, n = 1)

    panel.lines(
      x = c(h, ellipse[1, 1]),
      y = c(k, ellipse[1, 2]),
      lty = 2,
      col = "grey50"
    )
    panel.lines(
      x = c(h, theta[1, 1]),
      y = c(k, theta[1, 2]),
      lty = 2,
      col = "grey50"
    )
    panel.lines(arc, col = "grey50")
    panel.text(
      x = arc[nrow(arc) / 2, 1] + 0.06,
      y = arc[nrow(arc) / 2, 2] + 0.02,
      labels = expression(italic(phi))
    )

    # semiaxes
    pBrackets::grid.brackets(
      h,
      k,
      ellipse[n / 2, 1],
      ellipse[n / 2, 2],
      col = "grey50"
    )
    pBrackets::grid.brackets(
      h,
      k,
      ellipse[n / 4, 1],
      ellipse[n / 4, 2],
      h = 0.04,
      col = "grey50"
    )

    panel.text(0.37, 0.12, labels = "a", font = 3)
    panel.text(0.43, 0.68, labels = "b", font = 3)

    # center
    panel.text(x = h, y = k + 0.025, pos = 3, label = c("h,k"), font = 3)
    panel.points(x = h, y = k, pch = 21, fill = "white", col = "black")

    # ellipse
    panel.lines(ellipse, col = 1)
  }
)

## -----------------------------------------------------------------------------
C1 <- ellipseToConicMatrix(c(8, 2), c(0, 0), -pi / 3)
C2 <- ellipseToConicMatrix(c(5, 2), c(1, -2), pi / 5)

ellipses <- data.frame(rbind(
  ellipse(c(8, 2), c(0, 0), -pi / 3),
  ellipse(c(5, 2), c(1, -2), pi / 5)
))
colnames(ellipses) <- c("x", "y")
ellipses$fac <- rep(c("A", "B"), each = 201)
pp <- intersectConicConic(C1, C2)
x <- pp[1, ]
y <- pp[2, ]
a <- double(6)
b <- double(6)
k <- 1
for (i in 1:3) {
  for (j in (i + 1):4) {
    b[k] <- (y[j] - y[i]) / (x[j] - x[i])
    a[k] <- b[k] * (-x[i]) + y[i]
    k <- k + 1
  }
}
a <- a[c(1, 6, 3, 4, 2, 5)]
b <- b[c(1, 6, 3, 4, 2, 5)]

xyplot(
  y ~ x,
  data = ellipses,
  type = "l",
  groups = fac,
  asp = 1,
  xlab = NULL,
  ylab = NULL,
  scales = list(draw = FALSE, axes = FALSE),
  par.settings = list(axis.line = list(col = "transparent")),
  panel = function(x, y, ...) {
    panel.abline(a = a[3], b = b[3], col = "#FEFEFE")
    panel.abline(a = a[4], b = b[4], col = "#FEFEFE")
    panel.xyplot(x, y, ..., col = 1)
  }
)

xyplot(
  y ~ x,
  data = ellipses,
  type = "l",
  groups = fac,
  asp = 1,
  xlab = NULL,
  ylab = NULL,
  scales = list(draw = FALSE, axes = FALSE),
  par.settings = list(axis.line = list(col = "transparent")),
  panel = function(x, y, ...) {
    panel.xyplot(x, y, ..., col = 1)
    for (i in seq_along(a)) {
      panel.abline(a = a[i], b = b[i], col = pal[ceiling(i / 2) + 1])
    }
  }
)

xyplot(
  y ~ x,
  data = ellipses,
  type = "l",
  groups = fac,
  asp = 1,
  xlab = NULL,
  ylab = NULL,
  scales = list(draw = FALSE, axes = FALSE),
  par.settings = list(axis.line = list(col = "transparent")),
  panel = function(x, y, ...) {
    panel.xyplot(x, y, ..., col = c("transparent", "black"))
    panel.abline(a = a[3], b = b[3], col = pal[3])
    panel.abline(a = a[4], b = b[4], col = pal[3])
    panel.points(t(pp[1:2, ]), col = 1, pch = 21, fill = "white")
  }
)

## -----------------------------------------------------------------------------
x <- c(0, -0.3, 0.2)
y <- c(0, 0.1, 0.3)
ra <- a <- c(0.3, 0.5, 0.4)
rb <- b <- c(0.3, 0.3, 0.6)
phi <- c(-pi / 6, 2, -2)

ee <- data.frame(x, y, ra, rb, phi)

tx <- atan2(-b * tan(phi), a)
ty <- atan2(b * tan(pi / 2L - phi), a)

xlim <- range(
  x + a * cos(tx) * cos(phi) - b * sin(tx) * sin(phi),
  x + a * cos(tx + pi) * cos(phi) - b * sin(tx + pi) * sin(phi)
)
ylim <- range(
  y + b * sin(ty) * cos(phi) + a * cos(ty) * sin(phi),
  y + b * sin(ty + pi) * cos(phi) + a * cos(ty + pi) * sin(phi)
)

pp <- matrix(NA, ncol = 4, nrow = 0)

for (i in 1:2) {
  for (j in (i + 1):3) {
    e1 <- ellipseToConicMatrix(c(ra[i], rb[i]), c(x[i], y[i]), phi[i])
    e2 <- ellipseToConicMatrix(c(ra[j], rb[j]), c(x[j], y[j]), phi[j])
    pp <- rbind(pp, cbind(t(intersectConicConic(e1, e2)[1:2, ]), i, j))
  }
}

sel <- logical(nrow(pp))
for (k in seq_len(nrow(pp))) {
  in_which <- ((pp[k, 1] - x) * cos(phi) + (pp[k, 2] - y) * sin(phi))^2 /
    ra^2 +
    ((pp[k, 1] - x) * sin(phi) - (pp[k, 2] - y) * cos(phi))^2 / rb^2 <=
    1 + 0.1
  sel[k] <- all(in_which)
}

pp <- pp[sel, ]
mid <- cbind(mean(pp[, 1]), mean(pp[, 2]))
seglines <- matrix(NA, ncol = 2, nrow = 0)

ang <- atan2(pp[, 1] - mid[1], pp[, 2] - mid[2])
ord <- order(ang)

pp <- pp[ord, ]

j <- nrow(pp)
for (i in seq_len(nrow(pp))) {
  k <- intersect(pp[i, 3:4], pp[j, 3:4])
  start <- atan2(pp[j, 2] - y[k], pp[j, 1] - x[k])
  stop <- atan2(pp[i, 2] - y[k], pp[i, 1] - x[k])
  arc <- ellipse_arc(
    c(a[k], b[k]),
    c(x[k], y[k]),
    theta = phi[k],
    rng = c(start, stop)
  )
  seglines <- rbind(seglines, arc)
  j <- i
}

theta <- seq.int(0, 2 * pi, length.out = 200L)
x0 <- as.vector(sapply(seq_along(x), function(i) {
  x[i] + ra[i] * cos(theta) * cos(phi[i]) - rb[i] * sin(theta) * sin(phi[i])
}))
y0 <- as.vector(sapply(seq_along(y), function(i) {
  y[i] + rb[i] * sin(theta) * cos(phi[i]) + ra[i] * cos(theta) * sin(phi[i])
}))

xyplot(
  y ~ x,
  data = ee,
  asp = "iso",
  xlim = extendrange(xlim, f = 0.01),
  ylim = extendrange(ylim, f = 0.01),
  scales = list(draw = FALSE),
  xlab = NULL,
  ylab = NULL,
  par.settings = list(axis.line = list(col = "transparent")),
  panel = function(x, y, ...) {
    grid.polygon(
      x0,
      y0,
      id.lengths = rep(200, 3),
      default.units = "native",
      gp = grid::gpar(fill = "transparent")
    )
    panel.polygon(seglines, col = "slategray2")
    panel.polygon(pp[, 1:2], col = "grey90")
    panel.points(mid, pch = 4, col = 1, cex = 1.5)
    panel.points(
      pp[, 1:2, drop = FALSE],
      col = 1,
      pch = 21,
      fill = "white",
      cex = 1.5
    )
  }
)

## -----------------------------------------------------------------------------
ellipse <- ellipse(c(1, 0.6), c(0, 0), 0)
i0 <- 15
i1 <- 45

tri <- rbind(
  cbind(0, 0),
  ellipse[i0, , drop = FALSE],
  ellipse[i1, , drop = FALSE]
)

xyplot(
  1 ~ 1,
  xlim = c(-1.2, 1.2),
  ylim = c(-0.8, 0.8),
  asp = "iso",
  xlab = NULL,
  ylab = NULL,
  par.settings = list(axis.line = list(col = "transparent")),
  scales = list(draw = FALSE),
  panel = function(x, y, ...) {
    panel.grid(x = 0, y = 0, h = 1, v = 1)
    panel.lines(ellipse, col = 1)
    panel.polygon(ellipse[i0:i1, ], col = "steelblue1", alpha = 0.5)
    panel.polygon(tri, col = "grey95")
    panel.points(ellipse[c(i0, i1), ], col = 1, pch = 21, fill = "white")
    panel.text(
      ellipse[c(i0, i1), ],
      adj = c(-0.5, -0.5),
      labels = c(
        expression(italic(theta[0])),
        expression(italic(theta[1]))
      )
    )
    pBrackets::grid.brackets(1, 0, 0, 0, h = 0.05, type = 1)
    pBrackets::grid.brackets(0, 0, 0, 0.6, h = 0.04)
    panel.text(0.5, unit(-0.05, "npc"), labels = "a", font = 3, pos = 1)
    panel.text(-0.05, unit(0.3, "native"), labels = "b", font = 3, pos = 2)
  }
)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## -----------------------------------------------------------------------------
combos <- c(
  "agc"                  = 9,
  "camk"                 = 17,
  "cmgc"                 = 16,
  "tk"                   = 16,
  "tkl"                  = 23,
  "agc&camk"             = 1,
  "camk&tk"              = 1,
  "tk&tkl"               = 1,
  "camk&cmgc&tkl"        = 1,
  "camk&tk&tkl"          = 2,
  "agc&camk&tk&tkl"      = 1,
  "camk&cmgc&tk&tkl"     = 3,
  "agc&camk&cmgc&tk&tkl" = 1
)

## ----fig.cap = "An Euler diagram fit to the combination given earlier on, showing that only 1-by-1 intersections are present. This fit uses the default loss function, the sum of squared errors."----
library(eulerr)

fit <- euler(combos)
plot(fit)

## ----fig.cap = "Euler diagrams fit to the combination above, using different loss function", fig.show = "hold", fig.width = 3, fig.height = 3----
losses <- c("square", "abs", "region")
aggregators <- c("sum", "max")

for (loss in losses) {
  for (aggregator in aggregators) {
    fit <- euler(combos, loss = loss, loss_aggregator = aggregator)
    print(plot(fit, main = paste(aggregator, loss, sep = ", ")))
  }
}

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

options(digits = 4)

## ----input--------------------------------------------------------------------
library(eulerr)

# Input in the form of a named numeric vector
fit1 <- euler(c(
  "A" = 25,
  "B" = 5,
  "C" = 5,
  "A&B" = 5,
  "A&C" = 5,
  "B&C" = 3,
  "A&B&C" = 3
))

# Input as a matrix of logicals
set.seed(1)
mat <- cbind(
  A = sample(c(TRUE, TRUE, FALSE), 50, TRUE),
  B = sample(c(TRUE, FALSE), 50, TRUE),
  C = sample(c(TRUE, FALSE, FALSE, FALSE), 50, TRUE)
)
fit2 <- euler(mat)

## -----------------------------------------------------------------------------
fit2

## ----residual-plot, fig.cap = "Residuals for the fit diagram."----------------
# Cleveland dot plot of the residuals
dotchart(resid(fit2))

## ----error-plot, fig.cap = "A plot from `error_plot()`."----------------------
error_plot(fit2)

## ----coefs--------------------------------------------------------------------
coef(fit2)

## ----fig.cap = "A difficult combination from Wilkinson 2012."-----------------
wilkinson2012 <- c(
  A = 4,
  B = 6,
  C = 3,
  D = 2,
  E = 7,
  F = 3,
  "A&B" = 2,
  "A&F" = 2,
  "B&C" = 2,
  "B&D" = 1,
  "B&F" = 2,
  "C&D" = 1,
  "D&E" = 1,
  "E&F" = 1,
  "A&B&F" = 1,
  "B&C&D" = 1
)
fit3 <- euler(wilkinson2012, shape = "ellipse")
plot(fit3)

## ----eulerr-plot, fig.cap = "Customizing Euler plots is a breeze in eulerr."----
plot(fit2)

# Remove fills, vary borders, display quantities, and switch font.
plot(
  fit2,
  quantities = TRUE,
  fill = "transparent",
  lty = 1:3,
  labels = list(font = 4)
)

## -----------------------------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

library(eulerr)

# The benchmark numbers are precomputed by `data-raw/benchmarks.R` (which needs
# the competitor packages and is not part of the package build) and stored in
# `benchmark_results.rds`. This vignette only reads and displays them, so it
# builds with eulerr, knitr, and lattice alone.
results_path <- "benchmark_results.rds"
have_results <- file.exists(results_path)
if (have_results) {
  results <- readRDS(results_path)
}

## -----------------------------------------------------------------------------
# Reshape a long accuracy/timing slice to a dataset-by-package matrix, dropping
# competitor columns that were never run (all NA) so the scaffolded state (only
# eulerr available) still produces a clean table.
to_matrix <- function(df, value_col) {
  m <- tapply(
    df[[value_col]],
    list(df$dataset, df$package),
    FUN = function(x) x[1]
  )
  m <- m[, colSums(!is.na(m)) > 0, drop = FALSE]
  # Order datasets as first seen, eulerr columns first.
  m <- m[unique(df$dataset), , drop = FALSE]
  pkg_order <- order(!grepl("^eulerr", colnames(m)), colnames(m))
  m[, pkg_order, drop = FALSE]
}

fmt <- function(x) {
  ifelse(is.na(x), "---", formatC(x, format = "g", digits = 2))
}

## -----------------------------------------------------------------------------
if (!have_results) {
  cat(
    "> **Benchmark results are not available.** Run",
    "`Rscript data-raw/benchmarks.R` to generate `benchmark_results.rds`,",
    "then rebuild this vignette.\n"
  )
}

## -----------------------------------------------------------------------------
run <- results$meta$competitors_run
if (length(run) == 0) {
  cat(
    "> The stored results were generated **without any competitor package",
    "installed**, so only eulerr's own numbers are shown below. Install",
    "venneuler, BioVenn, and nVennR and re-run `data-raw/benchmarks.R` for the",
    "full comparison.\n"
  )
} else {
  cat(
    "> Competitors benchmarked:",
    paste(run, collapse = ", "),
    paste0("(generated ", results$meta$generated, ").\n")
  )
}

## -----------------------------------------------------------------------------
acc <- results$accuracy
for (cmp in unique(acc$comparison)) {
  sub <- acc[acc$comparison == cmp, ]
  metric <- sub$metric[1]
  m <- to_matrix(sub, "value")
  cat("\n**", cmp, "** --- lower ", metric, " is better.\n\n", sep = "")
  print(knitr::kable(
    apply(m, 2, fmt),
    align = "r",
    caption = NULL
  ))
  cat("\n")
}

## -----------------------------------------------------------------------------
library(lattice)

acc <- results$accuracy
acc$package <- factor(acc$package, levels = unique(acc$package))

lattice::barchart(
  value ~ dataset | comparison,
  groups = package,
  data = acc,
  scales = list(x = list(rot = 45), y = list(relation = "free")),
  layout = c(3, 1),
  ylab = "fit error",
  auto.key = list(columns = 1, space = "right"),
  par.settings = lattice::simpleTheme(col = eulerr_options()$fills$fill(8))
)

## -----------------------------------------------------------------------------
tim <- results$timing
# Average eulerr-circle / competitor timings across datasets for a compact view.
m <- to_matrix(tim, "time_ms")
cat("Median fit time (ms), by dataset and package:\n\n")
knitr::kable(
  apply(m, 2, function(x) formatC(x, format = "f", digits = 1)),
  align = "r"
)

## -----------------------------------------------------------------------------
tim <- results$timing
tim$package <- factor(tim$package, levels = unique(tim$package))

lattice::barchart(
  time_ms ~ dataset,
  groups = package,
  data = tim[!duplicated(tim[c("dataset", "package")]), ],
  scales = list(x = list(rot = 45), y = list(log = 10)),
  ylab = "median time (ms)",
  auto.key = list(columns = 1, space = "right"),
  par.settings = lattice::simpleTheme(col = eulerr_options()$fills$fill(8))
)

## -----------------------------------------------------------------------------
qual <- data.frame(
  Package = c(
    "eulerr",
    "venneuler",
    "nVennR",
    "BioVenn",
    "vennplot",
    "VennDiagram"
  ),
  Shapes = c(
    "circle, ellipse, rectangle, square",
    "circle",
    "irregular polygon",
    "circle",
    "circle (2D), sphere (3D)",
    "circle"
  ),
  `Max sets` = c("many", "many", "many", "2--3", "2--3", "4 (2 scaled)"),
  Proportional = c(
    "approximate",
    "approximate",
    "quasi",
    "accurate (2--3)",
    "approximate",
    "2 sets only"
  ),
  Input = c(
    "vectors, data frames, matrices, tables, lists",
    "named vector, data frame",
    "lists",
    "ID lists",
    "counts, lists",
    "counts"
  ),
  `Fit reported` = c(
    "stress, diagError, regionError",
    "stress",
    "none",
    "none",
    "none",
    "none"
  ),
  `Key dependency` = c(
    "none (Rust)",
    "rJava / Java",
    "C++",
    "none",
    "Rcpp, rgl",
    "none"
  ),
  Source = c(
    "CRAN",
    "CRAN",
    "GitHub",
    "CRAN",
    "CRAN (dormant)",
    "CRAN"
  ),
  check.names = FALSE,
  stringsAsFactors = FALSE
)

knitr::kable(qual, align = "l")

## -----------------------------------------------------------------------------
excl <- data.frame(
  Package = c(
    "venn, ggVennDiagram, ggvenn, RVenn, gplots",
    "UpSetR",
    "colorfulVennPlot",
    "VennMaster",
    "Vennerable"
  ),
  Reason = c(
    "Draw fixed, schematic shapes; quantity shown via labels or color, not area",
    "Not a Venn/Euler diagram at all (UpSet matrix/bar charts)",
    "Archived from CRAN; only a 2-set helper, never a general fitter",
    "Area-proportional, but a standalone Java application --- not an R package",
    "Area-weighted, but hosted on R-Forge/GitHub, not CRAN or Bioconductor"
  ),
  check.names = FALSE,
  stringsAsFactors = FALSE
)

knitr::kable(excl, align = "l")

#' Plot Euler and Venn diagrams
#'
#' Plot diagrams fit with [euler()] and [venn()] using [grid::Grid()] graphics.
#' This
#' function sets up all the necessary plot parameters and computes
#' the geometry of the diagram. [plot.eulergram()], meanwhile,
#' does the actual plotting of the diagram. Please see the **Details** section
#' to learn about the individual settings for each argument.
#'
#' The only difference between [plot.euler()] and [plot.venn()] is that
#' `quantities` is set to `TRUE` by default in the latter and `FALSE` in
#' the former.
#'
#' Most of the arguments to this function accept either a logical, a vector, or
#' a list where
#'
#' * logical values set the attribute on or off,
#' * vectors are shortcuts to commonly used options (see the individual
#' parameters), and
#' * lists enable fine-grained control, including graphical
#' parameters as described in [grid::gpar()] and control
#' arguments that are specific to each argument.
#'
#' The various [grid::gpar()] values that are available for each argument
#' are:
#'
#' \tabular{lcccccccc}{
#'              \tab fills \tab edges \tab labels \tab quantities  \tab annotations \tab strips \tab legend \tab main \cr
#'   col        \tab       \tab x     \tab x      \tab x           \tab x           \tab x      \tab x      \tab x    \cr
#'   fill       \tab x     \tab       \tab        \tab             \tab             \tab        \tab        \tab      \cr
#'   alpha      \tab x     \tab x     \tab x      \tab x           \tab x           \tab x      \tab x      \tab x    \cr
#'   lty        \tab       \tab x     \tab        \tab             \tab             \tab        \tab        \tab      \cr
#'   lwd        \tab       \tab x     \tab        \tab             \tab             \tab        \tab        \tab      \cr
#'   lex        \tab       \tab x     \tab        \tab             \tab             \tab        \tab        \tab      \cr
#'   fontsize   \tab       \tab       \tab x      \tab x           \tab x           \tab x      \tab x      \tab x    \cr
#'   cex        \tab       \tab       \tab x      \tab x           \tab x           \tab x      \tab x      \tab x    \cr
#'   fontfamily \tab       \tab       \tab x      \tab x           \tab x           \tab x      \tab x      \tab x    \cr
#'   lineheight \tab       \tab       \tab x      \tab x           \tab x           \tab x      \tab x      \tab x    \cr
#'   font       \tab       \tab       \tab x      \tab x           \tab x           \tab x      \tab x      \tab x    \cr
#' }
#'
#' Defaults for these values, as well as other parameters of the plots, can
#' be set globally using [eulerr_options()].
#'
#' If the diagram has been fit using the `data.frame` or `matrix` methods
#' and using the `by` argument, the plot area will be split into panels for
#' each combination of the one to two factors. The `fills`, `patterns`, `edges`,
#' `labels`, `quantities`, and `annotations` arguments each accept an optional
#' `by_group` entry: a named list of override lists keyed by panel name (the
#' names of the fitted object). For multi-`by` fits the panel name is the
#' levels joined by `.`, e.g. `"Male.German"`. Panels not listed in `by_group`
#' use the top-level settings unchanged. Only graphical fields (and `rot` for
#' `labels`, `quantities`, and `annotations`) may be overridden per panel;
#' structural fields such as `quantities$type`, `quantities$format`,
#' `annotations$labels`, or named-by-subset `fills$fill` must be set at the
#' top level.
#'
#' For users who are looking to plot their diagram using another package,
#' all the necessary parameters can be collected if the result of this
#' function is assigned to a variable (rather than printed to screen).
#'
#' @param x an object of class `'euler'`, generated from [euler()]
#' @param fills a logical, vector, or list of graphical parameters for the fills
#'   in the diagram. Vectors are assumed to be colors for the fills.
#'   See [grid::grid.path()]. Named fill vectors can be matched in
#'   `fills$mode = "disjoint"` (default) or `fills$mode = "union"`.
#' @param patterns a logical, vector, or list of graphical parameters for
#'   fill patterns in the diagram. Vectors are assumed to be pattern types
#'   (currently `"stripes"` or `NA`), where `NA` means no pattern.
#'   Supported list items are `type`, `angle`, `col`, `lwd`, and `alpha`.
#'   Named pattern vectors can be matched in
#'   `patterns$mode = "disjoint"` (default) or `patterns$mode = "union"`.
#' @param edges a logical, vector, or list of graphical parameters for the edges
#'   in the diagram. Vectors are assumed to be colors for the edges.
#'   See [grid::grid.polyline()].
#' @param legend a logical scalar or list. If a list, the item `side` can be
#'   used to set the location of the legend and `symbol_size` can be used to scale
#'   the legend symbols independently of the text size. See
#'   [grid::grid.legend()].
#' @param labels a logical, vector, or list. Vectors are assumed to be
#'   text for the labels. See [grid::grid.text()]. In addition to the
#'   `grid::gpar()` fields, the following placement controls are
#'   supported (delegated to the `eunoia` Rust crate):
#'   `labels$placement` (`"raycast"` (default), `"force_directed"`, or
#'   `"elbow"`) selects the strategy used when a label does not fit
#'   inside its region. `"raycast"` and `"force_directed"` produce
#'   straight leader lines (the former places the label along the
#'   centroid→POI ray, the latter relaxes labels with a polygon-aware
#'   force solver). `"elbow"` produces d3-pie style orthogonal leaders,
#'   stacking exterior labels in left/right columns reached by a
#'   three-segment polyline. `labels$margin` (numeric) overrides the
#'   per-region margin between an exterior label and the diagram
#'   (default is half the larger of the label's width and height);
#'   `labels$tether` (`"poi"` (default) or `"boundary"`) chooses where
#'   the leader line attaches on the source region; `labels$gap`
#'   controls the visible gap between the leader tip and the label box
#'   edge — a bare numeric is interpreted as `lines` (font-relative),
#'   a [grid::unit()] is honored as given, and the default `NULL`
#'   tracks `eulerr_options()$padding` so the gap matches the spacing
#'   between label and quantity; `labels$leader` is a list (`col`,
#'   `alpha`, `lwd`, `lty`, `lex`) styling the leader line drawn from
#'   the tether to the exterior label. Strategy-specific knobs live in
#'   their own sublists: `labels$force_directed = list(iterations = ...)`
#'   sets the iteration cap for the force-directed solver, and
#'   `labels$elbow = list(min_gap = ...)` sets the minimum vertical
#'   centre-to-centre spacing between stacked label boxes in an elbow
#'   column.
#' @param quantities a logical, vector, or list. Vectors are assumed to be
#'   text for the quantities' labels, which by
#'   default are the original values in the input to [euler()]. In addition
#'   to plain vectors, `quantities$labels` can also be a named vector keyed by
#'   subset names (e.g., `"A"`, `"B"`, `"A&B"`), which is useful for supplying
#'   custom text for overlap regions. If `quantities$labels` is `NULL`,
#'   `quantities$format` can be used to control number formatting as a list with
#'   an item `fun` (a function such as [signif()] or [round()]) and optional
#'   extra arguments passed to that function (for example,
#'   `list(fun = prettyNum, big.mark = ",")`). `quantities$total` can be used to
#'   set an external
#'   denominator for percent/fraction quantities (instead of the plotted total).
#'   to arguments that apply to [grid::grid.text()], an argument `type` may
#'   also be used which should be a combination of `"counts"`, `"percent"`,
#'   and `"fraction"`. The first item will be printed first and the second
#'   will be printed thereafter inside brackets. The default is
#'   `type = "counts"`.
#'   For finer control over the rendered text, set `quantities$template` to
#'   a string with `{counts}`, `{percent}`, and/or `{fraction}` placeholders,
#'   for example `"{counts}\n{percent}"` to put the count and percentage on
#'   separate lines or `"n={counts} ({percent})"` for arbitrary layout. When
#'   `template` is set it overrides `type`; the set of placeholders in the
#'   template determines which values are computed.
#' @param annotations free-form per-region text rendered as a third
#'   stacked element below the quantity (or below the label when no
#'   quantity is drawn). Accepts a named character vector keyed by
#'   subset name (e.g. `c(A = "n = 12", "A&B" = "n = 3")`) as a
#'   shorthand for `list(labels = <vector>)`, or a list with `labels`
#'   plus [grid::gpar()] fields (`col`, `alpha`, `fontsize`, `cex`,
#'   `fontfamily`, `lineheight`, `font`, `rot`). Regions absent from
#'   `labels` are not annotated. The composite tag bbox grows to
#'   include the annotation, so exterior placement and leader lines
#'   adapt automatically. Defaults to slightly smaller text than
#'   `labels` / `quantities` (`cex = 0.8`).
#' @param strips a list, ignored unless the `'by'` argument
#'   was used in [euler()]. In addition to graphical parameters, this
#'   argument can include `labels = list(top = ..., left = ...)` for custom
#'   strip labels. Unnamed labels are interpreted in display order. Named
#'   labels are matched by factor levels and then reordered to display order.
#' @param bg a logical, character, or list controlling the background grob.
#'   Character values are interpreted as the background fill color.
#' @param complement a logical, character, or list controlling the
#'   container box and complement region for diagrams fit with
#'   `complement =` in [euler()]. `TRUE` (default) draws the container
#'   with a dashed outline (`lty = 2`), no fill, and the complement count
#'   inside the complement region. `FALSE` suppresses the container and
#'   its label entirely. A character value is treated as a fill color
#'   shorthand. A list accepts `fill`, `alpha`, `col`, `lty`, `lwd`,
#'   `lex` (outline + label gpar), `fontsize`, `cex`, `font`, `fontfamily`,
#'   `lineheight` (label only), and `label` (custom text — defaults to the
#'   complement count). Also accepts the same placement controls as
#'   `labels` (`placement`, `margin`, `tether`, `gap`, `leader`,
#'   `force_directed`, `elbow`)
#'   for the complement count label. Has no effect if the diagram was
#'   fit without `complement =`. Defaults can be set via
#'   `eulerr_options(complement = ...)`.
#' @param n number of vertices for the `edges` and `fills`
#' @param main a title for the plot in the form of a
#'   character, expression, list or something that can be
#'   sensibly converted to a label via [grDevices::as.graphicsAnnot()]. A
#'   list of length one can be provided, in which case its only element
#'   is used as the label. If a list of longer length is provided, an item
#'   named `'label'` must be provided (and will be used for the actual text).
#' @param ... parameters to update `fills` and `edges` with and thereby a shortcut
#'   to set these parameters
#'   [grid::grid.text()].
#' @param rotate a numeric value giving the angle in degrees by which to rotate
#'   the entire diagram layout. Positive values rotate counter-clockwise.
#'   Defaults to `0` (no rotation).
#' @param adjust_labels a logical. If `TRUE`, adjustment will be made to avoid
#'   overlaps or out-of-limits plotting of labels, quantities, and
#'   percentages.
#'
#' @seealso [euler()], [plot.eulergram()], [grid::gpar()],
#'   [grid::grid.polyline()], [grid::grid.path()],
#'   [grid::grid.legend()], [grid::grid.text()]
#'
#' @return Provides an object of class `'eulergram'` , which is a
#'   description of the diagram to be drawn. [plot.eulergram()] does the actual
#'   drawing of the diagram.
#' @export
#' @examples
#' fit <- euler(c("A" = 10, "B" = 5, "A&B" = 3))
#'
#' # Customize colors, remove borders, bump alpha, color labels white
#' plot(fit,
#'      fills = list(fill = c("red", "steelblue4"), alpha = 0.5),
#'      labels = list(col = "white", font = 4))
#'
#' # Add quantities to the plot
#' plot(fit, quantities = TRUE)
#'
#' # Add free-form per-region annotations below the counts
#' plot(
#'   fit,
#'   quantities = TRUE,
#'   annotations = c(A = "mean = 35", "A&B" = "mean = 41")
#' )
#'
#' # Add a custom legend and retain quantities
#' plot(fit, quantities = TRUE, legend = list(labels = c("foo", "bar")))
#'
#' # Plot without fills and distinguish sets with border types instead
#' plot(fit, fills = "transparent", lty = 1:2)
#'
#' # Save plot parameters to plot using some other method
#' diagram_description <- plot(fit)
#'
#' # Plots using 'by' argument
#' plot(euler(fruits[, 1:4], by = list(sex)), legend = TRUE)
#'
#' # Per-panel styling with `by_group`
#' plot(
#'   venn(fruits[, 1:4], by = list(sex)),
#'   quantities = list(
#'     by_group = list(
#'       male = list(col = "steelblue"),
#'       female = list(col = "tomato")
#'     )
#'   )
#' )
plot.euler <- function(
  x,
  fills = TRUE,
  patterns = FALSE,
  edges = TRUE,
  legend = FALSE,
  labels = identical(legend, FALSE),
  quantities = FALSE,
  annotations = NULL,
  strips = NULL,
  bg = FALSE,
  main = NULL,
  complement = TRUE,
  rotate = 0,
  n = 200L,
  adjust_labels = TRUE,
  ...
) {
  if (!missing(adjust_labels)) {
    warning("`adjust_labels` is deprecated and no longer has any effect.")
  }

  # retrieve default options
  opar <- eulerr_options()

  groups <- attr(x, "groups")
  dots <- list(...)

  do_custom_legend <- grid::is.grob(legend)

  do_fills <- !is_false(fills) && !is.null(fills)
  do_patterns <- !is_false(patterns) && !is.null(patterns)
  do_edges <- !is_false(edges) && !is.null(edges)
  do_labels <- !is_false(labels) && !is.null(labels)
  do_quantities <- !is_false(quantities) && !is.null(quantities)
  do_annotations <- !is_false(annotations) && !is.null(annotations)
  do_legend <- !is_false(legend) && !is.null(legend)
  do_groups <- !is.null(groups)
  do_strips <- !is_false(strips) && do_groups
  do_bg <- !is_false(bg) && !is.null(bg)
  do_main <- is.character(main) || is.expression(main) || is.list(main)

  has_container <- if (do_groups) {
    any(vapply(x, function(xi) !is.null(xi$container), logical(1)))
  } else {
    !is.null(x$container)
  }
  do_complement <- has_container &&
    !is_false(complement) &&
    !is.null(complement)

  # Extract per-panel overrides (`by_group`) from each styling parameter so the
  # existing normalization sees a clean list. These are re-applied later, once
  # per panel, inside the setup_grobs loop.
  panel_override_fields <- list(
    fills = c("fill", "alpha"),
    patterns = c("angle", "col", "lwd", "alpha"),
    edges = c("col", "alpha", "lex", "lwd", "lty"),
    labels = c(
      "col",
      "alpha",
      "fontsize",
      "cex",
      "fontfamily",
      "lineheight",
      "font",
      "rot"
    ),
    quantities = c(
      "col",
      "alpha",
      "fontsize",
      "cex",
      "fontfamily",
      "lineheight",
      "font",
      "rot"
    ),
    annotations = c(
      "col",
      "alpha",
      "fontsize",
      "cex",
      "fontfamily",
      "lineheight",
      "font",
      "rot"
    )
  )
  pop_by_group <- function(param, name) {
    if (!is.list(param) || is.null(param$by_group)) {
      return(list(param = param, by_group = NULL))
    }
    by_group <- param$by_group
    param$by_group <- NULL
    if (!do_groups) {
      stop(
        "`",
        name,
        "$by_group` requires a diagram fit with `by =`."
      )
    }
    if (!is.list(by_group) || length(by_group) == 0L) {
      stop("`", name, "$by_group` must be a non-empty named list.")
    }
    keys <- names(by_group)
    if (is.null(keys) || any(!nzchar(keys))) {
      stop("`", name, "$by_group` must be a fully named list.")
    }
    valid_keys <- names(x)
    unknown <- setdiff(keys, valid_keys)
    if (length(unknown) > 0L) {
      stop(
        "`",
        name,
        "$by_group` has unknown keys: ",
        paste(unknown, collapse = ", "),
        ". Valid keys: ",
        paste(valid_keys, collapse = ", "),
        "."
      )
    }
    allowed <- panel_override_fields[[name]]
    for (key in keys) {
      override <- by_group[[key]]
      if (!is.list(override)) {
        stop(
          "`",
          name,
          "$by_group$",
          key,
          "` must be a list."
        )
      }
      bad <- setdiff(names(override), allowed)
      if (length(bad) > 0L) {
        stop(
          "`",
          name,
          "$by_group$",
          key,
          "` contains fields that cannot be overridden per panel: ",
          paste(bad, collapse = ", "),
          ". Allowed fields: ",
          paste(allowed, collapse = ", "),
          "."
        )
      }
    }
    list(param = param, by_group = by_group)
  }

  fills_split <- pop_by_group(fills, "fills")
  fills <- fills_split$param
  fills_by_group <- fills_split$by_group

  patterns_split <- pop_by_group(patterns, "patterns")
  patterns <- patterns_split$param
  patterns_by_group <- patterns_split$by_group

  edges_split <- pop_by_group(edges, "edges")
  edges <- edges_split$param
  edges_by_group <- edges_split$by_group

  labels_split <- pop_by_group(labels, "labels")
  labels <- labels_split$param
  labels_by_group <- labels_split$by_group

  quantities_split <- pop_by_group(quantities, "quantities")
  quantities <- quantities_split$param
  quantities_by_group <- quantities_split$by_group

  annotations_split <- pop_by_group(annotations, "annotations")
  annotations <- annotations_split$param
  annotations_by_group <- annotations_split$by_group

  fills_user <- fills

  shapes <- if (do_groups) x[[1L]]$shapes else x$shapes

  n_e <- NROW(shapes)

  setnames <- rownames(shapes)
  setnames_orig <- setnames

  # Build a sparse combo_labels that places singletons (in input order) first,
  # then multi-set combos in cardinality + lexicographic order. This is the
  # working set of regions for the entire plot pipeline.
  if (do_groups) {
    all_labels <- unique(unlist(
      lapply(x, function(xi) names(xi$fitted.values)),
      use.names = FALSE
    ))
  } else {
    all_labels <- names(x$fitted.values)
  }
  singletons_present <- intersect(setnames_orig, all_labels)
  multi_labels <- setdiff(all_labels, singletons_present)
  multi_card <- lengths(strsplit(multi_labels, "&", fixed = TRUE))
  multi_labels <- multi_labels[order(multi_card, multi_labels)]
  combo_labels <- c(singletons_present, multi_labels)
  combo_sets <- strsplit(combo_labels, "&", fixed = TRUE)
  n_id <- length(combo_labels)

  # Sparse equivalent of the legacy bit_indexr `id` matrix: rows are populated
  # combinations (in `combo_labels` order), columns are sets (in `setnames_orig`
  # order). id[i, j] is TRUE iff combination i includes set j.
  if (n_id > 0L && n_e > 0L) {
    id <- t(vapply(
      combo_sets,
      function(s) setnames_orig %in% s,
      logical(n_e)
    ))
    if (n_id == 1L) {
      id <- matrix(id, nrow = 1L)
    }
    dimnames(id) <- list(combo_labels, setnames_orig)
  } else {
    id <- matrix(
      FALSE,
      nrow = n_id,
      ncol = n_e,
      dimnames = list(combo_labels, setnames_orig)
    )
  }

  align_fitted <- function(xi) {
    out <- xi$fitted.values[combo_labels]
    out[is.na(out)] <- 0
    out
  }

  if (do_groups) {
    res <- lapply(x, function(xi) is.na(xi$shapes$h))
    empty_sets <- apply(do.call(rbind, res), 2, all)

    empty_subsets <- if (any(empty_sets)) {
      rowSums(id[, empty_sets, drop = FALSE]) > 0
    } else {
      logical(n_id)
    }

    res <- lapply(x, function(xi) {
      fitted <- align_fitted(xi)[!empty_subsets]
      nonzero <- nonzero_fit(fitted)
      ifelse(is.na(nonzero), FALSE, nonzero)
    })

    nonzero <- apply(do.call(rbind, res), 2, any)
  } else {
    empty_sets <- is.na(x$shapes$h)
    empty_subsets <- if (any(empty_sets)) {
      rowSums(id[, empty_sets, drop = FALSE]) > 0
    } else {
      logical(n_id)
    }
    fitted <- align_fitted(x)[!empty_subsets]
    nonzero <- nonzero_fit(fitted)
    nonzero <- ifelse(is.na(nonzero), FALSE, nonzero)
  }

  merged_sets <- rep(FALSE, length(setnames))

  if (!do_groups && any(nonzero)) {
    n_overlaps <- integer(n_id)
    single_mass <- logical(n_e)
    nz <- nonzero_fit(align_fitted(x))

    for (i in seq_len(n_e)) {
      nzi <- nz & id[, i]

      if (sum(nzi) == 1) {
        n_overlaps[which(nzi)] <- n_overlaps[which(nzi)] + 1
      }

      single_mass[i] <- sum(nzi) == 1
    }

    complete_overlaps <- n_overlaps > 1
    co_idx <- which(complete_overlaps)

    if (length(co_idx) > 0L) {
      for (idx in co_idx) {
        merge_sets <- id[idx, ] & single_mass
        if (any(merge_sets)) {
          setnames[merge_sets] <- paste(setnames[merge_sets], collapse = ",")
          merged_sets[which(merge_sets)[length(which(merge_sets))]] <- TRUE
        }
      }
    }
  }

  stopifnot(n > 0, is.numeric(n) && length(n) == 1)

  fills_out <- NULL

  # setup fills
  if (do_fills) {
    fills_out <- replace_list(
      list(
        fill = opar$fills$fill,
        alpha = opar$fills$alpha,
        mode = opar$fills$mode
      ),
      if (is.list(fills)) {
        fills
      } else if (isTRUE(fills)) {
        list()
      } else {
        list(fill = fills)
      }
    )
    fills_out <- replace_list(fills_out, dots)
    fills_out$col <- "transparent"

    if (is.function(fills_out$fill)) {
      fills_out$fill <- fills_out$fill(n_e)
    }

    fill_names <- names(fills_out$fill)
    if (!is.null(fill_names)) {
      all_named <- all(nzchar(fill_names))
      any_named <- any(nzchar(fill_names))
      if (any_named && !all_named) {
        stop("`fills$fill` must be either fully named or fully unnamed.")
      }
      if (all_named) {
        if (!fills_out$mode %in% c("disjoint", "union")) {
          stop("`fills$mode` must be either 'disjoint' or 'union'.")
        }
        subset_names <- rownames(id)
        valid_fill_names <- c(setnames, subset_names)
        unknown <- setdiff(fill_names, valid_fill_names)
        if (length(unknown) > 0L) {
          stop(
            "`fills$fill` has unknown names: ",
            paste(unknown, collapse = ", ")
          )
        }

        default_fill <- opar$fills$fill
        if (is.function(default_fill)) {
          default_fill <- default_fill(n_e)
        }
        n_default <- length(default_fill)
        if (n_default == n_e && n_default != n_id) {
          per_set <- default_fill
          default_fill <- character(n_id)
          for (ii in seq_len(n_id)) {
            set_idx <- which(id[ii, ])
            if (length(set_idx) == 1L) {
              default_fill[ii] <- per_set[set_idx]
            } else if (length(set_idx) > 1L) {
              default_fill[ii] <- mix_colors(per_set[set_idx])
            }
          }
        } else if (n_default == 1L || n_default == n_id) {
          default_fill <- rep_len(default_fill, n_id)
        } else {
          stop("Default `fills$fill` must have length 1, n_sets, or n_subsets.")
        }

        fill_map <- default_fill
        names(fill_map) <- subset_names
        named_sets <- intersect(fill_names, setnames)
        if (identical(fills_out$mode, "union") && length(named_sets) > 0L) {
          for (set_name in named_sets) {
            fill_map[id[, set_name]] <- fills_out$fill[[set_name]]
          }
        }
        named_subsets <- intersect(fill_names, subset_names)
        fill_map[named_subsets] <- unname(fills_out$fill[named_subsets])
        fills_out$fill <- fill_map
      }
    }

    n_fills <- length(fills_out$fill)
    if (n_fills == n_e && n_fills != n_id) {
      per_set <- fills_out$fill
      fills_out$fill <- character(n_id)
      for (i in seq_len(n_id)) {
        set_idx <- which(id[i, ])
        if (length(set_idx) == 1L) {
          fills_out$fill[i] <- per_set[set_idx]
        } else if (length(set_idx) > 1L) {
          fills_out$fill[i] <- mix_colors(per_set[set_idx])
        }
      }
    } else if (!(n_fills %in% c(1L, n_id))) {
      stop("`fills$fill` must have length 1, n_sets, or n_subsets.")
    }
    fills <- list()
    fills_gp <- fills_out
    fills_gp$mode <- NULL
    fills$gp <- setup_gpar(fills_gp, list(), n_id)
  } else {
    fills <- NULL
  }

  legacy_patterns <- list(
    type = if (is.list(fills_user) && !is.null(fills_user$pattern)) {
      fills_user$pattern
    } else {
      NULL
    },
    angle = if (is.list(fills_user) && !is.null(fills_user$angle)) {
      fills_user$angle
    } else {
      NULL
    },
    col = if (is.list(fills_user) && !is.null(fills_user$pattern_col)) {
      fills_user$pattern_col
    } else {
      NULL
    },
    lwd = if (is.list(fills_user) && !is.null(fills_user$pattern_lwd)) {
      fills_user$pattern_lwd
    } else {
      NULL
    }
  )
  do_legacy_patterns <- any(!vapply(legacy_patterns, is.null, logical(1)))
  do_patterns <- do_patterns || do_legacy_patterns

  # setup patterns
  if (do_patterns && !do_fills) {
    stop("`patterns` requires `fills` to be enabled.")
  }

  if (do_patterns) {
    patterns_out <- replace_list(
      list(
        type = opar$patterns$type,
        angle = opar$patterns$angle,
        col = opar$patterns$col,
        lwd = opar$patterns$lwd,
        alpha = opar$patterns$alpha,
        mode = opar$patterns$mode
      ),
      if (is.list(patterns)) {
        patterns
      } else if (isTRUE(patterns)) {
        list(type = "stripes")
      } else {
        list(type = patterns)
      }
    )

    if (do_legacy_patterns) {
      warning(
        "`fills$pattern`, `fills$angle`, `fills$pattern_col`, and ",
        "`fills$pattern_lwd` are deprecated; use `patterns` instead."
      )
      if (!is.null(legacy_patterns$type)) {
        patterns_out$type <- legacy_patterns$type
      }
      if (!is.null(legacy_patterns$angle)) {
        patterns_out$angle <- legacy_patterns$angle
      }
      if (!is.null(legacy_patterns$col)) {
        patterns_out$col <- legacy_patterns$col
      }
      if (!is.null(legacy_patterns$lwd)) patterns_out$lwd <- legacy_patterns$lwd
    }

    patterns_out <- replace_list(patterns_out, dots)

    pattern_type_names <- names(patterns_out$type)
    if (!is.null(pattern_type_names)) {
      all_named <- all(nzchar(pattern_type_names))
      any_named <- any(nzchar(pattern_type_names))
      if (any_named && !all_named) {
        stop("`patterns$type` must be either fully named or fully unnamed.")
      }
      if (all_named) {
        if (!patterns_out$mode %in% c("disjoint", "union")) {
          stop("`patterns$mode` must be either 'disjoint' or 'union'.")
        }
        subset_names <- rownames(id)
        valid_pattern_names <- c(setnames, subset_names)
        unknown <- setdiff(pattern_type_names, valid_pattern_names)
        if (length(unknown) > 0L) {
          stop(
            "`patterns$type` has unknown names: ",
            paste(unknown, collapse = ", ")
          )
        }
        named_sets <- intersect(pattern_type_names, setnames)
        named_subsets <- intersect(pattern_type_names, subset_names)

        if (
          identical(patterns_out$mode, "union") &&
            all(pattern_type_names %in% setnames)
        ) {
          set_default <- rep_len(opar$patterns$type, n_e)
          names(set_default) <- setnames
          set_default[named_sets] <- unname(patterns_out$type[named_sets])
          patterns_out$type <- unname(set_default)
        } else {
          default_type <- opar$patterns$type
          n_default <- length(default_type)
          if (n_default == n_e && n_default != n_id) {
            per_set <- default_type
            default_type <- rep(NA_character_, n_id)
            for (ii in seq_len(n_id)) {
              set_idx <- which(id[ii, ])
              if (length(set_idx) == 1L) {
                default_type[ii] <- per_set[set_idx]
              }
            }
          } else if (n_default == 1L || n_default == n_id) {
            default_type <- rep_len(default_type, n_id)
          } else {
            stop(
              "Default `patterns$type` must have length 1, n_sets, or n_subsets."
            )
          }

          type_map <- default_type
          names(type_map) <- subset_names
          if (
            identical(patterns_out$mode, "union") && length(named_sets) > 0L
          ) {
            for (set_name in named_sets) {
              type_map[id[, set_name]] <- patterns_out$type[[set_name]]
            }
          }
          type_map[named_subsets] <- unname(patterns_out$type[named_subsets])
          patterns_out$type <- type_map
        }
      }
    }

    expand_pattern_param <- function(x, name, by_set, default = NULL) {
      n_x <- length(x)
      if (!(n_x %in% c(1L, n_e, n_id))) {
        stop("`patterns$", name, "` must have length 1, n_sets, or n_subsets.")
      }
      if (n_x == 1L || n_x == n_id) {
        return(rep_len(x, n_id))
      }

      out <- vector(mode = mode(x), length = n_id)
      for (ii in seq_len(n_id)) {
        active <- which(id[ii, ] & by_set)
        if (length(active) == 0L) {
          if (is.null(default)) {
            out[ii] <- x[1L]
          } else if (is.character(default)) {
            out[ii] <- default[1L]
          } else {
            out[ii] <- as.vector(default)[1L]
          }
        } else if (is.character(x)) {
          out[ii] <- if (length(active) == 1L) {
            x[active]
          } else {
            mix_colors(x[active])
          }
        } else {
          out[ii] <- mean(x[active])
        }
      }
      out
    }

    n_types <- length(patterns_out$type)
    if (!(n_types %in% c(1L, n_e, n_id))) {
      stop("`patterns$type` must have length 1, n_sets, or n_subsets.")
    }

    if (n_types %in% c(1L, n_id)) {
      pattern_mode <- "intersection"
      patterns_out$type <- rep_len(patterns_out$type, n_id)
      patterns_out$angle <- expand_pattern_param(
        patterns_out$angle,
        "angle",
        rep(TRUE, n_e)
      )
      patterns_out$col <- expand_pattern_param(
        patterns_out$col,
        "col",
        rep(TRUE, n_e),
        default = NA_character_
      )
      patterns_out$lwd <- expand_pattern_param(
        patterns_out$lwd,
        "lwd",
        rep(TRUE, n_e)
      )
      patterns_out$alpha <- expand_pattern_param(
        patterns_out$alpha,
        "alpha",
        rep(TRUE, n_e)
      )
    } else {
      pattern_mode <- "shape"
      patterns_out$type <- tolower(patterns_out$type)
      patterns_out$angle <- rep_len(patterns_out$angle, n_e)
      patterns_out$col <- rep_len(patterns_out$col, n_e)
      patterns_out$lwd <- rep_len(patterns_out$lwd, n_e)
      patterns_out$alpha <- rep_len(patterns_out$alpha, n_e)
    }

    patterns <- list()
    patterns$mode <- pattern_mode

    if (pattern_mode == "intersection") {
      patterns_out$type[is.na(patterns_out$type)] <- "none"
      patterns_out$type <- tolower(patterns_out$type)

      if (any(!patterns_out$type %in% c("none", "stripes"))) {
        stop("`patterns$type` must be one of: 'stripes', NA.")
      }

      if (all(is.na(patterns_out$col))) {
        patterns_out$col <- fills$gp$fill
      } else {
        na_col <- is.na(patterns_out$col) | patterns_out$col == "NA"
        patterns_out$col[na_col] <- fills$gp$fill[na_col]
      }

      patterns_gp <- patterns_out
      patterns_gp$mode <- NULL
      patterns$gp <- setup_gpar(patterns_gp, list(), n_id)
    } else {
      patterns_out$type[is.na(patterns_out$type)] <- "none"
      if (any(!patterns_out$type %in% c("none", "stripes"))) {
        stop("`patterns$type` must be one of: 'stripes', NA.")
      }
      if (all(is.na(patterns_out$col))) {
        patterns_out$col <- fills$gp$fill[seq_len(n_e)]
      } else {
        na_col <- is.na(patterns_out$col) | patterns_out$col == "NA"
        patterns_out$col[na_col] <- fills$gp$fill[which(na_col)]
      }
      patterns_set_gp <- patterns_out
      patterns_set_gp$mode <- NULL
      patterns$set_gp <- setup_gpar(patterns_set_gp, list(), n_e)
      patterns$gp <- patterns$set_gp
    }
  } else {
    patterns <- NULL
  }

  # setup edges
  if (do_edges) {
    if (isTRUE(edges)) {
      edges_out <- list()
    } else if (!is.list(edges)) {
      edges_out <- list(col = edges)
    } else {
      edges_out <- edges
    }

    edges <- list()
    edges$gp <- setup_gpar(
      list(
        col = opar$edges$col,
        alpha = opar$edges$alpha,
        lex = opar$edges$lex,
        lwd = opar$edges$lwd,
        lty = opar$edges$lty
      ),
      update_list(edges_out, dots),
      n_e
    )
  } else {
    edges <- NULL
  }

  # setup strips
  if (do_groups) {
    group_names <- lapply(groups, levels)
    n_levels <- sum(lengths(group_names))
  }

  if (do_strips) {
    strip_labels <- NULL
    if (is.list(strips) && "labels" %in% names(strips)) {
      strip_labels <- strips$labels
      strips$labels <- NULL

      if (!is.null(strip_labels)) {
        if (!is.list(strip_labels)) {
          stop(
            "`strips$labels` must be a list with optional `top` and `left` entries."
          )
        }
        strip_label_names <- names(strip_labels)
        if (is.null(strip_label_names) || any(!nzchar(strip_label_names))) {
          stop(
            "`strips$labels` must be a named list with optional `top` and `left` entries."
          )
        }
        bad_labels <- setdiff(strip_label_names, c("top", "left"))
        if (length(bad_labels) > 0L) {
          stop("`strips$labels` only supports `top` and `left` entries.")
        }
      }
    }

    strips <- list(
      gp = setup_gpar(opar$strips, strips, n_levels),
      groups = groups,
      labels = strip_labels
    )
  } else {
    strips <- NULL
  }

  # setup labels
  if (do_labels) {
    if (is.list(labels)) {
      labels <- update_list(
        list(labels = setnames, rot = opar$labels$rot),
        labels
      )
    } else if (isTRUE(labels)) {
      labels <- list(labels = setnames, rot = opar$labels$rot)
    } else {
      labels <- list(labels = labels, rot = opar$labels$rot)
    }

    # Pull placement-related fields out so they don't leak into gpar.
    placement_fields <- c(
      "placement",
      "margin",
      "tether",
      "gap",
      "leader",
      "force_directed",
      "elbow"
    )
    placement_user <- labels[intersect(names(labels), placement_fields)]
    labels[placement_fields] <- NULL

    labels$rot <- rep_len(labels$rot, n_e)
    labels$gp <- setup_gpar(
      list(
        col = opar$labels$col,
        alpha = opar$labels$alpha,
        fontsize = opar$labels$fontsize,
        cex = opar$labels$cex,
        fontfamily = opar$labels$fontfamily,
        lineheight = opar$labels$lineheight,
        font = opar$labels$font
      ),
      labels,
      n_e
    )

    labels$placement <- placement_user$placement %||%
      opar$labels$placement
    labels$margin <- placement_user$margin %||% opar$labels$margin
    labels$tether <- placement_user$tether %||% opar$labels$tether
    labels$gap <- placement_user$gap %||% opar$labels$gap
    labels$force_directed <- update_list(
      opar$labels$force_directed %||% list(),
      placement_user$force_directed %||% list()
    )
    labels$elbow <- update_list(
      opar$labels$elbow %||% list(),
      placement_user$elbow %||% list()
    )
    labels$leader <- update_list(
      opar$labels$leader %||% list(),
      placement_user$leader %||% list()
    )
  } else {
    labels <- NULL
  }

  # setup quantities
  if (do_quantities) {
    normalize_quantity_formatter <- function(formatter) {
      if (is.null(formatter)) {
        formatter <- list()
      }

      if (!is.list(formatter)) {
        stop("`quantities$format` must be a list.")
      }

      if (is.null(formatter$fun)) {
        fun <- NULL
      } else {
        fun <- formatter$fun
      }

      if (!is.null(fun) && !is.function(fun)) {
        stop("`quantities$format$fun` must be a function.")
      }

      args <- formatter[setdiff(names(formatter), "fun")]
      list(fun = fun, args = args)
    }

    if (is.list(quantities)) {
      if (!is.null(quantities$type)) {
        quantities$type[quantities$type == "numbers"] <- "counts"
        if (!all(quantities$type %in% c("counts", "percent", "fraction"))) {
          stop(
            "'type' must be one or more of 'counts', 'percent', and 'fraction'"
          )
        }

        quantities_type <- match.arg(
          quantities$type,
          c("counts", "percent", "fraction"),
          several.ok = TRUE
        )
      } else {
        quantities_type <- opar$quantities$type
      }

      quantities <- update_list(
        list(
          labels = NULL,
          type = quantities_type,
          template = opar$quantities$template,
          rot = opar$quantities$rot,
          format = NULL,
          total = NULL
        ),
        quantities
      )
    } else if (isTRUE(quantities)) {
      quantities <- list(
        labels = NULL,
        type = opar$quantities$type,
        template = opar$quantities$template,
        rot = opar$quantities$rot,
        format = NULL,
        total = NULL
      )
    } else {
      quantities <- list(
        labels = quantities,
        type = opar$quantities$type,
        template = opar$quantities$template,
        rot = opar$quantities$rot,
        format = NULL,
        total = NULL
      )
    }

    quantities$type[quantities$type == "numbers"] <- "counts"
    quantities$format <- normalize_quantity_formatter(quantities$format)

    if (!is.null(quantities$labels)) {
      lbl_names <- names(quantities$labels)
      if (!is.null(lbl_names)) {
        all_named <- all(nzchar(lbl_names))
        any_named <- any(nzchar(lbl_names))
        if (any_named && !all_named) {
          stop(
            "`quantities$labels` must be either fully named or fully unnamed."
          )
        }
      }
    }

    if (
      !is.null(quantities$total) &&
        (!is.numeric(quantities$total) ||
          length(quantities$total) != 1 ||
          is.na(quantities$total) ||
          quantities$total <= 0)
    ) {
      stop("`quantities$total` must be a single positive number.")
    }

    quantities$rot <- rep_len(quantities$rot, n_id)

    quantities$gp <- setup_gpar(
      list(
        col = opar$quantities$col,
        alpha = opar$quantities$alpha,
        fontsize = opar$quantities$fontsize,
        cex = opar$quantities$cex,
        fontfamily = opar$quantities$fontfamily,
        lineheight = opar$quantities$lineheight,
        font = opar$quantities$font
      ),
      quantities,
      n_id
    )
  } else {
    quantities <- NULL
  }

  # setup annotations
  if (do_annotations) {
    if (is.list(annotations)) {
      annotations <- update_list(
        list(labels = NULL, rot = opar$annotations$rot),
        annotations
      )
    } else if (isTRUE(annotations)) {
      annotations <- list(labels = NULL, rot = opar$annotations$rot)
    } else {
      annotations <- list(labels = annotations, rot = opar$annotations$rot)
    }

    if (!is.null(annotations$labels)) {
      if (!is.character(annotations$labels)) {
        stop("`annotations$labels` must be a character vector.")
      }
      lbl_names <- names(annotations$labels)
      if (is.null(lbl_names) || any(!nzchar(lbl_names))) {
        stop(
          "`annotations$labels` must be a fully named character vector keyed by subset (e.g. `c(A = \"...\", \"A&B\" = \"...\")`)."
        )
      }
    }

    annotations$rot <- rep_len(annotations$rot, n_id)

    annotations$gp <- setup_gpar(
      list(
        col = opar$annotations$col,
        alpha = opar$annotations$alpha,
        fontsize = opar$annotations$fontsize,
        cex = opar$annotations$cex,
        fontfamily = opar$annotations$fontfamily,
        lineheight = opar$annotations$lineheight,
        font = opar$annotations$font
      ),
      annotations,
      n_id
    )
  } else {
    annotations <- NULL
  }

  # setup legend
  if (do_custom_legend) {
    legend <- legend
  } else if (do_legend) {
    # TODO: create a better, custom legend

    legend <- update_list(
      list(
        labels = setnames[!empty_sets & !merged_sets],
        side = opar$legend$side,
        nrow = sum(!empty_sets),
        ncol = 1L,
        byrow = opar$legend$byrow,
        do.lines = opar$legend$do.lines,
        lines.first = opar$legend$lines.first,
        hgap = opar$legend$hgap,
        vgap = opar$legend$vgap,
        default.units = opar$legend$default.units,
        pch = opar$legend$pch,
        symbol_size = opar$legend$symbol_size
      ),
      legend
    )

    symbol_size <- if (!is.null(legend$symbol_size)) {
      legend$symbol_size
    } else {
      opar$legend$cex
    }
    legend$symbol_size <- NULL

    legend$gp <- setup_gpar(
      list(
        fill = if (do_fills) {
          fills$gp$fill[!empty_sets & !merged_sets]
        } else {
          "transparent"
        },
        alpha = if (do_fills) {
          fills$gp$alpha[!empty_sets & !merged_sets]
        } else if (do_edges) {
          edges$gp$alpha[!empty_sets & !merged_sets]
        } else {
          0
        },
        cex = opar$legend$cex,
        fontsize = opar$legend$fontsize,
        font = opar$legend$font,
        fontfamily = opar$legend$fontfamily,
        lwd = if (do_edges) edges$gp$lwd[!empty_sets & !merged_sets] else 0,
        lex = if (do_edges) edges$gp$lex[!empty_sets & !merged_sets] else 0,
        col = if (do_edges) {
          edges$gp$col[!empty_sets & !merged_sets]
        } else {
          "transparent"
        },
        pattern_type = if (do_patterns) {
          patterns$gp$type[!empty_sets & !merged_sets]
        } else {
          "none"
        },
        pattern_angle = if (do_patterns) {
          patterns$gp$angle[!empty_sets & !merged_sets]
        } else {
          45
        },
        pattern_col = if (do_patterns) {
          patterns$gp$col[!empty_sets & !merged_sets]
        } else {
          "transparent"
        },
        pattern_lwd = if (do_patterns) {
          patterns$gp$lwd[!empty_sets & !merged_sets]
        } else {
          0
        },
        pattern_alpha = if (do_patterns) {
          patterns$gp$alpha[!empty_sets & !merged_sets]
        } else {
          0
        }
      ),
      legend,
      sum(!empty_sets)
    )
  } else {
    legend <- NULL
  }

  if (do_main) {
    if (is.list(main)) {
      if (length(main) == 1 && is.null(names(main))) {
        label <- main[[1]]
      } else {
        label <- main$label
      }

      if (is.null(label)) {
        stop("you need to provide a 'label' item if 'main' is a list.")
      }
    } else {
      label <- main
    }

    main <- update_list(
      list(
        label = label,
        x = opar$main$x,
        y = opar$main$y,
        just = opar$main$just,
        hjust = opar$main$hjust,
        vjust = opar$main$vjust,
        rot = opar$main$rot,
        check.overlap = opar$main$check.overlap,
        default.units = opar$main$default.units
      ),
      main
    )

    main$gp <- setup_gpar(
      list(
        cex = opar$main$cex,
        fontsize = opar$main$fontsize,
        font = opar$main$font,
        fontfamily = opar$main$fontfamily,
        col = opar$main$col,
        lineheight = opar$main$lineheight,
        alpha = opar$main$alpha
      ),
      main,
      1
    )
  } else {
    main <- NULL
  }

  if (do_bg) {
    if (isTRUE(bg)) {
      bg <- list()
    } else if (!is.list(bg)) {
      bg <- list(fill = bg)
    }

    bg_grob <- grid::rectGrob(
      gp = setup_gpar(
        list(fill = "white", col = "transparent", alpha = 1),
        bg,
        1
      ),
      name = "bg.grob"
    )
  }

  # setup complement (container box + complement count label) styling.
  # The label text override `label` lives outside the gpar list so it isn't
  # passed to grid::gpar.
  if (do_complement) {
    complement_user <- if (is.list(complement)) {
      complement
    } else if (isTRUE(complement)) {
      list()
    } else {
      list(fill = complement)
    }
    complement_label <- complement_user$label
    complement_user$label <- NULL

    # Pull placement fields out so they don't leak into gpar.
    cplace_fields <- c(
      "placement",
      "margin",
      "tether",
      "gap",
      "leader",
      "force_directed",
      "elbow"
    )
    cplace_user <- complement_user[intersect(
      names(complement_user),
      cplace_fields
    )]
    complement_user[cplace_fields] <- NULL

    complement <- list(label = complement_label)
    complement$gp <- setup_gpar(
      list(
        fill = opar$complement$fill,
        alpha = opar$complement$alpha,
        col = opar$complement$col,
        lty = opar$complement$lty,
        lwd = opar$complement$lwd,
        lex = opar$complement$lex,
        fontsize = opar$complement$fontsize,
        cex = opar$complement$cex,
        font = opar$complement$font,
        fontfamily = opar$complement$fontfamily,
        lineheight = opar$complement$lineheight
      ),
      complement_user,
      1
    )
    complement$placement <- cplace_user$placement %||%
      opar$complement$placement
    complement$margin <- cplace_user$margin %||%
      opar$complement$margin
    complement$tether <- cplace_user$tether %||%
      opar$complement$tether
    complement$gap <- cplace_user$gap %||% opar$complement$gap
    complement$force_directed <- update_list(
      opar$complement$force_directed %||% list(),
      cplace_user$force_directed %||% list()
    )
    complement$elbow <- update_list(
      opar$complement$elbow %||% list(),
      cplace_user$elbow %||% list()
    )
    complement$leader <- update_list(
      opar$complement$leader %||% list(),
      cplace_user$leader %||% list()
    )
  } else {
    complement <- NULL
  }

  # apply layout rotation if requested
  if (rotate != 0) {
    theta <- rotate * pi / 180
    ct <- cos(theta)
    st <- sin(theta)
    # Rotation only updates `phi` for ellipse-like shapes — rectangles and
    # squares are axis-aligned in eunoia, so there's no in-plane rotation
    # angle to advance. The geometry itself is still rotated via the (h, k)
    # update, but the result rotates the centers without re-orienting the
    # boxes; this matches eunoia's axis-aligned model.
    rotate_shapes <- function(dd) {
      h_new <- dd$h * ct - dd$k * st
      dd$k <- dd$h * st + dd$k * ct
      dd$h <- h_new
      if (NROW(dd) > 0L) {
        type <- dd$type[1L]
        if (type %in% c("circle", "ellipse")) {
          dd$phi <- dd$phi + theta
        }
      }
      dd
    }
    rotate_ellipse_frame <- function(dd) {
      if (is.null(dd) || NROW(dd) == 0L) {
        return(dd)
      }
      h_new <- dd$h * ct - dd$k * st
      dd$k <- dd$h * st + dd$k * ct
      dd$h <- h_new
      dd$phi <- dd$phi + theta
      dd
    }
    if (do_groups) {
      x <- lapply(x, function(xi) {
        xi$shapes <- rotate_shapes(xi$shapes)
        if (!is.null(xi$ellipses)) {
          xi$ellipses <- rotate_ellipse_frame(xi$ellipses)
        }
        xi
      })
    } else {
      x$shapes <- rotate_shapes(x$shapes)
      if (!is.null(x$ellipses)) {
        x$ellipses <- rotate_ellipse_frame(x$ellipses)
      }
    }
  }

  # set up geometry for diagrams. Per-strategy sublists collapse to the
  # flat shape that `place_euler_labels` (and `apply_label_placement`'s
  # `placement_opts`) expect: `iterations` is only forwarded when
  # `force_directed` is the active strategy, `min_gap` only when `elbow`
  # is.
  strat <- labels$placement
  fd_opts <- labels$force_directed %||% list()
  el_opts <- labels$elbow %||% list()
  placement_opts <- list(
    placement = strat,
    margin = labels$margin,
    iterations = if (identical(strat, "force_directed")) {
      fd_opts$iterations
    } else {
      NULL
    },
    min_gap = if (identical(strat, "elbow")) el_opts$min_gap else NULL,
    tether = labels$tether,
    gap = labels$gap
  )
  if (do_groups) {
    data <- lapply(
      x,
      setup_geometry,
      fills = fills,
      edges = edges,
      labels = labels,
      quantities = quantities,
      annotations = annotations,
      n = n,
      merged_sets = merged_sets,
      placement_opts = placement_opts,
      do_complement_label = isTRUE(do_complement)
    )
  } else {
    data <- setup_geometry(
      x,
      fills = fills,
      edges = edges,
      labels = labels,
      quantities = quantities,
      annotations = annotations,
      n = n,
      merged_sets = merged_sets,
      placement_opts = placement_opts,
      do_complement_label = isTRUE(do_complement)
    )
  }

  # start setting up grobs

  if (do_groups) {
    n_groups <- length(data)
    group_keys <- names(x)
    panel_override <- function(by_group, key) {
      if (is.null(by_group)) {
        return(NULL)
      }
      by_group[[key]]
    }
    euler_grob_children <- grid::gList()
    for (i in seq_len(n_groups)) {
      key_i <- group_keys[i]
      fills_i <- apply_panel_overrides(
        fills,
        panel_override(fills_by_group, key_i)
      )
      patterns_i <- apply_panel_overrides(
        patterns,
        panel_override(patterns_by_group, key_i)
      )
      edges_i <- apply_panel_overrides(
        edges,
        panel_override(edges_by_group, key_i)
      )
      labels_i <- apply_panel_overrides(
        labels,
        panel_override(labels_by_group, key_i)
      )
      quantities_i <- apply_panel_overrides(
        quantities,
        panel_override(quantities_by_group, key_i)
      )
      annotations_i <- apply_panel_overrides(
        annotations,
        panel_override(annotations_by_group, key_i)
      )
      euler_grob_children[[i]] <- setup_grobs(
        data[[i]],
        fills = fills_i,
        patterns = patterns_i,
        edges = edges_i,
        labels = labels_i,
        quantities = quantities_i,
        annotations = annotations_i,
        complement = complement,
        number = i,
        merged_sets = merged_sets,
        n_vertices = as.integer(n),
        placement_opts = placement_opts
      )
    }
    euler_grob <- grid::gTree(
      grid::nullGrob(),
      name = "canvas.grob",
      children = euler_grob_children
    )
    pos <- vapply(groups, as.numeric, numeric(NROW(groups)), USE.NAMES = FALSE)
    layout <- lengths(lapply(groups, unique))
    if (length(layout) == 1L) {
      layout <- c(1L, layout)
    }
    xlim <- range(unlist(lapply(data, "[[", "xlim")))
    ylim <- range(unlist(lapply(data, "[[", "ylim")))
  } else {
    euler_grob <- setup_grobs(
      data,
      fills = fills,
      patterns = patterns,
      edges = edges,
      labels = labels,
      quantities = quantities,
      annotations = annotations,
      complement = complement,
      number = 1,
      merged_sets = merged_sets,
      n_vertices = as.integer(n),
      placement_opts = placement_opts
    )
    euler_grob <- grid::grobTree(euler_grob, name = "canvas.grob")

    xlim <- data$xlim
    ylim <- data$ylim
    pos <- c(1L, 1L)
    layout <- c(1L, 1L)
  }

  xrng <- abs(xlim[1L] - xlim[2L])
  yrng <- abs(ylim[1L] - ylim[2L])

  if (xrng == 0 || yrng == 0) {
    xrng <- yrng <- 1
    ylim <- xlim <- c(0, 1)
  }

  ar <- xrng / yrng

  # Reserve a fixed physical margin inside the canvas so shape outlines
  # (which straddle the geometric boundary by half a stroke width) aren't
  # clipped by the canvas edge. The narrower data axis gets the minimum
  # `panel_pad`; the wider axis is scaled up so the resulting inset
  # preserves the data aspect ratio (otherwise circles would render
  # slightly elliptical).
  panel_pad <- grid::unit(2, "pt")
  if (ar >= 1) {
    pad_x <- panel_pad * ar
    pad_y <- panel_pad
  } else {
    pad_x <- panel_pad
    pad_y <- panel_pad / ar
  }
  # adjust <- layout[1L]/layout[2]

  do_strip_left <- layout[1L] > 1L && do_strips
  do_strip_top <- layout[2L] > 1L && do_strips

  strip_top_row <- strip_top_col <- strip_left_row <- strip_left_col <- 1

  nrow <- ncol <- 1
  heights <- grid::unit(1, "null")
  widths <- grid::unit(1 * ar * layout[2] / layout[1], "null")
  diagram_col <- 1
  diagram_row <- 1

  if (do_main) {
    diagram_row <- diagram_row + 2
    nrow <- nrow + 2
    strip_left_row <- strip_left_row + 2
    strip_top_row <- strip_top_row + 2
  }

  if (do_strip_left) {
    diagram_col <- diagram_col + 1
    ncol <- ncol + 1
  }
  if (do_strip_top) {
    diagram_row <- diagram_row + 1
    nrow <- nrow + 1
  }

  if (do_strip_left && do_strip_top) {
    strip_top_col <- strip_top_col + 1
    strip_left_row <- strip_left_row + 1
  }

  # draw strips
  if (do_strip_top) {
    strip_top_vp <-
      grid::viewport(
        layout.pos.row = strip_top_row,
        layout.pos.col = strip_top_col,
        name = "strip.top.vp",
        layout = grid::grid.layout(nrow = 1, ncol = layout[2])
      )

    top_group_name <- names(layout)[[2L]]
    top_levels <- levels(strips$groups[[top_group_name]])
    lvls <- resolve_strip_labels(
      labels = strips$labels$top,
      levels = top_levels,
      display_levels = top_levels,
      axis = "top"
    )
    n_lvls <- length(lvls)
    step <- 1 / n_lvls

    strip_top_grob <- grid::textGrob(
      lvls,
      x = step / 2 + (seq(0, n_lvls - 1) * step),
      name = "strip.top.grob",
      gp = do.call(grid::gpar, strips$gp),
      vp = strip_top_vp
    )

    heights <- grid::unit.c(
      grid::unit(2, "grobheight", list(strip_top_grob)),
      heights
    )
  }

  if (do_strip_left) {
    strip_left_vp <-
      grid::viewport(
        layout.pos.row = strip_left_row,
        layout.pos.col = strip_left_col,
        name = "strip.left.vp",
        layout = grid::grid.layout(nrow = layout[1], ncol = 1)
      )

    left_group_name <- names(layout)[[1L]]
    left_levels <- levels(strips$groups[[left_group_name]])
    lvls <- resolve_strip_labels(
      labels = strips$labels$left,
      levels = left_levels,
      display_levels = rev(left_levels),
      axis = "left"
    )
    n_lvls <- length(lvls)
    step <- 1 / n_lvls

    strip_left_grob <- grid::textGrob(
      lvls,
      y = step / 2 + (seq(0, n_lvls - 1) * step),
      name = "strip.left.grob",
      rot = 90,
      gp = do.call(grid::gpar, strips$gp),
      vp = strip_left_vp
    )

    widths <- grid::unit.c(
      grid::unit(2, "grobwidth", list(strip_left_grob)),
      widths
    )
  }

  if (do_legend) {
    if (do_custom_legend) {
      legend_grob <- legend
      legend <- list(side = "right")
    } else {
      legend_grob <- euler_legend_grob(
        labels = legend$labels,
        gp = legend$gp,
        symbol_size = symbol_size,
        hgap = legend$hgap,
        vgap = legend$vgap,
        ncol = legend$ncol,
        nrow = legend$nrow,
        byrow = legend$byrow
      )
    }

    legend_grob$name <- "legend.grob"
    if (do_strip_top) {
      legend_row <- 2 + 2 * do_main
    }

    if (legend$side == "right") {
      # legend on right (default)
      ncol <- ncol + 2L
      legend_row <- nrow
      legend_col <- ncol
      widths <- grid::unit.c(
        widths,
        grid::unit(c(1, 1), c("lines", "grobwidth"), list(NULL, legend_grob))
      )
    } else if (legend$side == "left") {
      # legend on left
      ncol <- ncol + 2L
      legend_row <- if (do_strip_top) 2L + 2 * do_main else 1L + 2 * do_main
      legend_col <- 1
      diagram_col <- diagram_col + 2L
      if (do_strip_left) {
        strip_left_col <- strip_left_col + 2L
      }
      if (do_strip_top) {
        strip_top_col <- strip_top_col + 2L
      }
      widths <- grid::unit.c(
        grid::unit(c(1, 1), c("grobwidth", "lines"), list(legend_grob, NULL)),
        widths
      )
    } else if (legend$side == "top") {
      # legend on top
      nrow <- nrow + 2L
      legend_row <- 1L + do_main * 2
      legend_col <- if (do_strip_left) 2L else 1L
      diagram_row <- diagram_row + 2L
      if (do_strip_top) {
        strip_top_row <- strip_top_row + 2L
      }
      if (do_strip_left) {
        strip_left_row <- strip_left_row + 2L
      }
      heights <- grid::unit.c(
        grid::unit(c(1, 1), c("grobheight", "lines"), list(legend_grob, NULL)),
        heights
      )
    } else {
      # legend on bottom
      nrow <- nrow + 2L
      legend_row <- nrow
      legend_col <- if (do_strip_left) 2L else 1L
      heights <- grid::unit.c(
        heights,
        grid::unit(c(1, 1), c("lines", "grobheight"), list(NULL, legend_grob))
      )
    }
    legend_grob$vp <- grid::viewport(
      layout.pos.row = legend_row,
      layout.pos.col = legend_col,
      name = "legend.vp"
    )
  }

  if (do_main) {
    main_grob <- grid::textGrob(
      label = main$label,
      x = main$x,
      y = main$y,
      just = main$just,
      hjust = main$hjust,
      vjust = main$vjust,
      rot = main$rot,
      check.overlap = FALSE,
      default.units = main$default.units,
      gp = main$gp,
      name = "main.grob"
    )
    heights <- grid::unit.c(
      grid::unit(c(1, 1), c("lines", "grobheight"), list(NULL, main_grob)),
      heights
    )
    main_grob$vp <- grid::viewport(
      layout.pos.row = 1,
      layout.pos.col = diagram_col,
      name = "main.vp"
    )
  }

  canvas_vp <- grid::viewport(
    layout.pos.row = diagram_row,
    layout.pos.col = diagram_col,
    width = grid::unit(1, "npc") - pad_x - pad_x,
    height = grid::unit(1, "npc") - pad_y - pad_y,
    name = "canvas.vp",
    layout = grid::grid.layout(
      nrow = layout[1L],
      ncol = layout[2L],
      widths = rep(1 / ar, layout[1L]),
      heights = rep(1, layout[2L])
    )
  )

  for (i in seq_along(euler_grob$children)) {
    if (NCOL(pos) == 2L) {
      j <- pos[i, 1L]
      k <- pos[i, 2L]
    } else {
      j <- 1L
      k <- pos[i]
    }
    euler_grob$children[[i]]$vp <- grid::viewport(
      layout.pos.row = j,
      layout.pos.col = k,
      xscale = if (xlim[1] == -Inf) c(-1, 1) else xlim,
      yscale = if (ylim[1] == -Inf) c(-1, 1) else ylim,
      name = paste0("panel.vp.", j, ".", k)
    )
  }

  euler_grob$vp <- canvas_vp

  # return a gTree object
  children <- gList(
    if (do_bg) bg_grob = bg_grob,
    if (do_main) main_grob = main_grob,
    if (do_strip_top) strip_top_grob = strip_top_grob,
    if (do_strip_left) strip_left_grob = strip_left_grob,
    if (do_legend) legend_grob = legend_grob,
    euler_grob = euler_grob
  )

  grid::gTree(
    data = data,
    children = children,
    vp = grid::viewport(
      layout = grid::grid.layout(
        nrow = nrow,
        ncol = ncol,
        widths = widths,
        heights = heights,
        respect = TRUE
      ),
      name = "euler.vp"
    ),
    cl = "eulergram",
    name = "euler.diagram"
  )
}

resolve_strip_labels <- function(labels, levels, display_levels, axis) {
  if (is.null(labels)) {
    return(display_levels)
  }

  label_names <- names(labels)
  has_names <- !is.null(label_names) && any(nzchar(label_names))

  if (has_names) {
    if (any(!nzchar(label_names))) {
      stop(sprintf(
        "`strips$labels$%s` must be fully named when names are used.",
        axis
      ))
    }
    if (anyDuplicated(label_names)) {
      stop(sprintf("`strips$labels$%s` must have unique names.", axis))
    }

    unknown_levels <- setdiff(label_names, levels)
    if (length(unknown_levels) > 0L) {
      stop(sprintf("`strips$labels$%s` has unknown level names.", axis))
    }

    missing_levels <- setdiff(levels, label_names)
    if (length(missing_levels) > 0L) {
      stop(sprintf(
        "`strips$labels$%s` must provide labels for all levels.",
        axis
      ))
    }

    labels <- labels[match(display_levels, label_names)]
    names(labels) <- NULL
    return(labels)
  }

  if (length(labels) != length(display_levels)) {
    stop(
      sprintf(
        "`strips$labels$%s` must have length %d.",
        axis,
        length(display_levels)
      )
    )
  }

  labels
}

#' @rdname plot.euler
#' @export
plot.eulerr_venn <- function(
  x,
  fills = TRUE,
  patterns = FALSE,
  edges = TRUE,
  legend = FALSE,
  labels = identical(legend, FALSE),
  quantities = TRUE,
  strips = NULL,
  bg = FALSE,
  main = NULL,
  complement = TRUE,
  n = 200L,
  adjust_labels = TRUE,
  ...
) {
  if (!missing(adjust_labels)) {
    warning("`adjust_labels` is deprecated and no longer has any effect.")
  }

  plot.euler(
    x = x,
    fills = fills,
    patterns = patterns,
    edges = edges,
    legend = legend,
    labels = labels,
    quantities = quantities,
    strips = strips,
    bg = bg,
    main = main,
    complement = complement,
    n = n,
    ...
  )
}

#' @rdname plot.euler
#' @export
plot.venn <- function(...) {
  warning(
    "`plot.venn()` is deprecated and will be removed in a future release. ",
    "Use `plot()` on objects from `venn()` instead."
  )
  plot.eulerr_venn(...)
}

#' Print (plot) Euler diagram
#'
#' This function is responsible for the actual drawing of
#' `'eulergram'` objects created through [plot.euler()]. [print.eulergram()]
#' is an alias for [plot.eulergram()], which has been provided so that
#' [plot.euler()] gets called automatically.
#'
#' @param x an object of class `'eulergram'`, usually the output of
#'   [plot.euler()]
#' @param newpage if `TRUE`, opens a new page via [grid.newpage()] to draw on
#' @param ... ignored
#' @return A plot is drawn on the current device using [grid::Grid()] graphics.
#' @export
plot.eulergram <- function(x, newpage = TRUE, ...) {
  if (isTRUE(newpage)) {
    grid::grid.newpage()
  }
  grid::grid.draw(x)
}

#' @rdname plot.eulergram
#' @export
print.eulergram <- function(x, ...) {
  graphics::plot(x, ...)
}

test_that("normal plotting works without errors", {
  tmp <- tempfile()
  png(tmp)

  f1 <- euler(c("A" = 10, "B" = 5, "A&B" = 2))

  expect_silent(plot(
    f1,
    fills = list(fill = c("black", "blue"), alpha = 0.3),
    labels = list(font = 4)
  ))

  expect_silent(plot(f1))
  expect_silent(plot(f1, legend = TRUE, quantities = TRUE))
  expect_silent(plot(f1, legend = FALSE, quantities = TRUE))
  expect_silent(plot(f1, legend = TRUE, labels = FALSE))
  expect_silent(plot(f1, labels = c("asdf", "qwer")))
  expect_silent(plot(f1, labels = str2expression(c("A", "B"))))
  expect_silent(plot(f1, labels = list(labels = str2expression(c("A", "B")))))
  expect_silent(plot(f1, main = "Hello"))
  expect_silent(plot(f1, expression = "phi[1]"))
  expect_silent(plot(f1, edges = c("white", "blue")))
  expect_silent(plot(f1, bg = "grey95"))
  expect_silent(plot(f1, bg = list(fill = "grey95", alpha = 0.8)))
  expect_silent(plot(f1, quantities = list(type = "percent")))
  expect_silent(plot(f1, quantities = list(type = "counts")))
  expect_silent(plot(f1, quantities = list(type = c("percent", "counts"))))
  expect_silent(plot(f1, quantities = list(type = c("counts", "percent"))))
  expect_silent(plot(
    f1,
    quantities = list(labels = c(A = "foo", B = "bar", "A&B" = "baz"))
  ))
  expect_silent(plot(
    f1,
    quantities = c(A = "foo", "A&B" = "baz")
  ))

  expect_error(plot(f1, quantities = list(type = c("asdf"))))
  expect_error(plot(
    f1,
    quantities = list(labels = c("foo", A = "bar"))
  ))
  expect_error(plot(
    f1,
    quantities = c("foo", A = "bar")
  ))
  p_named_fill <- plot(
    f1,
    fills = c(A = "black"),
    labels = FALSE,
    quantities = FALSE,
    edges = FALSE,
    legend = FALSE
  )
  fill_children <- p_named_fill$children$canvas.grob$children$diagram.grob.1$children
  fill_cols <- vapply(
    fill_children,
    function(g) {
      if (inherits(g, "gTree")) g$children[[1]]$gp$fill else g$gp$fill
    },
    character(1)
  )
  expect_equal(unname(fill_cols[1]), "black")
  expect_false(all(fill_cols == "black"))

  p_named_pattern <- plot(
    f1,
    fills = TRUE,
    patterns = c(A = "stripes"),
    labels = FALSE,
    quantities = FALSE,
    edges = FALSE,
    legend = FALSE
  )
  pattern_children <- p_named_pattern$children$canvas.grob$children$diagram.grob.1$children
  pattern_count <- sum(vapply(
    pattern_children,
    function(g) {
      inherits(g, "gTree") && length(g$children) > 1L
    },
    logical(1)
  ))
  expect_equal(pattern_count, 1L)

  p_named_pattern_union <- plot(
    f1,
    fills = TRUE,
    patterns = list(type = c(A = "stripes"), mode = "union"),
    labels = FALSE,
    quantities = FALSE,
    edges = FALSE,
    legend = FALSE
  )
  pattern_children_union <- p_named_pattern_union$children$canvas.grob$children$diagram.grob.1$children
  pattern_count_union <- sum(vapply(
    pattern_children_union,
    function(g) {
      inherits(g, "gTree") && length(g$children) > 1L
    },
    logical(1)
  ))
  expect_equal(pattern_count_union, 0L)

  expect_error(plot(
    f1,
    fills = list(fill = c("red", "blue", "green", "black"))
  ))
  bad_named_fills <- c("black", "blue")
  names(bad_named_fills) <- c("A", "")
  expect_error(plot(f1, fills = bad_named_fills))
  expect_error(plot(f1, fills = list(fill = c(A = "black"), mode = "asdf")))
  bad_named_patterns <- c("stripes", NA_character_)
  names(bad_named_patterns) <- c("A", "")
  expect_error(plot(f1, patterns = bad_named_patterns))
  expect_error(plot(
    f1,
    patterns = list(type = c(A = "stripes"), mode = "asdf")
  ))
  expect_error(plot(
    f1,
    patterns = list(type = c("stripes", NA, "stripes", NA))
  ))

  grid <- expand.grid(
    labels = c(TRUE, FALSE),
    quantities = c(TRUE, FALSE),
    legend = c(TRUE, FALSE),
    fills = c(TRUE, FALSE),
    edges = c(TRUE, FALSE)
  )

  for (i in seq_len(nrow(grid))) {
    expect_silent(plot(
      f1,
      labels = grid$labels[!!i],
      quantities = grid$quantities[!!i],
      legend = grid$legend[!!i],
      fills = grid$fills[!!i],
      edges = grid$edges[!!i]
    ))
  }

  dat <- data.frame(
    Liberal = sample(c(TRUE, FALSE), size = 100, replace = TRUE),
    Conservative = sample(c(TRUE, TRUE, FALSE), size = 100, replace = TRUE),
    Gender = sample(c("Men", "Women"), size = 100, replace = TRUE),
    Nation = sample(c("Sweden", "Denmark"), size = 100, replace = TRUE)
  )

  f2 <- euler(dat[, 1:3], by = list(Gender))
  f3 <- euler(dat, by = list(Gender, Nation))

  expect_silent(plot(
    f2,
    edges = list(fill = "transparent", lty = c(1, 2), lwd = c(1, 2))
  ))

  grid <- expand.grid(
    side = c(NA, "right", "left", "top", "bottom"),
    main = c("Title", FALSE),
    stringsAsFactors = FALSE
  )

  for (i in seq_len(nrow(grid))) {
    if (is.na(grid$side[i])) {
      legend <- FALSE
    } else {
      legend <- list(side = grid$side[i])
    }

    expect_silent(plot(f1, legend = legend, main = grid$main[!!i]))
    expect_silent(plot(f2, legend = legend, main = grid$main[!!i]))
    expect_silent(plot(f3, legend = legend, main = grid$main[!!i]))
  }

  f4 <- euler(c(A = 1))
  expect_silent(plot(f4))

  dev.off()
  unlink(tmp)
})

test_that("plotting zero-fits works", {
  tmp <- tempfile()
  png(tmp)

  s <- c(a = 0, b = 0)
  expect_is(plot(euler(s)), "gTree")
  dev.off()
  unlink(tmp)
})

test_that("stripe fill patterns can be added", {
  tmp <- tempfile()
  png(tmp)

  f <- euler(c(A = 10, B = 8, "A&B" = 3))
  expect_silent(plot(
    f,
    fills = list(
      fill = c("grey90", "grey90")
    ),
    patterns = list(type = "stripes", angle = 30)
  ))

  dev.off()
  unlink(tmp)
})

test_that("set-level pattern types propagate to intersections", {
  tmp <- tempfile()
  png(tmp)

  f <- euler(c(A = 10, B = 8, "A&B" = 3))
  p <- plot(
    f,
    fills = list(fill = c("grey90", "grey90")),
    patterns = list(
      type = c("stripes", NA),
      angle = c(25, 0),
      col = "black",
      lwd = 1
    ),
    labels = FALSE,
    quantities = FALSE,
    edges = FALSE,
    legend = FALSE
  )

  g <- p$children$canvas.grob$children$diagram.grob.1$children
  expect_true(any(grepl("^set\\.pattern\\.grob\\.", names(g))))

  dev.off()
  unlink(tmp)
})

test_that("legacy fill pattern arguments still work with deprecation warning", {
  tmp <- tempfile()
  png(tmp)

  f <- euler(c(A = 10, B = 8, "A&B" = 3))
  expect_warning(
    plot(
      f,
      fills = list(
        fill = c("grey90", "grey90"),
        pattern = "stripes",
        angle = 30,
        pattern_col = "black",
        pattern_lwd = 0.8
      )
    ),
    "deprecated"
  )

  dev.off()
  unlink(tmp)
})

test_that("legend keys support patterns", {
  tmp <- tempfile()
  png(tmp)

  f <- euler(c(A = 10, B = 8, "A&B" = 3))
  p <- plot(
    f,
    fills = list(fill = c("grey90", "grey90")),
    patterns = list(type = "stripes", col = "black", lwd = 0.8),
    legend = TRUE,
    quantities = FALSE
  )

  legend_grob <- p$children$legend.grob
  point_cells <- legend_grob$children[vapply(
    legend_grob$children,
    function(cell) {
      inherits(cell$children[[1]], "gTree")
    },
    logical(1)
  )]

  expect_true(length(point_cells) > 0)

  dev.off()
  unlink(tmp)
})

test_that("quantity formatters can be customized", {
  tmp <- tempfile()
  png(tmp)

  f <- euler(c(A = 10, B = 8, "A&B" = 3), input = "disjoint")

  p <- plot(
    f,
    quantities = list(
      type = "percent",
      format = list(fun = round, digits = 1)
    )
  )
  q <- p$data$centers$quantities
  names(q) <- rownames(p$data$centers)
  expect_equal(unname(q[c("A", "B", "A&B")]), c("47.6 %", "38.1 %", "14.3 %"))

  p <- plot(
    f,
    quantities = list(
      type = "counts",
      format = list(fun = round, digits = -1)
    )
  )
  q <- p$data$centers$quantities
  names(q) <- rownames(p$data$centers)
  expect_equal(unname(q[c("A", "B", "A&B")]), c("10", "10", "0"))

  p <- plot(
    f,
    quantities = list(
      type = "percent",
      total = 100,
      format = list(fun = round, digits = 1)
    )
  )
  q <- p$data$centers$quantities
  names(q) <- rownames(p$data$centers)
  expect_equal(unname(q[c("A", "B", "A&B")]), c("10 %", "8 %", "3 %"))

  p <- plot(
    f,
    quantities = list(
      type = "fraction",
      total = 100,
      format = list(fun = round, digits = 2)
    )
  )
  q <- p$data$centers$quantities
  names(q) <- rownames(p$data$centers)
  expect_equal(unname(q[c("A", "B", "A&B")]), c("0.1", "0.08", "0.03"))

  p <- plot(
    f,
    quantities = list(
      type = c("counts", "fraction"),
      total = 100,
      format = list(fun = round, digits = 2)
    )
  )
  q <- p$data$centers$quantities
  names(q) <- rownames(p$data$centers)
  expect_equal(
    unname(q[c("A", "B", "A&B")]),
    c("10 (0.1)", "8 (0.08)", "3 (0.03)")
  )

  expect_error(plot(f, quantities = list(format = 1)))
  expect_error(plot(f, quantities = list(format = list(fun = 1))))
  expect_error(plot(f, quantities = list(total = 0)))

  dev.off()
  unlink(tmp)
})

test_that("quantities$template controls rendered text", {
  tmp <- tempfile()
  png(tmp)

  f <- euler(c(A = 10, B = 8, "A&B" = 3), input = "disjoint")

  p <- plot(
    f,
    quantities = list(template = "{counts}\n{percent}")
  )
  q <- p$data$centers$quantities
  names(q) <- rownames(p$data$centers)
  expect_equal(
    unname(q[c("A", "B", "A&B")]),
    c("10\n48 %", "8\n38 %", "3\n14 %")
  )

  p <- plot(
    f,
    quantities = list(template = "n={counts} ({percent})")
  )
  q <- p$data$centers$quantities
  names(q) <- rownames(p$data$centers)
  expect_equal(
    unname(q[c("A", "B", "A&B")]),
    c("n=10 (48 %)", "n=8 (38 %)", "n=3 (14 %)")
  )

  # Template overrides type.
  p <- plot(
    f,
    quantities = list(type = "counts", template = "{percent}")
  )
  q <- p$data$centers$quantities
  names(q) <- rownames(p$data$centers)
  expect_equal(unname(q[c("A", "B", "A&B")]), c("48 %", "38 %", "14 %"))

  expect_error(
    plot(f, quantities = list(template = "{counts} ({foo})")),
    "Unknown placeholder"
  )
  expect_error(
    plot(f, quantities = list(template = c("{counts}", "{percent}"))),
    "single string"
  )

  dev.off()
  unlink(tmp)
})

test_that("error_plot functions normally", {
  tmp <- tempfile()
  png(tmp)

  f <- euler(organisms)

  expect_silent(error_plot(f))
  expect_silent(error_plot(
    f,
    pal = grDevices::colorRampPalette(c("red", "blue"))
  ))
  expect_silent(error_plot(f, quantities = FALSE))
  expect_silent(error_plot(f, edges = FALSE))

  dev.off()
  unlink(tmp)
})

test_that("plots with euler lists works", {
  tmp <- tempfile()
  png(tmp)

  f1 <- euler(fruits[, 1:5], by = age)
  f2 <- euler(fruits[, 1:5], by = list(age, sex))

  expect_silent(plot(f1, legend = TRUE, strips = FALSE))
  expect_silent(plot(f2, strips = list(cex = 2, fontface = "bold")))
  expect_silent(plot(f1))

  dev.off()
  unlink(tmp)
})

test_that("strip labels can be customized", {
  tmp <- tempfile()
  png(tmp)

  f <- euler(fruits[, 1:5], by = list(age, sex))

  p <- plot(
    f,
    strips = list(
      labels = list(
        top = c("Women", "Men"),
        left = c("Children", "Adults")
      )
    )
  )
  expect_equal(p$children$strip.top.grob$label, c("Women", "Men"))
  expect_equal(p$children$strip.left.grob$label, c("Children", "Adults"))

  top_named <- c(male = "Men", female = "Women")
  left_named <- c(adult = "Adults", child = "Children")
  p_named <- plot(
    f,
    strips = list(labels = list(top = top_named, left = left_named))
  )
  expect_equal(p_named$children$strip.top.grob$label, c("Women", "Men"))
  expect_equal(p_named$children$strip.left.grob$label, c("Children", "Adults"))

  top_expr <- str2expression(c("alpha[1]", "beta[2]"))
  left_expr <- str2expression(c("gamma", "delta"))
  p_expr <- plot(
    f,
    strips = list(labels = list(top = top_expr, left = left_expr))
  )
  expect_equal(p_expr$children$strip.top.grob$label, top_expr)
  expect_equal(p_expr$children$strip.left.grob$label, left_expr)

  expect_error(
    plot(f, strips = list(labels = list(top = "Women"))),
    "`strips\\$labels\\$top` must have length 2"
  )
  expect_error(
    plot(f, strips = list(labels = "foo")),
    "`strips\\$labels` must be a list"
  )
  expect_error(
    plot(f, strips = list(labels = list("Women", "Men"))),
    "`strips\\$labels` must be a named list"
  )
  expect_error(
    plot(
      f,
      strips = list(labels = list(top = c(female = "Women", "Men")))
    ),
    "must be fully named"
  )
  expect_error(
    plot(
      f,
      strips = list(labels = list(top = c(female = "Women", female = "Men")))
    ),
    "must have unique names"
  )
  expect_error(
    plot(
      f,
      strips = list(labels = list(top = c(female = "Women", foo = "Men")))
    ),
    "unknown level names"
  )
  expect_error(
    plot(f, strips = list(labels = list(top = c(female = "Women")))),
    "must provide labels for all levels"
  )

  dev.off()
  unlink(tmp)
})

test_that("venn plotting prefers eulerr class dispatch", {
  tmp <- tempfile()
  png(tmp)

  f <- venn(c(A = 10, B = 5, "A&B" = 2))
  expect_is(f, "eulerr_venn")
  expect_silent(plot(f))

  dev.off()
  unlink(tmp)
})

test_that("empty singleton sets do not override overlap fill colors", {
  tmp <- tempfile()
  png(tmp)

  fit <- euler(c(
    A = 0,
    B = 16,
    C = 12,
    "A&B" = 25,
    "A&C" = 39,
    "B&C" = 1,
    "A&B&C" = 40
  ))
  p <- plot(
    fit,
    fills = list(fill = c("#E41A1C", "#377EB8", "#4DAF4A")),
    labels = FALSE,
    quantities = FALSE,
    edges = FALSE,
    legend = FALSE
  )

  g <- p$children$canvas.grob$children$diagram.grob.1$children
  fills <- vapply(
    g[grepl("^fills\\.grob\\.", names(g))],
    function(x) if (length(x$gp$fill) == 0) NA_character_ else x$gp$fill,
    character(1)
  )

  expect_equal(unname(fills[["fills.grob.4"]]), "#AF5E6A")
  expect_equal(unname(fills[["fills.grob.5"]]), "#AE7E31")

  dev.off()
  unlink(tmp)
})

test_that("legacy plot.venn warns", {
  tmp <- tempfile()
  png(tmp)

  f <- venn(c(A = 10, B = 5, "A&B" = 2))
  expect_warning(plot.venn(f), "deprecated")

  dev.off()
  unlink(tmp)
})

test_that("rotate parameter works", {
  tmp <- tempfile()
  png(tmp)

  fit <- euler(c(A = 10, B = 5, "A&B" = 3))
  expect_silent(plot(fit, rotate = 0))
  expect_silent(plot(fit, rotate = 90))
  expect_silent(plot(fit, rotate = 45))
  expect_silent(plot(fit, rotate = -30))

  dev.off()
  unlink(tmp)
})

test_that("by_group applies per-panel overrides", {
  tmp <- tempfile()
  png(tmp)

  fit <- euler(fruits[, 1:4], by = list(sex))

  # Helper: extract the colors used by quantity textGrobs in panel `i`.
  panel_quantity_cols <- function(g, i) {
    panel <- g$children$canvas.grob$children[[i]]
    tags <- panel$children$tags$children
    out <- character(0)
    for (tag in tags) {
      for (child in tag$children) {
        if (
          inherits(child, "text") &&
            grepl("^tag\\.quantity\\.", child$name)
        ) {
          out <- c(out, child$gp$col)
        }
      }
    }
    unique(out)
  }

  # Per-panel overrides for `quantities`
  g <- plot(
    fit,
    quantities = list(
      by_group = list(
        male = list(col = "blue"),
        female = list(col = "red")
      )
    )
  )
  female_idx <- which(names(fit) == "female")
  male_idx <- which(names(fit) == "male")
  expect_equal(panel_quantity_cols(g, female_idx), "red")
  expect_equal(panel_quantity_cols(g, male_idx), "blue")

  # Top-level value applies to panels not listed in `by_group`
  g <- plot(
    fit,
    quantities = list(
      col = "gray",
      by_group = list(male = list(col = "blue"))
    )
  )
  expect_equal(panel_quantity_cols(g, female_idx), "gray")
  expect_equal(panel_quantity_cols(g, male_idx), "blue")

  # Multi-`by` uses dotted keys (factor levels joined by ".")
  fit2 <- euler(fruits, by = list(sex, age))
  g <- plot(
    fit2,
    quantities = list(
      col = "darkgreen",
      by_group = list("male.adult" = list(col = "navy"))
    )
  )
  ma_idx <- which(names(fit2) == "male.adult")
  fa_idx <- which(names(fit2) == "female.adult")
  expect_equal(panel_quantity_cols(g, ma_idx), "navy")
  expect_equal(panel_quantity_cols(g, fa_idx), "darkgreen")

  # All five params accept `by_group` without error
  expect_silent(plot(
    fit,
    fills = list(by_group = list(male = list(fill = "steelblue"))),
    edges = list(by_group = list(male = list(lwd = 2))),
    labels = list(by_group = list(male = list(col = "purple"))),
    quantities = list(by_group = list(male = list(col = "purple"))),
    patterns = list(
      type = "stripes",
      by_group = list(male = list(angle = 60))
    )
  ))

  dev.off()
  unlink(tmp)
})

test_that("by_group rejects invalid input", {
  fit <- euler(fruits[, 1:4], by = list(sex))

  # Unknown panel key
  expect_error(
    plot(fit, quantities = list(by_group = list(bogus = list(col = "red")))),
    "unknown keys"
  )

  # Structural field
  expect_error(
    plot(
      fit,
      quantities = list(by_group = list(male = list(type = "percent")))
    ),
    "cannot be overridden per panel"
  )

  # by_group without `by =`
  fit_plain <- euler(c(A = 1, B = 2, "A&B" = 1))
  expect_error(
    plot(
      fit_plain,
      quantities = list(by_group = list(any = list(col = "red")))
    ),
    "requires a diagram fit with `by ="
  )

  # Override is not a list
  expect_error(
    plot(fit, quantities = list(by_group = list(male = "red"))),
    "must be a list"
  )

  # by_group itself unnamed
  expect_error(
    plot(fit, quantities = list(by_group = list(list(col = "red")))),
    "fully named list"
  )
})

test_that("annotations render as a third stacked text element", {
  tmp <- tempfile()
  png(tmp)
  on.exit({
    dev.off()
    unlink(tmp)
  })

  tag_text_grobs <- function(tag, kind) {
    nm_prefix <- paste0("tag.", kind, ".")
    out <- list()
    for (child in tag$children) {
      if (inherits(child, "text") && startsWith(child$name, nm_prefix)) {
        out[[length(out) + 1L]] <- child
      }
    }
    out
  }

  fit <- euler(c(A = 10, B = 5, "A&B" = 3))
  p <- plot(
    fit,
    quantities = TRUE,
    annotations = c(A = "mean=35", "A&B" = "n=3")
  )

  # The centers data.frame carries the per-region annotation text.
  expect_equal(p$data$centers["A", "annotations"], "mean=35")
  expect_equal(p$data$centers["A&B", "annotations"], "n=3")
  expect_true(is.na(p$data$centers["B", "annotations"]))

  tags <- p$children$canvas.grob$children$diagram.grob.1$children$tags$children
  combos <- vapply(tags, function(t) t$combo_key %||% "", character(1))

  # A: all three slots populated
  a <- tags[[which(combos == "A")]]
  expect_true(isTRUE(a$has_label))
  expect_true(isTRUE(a$has_quantity))
  expect_true(isTRUE(a$has_annotation))
  expect_length(tag_text_grobs(a, "annotation"), 1L)
  expect_equal(tag_text_grobs(a, "annotation")[[1L]]$label, "mean=35")

  # B: no annotation requested -> nullGrob placeholder
  b <- tags[[which(combos == "B")]]
  expect_false(isTRUE(b$has_annotation))
  expect_length(tag_text_grobs(b, "annotation"), 0L)

  # A&B: quantity at anchor, annotation below (no label)
  ab <- tags[[which(combos == "A&B")]]
  expect_false(isTRUE(ab$has_label))
  expect_true(isTRUE(ab$has_quantity))
  expect_true(isTRUE(ab$has_annotation))
})

test_that("annotations accept named-vector shorthand and list with styling", {
  tmp <- tempfile()
  png(tmp)
  on.exit({
    dev.off()
    unlink(tmp)
  })

  fit <- euler(c(A = 10, B = 5, "A&B" = 3))

  # Shorthand named vector
  p1 <- plot(fit, annotations = c(A = "a"))
  # Equivalent list form
  p2 <- plot(fit, annotations = list(labels = c(A = "a")))

  expect_equal(p1$data$centers$annotations, p2$data$centers$annotations)

  # List form with gpar override
  p3 <- plot(
    fit,
    annotations = list(labels = c(A = "a"), col = "purple", cex = 1.5)
  )
  ann_tag <- p3$children$canvas.grob$children$diagram.grob.1$children$tags$children[[1L]]
  expect_equal(ann_tag$annotation_gp$col, "purple")
  expect_equal(ann_tag$annotation_gp$cex, 1.5)
})

test_that("annotation anchor falls back when quantity or label is absent", {
  tmp <- tempfile()
  png(tmp)
  on.exit({
    dev.off()
    unlink(tmp)
  })

  fit <- euler(c(A = 10, B = 5, "A&B" = 3))

  # quantity off, label on -> label is the anchor, annotation stacks below it
  p_no_quant <- plot(
    fit,
    quantities = FALSE,
    labels = TRUE,
    annotations = c(A = "note")
  )
  tags <- p_no_quant$children$canvas.grob$children$diagram.grob.1$children$tags$children
  combos <- vapply(tags, function(t) t$combo_key %||% "", character(1))
  a <- tags[[which(combos == "A")]]
  expect_true(isTRUE(a$has_label))
  expect_false(isTRUE(a$has_quantity))
  expect_true(isTRUE(a$has_annotation))

  # labels off, quantities off -> annotation IS the anchor
  p_solo <- plot(
    fit,
    quantities = FALSE,
    labels = FALSE,
    annotations = c(A = "only")
  )
  tags <- p_solo$children$canvas.grob$children$diagram.grob.1$children$tags$children
  combos <- vapply(tags, function(t) t$combo_key %||% "", character(1))
  a <- tags[[which(combos == "A")]]
  expect_false(isTRUE(a$has_label))
  expect_false(isTRUE(a$has_quantity))
  expect_true(isTRUE(a$has_annotation))
})

test_that("by_group panel overrides apply to annotations", {
  tmp <- tempfile()
  png(tmp)
  on.exit({
    dev.off()
    unlink(tmp)
  })

  panel_annotation_cols <- function(g, i) {
    panel <- g$children$canvas.grob$children[[i]]
    tags <- panel$children$tags$children
    out <- character(0)
    for (tag in tags) {
      for (child in tag$children) {
        if (
          inherits(child, "text") &&
            grepl("^tag\\.annotation\\.", child$name)
        ) {
          out <- c(out, child$gp$col)
        }
      }
    }
    unique(out)
  }

  fit <- euler(fruits[, 1:4], by = list(sex))
  female_idx <- which(names(fit) == "female")
  male_idx <- which(names(fit) == "male")

  g <- plot(
    fit,
    quantities = TRUE,
    annotations = list(
      labels = c(banana = "tropical"),
      by_group = list(
        female = list(col = "purple"),
        male = list(col = "navy")
      )
    )
  )
  expect_equal(panel_annotation_cols(g, female_idx), "purple")
  expect_equal(panel_annotation_cols(g, male_idx), "navy")
})

test_that("annotations reject unnamed and non-character input", {
  fit <- euler(c(A = 1, B = 1, "A&B" = 1))
  expect_error(
    plot(fit, annotations = c("foo", "bar")),
    "fully named character vector"
  )
  expect_error(
    plot(fit, annotations = list(labels = c(A = 1, B = 2))),
    "must be a character vector"
  )
  # by_group requires `by =`
  expect_error(
    plot(fit, annotations = list(by_group = list(any = list(col = "red")))),
    "requires a diagram fit with `by ="
  )
})

test_that("eulergrams can be composed with | and /", {
  tmp <- tempfile()
  png(tmp)
  on.exit({
    dev.off()
    unlink(tmp)
  })

  p1 <- plot(euler(c(A = 1, B = 8, "A&B" = 1)))
  p2 <- plot(euler(c(A = 1, C = 1, "A&C" = 1)))
  p3 <- plot(euler(c(X = 3, Y = 2, "X&Y" = 1)))

  horiz <- p1 | p2
  expect_s3_class(horiz, "eulergram")
  expect_equal(horiz$name, "euler.composed")
  expect_length(horiz$children, 2L)
  expect_equal(horiz$children[[1]]$vp$layout.pos.col[1], 1L)
  expect_equal(horiz$children[[2]]$vp$layout.pos.col[1], 3L)
  expect_equal(horiz$vp$layout$ncol, 3L)
  expect_equal(horiz$vp$layout$nrow, 1L)

  vert <- p1 / p2
  expect_s3_class(vert, "eulergram")
  expect_equal(vert$vp$layout$nrow, 3L)
  expect_equal(vert$vp$layout$ncol, 1L)
  expect_equal(vert$children[[1]]$vp$layout.pos.row[1], 1L)
  expect_equal(vert$children[[2]]$vp$layout.pos.row[1], 3L)

  nested <- (p1 | p2) / p3
  expect_s3_class(nested, "eulergram")
  expect_equal(nested$vp$layout$nrow, 3L)
  inner <- nested$children[[1]]$children[[1]]
  expect_s3_class(inner, "eulergram")
  expect_equal(inner$vp$layout$ncol, 3L)

  expect_silent({
    grid::grid.newpage()
    grid::grid.draw(nested)
  })
})

test_that("composition spacing follows eulerr_options()", {
  tmp <- tempfile()
  png(tmp)
  on.exit({
    dev.off()
    unlink(tmp)
    eulerr_options(composition = list(spacing = grid::unit(1, "lines")))
  })

  p1 <- plot(euler(c(A = 1, B = 1, "A&B" = 1)))
  p2 <- plot(euler(c(C = 1, D = 1, "C&D" = 1)))

  eulerr_options(composition = list(spacing = grid::unit(2, "cm")))
  composed <- p1 | p2
  widths <- composed$vp$layout$widths
  expect_match(as.character(widths[2]), "2")
  expect_equal(grid::unitType(widths)[2], "cm")
})

test_that("composing non-eulergram operands fails", {
  tmp <- tempfile()
  png(tmp)
  on.exit({
    dev.off()
    unlink(tmp)
  })

  p1 <- plot(euler(c(A = 1, B = 1, "A&B" = 1)))

  expect_error(p1 | 1, "Both operands must be `eulergram`")
  expect_error(p1 / "x", "Both operands must be `eulergram`")
})

.eulerr_env <- new.env(parent = emptyenv())
assign("options", list(), envir = .eulerr_env)

.onLoad <- function(libname, pkgname) {
  eulerr_options(eulerr_default_options())
}

.onUnload <- function(libpath) {
  library.dynam.unload("eulerr", libpath)
}

fit_diagram <- function(
  combinations,
  type = c("euler", "venn"),
  input = c("disjoint", "union"),
  shape = c("circle", "ellipse", "rectangle", "square"),
  loss = c(
    "sum_squared",
    "sum_absolute",
    "sum_absolute_region_error",
    "sum_squared_region_error",
    "max_absolute",
    "max_squared",
    "root_mean_squared",
    "stress",
    "diag_error"
  ),
  loss_aggregator = NULL,
  complement = NULL,
  control = list(),
  ...
) {
  input <- match.arg(input)
  shape <- match.arg(shape)
  type <- match.arg(type)
  loss <- resolve_loss(loss, loss_aggregator)

  if (!is.numeric(combinations)) {
    stop("`combinations` must be numeric")
  }

  if (any(combinations < 0)) {
    stop("values in `combinations` cannot be negative")
  }

  if (is.null(attr(combinations, "names")) || any(names(combinations) == "")) {
    stop("every element in `combinations` needs to be named")
  }

  if (any(duplicated(names(combinations)))) {
    stop("names of elements in `combinations` cannot be duplicated")
  }

  if (!is.null(complement)) {
    if (
      !is.numeric(complement) ||
        length(complement) != 1L ||
        !is.finite(complement) ||
        complement < 0
    ) {
      stop("`complement` must be a single non-negative number.")
    }
    complement <- as.double(complement)
  }

  combo_name_parts <- strsplit(names(combinations), split = "&", fixed = TRUE)
  setnames <- unique(unlist(combo_name_parts, use.names = FALSE))
  n <- length(setnames)

  if (type == "venn" && !is.null(complement)) {
    stop("`complement` is not supported for `venn()` diagrams.")
  }

  # Venn early return (uses lookup table, no optimization)
  if (type == "venn") {
    combo_labels <- all_set_combinations(setnames)
    N <- length(combo_labels)
    combo_set_lists <- strsplit(combo_labels, "&", fixed = TRUE)
    cards <- lengths(combo_set_lists)

    areas <- numeric(N)
    for (j in seq_along(combo_name_parts)) {
      parts <- combo_name_parts[[j]]
      hit <- vapply(
        combo_set_lists,
        function(s) length(s) == length(parts) && all(s %in% parts),
        logical(1)
      )
      areas[hit] <- combinations[j]
    }

    if (input == "disjoint") {
      areas_disjoint <- areas
    } else {
      areas_disjoint <- numeric(N)
      for (i in order(cards, decreasing = TRUE)) {
        this_sets <- combo_set_lists[[i]]
        is_strict_super <- cards > cards[i] &
          vapply(combo_set_lists, function(s) all(this_sets %in% s), logical(1))
        areas_disjoint[i] <- areas[i] - sum(areas_disjoint[is_strict_super])
      }
      if (any(areas_disjoint < 0)) {
        stop("Check your set configuration. Some disjoint areas are negative.")
      }
    }

    orig <- stats::setNames(areas_disjoint, combo_labels)
    fit <- stats::setNames(rep.int(1, N), combo_labels)

    fpar <- venn_spec[[n]]
    rownames(fpar) <- setnames
    shapes <- ellipse_frame_to_shapes(fpar, "ellipse")

    return(structure(
      list(
        shapes = shapes,
        ellipses = fpar,
        original.values = orig,
        fitted.values = fit
      ),
      class = c("eulerr_venn", "venn", "euler", "list")
    ))
  }

  # Euler fit: delegate to Rust
  control <- utils::modifyList(
    list(
      extraopt = (n == 3) && (shape == "ellipse"),
      extraopt_threshold = 0.001,
      extraopt_control = list(),
      tolerance = 1e-3,
      max_sets = NULL
    ),
    control
  )

  # Keep the global-search fallback gated on ellipse fits for now — eunoia's
  # rectangle/square fitters haven't been tuned to benefit from CMA-ES the
  # way the ellipse path has.
  extraopt_threshold <- if (isTRUE(control$extraopt) && shape == "ellipse") {
    as.numeric(control$extraopt_threshold)
  } else {
    NULL
  }

  tolerance <- if (is.null(control$tolerance)) {
    NULL
  } else {
    as.numeric(control$tolerance)
  }

  hard_cap <- max_sets_hard_cap()
  max_sets <- control$max_sets
  if (!is.null(max_sets)) {
    if (
      !is.numeric(max_sets) ||
        length(max_sets) != 1L ||
        !is.finite(max_sets) ||
        max_sets < 1 ||
        max_sets != as.integer(max_sets)
    ) {
      stop("`control$max_sets` must be a positive integer scalar")
    }
    max_sets <- as.integer(max_sets)
    if (max_sets > hard_cap) {
      stop(sprintf(
        "`control$max_sets` (%d) exceeds the hard cap of %d",
        max_sets,
        hard_cap
      ))
    }
  }

  effective_cap <- if (is.null(max_sets)) max_sets_default() else max_sets
  if (n > effective_cap) {
    stop(sprintf(
      "too many sets: %d requested, but maximum supported is %d (raise `control$max_sets` up to %d to override)",
      n,
      effective_cap,
      hard_cap
    ))
  }

  # Derive integer seed from R's RNG so set.seed() works
  seed <- sample.int(.Machine$integer.max, 1L)

  result <- fit_euler_diagram(
    combo_names = names(combinations),
    combo_values = as.double(combinations),
    input = input,
    shape = shape,
    loss = loss,
    extraopt_threshold = extraopt_threshold,
    tolerance = tolerance,
    max_sets = if (is.null(max_sets)) NULL else as.numeric(max_sets),
    complement = complement,
    seed = seed
  )

  # Empty sets stay NA so downstream code (setup_geometry, plotting) detects
  # them via `is.na(shapes$h)` and skips polygonization — eunoia rejects
  # zero-area shapes, so we must not feed those through.
  n_all <- length(result$all_set_names)

  shapes <- new_shape_frame(shape, n_all, result$all_set_names)

  if (length(result$fitted_set_names) > 0L) {
    fidx <- match(result$fitted_set_names, result$all_set_names)
    shapes$h[fidx] <- result$h
    shapes$k[fidx] <- result$k
    shapes$a[fidx] <- result$a
    shapes$b[fidx] <- result$b
    shapes$phi[fidx] <- result$phi
    shapes$width[fidx] <- result$width
    shapes$height[fidx] <- result$height
    shapes$side[fidx] <- result$side
  }

  # The legacy `$ellipses` column is still populated for circle/ellipse fits
  # so consumers reading `fit$ellipses$h` etc. keep working without change.
  # For rectangle/square fits there is no equivalent ellipse representation,
  # so `$ellipses` is omitted — callers must read `$shapes` (see ?euler).
  fpar <- if (shape %in% c("circle", "ellipse")) {
    shapes_to_ellipse_frame(shapes)
  } else {
    NULL
  }

  labs <- result$combo_labels
  orig <- stats::setNames(result$original_values, labs)
  fit <- stats::setNames(result$fitted_values, labs)

  container <- if (isTRUE(result$has_container)) {
    container_area <- result$container_width * result$container_height
    list(
      h = result$container_h,
      k = result$container_k,
      width = result$container_width,
      height = result$container_height,
      complement = complement,
      complement_fitted = container_area - sum(result$fitted_values)
    )
  } else if (!is.null(complement)) {
    # eunoia rejects 0-/1-set specs with a complement, so synthesise a
    # square container around the (possibly empty) closed-form layout.
    fitted_total <- sum(result$fitted_values)
    side <- sqrt(fitted_total + complement)
    if (length(result$h) >= 1L) {
      cx <- mean(result$h)
      cy <- mean(result$k)
    } else {
      cx <- 0
      cy <- 0
    }
    list(
      h = cx,
      k = cy,
      width = side,
      height = side,
      complement = complement,
      complement_fitted = complement
    )
  } else {
    NULL
  }

  out <- list(
    shapes = shapes,
    ellipses = fpar,
    original.values = orig,
    fitted.values = fit,
    residuals = stats::setNames(result$residuals, labs),
    regionError = stats::setNames(result$region_error, labs),
    diagError = result$diag_error,
    stress = result$stress,
    container = container
  )
  # Drop `$ellipses` rather than carry an explicit NULL slot — `coef()` and
  # any other consumer that checks `is.null(fit$ellipses)` then sees a
  # missing field, which is the deprecation contract.
  if (is.null(fpar)) {
    out$ellipses <- NULL
  }

  structure(out, class = c("euler", "list"))
}

#' Allocate a fresh `$shapes` data frame for `n_all` sets. Rows for empty
#' sets keep NA in every column so downstream plotting can detect them via
#' `is.na(shapes$h)` regardless of shape kind.
#' @keywords internal
new_shape_frame <- function(shape, n_all, row_names) {
  data.frame(
    type = rep(shape, n_all),
    h = rep(NA_real_, n_all),
    k = rep(NA_real_, n_all),
    a = rep(NA_real_, n_all),
    b = rep(NA_real_, n_all),
    phi = rep(NA_real_, n_all),
    width = rep(NA_real_, n_all),
    height = rep(NA_real_, n_all),
    side = rep(NA_real_, n_all),
    row.names = row_names,
    stringsAsFactors = FALSE
  )
}

#' Promote a legacy 5-column ellipse data frame (h, k, a, b, phi) into the
#' wide `$shapes` schema. Used by the `venn()` path, where the precomputed
#' venn-shape lookup is still expressed as ellipses.
#' @keywords internal
ellipse_frame_to_shapes <- function(fpar, shape) {
  shapes <- new_shape_frame(shape, NROW(fpar), rownames(fpar))
  shapes$h <- fpar$h
  shapes$k <- fpar$k
  shapes$a <- fpar$a
  shapes$b <- fpar$b
  shapes$phi <- fpar$phi
  shapes
}

#' Project the wide `$shapes` schema back into the legacy 5-column
#' (h, k, a, b, phi) ellipse data frame for circle/ellipse fits. Preserves
#' the row order and row names so back-compat consumers see exactly the
#' shape they used to.
#' @keywords internal
shapes_to_ellipse_frame <- function(shapes) {
  data.frame(
    h = shapes$h,
    k = shapes$k,
    a = shapes$a,
    b = shapes$b,
    phi = shapes$phi,
    row.names = rownames(shapes),
    stringsAsFactors = TRUE
  )
}

# Translate the legacy (loss, loss_aggregator) pair into a single
# eunoia-style loss name, warning on use of either deprecated form.
resolve_loss <- function(loss, loss_aggregator) {
  loss_choices <- c(
    "sum_squared",
    "sum_absolute",
    "sum_absolute_region_error",
    "sum_squared_region_error",
    "max_absolute",
    "max_squared",
    "root_mean_squared",
    "stress",
    "diag_error"
  )
  legacy_loss <- c("square", "abs", "region")

  # The unmodified default vector means the caller supplied no `loss`; pick
  # the first option without requiring the upstream defaults to be a literal
  # match for `loss_choices`.
  if (length(loss) > 1L) {
    if (!is.null(loss_aggregator)) {
      warning(
        "`loss_aggregator` is deprecated and will be removed in a future ",
        "version of eulerr. Pick a `loss` value directly instead.",
        call. = FALSE
      )
    }
    return(loss_choices[1L])
  }

  if (!is.null(loss_aggregator)) {
    warning(
      "`loss_aggregator` is deprecated and will be removed in a future ",
      "version of eulerr. Pick a `loss` value directly instead.",
      call. = FALSE
    )
    loss_aggregator <- match.arg(loss_aggregator, c("sum", "max"))
  }

  if (loss %in% legacy_loss) {
    agg <- if (is.null(loss_aggregator)) "sum" else loss_aggregator
    new_loss <- switch(
      paste(loss, agg, sep = "_"),
      square_sum = "sum_squared",
      square_max = "max_squared",
      abs_sum = "sum_absolute",
      abs_max = "max_absolute",
      region_sum = "sum_absolute_region_error",
      region_max = "diag_error"
    )
    warning(
      "Loss value '",
      loss,
      "' is deprecated. Use loss = '",
      new_loss,
      "' instead.",
      call. = FALSE
    )
    loss <- new_loss
  }

  match.arg(loss, loss_choices)
}

stripe_segments <- function(
  xlim,
  ylim,
  angle = 45,
  spacing = NULL
) {
  xrng <- diff(xlim)
  yrng <- diff(ylim)
  if (is.null(spacing)) {
    spacing <- max(xrng, yrng) / 25
  }
  if (spacing <= 0) {
    return(NULL)
  }

  theta <- (angle %% 180) * pi / 180
  n_vec <- c(-sin(theta), cos(theta))
  d_vec <- c(cos(theta), sin(theta))

  corners <- rbind(
    c(xlim[1], ylim[1]),
    c(xlim[1], ylim[2]),
    c(xlim[2], ylim[1]),
    c(xlim[2], ylim[2])
  )
  tvals <- as.numeric(corners %*% n_vec)
  t_seq <- seq(min(tvals) - spacing, max(tvals) + spacing, by = spacing)

  bbox_intersections <- function(t0) {
    pts <- list()
    if (abs(d_vec[1]) > .Machine$double.eps) {
      s <- (xlim[1] - t0 * n_vec[1]) / d_vec[1]
      y <- t0 * n_vec[2] + s * d_vec[2]
      if (y >= ylim[1] - 1e-9 && y <= ylim[2] + 1e-9) {
        pts[[length(pts) + 1L]] <- c(xlim[1], y)
      }
      s <- (xlim[2] - t0 * n_vec[1]) / d_vec[1]
      y <- t0 * n_vec[2] + s * d_vec[2]
      if (y >= ylim[1] - 1e-9 && y <= ylim[2] + 1e-9) {
        pts[[length(pts) + 1L]] <- c(xlim[2], y)
      }
    }
    if (abs(d_vec[2]) > .Machine$double.eps) {
      s <- (ylim[1] - t0 * n_vec[2]) / d_vec[2]
      x <- t0 * n_vec[1] + s * d_vec[1]
      if (x >= xlim[1] - 1e-9 && x <= xlim[2] + 1e-9) {
        pts[[length(pts) + 1L]] <- c(x, ylim[1])
      }
      s <- (ylim[2] - t0 * n_vec[2]) / d_vec[2]
      x <- t0 * n_vec[1] + s * d_vec[1]
      if (x >= xlim[1] - 1e-9 && x <= xlim[2] + 1e-9) {
        pts[[length(pts) + 1L]] <- c(x, ylim[2])
      }
    }
    if (length(pts) < 2L) {
      return(NULL)
    }
    m <- do.call(rbind, pts)
    m <- unique(round(m, 9))
    if (NROW(m) < 2L) {
      return(NULL)
    }
    dist2 <- as.matrix(stats::dist(m))^2
    idx <- which(dist2 == max(dist2), arr.ind = TRUE)[1L, ]
    list(p1 = m[idx[1], ], p2 = m[idx[2], ])
  }

  out <- lapply(t_seq, bbox_intersections)
  out <- out[!vapply(out, is.null, logical(1))]
  if (length(out) == 0L) {
    return(NULL)
  }
  list(
    x0 = vapply(out, function(z) z$p1[1], numeric(1)),
    y0 = vapply(out, function(z) z$p1[2], numeric(1)),
    x1 = vapply(out, function(z) z$p2[1], numeric(1)),
    y1 = vapply(out, function(z) z$p2[2], numeric(1))
  )
}

split_fill_paths <- function(fill_data) {
  idx <- cumsum(fill_data$id.lengths)
  starts <- c(1L, utils::head(idx + 1L, -1L))
  mapply(
    function(s, e) {
      list(
        x = fill_data$x[s:e],
        y = fill_data$y[s:e]
      )
    },
    starts,
    idx,
    SIMPLIFY = FALSE
  )
}

stripe_polygons <- function(
  xlim,
  ylim,
  angle = 45,
  spacing = NULL,
  stripe_width = NULL
) {
  seg <- stripe_segments(xlim, ylim, angle = angle, spacing = spacing)
  if (is.null(seg)) {
    return(list())
  }

  if (is.null(spacing)) {
    spacing <- max(diff(xlim), diff(ylim)) / 25
  }
  if (is.null(stripe_width)) {
    stripe_width <- spacing / 3
  }
  stripe_width <- max(stripe_width, .Machine$double.eps)

  dx <- seg$x1 - seg$x0
  dy <- seg$y1 - seg$y0
  len <- sqrt(dx^2 + dy^2)
  ok <- len > .Machine$double.eps
  if (!any(ok)) {
    return(list())
  }

  dx <- dx[ok]
  dy <- dy[ok]
  len <- len[ok]
  x0 <- seg$x0[ok]
  y0 <- seg$y0[ok]
  x1 <- seg$x1[ok]
  y1 <- seg$y1[ok]

  nx <- -dy / len
  ny <- dx / len
  h <- stripe_width / 2

  lapply(seq_along(x0), function(i) {
    list(
      x = c(
        x0[i] + h * nx[i],
        x1[i] + h * nx[i],
        x1[i] - h * nx[i],
        x0[i] - h * nx[i]
      ),
      y = c(
        y0[i] + h * ny[i],
        y1[i] + h * ny[i],
        y1[i] - h * ny[i],
        y0[i] - h * ny[i]
      )
    )
  })
}

apply_stripe_pattern <- function(fill_data, pattern_gp, spacing_scale = 25) {
  if (is.null(pattern_gp$type) || !identical(pattern_gp$type, "stripes")) {
    return(NULL)
  }

  spacing <- max(diff(range(fill_data$x)), diff(range(fill_data$y))) /
    spacing_scale
  stripe_width <- spacing * pattern_gp$lwd / 2

  stripes <- stripe_polygons(
    xlim = range(fill_data$x),
    ylim = range(fill_data$y),
    angle = pattern_gp$angle,
    spacing = spacing,
    stripe_width = stripe_width
  )

  if (length(stripes) == 0L) {
    return(NULL)
  }

  fill_paths <- split_fill_paths(fill_data)
  subject_x <- unlist(lapply(fill_paths, "[[", "x"), use.names = FALSE)
  subject_y <- unlist(lapply(fill_paths, "[[", "y"), use.names = FALSE)
  subject_id_lengths <- as.integer(lengths(lapply(fill_paths, "[[", "x")))

  clipped <- list()
  idx <- 1L
  for (stripe in stripes) {
    piece <- polygon_clip_rust(
      subject_x = subject_x,
      subject_y = subject_y,
      subject_id_lengths = subject_id_lengths,
      clip_x = stripe$x,
      clip_y = stripe$y,
      op = "intersection"
    )
    n_pieces <- length(piece$id_lengths)
    if (n_pieces > 0L) {
      starts <- c(1L, utils::head(cumsum(piece$id_lengths) + 1L, -1L))
      ends <- cumsum(piece$id_lengths)
      for (j in seq_len(n_pieces)) {
        s <- starts[j]
        e <- ends[j]
        clipped[[idx]] <- list(x = piece$x[s:e], y = piece$y[s:e])
        idx <- idx + 1L
      }
    }
  }

  if (length(clipped) == 0L) {
    return(NULL)
  }

  clipped
}

add_fill_pattern <- function(fill_grob, fill_data, pattern_gp) {
  clipped <- apply_stripe_pattern(fill_data, pattern_gp, spacing_scale = 25)
  if (is.null(clipped)) {
    return(fill_grob)
  }

  stripe_grob <- grid::pathGrob(
    x = unlist(lapply(clipped, "[[", "x"), use.names = FALSE),
    y = unlist(lapply(clipped, "[[", "y"), use.names = FALSE),
    id.lengths = lengths(lapply(clipped, "[[", "x")),
    default.units = "native",
    gp = grid::gpar(
      fill = pattern_gp$col,
      col = "transparent",
      alpha = pattern_gp$alpha
    )
  )

  grid::grobTree(fill_grob, stripe_grob)
}

#' Grobify Euler objects
#'
#' @param x geometry data
#' @param fills fills params
#' @param patterns pattern params
#' @param edges edges params
#' @param labels labels params
#' @param quantities quantities params
#' @param number current diagram number
#' @param merged_sets sets that are the same and have been merged
#'
#' @return A [grid::gList()] is returned.
#' @keywords internal
setup_grobs <- function(
  x,
  fills,
  patterns,
  edges,
  labels,
  quantities,
  annotations = NULL,
  complement = NULL,
  number,
  merged_sets,
  n_vertices = 200L,
  placement_opts = NULL
) {
  data_edges <- x$edges
  data_fills <- x$fills
  data_tags <- x$centers
  fitted <- x$fitted.values
  empty_sets <- x$empty_sets
  empty_subsets <- x$empty_subsets

  do_tags <- !is.null(data_tags)
  do_edges <- !is.null(data_edges)
  do_fills <- !is.null(data_fills)
  do_patterns <- !is.null(patterns)
  do_labels <- !is.null(labels)
  do_quantities <- !is.null(quantities)
  do_annotations <- !is.null(annotations)

  xlim <- x$xlim
  ylim <- x$ylim

  n_e <- NROW(x$shapes)
  n_id <- length(x$fitted.values)

  #edges
  if (do_edges) {
    # edges
    if (is.null(data_edges$x)) {
      edges_grob <- grid::nullGrob()
    } else {
      edges_grob <- grid::polylineGrob(
        data_edges$x,
        data_edges$y,
        id.lengths = data_edges$id.lengths,
        default.units = "native",
        name = "edges.grob",
        gp = edges$gp[which(!empty_sets & !merged_sets)]
      )
    }
  }

  # patterns by shape
  if (do_patterns && identical(patterns$mode, "shape")) {
    data_sets <- x$set_polygons
    pattern_grobs <- vector("list", n_e)

    for (i in seq_len(n_e)) {
      set_data <- data_sets[[i]]
      set_fill <- list(
        x = set_data$x,
        y = set_data$y,
        id.lengths = length(set_data$x)
      )
      clipped <- apply_stripe_pattern(
        set_fill,
        patterns$set_gp[i],
        spacing_scale = 25
      )
      if (is.null(clipped)) {
        pattern_grobs[[i]] <- grid::nullGrob(
          name = paste0("set.pattern.grob.", i)
        )
      } else {
        pattern_grobs[[i]] <- grid::pathGrob(
          x = unlist(lapply(clipped, "[[", "x"), use.names = FALSE),
          y = unlist(lapply(clipped, "[[", "y"), use.names = FALSE),
          id.lengths = lengths(lapply(clipped, "[[", "x")),
          default.units = "native",
          name = paste0("set.pattern.grob.", i),
          gp = grid::gpar(
            fill = patterns$set_gp[i]$col,
            col = "transparent",
            alpha = patterns$set_gp[i]$alpha
          )
        )
      }
    }
    patterns_grob <- do.call(grid::gList, pattern_grobs)
  }

  # fills
  if (do_fills) {
    if (n_e == 0) {
      fills_grob <- grid::nullGrob()
    } else if (n_e == 1) {
      fill_idx <- which(!empty_subsets)[1L]
      fill_grob <- grid::polygonGrob(
        data_fills[[1]]$x,
        data_fills[[1]]$y,
        default.units = "native",
        name = "fills.grob",
        gp = fills$gp[fill_idx]
      )
      fill_grob <- add_fill_pattern(
        fill_grob = fill_grob,
        fill_data = data_fills[[1]],
        pattern_gp = if (
          do_patterns && identical(patterns$mode, "intersection")
        ) {
          patterns$gp[fill_idx]
        } else {
          NULL
        }
      )
      fills_grob <- grid::gList(fill_grob)
    } else {
      fills_grob <- vector("list", n_id)

      for (i in seq_len(n_id)) {
        if (is.null(data_fills[[i]])) {
          fills_grob[[i]] <- grid::nullGrob(name = paste0("fills.grob.", i))
        } else {
          fill_idx <- which(!empty_subsets)[i]
          fill_grob <- grid::pathGrob(
            data_fills[[i]]$x,
            data_fills[[i]]$y,
            id.lengths = data_fills[[i]]$id.lengths,
            rule = "winding",
            default.units = "native",
            name = paste0("fills.grob.", i),
            gp = fills$gp[fill_idx]
          )
          fills_grob[[i]] <- add_fill_pattern(
            fill_grob = fill_grob,
            fill_data = data_fills[[i]],
            pattern_gp = if (
              do_patterns && identical(patterns$mode, "intersection")
            ) {
              patterns$gp[fill_idx]
            } else {
              NULL
            }
          )
        }
      }
      fills_grob <- do.call(grid::gList, fills_grob)
    }
  }

  container_data <- x$container
  do_container <- !is.null(container_data) && !is.null(complement)

  container_fill_grob <- NULL
  container_edge_grob <- NULL

  if (do_container) {
    cgp <- complement$gp

    cp <- container_data$complement_polygon
    if (length(cp$id_lengths) > 0L) {
      container_fill_grob <- grid::pathGrob(
        cp$x,
        cp$y,
        id.lengths = cp$id_lengths,
        rule = "winding",
        default.units = "native",
        name = "complement.fill.grob",
        gp = grid::gpar(
          fill = cgp$fill[1L],
          col = "transparent",
          alpha = cgp$alpha[1L]
        )
      )
    }

    container_edge_grob <- grid::polylineGrob(
      container_data$outline$x,
      container_data$outline$y,
      default.units = "native",
      name = "container.edge.grob",
      gp = grid::gpar(
        col = cgp$col[1L],
        lwd = cgp$lwd[1L],
        lty = cgp$lty[1L],
        lex = cgp$lex[1L],
        alpha = cgp$alpha[1L]
      )
    )
  }

  # ------------------------------------------------------------------
  # EulerTags: one gTree per panel holding every label/quantity/leader
  # plus the optional complement count. makeContent.EulerTags re-runs
  # the eunoia placement at draw time so resizing the device replaces
  # labels cleanly — no clipping on shrink.
  # ------------------------------------------------------------------
  tag_grobs <- gList()
  if ((do_quantities || do_labels || do_annotations) && !is.null(data_tags)) {
    for (i in seq_len(NROW(data_tags))) {
      tag_grobs[[length(tag_grobs) + 1L]] <- setup_tag(
        data_tags[i, , drop = FALSE],
        labels,
        quantities,
        annotations,
        number = i
      )
    }
  }
  if (do_container) {
    comp_tag <- setup_complement_tag(
      container_data,
      complement,
      number = length(tag_grobs) + 1L
    )
    if (!is.null(comp_tag)) {
      tag_grobs[[length(tag_grobs) + 1L]] <- comp_tag
    }
  }

  do_tags <- length(tag_grobs) > 0L
  tags_gtree <- NULL
  if (do_tags) {
    label_precision <- max(diff(xlim), diff(ylim)) / 100
    tags_gtree <- gTree(
      xlim = xlim,
      ylim = ylim,
      shapes = x$shapes,
      container = container_data,
      n_vertices = as.integer(n_vertices),
      placement_opts = placement_opts,
      label_precision = label_precision,
      padding = eulerr_options()$padding,
      children = tag_grobs,
      name = "tags",
      cl = "EulerTags"
    )
  }

  panel_children <- grid::gList(
    if (do_container && !is.null(container_fill_grob)) container_fill_grob,
    if (do_fills) fills_grob,
    if (do_patterns && identical(patterns$mode, "shape")) patterns_grob,
    if (do_edges) edges_grob,
    if (do_container && !is.null(container_edge_grob)) container_edge_grob,
    if (do_tags) tags_gtree
  )

  # Wrap each panel in a custom `EulerPanel` gTree so its viewport can
  # be sized at draw time. `makeContext.EulerPanel` measures labels
  # against the live device, runs eunoia's placement, and rewrites
  # the panel viewport's `xscale`/`yscale` to encompass the labels.
  # On window resize grid re-fires `makeContext`, so the bbox tracks
  # the current device and labels can't be clipped at the panel edge.
  #
  # The PURE geometry bbox (no labels) is used to seed the measurement
  # viewport — that's the smallest plausible viewport, which gives the
  # smallest label native size and the most conservative placement.
  # The (label-aware) `x$xlim`/`x$ylim` from setup_geometry are used as
  # the fallback when measurement isn't possible.
  if (NROW(x$shapes) > 0L) {
    geom_box <- get_bounding_box(x$shapes)
    geom_xlim <- geom_box$xlim
    geom_ylim <- geom_box$ylim
    if (!is.null(x$container)) {
      cx <- x$container$h
      cy <- x$container$k
      chw <- x$container$width / 2
      chh <- x$container$height / 2
      geom_xlim <- range(c(geom_xlim, cx - chw, cx + chw))
      geom_ylim <- range(c(geom_ylim, cy - chh, cy + chh))
    }
  } else {
    geom_xlim <- xlim
    geom_ylim <- ylim
  }
  label_precision_base <- max(diff(geom_xlim), diff(geom_ylim)) / 100
  grid::gTree(
    children = panel_children,
    shapes = x$shapes,
    container = if (do_container) x$container else NULL,
    n_vertices = as.integer(n_vertices),
    placement_opts = placement_opts,
    geom_xlim = geom_xlim,
    geom_ylim = geom_ylim,
    label_precision = label_precision_base,
    padding = eulerr_options()$padding,
    name = paste0("diagram.grob.", number),
    cl = "EulerPanel"
  )
}

#' Area-proportional Euler diagrams
#'
#' Fit Euler diagrams (a generalization of Venn diagrams) using numerical
#' optimization to find exact or approximate solutions to a specification of set
#' relationships. The shape of the diagram may be a circle, an ellipse, an
#' axis-aligned rectangle, or an axis-aligned square.
#'
#' If the input is a matrix or data frame and argument `by` is specified,
#' the function returns a list of euler diagrams.
#'
#' The function minimizes the residual sums of squares,
#' \deqn{
#'   \sum_{i=1}^n (A_i - \omega_i)^2,
#' }{
#'   \sum (A_i - \omega_i)^2,
#' }
#' by default, where \eqn{\omega_i} the size of the ith disjoint subset, and
#' \eqn{A_i} the corresponding area in the diagram, that is, the unique
#' contribution to the total area from this overlap. The loss function
#' can, however, be controlled via the `loss` argument.
#'
#' [euler()] also returns `stress` (from \pkg{venneuler}), as well as
#' `diagError`, and `regionError` from \pkg{eulerAPE}.
#'
#' The *stress* statistic is computed as
#'
#' \deqn{
#'   \frac{\sum_{i=1}^n (A_i - \beta\omega_i)^2}{\sum_{i=1}^n A_i^2},
#' }{
#'   \sum (A_i - \beta\omega_i)^2 / \sum A_i^2,
#' }
#' where
#' \deqn{
#'   \beta = \sum_{i=1}^n A_i\omega_i / \sum_{i=1}^n \omega_i^2.
#' }{
#'   \beta = \sum A_i\omega_i / \sum \omega_i^2.
#' }
#'
#' `regionError` is computed as
#'
#' \deqn{
#'   \left| \frac{A_i}{\sum_{i=1}^n A_i} - \frac{\omega_i}{\sum_{i=1}^n \omega_i}\right|.
#' }{
#'   max|A_i / \sum A  - \omega_i / \sum \omega|.
#' }
#'
#' `diagError` is simply the maximum of regionError.
#'
#' @param combinations set relationships as a named numeric vector, matrix, or
#'   data.frame (see **methods (by class)**)
#' @param by a factor or character matrix to be used in [base::by()] to
#'   split the data.frame or matrix of set combinations
#' @param input type of input: disjoint identities
#'   (`'disjoint'`) or unions (`'union'`).
#' @param shape geometric shape used in the diagram
#' @param loss type of loss to minimize over. The default,
#'   `"sum_squared"`, minimizes the sum of squared errors. The available
#'   options mirror the loss functions exposed by the `eunoia` Rust crate
#'   that powers the optimizer:
#'   * `"sum_squared"` --- normalized sum of squared errors (default).
#'   * `"sum_absolute"` --- normalized sum of absolute errors.
#'   * `"sum_absolute_region_error"` --- normalized sum of absolute
#'     region errors.
#'   * `"sum_squared_region_error"` --- normalized sum of squared region
#'     errors.
#'   * `"max_absolute"` --- normalized maximum absolute error.
#'   * `"max_squared"` --- normalized maximum squared error.
#'   * `"root_mean_squared"` --- normalized root-mean-squared error.
#'   * `"stress"` --- venneuler-style stress.
#'   * `"diag_error"` --- eulerAPE-style `diagError`.
#' @param loss_aggregator deprecated; use `loss` directly instead. Pre-1.0
#'   code that combined `loss` (`"square"`/`"abs"`/`"region"`) with
#'   `loss_aggregator` (`"sum"`/`"max"`) still works but emits a warning;
#'   the combination is mapped to the equivalent new `loss` value.
#' @param complement an optional single non-negative number giving the area
#'   of the *complement* — that is, the universe outside every named set.
#'   When supplied, the fitter jointly optimizes a containing rectangle
#'   together with the diagram shapes so that the area of the rectangle
#'   minus the union of (clipped) shapes matches `complement`. This is the
#'   classical "everything not in any set" region; see [plot.euler()] for
#'   how it is rendered. Defaults to `NULL` (no container; classical
#'   shape-only fit). Not supported for [venn()].
#' @param control a list of control parameters.
#'   * `extraopt`: should the global-search fallback optimizer (CMA-ES) kick
#'   in when the primary optimizer's `diagError` exceeds `extraopt_threshold`?
#'   The default is `TRUE` for three-set ellipse fits and `FALSE` otherwise.
#'   * `extraopt_threshold`: threshold, in terms of `diagError`, for when
#'   the CMA-ES fallback kicks in. A value of 0 means it will kick in for
#'   *any* error; a value of 1 means it will never kick in. Default `0.001`.
#'   * `tolerance`: convergence tolerance passed to the underlying solver.
#'   Tighter values give more accurate fits at higher cost. Default `1e-8`.
#'   * `max_sets`: maximum number of sets the underlying engine will accept.
#'   Defaults to `NULL`, which uses the engine's built-in default of 32.
#'   Region masks are stored in a bitset, so values may be raised up to 63
#'   (the absolute hard cap). Going higher is rarely useful in practice
#'   since fully-overlapping diagrams have `2^n - 1` regions.
#' @param ... arguments passed down to other methods
#'
#' @return A list object of class `'euler'` with the following parameters.
#'   \item{shapes}{a data frame of fitted shape parameters. One row per set
#'     with a `type` column (one of `"circle"`, `"ellipse"`, `"rectangle"`,
#'     `"square"`), the center coordinates `h` and `k`, and the
#'     shape-specific columns: `a`, `b`, `phi` for ellipses/circles; `width`
#'     and `height` for rectangles; `side` (plus mirrored `width`/`height`)
#'     for squares. Columns that don't apply to the chosen shape are `NA`.}
#'   \item{ellipses}{for `shape = "circle"` and `shape = "ellipse"` fits,
#'     the legacy 5-column data frame of `h`, `k`, `a`, `b`, `phi`. This
#'     slot is deprecated in favour of `shapes` and is not populated for
#'     rectangle/square fits.}
#'   \item{original.values}{set relationships in the input}
#'   \item{fitted.values}{set relationships in the solution}
#'   \item{residuals}{residuals}
#'   \item{regionError}{the difference in percentage points between each
#'     disjoint subset in the input and the respective area in the output}
#'   \item{diagError}{the largest `regionError`}
#'   \item{stress}{normalized residual sums of squares}
#'
#' @seealso [plot.euler()], [print.euler()], [eulerr_options()], [venn()]
#'
#' @examples
#' # Fit a diagram with circles
#' combo <- c(A = 2, B = 2, C = 2, "A&B" = 1, "A&C" = 1, "B&C" = 1)
#' fit1 <- euler(combo)
#'
#' # Investigate the fit
#' fit1
#'
#' # Refit using ellipses instead
#' fit2 <- euler(combo, shape = "ellipse")
#'
#' # Investigate the fit again (which is now exact)
#' fit2
#'
#' # Plot it
#' plot(fit2)
#'
#' # A set with no perfect solution
#' euler(c(
#'   "a" = 3491, "b" = 3409, "c" = 3503,
#'   "a&b" = 120, "a&c" = 114, "b&c" = 132,
#'   "a&b&c" = 50
#' ))
#'
#' @references Wilkinson L. Exact and Approximate Area-Proportional Circular
#'   Venn and Euler Diagrams. IEEE Transactions on Visualization and Computer
#'   Graphics (Internet). 2012 Feb (cited 2016 Apr 9);18(2):321-31. Available
#'   from: \doi{10.1109/TVCG.2011.56}
#'
#'   Micallef L, Rodgers P. eulerAPE: Drawing Area-Proportional 3-Venn Diagrams
#'   Using Ellipses. PLOS ONE (Internet). 2014 Jul (cited 2016 Dec
#'   10);9(7):e101717. Available from:
#'   \doi{10.1371/journal.pone.0101717}
#'
#' @export
euler <- function(combinations, ...) UseMethod("euler")

#' @describeIn euler a named numeric vector, with
#'   combinations separated by an ampersand, for instance `A&B = 10`.
#'   Missing combinations are treated as being 0.
#'
#' @export
euler.default <- function(
  combinations,
  input = c("disjoint", "union"),
  shape = c("circle", "ellipse", "rectangle", "square"),
  loss = c(
    "sum_squared",
    "sum_absolute",
    "sum_absolute_region_error",
    "sum_squared_region_error",
    "max_absolute",
    "max_squared",
    "root_mean_squared",
    "stress",
    "diag_error"
  ),
  loss_aggregator = NULL,
  complement = NULL,
  control = list(),
  ...
) {
  fit_diagram(
    combinations,
    "euler",
    input,
    shape,
    loss,
    loss_aggregator,
    complement = complement,
    control = control,
    ...
  )
}

#' @describeIn euler a `data.frame` of logicals, binary integers, or
#'   factors.
#' @param weights a numeric vector of weights of the same length as
#'   the number of rows in `combinations`.
#' @param sep a character to use to separate the dummy-coded factors
#'   if there are factor or character vectors in 'combinations'.
#' @param factor_names whether to include factor names when
#'   constructing dummy codes
#' @export
#' @examples
#'
#' # Using grouping via the 'by' argument through the data.frame method
#' euler(fruits, by = list(sex, age))
#'
euler.data.frame <- function(
  combinations,
  weights = NULL,
  by = NULL,
  sep = "_",
  factor_names = TRUE,
  ...
) {
  by <- substitute(by)
  facs <- eval(by, combinations)

  combinations <- parse_dataframe(
    combinations,
    weights,
    by,
    facs,
    sep,
    factor_names
  )

  if (is.list(combinations)) {
    out <- lapply(combinations, euler, ...)
    class(out) <- c("euler", "list")
    attr(out, "groups") <- attr(combinations, "groups")
  } else {
    out <- euler(combinations, ...)
  }

  out
}

#' @describeIn euler a matrix that can be converted to a data.frame of logicals
#'   (as in the description above) via [base::as.data.frame.matrix()].
#' @export
#'
#' @examples
#'
#' # Using the matrix method
#' euler(organisms)
#'
#' # Using weights
#' euler(organisms, weights = c(10, 20, 5, 4, 8, 9, 2))
euler.matrix <- function(combinations, ...) {
  euler(as.data.frame(combinations), ...)
}

#' @describeIn euler A table with `max(dim(x)) < 3`.
#' @export
#'
#' @examples
#'
#' # The table method
#' euler(pain, factor_names = FALSE)
euler.table <- function(combinations, ...) {
  x <- as.data.frame(combinations)
  euler(x[, !(names(x) == "Freq")], weights = x$Freq, ...)
}

#' @describeIn euler a list of vectors, each vector giving the contents of
#'   that set (with no duplicates). Vectors in the list must be named.
#' @export
#' @examples
#'
#' # A euler diagram from a list of sample spaces (the list method)
#' euler(plants[c("erigenia", "solanum", "cynodon")])
euler.list <- function(combinations, ...) {
  out <- parse_list(combinations)
  euler(out, input = "disjoint", ...)
}

test_that("fitted and coef methods work as they should", {
  fit <- euler(c(A = 1, B = 2))
  expect_is(fitted(fit), "numeric")
  expect_is(coef(fit), "data.frame")
})

test_that("coef returns $shapes (canonical) for all shape kinds", {
  combo <- c(A = 5, B = 3, "A&B" = 1)
  for (shape in c("circle", "ellipse", "rectangle", "square")) {
    fit <- euler(combo, shape = shape)
    out <- coef(fit)
    expect_is(out, "data.frame")
    expect_true("type" %in% names(out))
    expect_identical(out, fit$shapes)
  }
})

test_that("$ellipses is populated for circle/ellipse and absent otherwise", {
  combo <- c(A = 5, B = 3, "A&B" = 1)
  fc <- euler(combo, shape = "circle")
  fe <- euler(combo, shape = "ellipse")
  fr <- euler(combo, shape = "rectangle")
  fs <- euler(combo, shape = "square")
  expect_false(is.null(fc$ellipses))
  expect_false(is.null(fe$ellipses))
  expect_null(fr$ellipses)
  expect_null(fs$ellipses)
})

test_that("complement attaches a fitted container", {
  set.seed(1)
  fit <- euler(c(A = 25, B = 25, "A&B" = 5), complement = 145)

  expect_false(is.null(fit$container))
  expect_equal(fit$container$complement, 145)

  container_area <- fit$container$width * fit$container$height
  fitted_total <- sum(fit$fitted.values)
  # Container area minus union of fitted shapes ≈ requested complement.
  expect_equal(
    container_area - fitted_total,
    145,
    tolerance = 1e-2
  )
  expect_equal(fit$container$complement_fitted, 145, tolerance = 1e-2)
})

test_that("no complement leaves container NULL", {
  fit <- euler(c(A = 5, B = 5, "A&B" = 1))
  expect_null(fit$container)
})

test_that("complement validates its input", {
  expect_error(
    euler(c(A = 1, B = 1, "A&B" = 0.5), complement = -1),
    "non-negative"
  )
  expect_error(
    euler(c(A = 1, B = 1, "A&B" = 0.5), complement = c(1, 2)),
    "single non-negative"
  )
  expect_error(
    euler(c(A = 1, B = 1, "A&B" = 0.5), complement = NA_real_),
    "non-negative"
  )
})

test_that("complement is rejected for venn()", {
  expect_error(
    venn(c(A = 1, B = 1, "A&B" = 0.5), complement = 5),
    "not supported for `venn"
  )
})

test_that("single-set complement synthesises a container", {
  fit <- euler(c(A = 30), complement = 70)
  expect_false(is.null(fit$container))
  # universe area = 30 + 70 = 100, square side = 10
  expect_equal(fit$container$width, 10, tolerance = 1e-6)
  expect_equal(fit$container$height, 10, tolerance = 1e-6)
})

test_that("plotting an euler with complement renders without error", {
  set.seed(1)
  fit <- euler(c(A = 25, B = 25, "A&B" = 5), complement = 145)
  tmp <- tempfile(fileext = ".png")
  png(tmp)
  on.exit({
    dev.off()
    unlink(tmp)
  })
  expect_silent(plot(fit, quantities = TRUE))
  expect_silent(plot(fit, complement = FALSE))
  expect_silent(plot(fit, complement = "grey90"))
  expect_silent(plot(
    fit,
    complement = list(
      fill = "lightyellow",
      col = "navy",
      lty = 1,
      lwd = 2,
      label = "outside",
      fontsize = 14,
      font = 2
    )
  ))
})

diagram_children <- function(g) {
  # Walk the diagram subtree and return a flat list keyed by grob name.
  # The complement label moved inside the EulerTags subtree so the
  # complement count can be re-placed at draw time alongside region
  # tags; the legacy lookup path returned only the top-level children
  # of `diagram.grob.<n>`.
  diagram <- g$children$canvas.grob$children[[1]]
  out <- list()
  walk <- function(node) {
    nm <- node$name
    if (!is.null(nm) && nzchar(nm)) {
      out[[nm]] <<- node
    }
    kids <- node$children
    if (!is.null(kids)) {
      for (i in seq_along(kids)) walk(kids[[i]])
    }
  }
  walk(diagram)
  out
}

test_that("complement = FALSE drops the container grobs", {
  set.seed(1)
  fit <- euler(c(A = 25, B = 25, "A&B" = 5), complement = 145)
  tmp <- tempfile(fileext = ".png")
  png(tmp)
  on.exit({
    dev.off()
    unlink(tmp)
  })
  with_box <- plot(fit, quantities = TRUE)
  no_box <- plot(fit, quantities = TRUE, complement = FALSE)

  expect_true("container.edge.grob" %in% names(diagram_children(with_box)))
  expect_false("container.edge.grob" %in% names(diagram_children(no_box)))
  expect_false(
    "complement.quantity.grob" %in% names(diagram_children(no_box))
  )
})

test_that("complement$label overrides the count text", {
  set.seed(1)
  fit <- euler(c(A = 25, B = 25, "A&B" = 5), complement = 145)
  tmp <- tempfile(fileext = ".png")
  png(tmp)
  on.exit({
    dev.off()
    unlink(tmp)
  })
  g <- plot(fit, complement = list(label = "n = 145"))
  label_grob <- diagram_children(g)[["complement.quantity.grob"]]
  expect_false(is.null(label_grob))
  expect_equal(label_grob$label, "n = 145")
})

test_that("complement gpar overrides flow through to the outline grob", {
  set.seed(1)
  fit <- euler(c(A = 25, B = 25, "A&B" = 5), complement = 145)
  tmp <- tempfile(fileext = ".png")
  png(tmp)
  on.exit({
    dev.off()
    unlink(tmp)
  })
  g <- plot(
    fit,
    complement = list(col = "navy", lty = 1, lwd = 3, fill = "lightyellow")
  )
  edge <- diagram_children(g)[["container.edge.grob"]]
  fill <- diagram_children(g)[["complement.fill.grob"]]
  expect_equal(edge$gp$col, "navy")
  expect_equal(edge$gp$lty, 1)
  expect_equal(edge$gp$lwd, 3)
  expect_equal(fill$gp$fill, "lightyellow")
})

test_that("default complement uses dashed outline (lty = 2)", {
  set.seed(1)
  fit <- euler(c(A = 25, B = 25, "A&B" = 5), complement = 145)
  tmp <- tempfile(fileext = ".png")
  png(tmp)
  on.exit({
    dev.off()
    unlink(tmp)
  })
  g <- plot(fit)
  edge <- diagram_children(g)[["container.edge.grob"]]
  expect_equal(edge$gp$lty, 2L)
})

test_that("euler_plot_data returns a complement polygon and label", {
  set.seed(1)
  fit <- euler(c(A = 25, B = 25, "A&B" = 5), complement = 145)
  shapes <- fit$shapes
  cd <- euler_plot_data(
    set_names = rownames(shapes),
    shape = shapes$type[1L],
    h = shapes$h,
    k = shapes$k,
    a = shapes$a,
    b = shapes$b,
    phi = shapes$phi,
    width = shapes$width,
    height = shapes$height,
    side = shapes$side,
    container_h = fit$container$h,
    container_k = fit$container$k,
    container_width = fit$container$width,
    container_height = fit$container$height,
    n_vertices = 200L,
    label_precision = 0.1
  )
  expect_true(cd$has_complement)
  expect_length(cd$container_outline_x, 5L)
  expect_length(cd$container_outline_y, 5L)
  expect_gt(length(cd$complement_polygon$x), 0L)
  # Label anchor must lie outside both fitted disks.
  d_to_A <- sqrt(
    (cd$complement_label_x - shapes$h[1])^2 +
      (cd$complement_label_y - shapes$k[1])^2
  )
  d_to_B <- sqrt(
    (cd$complement_label_x - shapes$h[2])^2 +
      (cd$complement_label_y - shapes$k[2])^2
  )
  expect_gt(d_to_A, shapes$a[1])
  expect_gt(d_to_B, shapes$a[2])
})

test_that("normal use of euler() returns no errors", {
  f <- euler(c(A = 10.923, B = 5.4, "A&B" = 0.43))

  expect_error(dont_print(f, round = 2), NA)

  # test both against stringsAsFactors = TRUE and FALSE
  dat <- data.frame(
    A = sample(c(TRUE, FALSE), size = 100, replace = TRUE),
    B = sample(c(TRUE, TRUE, FALSE), size = 100, replace = TRUE),
    x = sample(c("Men", "Women"), size = 100, replace = TRUE),
    stringsAsFactors = TRUE
  )

  expect_is(euler(dat), "euler")

  dat <- data.frame(
    A = sample(c(TRUE, FALSE), size = 100, replace = TRUE),
    B = sample(c(TRUE, TRUE, FALSE), size = 100, replace = TRUE),
    x = sample(c("Men", "Women"), size = 100, replace = TRUE),
    stringsAsFactors = FALSE
  )

  expect_is(euler(dat), "euler")

  expect_silent(euler(as.matrix(dat[, 1:2])))
  expect_is(euler(dat, by = x), "euler")
  expect_silent(dont_print(euler(dat, by = x)))
  expect_silent(dont_print(venn(dat, by = x)))

  d <- data.frame(
    Apple = c(1, 0, 0, 1),
    Banana = c(0, 1, 0, 1),
    Orange = c(1, 0, 1, 0)
  )
  expect_silent(euler(d))
})

test_that("normal use of venn() returns no errors", {
  expect_is(venn(organisms), "eulerr_venn")
  expect_silent(venn(organisms))
  expect_silent(venn(c(A = 1, B = 2)))
  expect_silent(venn(pain))
  expect_silent(venn(fruits, by = list(sex, age)))
  expect_silent(venn(organisms, weights = c(10, 20, 5, 4, 8, 9, 2)))
  expect_silent(venn(plants[c("erigenia", "solanum", "cynodon")]))
})

test_that("all venn diagram sizes work", {
  for (n in 2:5) {
    expect_silent(venn(n, names = letters[1:n]))
  }
})

test_that("normal use of venn() returns no errors", {
  expect_silent(venn(organisms))
  expect_silent(venn(c(A = 1, B = 2)))
  expect_silent(venn(pain))
  expect_silent(venn(fruits, by = list(sex, age)))
  expect_silent(venn(organisms, weights = c(10, 20, 5, 4, 8, 9, 2)))
  expect_silent(venn(plants[c("erigenia", "solanum", "cynodon")]))
})

test_that("all venn diagram sizes work", {
  for (n in 2:5) {
    expect_silent(venn(n, names = letters[1:n]))
  }
})

test_that("using weights works", {
  dat2 <- data.frame(
    A = c(TRUE, FALSE, TRUE, TRUE),
    B = c(FALSE, TRUE, TRUE, FALSE)
  )
  expect_is(euler(dat2, weights = c(3, 2, 1, 1)), "euler")
})

test_that("correct parallel options are asserted", {
  s <- c(a = 1, b = 2, "a&b" = 3)
  expect_silent(euler(s, n_threads = 1))
})

test_that("zero-sized input is allowed", {
  s <- c(A = 0)
  expect_silent(f <- euler(s))
  expect_equivalent(fitted(f), 0)

  s <- c(A = 0, B = 0)
  expect_silent(f <- euler(s))
  expect_equivalent(fitted(f), rep(0, length(fitted(f))))
})

test_that("factors in euler.data.frame() are handled appropriately", {
  d <- data.frame(
    A = sample(c(TRUE, FALSE), 90, TRUE),
    gender = gl(3, 30, 90, c("man", "woman", "transgender")),
    status = sample(gl(2, 45, 90, c("cohabitation", "single")))
  )
  expect_silent(f <- euler(d, by = list(status)))
})

test_that("list method works appropriately", {
  l <- list(A = c("a", "b", "c"), B = c("b", "e"), C = c("c"))
  expect_silent(euler(l))
})

test_that("table method works appropriately", {
  expect_silent(euler(as.table(apply(Titanic, 2:4, sum))))
})

test_that("sparse inputs with many sets do not blow up", {
  # 12 sets with only a few input overlaps. Pre-refactor this allocated a
  # 4095-row id matrix and 4095-length result vectors per call.
  sets <- paste0("S", 1:12)
  input <- c(
    stats::setNames(rep(10, 12), sets),
    c("S1&S2" = 3, "S2&S3" = 2, "S1&S2&S3" = 1)
  )
  f <- euler(input)
  expect_lt(length(f$fitted.values), 50L)
  expect_lt(length(f$original.values), 50L)
  expect_true(all(c("S1", "S1&S2", "S1&S2&S3") %in% names(f$fitted.values)))
  expect_silent(plot(f))
})

test_that("legacy loss / loss_aggregator arguments warn and still work", {
  s <- c(A = 2, B = 2, "A&B" = 1)

  # New-style loss values are silent
  expect_silent(euler(s, loss = "sum_squared"))
  expect_silent(euler(s, loss = "diag_error"))

  # Legacy loss values warn and translate
  expect_warning(euler(s, loss = "square"), "deprecated")
  expect_warning(euler(s, loss = "region"), "deprecated")

  # loss_aggregator on its own warns and is otherwise ignored
  expect_warning(
    euler(s, loss = "sum_squared", loss_aggregator = "sum"),
    "loss_aggregator.*deprecated"
  )

  # Legacy combination produces the same fit as its new equivalent
  set.seed(1)
  new_fit <- euler(s, loss = "diag_error")
  set.seed(1)
  legacy_fit <- suppressWarnings(
    euler(s, loss = "region", loss_aggregator = "max")
  )
  expect_equal(new_fit$shapes, legacy_fit$shapes)
  expect_equal(new_fit$ellipses, legacy_fit$ellipses)
})

test_that("fitted(dense = TRUE) expands to all 2^n - 1 combinations", {
  f <- euler(c(A = 1, B = 1, "A&B" = 0.5))
  sparse <- fitted(f)
  dense <- fitted(f, dense = TRUE)
  expect_setequal(names(dense), c("A", "B", "A&B"))
  expect_identical(dense[names(sparse)], sparse)

  # With absent combinations, dense fills 0.
  f2 <- euler(c(A = 1, B = 1, C = 1, "A&B" = 0.5))
  dense2 <- fitted(f2, dense = TRUE)
  expect_length(dense2, 2L^3L - 1L)
  expect_true(all(c("A&C", "B&C", "A&B&C") %in% names(dense2)))
  expect_equal(dense2[["A&C"]], 0)
})

test_that("rectangle fit populates width/height and leaves a/b/phi as NaN", {
  combo <- c(A = 5, B = 4, C = 6, "A&B" = 2, "A&C" = 1, "B&C" = 1, "A&B&C" = 0.5)
  fit <- euler(combo, shape = "rectangle")
  expect_is(fit, "euler")
  s <- fit$shapes
  expect_identical(unique(s$type), "rectangle")
  expect_true(all(is.finite(s$h)))
  expect_true(all(is.finite(s$k)))
  expect_true(all(is.finite(s$width)))
  expect_true(all(is.finite(s$height)))
  expect_true(all(is.nan(s$a)))
  expect_true(all(is.nan(s$b)))
  expect_true(all(is.nan(s$phi)))
  expect_true(all(is.nan(s$side)))
  expect_true(is.finite(fit$diagError))
  expect_null(fit$ellipses)
})

test_that("square fit populates side with width == height == side", {
  combo <- c(A = 5, B = 4, "A&B" = 1)
  fit <- euler(combo, shape = "square")
  s <- fit$shapes
  expect_identical(unique(s$type), "square")
  expect_true(all(is.finite(s$side)))
  expect_equal(s$width, s$side)
  expect_equal(s$height, s$side)
  expect_true(all(is.nan(s$a)))
})

test_that("rectangle/square diagrams plot without error", {
  combo <- c(A = 5, B = 4, "A&B" = 1)
  for (shape in c("rectangle", "square")) {
    fit <- euler(combo, shape = shape)
    pdf(NULL)
    on.exit(dev.off(), add = TRUE)
    g <- plot(fit)
    expect_s3_class(g, "eulergram")
  }
})

test_that("single-set edge cases return well-defined geometry per shape", {
  for (shape in c("circle", "ellipse", "rectangle", "square")) {
    fit <- euler(c(A = 5), shape = shape)
    s <- fit$shapes
    expect_equal(NROW(s), 1L)
    expect_true(is.finite(s$h))
    expect_true(is.finite(s$k))
    if (shape %in% c("circle", "ellipse")) {
      expect_true(is.finite(s$a))
      expect_true(is.finite(s$b))
    } else {
      expect_true(is.finite(s$width))
      expect_true(is.finite(s$height))
    }
  }
})

#' Compute geometries and label locations
#'
#' @param x an object of class 'euler'
#' @param fills fills
#' @param edges edges
#' @param labels labels
#' @param quantities quantities
#' @param n number of vertices to use to render each ellipse
#' @param merged_sets which sets have been merged?
#'
#' @return a list object with slots for the various objects
#' @keywords internal
setup_geometry <- function(
  x,
  fills,
  edges,
  labels,
  quantities,
  annotations,
  n,
  merged_sets,
  placement_opts = NULL,
  do_complement_label = FALSE
) {
  dd <- x$shapes
  empty_sets <- is.na(dd$h) & !merged_sets
  setnames_full <- rownames(dd)
  empty_set_names <- setnames_full[empty_sets]

  combo_labels_full <- names(x$fitted.values)
  combo_sets_full <- strsplit(combo_labels_full, "&", fixed = TRUE)
  empty_subsets <- vapply(
    combo_sets_full,
    function(s) any(s %in% empty_set_names),
    logical(1)
  )

  orig <- x$original.values[!empty_subsets]
  fitted <- x$fitted.values[!empty_subsets]
  combo_labels <- combo_labels_full[!empty_subsets]
  combo_sets <- combo_sets_full[!empty_subsets]

  dd <- dd[!empty_sets, , drop = FALSE]

  nonzero <- nonzero_fit(fitted)
  nonzero <- ifelse(is.na(nonzero), FALSE, nonzero)

  do_fills <- !is.null(fills)
  do_edges <- !is.null(edges)
  do_labels <- !is.null(labels)
  do_quantities <- !is.null(quantities)
  do_annotations <- !is.null(annotations)

  shape_type <- if (NROW(dd) > 0L) dd$type[1L] else "ellipse"

  n_e <- NROW(dd)
  n_id <- length(combo_labels)

  container <- x$container
  do_container <- !is.null(container)

  if (n_e > 0L) {
    limits <- get_bounding_box(dd)
  } else {
    limits <- list(xlim = c(-1, 1), ylim = c(-1, 1))
  }

  if (do_container) {
    cx_half <- container$width / 2
    cy_half <- container$height / 2
    container_xlim <- c(container$h - cx_half, container$h + cx_half)
    container_ylim <- c(container$k - cy_half, container$k + cy_half)
    limits$xlim <- range(c(limits$xlim, container_xlim))
    limits$ylim <- range(c(limits$ylim, container_ylim))
  }

  width <- abs(limits$xlim[1] - limits$xlim[2])
  height <- abs(limits$ylim[1] - limits$ylim[2])

  if (n_e > 0L) {
    plot_data <- euler_plot_data(
      set_names = rownames(dd),
      shape = shape_type,
      h = dd$h,
      k = dd$k,
      a = dd$a,
      b = dd$b,
      phi = dd$phi,
      width = dd$width,
      height = dd$height,
      side = dd$side,
      container_h = if (do_container) container$h else NULL,
      container_k = if (do_container) container$k else NULL,
      container_width = if (do_container) container$width else NULL,
      container_height = if (do_container) container$height else NULL,
      n_vertices = as.integer(n),
      label_precision = max(width, height) / 100
    )
    set_polygons <- plot_data$set_polygons
    region_labels_geom <- plot_data$region_labels
    region_polygons_geom <- plot_data$region_polygons
    region_centers_x_geom <- plot_data$region_centers_x
    region_centers_y_geom <- plot_data$region_centers_y

    align_idx <- match(combo_labels, region_labels_geom)
    region_polygons <- vector("list", n_id)
    region_centers_x <- rep(NA_real_, n_id)
    region_centers_y <- rep(NA_real_, n_id)
    has_geom <- !is.na(align_idx)
    if (any(has_geom)) {
      region_polygons[has_geom] <- region_polygons_geom[align_idx[has_geom]]
      region_centers_x[has_geom] <- region_centers_x_geom[align_idx[has_geom]]
      region_centers_y[has_geom] <- region_centers_y_geom[align_idx[has_geom]]
    }
  } else {
    plot_data <- NULL
    set_polygons <- list()
    region_polygons <- list()
    region_centers_x <- double(0)
    region_centers_y <- double(0)
  }

  e_x <- c(lapply(set_polygons, "[[", "x"), recursive = TRUE)
  e_y <- c(lapply(set_polygons, "[[", "y"), recursive = TRUE)

  # setup edges
  if (do_edges) {
    edges <- list(
      x = e_x,
      y = e_y,
      id.lengths = vapply(set_polygons, function(p) length(p$x), integer(1))
    )
  }

  if (do_fills || do_labels || do_quantities || do_annotations) {
    fills <- vector("list", n_id)
    for (i in seq_len(n_id)) {
      if (!nonzero[i]) {
        next
      }
      rp <- region_polygons[[i]]
      if (is.null(rp) || length(rp$id_lengths) == 0L) {
        next
      }
      fills[[i]]$x <- rp$x
      fills[[i]]$y <- rp$y
      fills[[i]]$id.lengths <- rp$id_lengths
    }
  }

  if (do_labels || do_quantities || do_annotations) {
    empty <- !nonzero_fit(fitted)

    centers_x <- region_centers_x
    centers_y <- region_centers_y
    centers_x[!nonzero] <- NA_real_
    centers_y[!nonzero] <- NA_real_
    centers_id <- seq_len(n_id)

    centers <- data.frame(
      x = centers_x,
      y = centers_y,
      id = centers_id,
      labels = rep(NA_character_, n_id),
      quantities = rep(NA, n_id),
      annotations = rep(NA_character_, n_id),
      labels_par_id = rep(NA_integer_, n_id),
      quantities_par_id = rep(NA_integer_, n_id),
      annotations_par_id = rep(NA_integer_, n_id),
      row.names = combo_labels,
      stringsAsFactors = FALSE
    )

    has_center <- !is.na(centers$x) & !is.na(centers$y)

    if (do_labels) {
      labels$labels <- labels$labels[which(!empty_sets)]
    }

    singles <- logical(NROW(centers))

    setnames <- rownames(dd)
    for (i in seq_len(n_e)) {
      set_i <- setnames[i]
      in_set <- vapply(combo_sets, function(s) set_i %in% s, logical(1))
      ind <- which(in_set & !empty & has_center)[1]

      if (!is.na(ind)) {
        if (do_labels) {
          centers$labels[ind] <- labels$labels[i]
          centers$labels_par_id[ind] <- i
        }
        singles[ind] <- TRUE
      }
    }

    others <- has_center & !singles & !empty

    if (do_quantities) {
      num <- orig[centers$id[singles | others]]

      if (is.null(quantities$labels)) {
        template <- quantities$template
        valid_types <- c("counts", "percent", "fraction")
        if (!is.null(template)) {
          if (!is.character(template) || length(template) != 1L) {
            stop("`quantities$template` must be a single string.")
          }
          placeholders <- regmatches(
            template,
            gregexpr("\\{([^{}]+)\\}", template)
          )[[1]]
          placeholders <- gsub("[{}]", "", placeholders)
          unknown <- setdiff(placeholders, valid_types)
          if (length(unknown) > 0L) {
            stop(
              "Unknown placeholder(s) in `quantities$template`: ",
              paste0("{", unknown, "}", collapse = ", "),
              ". Valid placeholders: ",
              paste0("{", valid_types, "}", collapse = ", "),
              "."
            )
          }
          type <- unique(placeholders)
        } else {
          type <- quantities$type
        }
        perc <- frac <- NULL
        fmt_fun <- quantities$format$fun
        fmt_args <- quantities$format$args

        total <- quantities$total
        if (is.null(total)) {
          total <- sum(num, na.rm = TRUE)
        }

        format_numeric <- function(x, default_fun, default_args = list()) {
          if (is.null(fmt_fun)) {
            out <- do.call(default_fun, c(list(x), default_args))
          } else {
            out <- do.call(fmt_fun, c(list(x), fmt_args))
          }
          as.character(out)
        }

        if ("percent" %in% type) {
          perc <- num / total * 100
          perc <- format_numeric(
            perc,
            function(x) ifelse(x >= 1, as.character(round(x)), "< 1")
          )
          perc <- sapply(perc[!is.na(perc)], function(x) paste0(x, " %"))
        }

        if ("fraction" %in% type) {
          frac <- num / total
          frac <- format_numeric(
            frac,
            signif,
            list(digits = options("digits")$digits)
          )
        }

        cnt <- format_numeric(
          num,
          signif,
          list(digits = options("digits")$digits)
        )

        values <- list(counts = cnt, percent = perc, fraction = frac)

        if (!is.null(template)) {
          n_q <- length(values[[type[1]]])
          qnt <- vapply(
            seq_len(n_q),
            function(i) {
              s <- template
              for (t in type) {
                s <- gsub(
                  paste0("{", t, "}"),
                  values[[t]][i],
                  s,
                  fixed = TRUE
                )
              }
              s
            },
            character(1)
          )
          centers$quantities[singles | others] <- qnt
        } else if (length(type) == 1) {
          centers$quantities[singles | others] <- values[[type]]
        } else if (length(type) == 2) {
          qnt <- paste0(values[[type[1]]], " (", values[[type[2]]], ")")
          centers$quantities[singles | others] <- qnt
        } else {
          qnt <- paste0(
            values[[type[1]]],
            " (",
            values[[type[2]]],
            "; ",
            values[[type[3]]],
            ")"
          )
          centers$quantities[singles | others] <- qnt
        }
      } else {
        if (!is.null(names(quantities$labels))) {
          named_quantities <- quantities$labels[rownames(centers)[
            singles | others
          ]]
          centers$quantities[singles | others] <- ifelse(
            is.na(named_quantities),
            NA_character_,
            unname(named_quantities)
          )
        } else {
          centers$quantities[singles | others] <-
            quantities$labels[which(!empty_subsets)][centers$id[
              singles | others
            ]]
        }
      }
    }

    if (do_annotations && !is.null(annotations$labels)) {
      rows <- singles | others
      annot_named <- annotations$labels[rownames(centers)[rows]]
      centers$annotations[rows] <- ifelse(
        is.na(annot_named),
        NA_character_,
        unname(annot_named)
      )
    }

    centers <- centers[has_center, , drop = FALSE]

    n_q <- sum(!is.na(centers$quantities))
    if (n_q > 0L) {
      centers$quantities_par_id[!is.na(centers$quantities)] <- seq_len(n_q)
    }

    n_a <- sum(!is.na(centers$annotations))
    if (n_a > 0L) {
      centers$annotations_par_id[!is.na(centers$annotations)] <- seq_len(n_a)
    }

    has_tag <- !is.na(centers$quantities_par_id) |
      !is.na(centers$labels_par_id) |
      !is.na(centers$annotations_par_id)

    centers <- centers[has_tag, , drop = FALSE]
  } else {
    centers <- NULL
  }

  container_data <- NULL
  if (do_container) {
    if (n_e > 0L) {
      cd <- list(
        outline_x = plot_data$container_outline_x,
        outline_y = plot_data$container_outline_y,
        complement_polygon = plot_data$complement_polygon,
        label_x = plot_data$complement_label_x,
        label_y = plot_data$complement_label_y
      )
    } else {
      # No fitted shapes: the complement *is* the whole container, so the
      # rectangle minus nothing is the rectangle itself. Synthesise it on the
      # R side rather than calling into the (shape-driven) Rust pipeline.
      cd <- list(
        outline_x = c(
          container$h - cx_half,
          container$h + cx_half,
          container$h + cx_half,
          container$h - cx_half,
          container$h - cx_half
        ),
        outline_y = c(
          container$k - cy_half,
          container$k - cy_half,
          container$k + cy_half,
          container$k + cy_half,
          container$k - cy_half
        ),
        complement_polygon = list(
          x = c(
            container$h - cx_half,
            container$h + cx_half,
            container$h + cx_half,
            container$h - cx_half
          ),
          y = c(
            container$k - cy_half,
            container$k - cy_half,
            container$k + cy_half,
            container$k + cy_half
          ),
          id_lengths = 4L
        ),
        label_x = container$h,
        label_y = container$k
      )
    }

    quantity_label <- NA_character_
    val <- container$complement
    if (is.null(val) || !is.finite(val)) {
      val <- container$complement_fitted
    }
    if (is.finite(val)) {
      quantity_label <- format(
        signif(val, digits = options("digits")$digits)
      )
    }

    container_data <- list(
      h = container$h,
      k = container$k,
      width = container$width,
      height = container$height,
      complement = container$complement,
      complement_fitted = container$complement_fitted,
      outline = list(x = cd$outline_x, y = cd$outline_y),
      complement_polygon = cd$complement_polygon,
      label_x = cd$label_x,
      label_y = cd$label_y,
      quantity_label = quantity_label
    )
  }

  # Refine label positions via eunoia's `place_labels` API: anchors come
  # from POI when the label fits inside its region, otherwise eunoia
  # places them exterior with a leader line. Also widens xlim/ylim so
  # exterior labels stay inside the panel viewport — labels are never
  # clipped.
  if (n_e > 0L) {
    placement_label <- do_complement_label &&
      !is.null(container_data) &&
      !is.null(container_data$quantity_label) &&
      !is.na(container_data$quantity_label)
    placed <- apply_label_placement(
      centers = centers,
      container_data = container_data,
      shapes = dd,
      labels = labels,
      quantities = quantities,
      annotations = annotations,
      placement_opts = placement_opts,
      do_complement_label = placement_label,
      limits = limits,
      n_vertices = as.integer(n),
      label_precision = max(width, height) / 100
    )
    centers <- placed$centers
    container_data <- placed$container_data
    limits <- placed$limits
  }

  list(
    shapes = dd,
    set_polygons = set_polygons,
    fitted.values = fitted,
    original.values = orig,
    fills = fills,
    edges = edges,
    labels = labels,
    quantities = quantities,
    annotations = annotations,
    centers = centers,
    empty_sets = empty_sets,
    empty_subsets = empty_subsets,
    container = container_data,
    xlim = limits$xlim,
    ylim = limits$ylim
  )
}

#' Error plot for `euler` objects
#'
#' This is a diagnostic tool for evaluating the fit from a call
#' to [euler()] visually. A color key is provided by default, which
#' represents the chosen error metric so that one can easily detect
#' which areas in the diagram to be skeptical about.
#'
#' Notice that this function is purely provided for diagnostic reasons
#' and does not come with the same kind of customization that
#' [plot.euler()] provides: the color legend can only be customized
#' in regards to its color palette and another key (instead of labels)
#' is completely turned off.
#'
#' @param x an object of class `euler`, typically the result of
#'   a call to [euler()].
#' @param type error metric. `'regionError'` is the difference in
#'   *percentage points* from the input
#' @param quantities whether to draw the error metric on the plot
#' @param pal color palette for the fills in the legend
#' @param ... arguments passed down to [plot.euler()]. Currently,
#'   providing `fills`, `legend`, or `strips` are not allowed and
#'   will return a warning.
#'
#' @return Returns an object of class `eulergram`, which will be
#' plotted on the device in the same manner as objects from
#' [plot.euler()]. See [plot.eulergram()] for details.
#'
#' @export
#'
#' @seealso [plot.euler()], [euler()],
#'   [plot.eulergram()]
#'
#' @examples
#' error_plot(euler(organisms), quantities = FALSE)
error_plot <- function(
  x,
  type = c("regionError", "residuals"),
  quantities = TRUE,
  pal = NULL,
  ...
) {
  type <- match.arg(type)
  dots <- list(...)
  n <- 101 # number of colors in legend

  if (!is.null(dots$fills)) {
    stop("values for 'fills' are not allowed here and will be ignored.")
  }

  if (!is.null(dots$legend)) {
    stop("values for 'legend' are not allowed here and will be ignored.")
  }

  if (!is.null(dots$strips)) {
    stop("values for 'strips' are not allowed here and will be ignored.")
  }

  r <- switch(
    type,
    residuals = stats::residuals(x),
    regionError = sign(stats::residuals(x)) * x$regionError
  )

  if (is.null(pal)) {
    pal <- grDevices::colorRampPalette(
      c(
        "#67001F",
        "#B2182B",
        "#D6604D",
        "#F4A582",
        "#FDDBC7",
        "#F7F7F7",
        "#D1E5F0",
        "#92C5DE",
        "#4393C3",
        "#2166AC",
        "#053061"
      ),
      space = "rgb"
    )
  }

  lim <- max(c(abs(r), ifelse(type == "regionError", 0.01, 0)))

  rng <- grDevices::extendrange(c(-lim, lim))

  labels <- grid::grid.pretty(rng)
  n_labels <- length(labels)

  s <- seq(rng[1], rng[2], length.out = n)

  ind <- findInterval(r, s)

  labels_grob <- grid::textGrob(
    label = labels,
    x = rep(0, length(labels)),
    y = labels,
    just = "left",
    vp = grid::viewport(yscale = rng),
    default.units = "native",
    name = paste("eulerr", "error_plot", sep = ".")
  )

  heights <- grid::unit(c(1, 1, 1), c("lines", "null", "lines"))
  widths <- grid::unit(
    c(1, 1, 1),
    c("lines", "lines", "grobwidth"),
    data = list(NULL, NULL, labels_grob)
  )

  layout <- grid::grid.layout(
    nrow = 3,
    ncol = 3,
    heights = heights,
    widths = widths,
    respect = TRUE
  )

  frame <- grid::frameGrob(
    layout = layout,
    vp = NULL,
    name = paste("eulerr", "error_plot", "key", sep = ".")
  )

  step <- diff(range(s)) / (n - 1)
  y_rect <- seq(min(s) + step / 2, max(s) - step / 2, length.out = n - 1)

  ticks_grob <- grid::segmentsGrob(
    x0 = rep(0, n_labels),
    y0 = labels,
    x1 = rep(0.7, n_labels),
    y1 = labels,
    default.units = "native",
    name = paste("eulerr", "error_plot", "key", "ticks", sep = "."),
    vp = grid::viewport(yscale = rng)
  )

  box_grob <- grid::rectGrob(
    vp = grid::viewport(yscale = rng),
    height = diff(rng),
    default.units = "native",
    name = paste("eulerr", "error_plot", "key", "box", sep = "."),
    gp = grid::gpar(fill = "transparent")
  )

  rect_grob <- grid::rectGrob(
    x = rep(0.5, length(y_rect)),
    y = y_rect,
    default.units = "native",
    height = step,
    vp = grid::viewport(yscale = rng),
    name = paste("eulerr", "error_plot", "key", "rect", sep = "."),
    gp = grid::gpar(fill = pal(n), col = "transparent")
  )

  frame <- grid::placeGrob(frame, rect_grob, row = 2, col = 1)
  frame <- grid::placeGrob(frame, box_grob, row = 2, col = 1)
  frame <- grid::placeGrob(frame, ticks_grob, row = 2, col = 2)
  frame <- grid::placeGrob(frame, labels_grob, row = 2, col = 3)

  graphics::plot(
    x,
    fills = pal(n)[ind],
    labels = TRUE,
    quantities = if (quantities) round(r, 3) else FALSE,
    legend = frame,
    ...
  )
}

#' Compose Euler Diagrams
#'
#' Arrange two `eulergram` objects side-by-side or stacked, building up
#' multi-panel layouts with operator syntax. Compositions can be nested
#' arbitrarily, e.g. `(p1 | p2) / p3`.
#'
#' @param e1,e2 `eulergram` objects, typically returned by [plot.euler()].
#'
#' @details
#' `|` arranges the two plots horizontally; `/` stacks them vertically.
#' The result is itself an `eulergram`, so further composition chains
#' naturally.
#'
#' The gap between adjacent plots is controlled by the
#' `composition$spacing` entry of [eulerr_options()], which must be a
#' [grid::unit()] and defaults to `grid::unit(1, "lines")`.
#'
#' Because composition is binary and recursive, panels at different
#' nesting levels are not size-aligned. In `(p1 | p2) / p3`, `p3` spans
#' the full bottom row while `p1` and `p2` split the top row equally.
#'
#' @return An `eulergram` containing the composed layout.
#'
#' @examples
#' p1 <- plot(euler(c(A = 1, B = 8, "A&B" = 1)))
#' p2 <- plot(euler(c(A = 1, C = 1, "A&C" = 1)))
#'
#' p1 | p2
#' p1 / p2
#'
#' p3 <- plot(euler(c(X = 3, Y = 2, "X&Y" = 1)))
#' (p1 | p2) / p3
#'
#' @name eulergram-compose
#' @seealso [plot.euler()], [eulerr_options()]
NULL

#' @rdname eulergram-compose
#' @export
"|.eulergram" <- function(e1, e2) {
  compose_eulergrams(e1, e2, horizontal = TRUE)
}

#' @rdname eulergram-compose
#' @export
"/.eulergram" <- function(e1, e2) {
  compose_eulergrams(e1, e2, horizontal = FALSE)
}

compose_eulergrams <- function(e1, e2, horizontal = TRUE) {
  if (!inherits(e1, "eulergram") || !inherits(e2, "eulergram")) {
    stop("Both operands must be `eulergram` objects.", call. = FALSE)
  }

  spacing <- .eulerr_env$options$composition$spacing
  if (is.null(spacing)) {
    spacing <- grid::unit(1, "lines")
  }
  if (!grid::is.unit(spacing)) {
    stop(
      "`eulerr_options()$composition$spacing` must be a `grid::unit()`.",
      call. = FALSE
    )
  }

  if (horizontal) {
    layout_vp <- grid::viewport(
      layout = grid::grid.layout(
        nrow = 1,
        ncol = 3,
        widths = grid::unit.c(
          grid::unit(1, "null"),
          spacing,
          grid::unit(1, "null")
        )
      ),
      name = "euler.composed.vp"
    )

    first <- grid::gTree(
      children = grid::gList(e1),
      vp = grid::viewport(layout.pos.col = 1),
      name = "euler.composed.left"
    )

    second <- grid::gTree(
      children = grid::gList(e2),
      vp = grid::viewport(layout.pos.col = 3),
      name = "euler.composed.right"
    )
  } else {
    layout_vp <- grid::viewport(
      layout = grid::grid.layout(
        nrow = 3,
        ncol = 1,
        heights = grid::unit.c(
          grid::unit(1, "null"),
          spacing,
          grid::unit(1, "null")
        )
      ),
      name = "euler.composed.vp"
    )

    first <- grid::gTree(
      children = grid::gList(e1),
      vp = grid::viewport(layout.pos.row = 1),
      name = "euler.composed.top"
    )

    second <- grid::gTree(
      children = grid::gList(e2),
      vp = grid::viewport(layout.pos.row = 3),
      name = "euler.composed.bottom"
    )
  }

  grid::gTree(
    children = grid::gList(first, second),
    vp = layout_vp,
    cl = "eulergram",
    name = "euler.composed"
  )
}

#' Get the bounding box for a row-wise collection of fitted shapes.
#'
#' Dispatches on the shape kind. Ellipse/circle use the rotated-ellipse
#' bounding-box formula; rectangle/square use width/height (or side)
#' directly since both are axis-aligned in eunoia.
#'
#' @param shapes the `$shapes` data frame of a fitted euler object (after
#'   dropping NA rows for empty sets), or — for legacy callers — the
#'   numeric `h` vector. When `h` is numeric the function falls back to
#'   the legacy ellipse signature for back-compat with external code.
#' @param k,a,b,phi legacy ellipse parameters; only used when `shapes` is
#'   numeric (the legacy signature).
#'
#' @return The bounding box as a list with `xlim` and `ylim`.
#' @keywords internal
get_bounding_box <- function(shapes, k = NULL, a = NULL, b = NULL, phi = NULL) {
  if (is.data.frame(shapes)) {
    return(shape_bounding_box(shapes))
  }
  # Legacy numeric signature: get_bounding_box(h, k, a, b, phi).
  h <- shapes
  if (is.null(b)) {
    b <- a
  }
  if (is.null(phi)) {
    phi <- 0
  }
  ellipse_bounding_box(h, k, a, b, phi)
}

#' Bounding box of a vector of rotated ellipses.
#' @keywords internal
ellipse_bounding_box <- function(h, k, a, b, phi) {
  xlim <- sqrt(a^2 * cos(phi)^2 + b^2 * sin(phi)^2)
  ylim <- sqrt(a^2 * sin(phi)^2 + b^2 * cos(phi)^2)

  list(
    xlim = range(xlim + h, -xlim + h),
    ylim = range(ylim + k, -ylim + k)
  )
}

#' Per-shape bounding box dispatch. Reads the `type` tag on `shapes` (rows
#' are assumed to share a tag since a diagram fixes one shape kind), then
#' picks the appropriate width/height calculation. Falls back to the
#' rotated-ellipse formula when the type is unknown so external callers
#' constructing ad-hoc `$shapes` frames still get a sensible box.
#' @keywords internal
shape_bounding_box <- function(shapes) {
  if (NROW(shapes) == 0L) {
    return(list(xlim = c(-1, 1), ylim = c(-1, 1)))
  }
  type <- shapes$type[1L]
  h <- shapes$h
  k <- shapes$k
  switch(
    type,
    rectangle = {
      half_w <- shapes$width / 2
      half_h <- shapes$height / 2
      list(
        xlim = range(c(h - half_w, h + half_w)),
        ylim = range(c(k - half_h, k + half_h))
      )
    },
    square = {
      half <- shapes$side / 2
      list(
        xlim = range(c(h - half, h + half)),
        ylim = range(c(k - half, k + half))
      )
    },
    ellipse_bounding_box(h, k, shapes$a, shapes$b, shapes$phi)
  )
}

test_that("bounding box computations work as expected", {
  f <- eulerr:::get_bounding_box(0, 0, 1, 1, 0)
  expect_equivalent(unlist(f), c(-1, 1, -1, 1))
  expect_equal(
    eulerr:::get_bounding_box(0, 1, 1),
    eulerr:::get_bounding_box(0, 1, 1, 1, pi)
  )

  expect_equal(
    eulerr:::get_bounding_box(0, 1, 1, 2, pi),
    eulerr:::get_bounding_box(0, 1, 1, 2, 2 * pi)
  )

  expect_equivalent(unlist(eulerr:::get_bounding_box(0, 0, 0)), rep(0, 4))

  expect_equal(
    eulerr:::get_bounding_box(0, 0, 1),
    eulerr:::get_bounding_box(c(0, 0), c(0, 0), c(1, 0.5))
  )
})

test_that("polygon_clip_rust delegates to eunoia for basic ops", {
  square_a <- list(x = c(0, 2, 2, 0), y = c(0, 0, 2, 2))
  square_b <- list(x = c(1, 3, 3, 1), y = c(1, 1, 3, 3))

  inter <- eulerr:::polygon_clip_rust(
    subject_x = square_a$x,
    subject_y = square_a$y,
    subject_id_lengths = length(square_a$x),
    clip_x = square_b$x,
    clip_y = square_b$y,
    op = "intersection"
  )
  expect_equal(length(inter$id_lengths), 1L)
  expect_equal(sum(inter$id_lengths), length(inter$x))

  empty <- eulerr:::polygon_clip_rust(
    subject_x = double(0),
    subject_y = double(0),
    subject_id_lengths = integer(0),
    clip_x = square_b$x,
    clip_y = square_b$y,
    op = "intersection"
  )
  expect_equal(length(empty$id_lengths), 0L)
})

#' Tally set relationships
#'
#' @param sets a data.frame with set relationships and weights
#' @param weights a numeric vector
#'
#' @return Calls [euler()] after the set relationships have been coerced to a
#'   named numeric vector.
#' @keywords internal
tally_combinations <- function(sets, weights) {
  if (!is.matrix(sets)) {
    sets <- as.matrix(sets)
  }
  setnames <- colnames(sets)

  if (NROW(sets) == 0L || NCOL(sets) == 0L) {
    return(numeric(0))
  }

  m <- sets != 0
  patterns <- apply(m, 1L, function(row) paste(setnames[row], collapse = "&"))
  keep <- nzchar(patterns)

  if (any(keep)) {
    tap <- tapply(weights[keep], patterns[keep], sum)
    counts <- as.numeric(tap)
    names(counts) <- names(tap)
  } else {
    counts <- numeric(0)
  }

  singletons <- stats::setNames(numeric(length(setnames)), setnames)
  shared <- intersect(setnames, names(counts))
  if (length(shared) > 0L) {
    singletons[shared] <- counts[shared]
  }
  multi <- counts[setdiff(names(counts), setnames)]

  c(singletons, multi)
}

#' Rescale values to new range
#'
#' @param x numeric vector
#' @param new_min new min
#' @param new_max new max
#'
#' @return Rescaled vector
#' @keywords internal
rescale <- function(x, new_min, new_max) {
  (new_max - new_min) / (max(x) - min(x)) * (x - max(x)) + new_max
}

#' Update list with input
#'
#' A wrapper for [utils::modifyList()] that attempts to coerce non-lists to
#' lists before updating.
#'
#' @param old a list to be updated
#' @param new stuff to update `x` with
#'
#' @seealso [utils::modifyList()]
#' @return Returns an updated list.
#' @keywords internal
update_list <- function(old, new) {
  if (is.null(old)) {
    old <- list()
  }
  if (!is.list(new)) {
    tryCatch(new <- as.list(new))
  }
  if (!is.list(old)) {
    tryCatch(old <- as.list(old))
  }
  utils::modifyList(old, new)
}

#' Replace (refresh) a list
#'
#' Unlike `update_list`, this function only modifies, and does not add,
#' items in the list.
#'
#' @param old the original list
#' @param new the things to update `old` with
#'
#' @return A refreshed list.
#' @keywords internal
replace_list <- function(old, new) {
  update_list(old, new[names(new) %in% names(old)])
}

#' Check if object is strictly FALSE
#'
#' @param x object to check
#'
#' @return A logical.
#' @keywords internal
is_false <- function(x) {
  identical(x, FALSE)
}

#' Null-coalesce: returns `y` when `x` is `NULL`, else `x`. Local
#' polyfill so we keep the R >= 4.2 floor (base `%||%` is 4.4+).
#' @keywords internal
#' @noRd
`%||%` <- function(x, y) {
  if (is.null(x)) y else x
}

#' Check if a vector is an integer
#'
#' @param x a vector
#'
#' @return TRUE of FALSE.
#' @keywords internal
is_integer <- function(x, tol = .Machine$double.eps^0.5) {
  all(abs(x - round(x)) < tol)
}

#' Check if vector is a real (numeric non-integer)
#'
#' @param x
#'
#' @return A logical.
#' @keywords internal
is_real <- function(x, tol = .Machine$double.eps^0.5) {
  is.numeric(x) && !is_integer(x, tol = )
}

#' Enumerate all 2^n - 1 combination labels for a set of names
#'
#' Generates labels in cardinality-first order: singletons first, then pairs,
#' then triples, etc. Within each cardinality, set names appear in their
#' original order. Used only by the Venn path (bounded at n = 5).
#'
#' @param setnames a character vector of set names
#'
#' @return A character vector of length 2^n - 1.
#' @keywords internal
all_set_combinations <- function(setnames) {
  n <- length(setnames)
  if (n == 0L) {
    return(character(0))
  }
  out <- character(0)
  for (k in seq_len(n)) {
    cmb <- utils::combn(setnames, k)
    labs <- if (k == 1L) {
      as.vector(cmb)
    } else {
      apply(cmb, 2L, paste, collapse = "&")
    }
    out <- c(out, labs)
  }
  out
}

#' Get the number of sets in he input
#'
#' @param combinations a vector of combinations (see [euler()])
#'
#' @return The number of sets in the input
#' @keywords internal
n_sets <- function(combinations) {
  combo_names <- strsplit(names(combinations), split = "&", fixed = TRUE)
  length(unique(unlist(combo_names, use.names = FALSE)))
}

#' Blend (average) colors
#'
#' @param rcol_in a vector of R colors
#'
#' @return A single R color
#' @keywords internal
mix_colors <- function(rcol_in) {
  rgb_in <- t(grDevices::col2rgb(rcol_in))
  lab_in <- grDevices::convertColor(rgb_in, "sRGB", "Lab", scale.in = 255)
  mean_col <- colMeans(lab_in)
  rgb_out <- grDevices::convertColor(mean_col, "Lab", "sRGB", scale.out = 1)
  grDevices::rgb(rgb_out)
}

#' Setup gpars
#'
#' @param default default values
#' @param user user-inputted values
#' @param n required number of items
#'
#' @return a `gpar` object
#' @keywords internal
setup_gpar <- function(default = list(), user = list(), n) {
  # set up gpars
  if (is.list(user)) {
    gp <- replace_list(default, user)
  } else {
    gp <- default
  }
  gp <- lapply(gp, function(x) if (is.function(x)) x(n) else x)

  do.call(grid::gpar, lapply(gp, rep_len, n))
}

#' Overlay per-panel overrides onto a styling parameter
#'
#' Used to apply `by_group` overrides to `fills`, `patterns`, `edges`, `labels`,
#' and `quantities` for a single panel produced by `euler(..., by = ...)`. The
#' override list contains gpar-level fields (and optionally `rot`), which are
#' overlaid onto `param$gp` (and `param$set_gp` for shape-mode patterns).
#' Structural fields are not overlaid here — they must be filtered upstream.
#'
#' @param param a list with `$gp` (a `grid::gpar`) and optionally `$rot` and/or
#'   `$set_gp`.
#' @param override a flat named list of fields to overlay.
#'
#' @return The param list with overlaid fields.
#' @keywords internal
apply_panel_overrides <- function(param, override) {
  if (is.null(param) || is.null(override) || length(override) == 0L) {
    return(param)
  }
  overlay_gp <- function(gp) {
    if (is.null(gp)) {
      return(gp)
    }
    n <- if (length(gp) > 0L) max(lengths(gp), 1L) else 1L
    for (nm in names(override)) {
      if (nm == "rot") {
        next
      }
      gp[[nm]] <- rep_len(override[[nm]], n)
    }
    gp
  }
  if (!is.null(param$gp)) {
    param$gp <- overlay_gp(param$gp)
  }
  if (!is.null(param$set_gp)) {
    param$set_gp <- overlay_gp(param$set_gp)
  }
  if (!is.null(override$rot) && !is.null(param$rot)) {
    param$rot <- rep_len(override$rot, length(param$rot))
  }
  param
}

#' Dummy code a data.frame
#'
#' @param x a data.frame
#' @param sep character for separating dummy code factors and their levels
#'   when constructing names
#' @param factor_names whether to include factor names when creating new
#'   names for dummy codes
#'
#' @return A dummy-coded version of x.
#'
#' @keywords internal
dummy_code <- function(x, sep = "_", factor_names = TRUE) {
  fac_chr <- vapply(x, function(x) is.character(x) || is.factor(x), logical(1))

  tmp <- x[, fac_chr, drop = FALSE]

  # convert characters into factors
  for (i in seq_len(ncol(tmp))) {
    tmp[, i] <- as.factor(tmp[, i])
  }

  lvls <- lapply(tmp, function(x) levels(x))

  n_levels <- vapply(lvls, function(x) length(x) - 1, double(1))
  dummy_levels <- lapply(lvls, function(x) x[-length(x)])
  n_levels_tot <- sum(n_levels)

  dummy_names <- dummy_levels

  if (isTRUE(factor_names)) {
    for (i in seq_along(dummy_names)) {
      dummy_names[[i]] <- paste(
        names(dummy_levels)[i],
        dummy_levels[[i]],
        sep = sep
      )
    }
  }

  dummy_names <- unlist(dummy_names)

  if (any(duplicated(dummy_names))) {
    stop(paste(
      "duplicated names for dummy coded factors were generated;",
      "please consider specifying 'factor_names = TRUE'."
    ))
  }

  out <- matrix(
    FALSE,
    nrow(x),
    n_levels_tot,
    dimnames = list(NULL, dummy_names)
  )

  k <- 1

  for (i in seq_along(n_levels)) {
    kk <- as.numeric(tmp[, i])
    for (j in seq_len(n_levels[i])) {
      out[, k] <- kk == j
      k <- 1 + k
    }
  }

  cbind(x[, !fac_chr, drop = FALSE], out)
}

nonzero_fit <- function(x) {
  abs(x) / sum(abs(x) + .Machine$double.eps) > 1e-4
}

#' Build the leader / label / quantity / annotation gList for one tag.
#'
#' Shared between [setup_tag()] (initial construction in [setup_grobs()])
#' and [makeContent.EulerTags()] (draw-time re-placement on resize).
#' Pure factory — no measurement; takes anchor + tether already in
#' native units, plus the stashed text / gpar bundle. The label /
#' quantity / annotation stack is centered vertically on `(ax, ay)`
#' so the bbox center matches the anchor eunoia placed and leader
#' endpoints land on the actual bbox edge.
#'
#' @keywords internal
build_tag_grobs <- function(
  ax,
  ay,
  kind,
  tx,
  ty,
  lend_x,
  lend_y,
  label_text,
  quantity_text,
  annotation_text,
  has_label,
  has_quantity,
  has_annotation,
  label_gp,
  quantity_gp,
  annotation_gp,
  label_rot,
  quantity_rot,
  annotation_rot,
  number,
  leader_gp_list,
  padding,
  waypoints_x = numeric(0),
  waypoints_y = numeric(0),
  name_prefix = "tag"
) {
  qname <- if (identical(name_prefix, "complement")) {
    "complement.quantity.grob"
  } else {
    paste0(name_prefix, ".quantity.", number)
  }
  lname <- if (identical(name_prefix, "complement")) {
    "complement.label.grob"
  } else {
    paste0(name_prefix, ".label.", number)
  }
  aname <- if (identical(name_prefix, "complement")) {
    "complement.annotation.grob"
  } else {
    paste0(name_prefix, ".annotation.", number)
  }
  leader_name <- if (identical(name_prefix, "complement")) {
    "complement.leader.grob"
  } else {
    paste0(name_prefix, ".leader.", number)
  }

  ax_unit <- grid::unit(ax, "native")
  ay_unit <- grid::unit(ay, "native")

  if (has_quantity) {
    quantity_grob <- grid::textGrob(
      quantity_text,
      x = ax_unit,
      y = ay_unit,
      rot = quantity_rot,
      gp = quantity_gp,
      name = qname
    )
  } else {
    quantity_grob <- grid::nullGrob(name = paste0(qname, ".null"))
  }

  if (has_label) {
    label_grob <- grid::textGrob(
      label_text,
      x = ax_unit,
      y = ay_unit,
      rot = label_rot,
      gp = label_gp,
      name = lname
    )
  } else {
    label_grob <- grid::nullGrob(name = paste0(lname, ".null"))
  }

  if (has_annotation) {
    annotation_grob <- grid::textGrob(
      annotation_text,
      x = ax_unit,
      y = ay_unit,
      rot = annotation_rot,
      gp = annotation_gp,
      name = aname
    )
  } else {
    annotation_grob <- grid::nullGrob(name = paste0(aname, ".null"))
  }

  # Center the label/quantity/annotation stack vertically on (ax, ay).
  # The bbox center then matches the anchor eunoia placed, so leader
  # endpoints (computed against an anchor-centered AABB) land on the
  # actual bbox edge instead of inside the stack.
  #
  # Each element's offset from `ay` is `0.5 * (below - above)`. We
  # only emit the terms that are actually present — adding a
  # `unit(0, "native")` to a y-position sum makes grid treat it as
  # an absolute position offset (not a zero height), which yanks the
  # resolution way off, so the zero-height branch must skip the term
  # entirely.
  build_above <- function(parts) {
    if (length(parts) == 0L) NULL else Reduce(`+`, parts)
  }
  if (has_label) {
    below_parts <- list()
    if (has_quantity) {
      below_parts <- c(below_parts, list(padding, grid::grobHeight(quantity_grob)))
    }
    if (has_annotation) {
      below_parts <- c(
        below_parts,
        list(padding, grid::grobHeight(annotation_grob))
      )
    }
    below <- build_above(below_parts)
    if (!is.null(below)) {
      label_grob$y <- ay_unit + 0.5 * below
    }
  }
  if (has_quantity) {
    above_parts <- list()
    below_parts <- list()
    if (has_label) {
      above_parts <- c(above_parts, list(grid::grobHeight(label_grob), padding))
    }
    if (has_annotation) {
      below_parts <- c(below_parts, list(padding, grid::grobHeight(annotation_grob)))
    }
    above <- build_above(above_parts)
    below <- build_above(below_parts)
    new_y <- ay_unit
    if (!is.null(below)) {
      new_y <- new_y + 0.5 * below
    }
    if (!is.null(above)) {
      new_y <- new_y - 0.5 * above
    }
    if (!is.null(above) || !is.null(below)) {
      quantity_grob$y <- new_y
    }
  }
  if (has_annotation) {
    above_parts <- list()
    if (has_quantity) {
      above_parts <- c(above_parts, list(grid::grobHeight(quantity_grob), padding))
    }
    if (has_label) {
      if (!has_quantity) {
        above_parts <- c(above_parts, list(padding))
      }
      above_parts <- c(above_parts, list(grid::grobHeight(label_grob)))
    }
    above <- build_above(above_parts)
    if (!is.null(above)) {
      annotation_grob$y <- ay_unit - 0.5 * above
    }
  }

  fallback_gp <- if (has_label) {
    label_gp
  } else if (has_quantity) {
    quantity_gp
  } else {
    annotation_gp
  }
  leader_grob <- build_leader_grob(
    ax = ax,
    ay = ay,
    kind = kind,
    tx = tx,
    ty = ty,
    lend_x = lend_x,
    lend_y = lend_y,
    waypoints_x = waypoints_x,
    waypoints_y = waypoints_y,
    leader_gp_list = leader_gp_list,
    fallback_gp = fallback_gp,
    name = leader_name
  )

  grid::gList(
    leader = leader_grob,
    label = label_grob,
    quantity = quantity_grob,
    annotation = annotation_grob
  )
}

#' Build the polyline leader for an exterior tag, or [grid::nullGrob()]
#' for interior / missing-tether placements.
#'
#' Draws the polyline `tether → waypoints[1..] → leader_end`. For
#' straight leaders (raycast / force-directed) `waypoints_*` are empty,
#' so the polyline collapses to the single `tether → leader_end`
#' segment. For elbow leaders eunoia emits one knee waypoint, producing
#' the orthogonal `tether → knee → leader_end` bend.
#'
#' Terminates at `(lend_x, lend_y)` — the point on the label box AABB
#' edge supplied by eunoia (`LabelPlacement::leader_end`). Falls back
#' to the anchor when the leader endpoint isn't finite so older /
#' partial placement results still draw something sensible.
#' @keywords internal
build_leader_grob <- function(
  ax,
  ay,
  kind,
  tx,
  ty,
  lend_x,
  lend_y,
  waypoints_x = numeric(0),
  waypoints_y = numeric(0),
  leader_gp_list,
  fallback_gp,
  name
) {
  is_exterior <- !is.null(kind) &&
    !is.na(kind) &&
    nzchar(kind) &&
    !identical(kind, "interior")
  if (!is_exterior) {
    return(grid::nullGrob(name = name))
  }
  if (
    is.null(tx) ||
      is.null(ty) ||
      !is.finite(tx) ||
      !is.finite(ty)
  ) {
    return(grid::nullGrob(name = name))
  }

  end_x <- if (!is.null(lend_x) && is.finite(lend_x)) lend_x else ax
  end_y <- if (!is.null(lend_y) && is.finite(lend_y)) lend_y else ay

  if (is.null(waypoints_x) || is.null(waypoints_y)) {
    waypoints_x <- numeric(0)
    waypoints_y <- numeric(0)
  }
  wp_ok <- is.finite(waypoints_x) & is.finite(waypoints_y)
  waypoints_x <- waypoints_x[wp_ok]
  waypoints_y <- waypoints_y[wp_ok]

  fallback_col <- if (
    !is.null(fallback_gp) &&
      !is.null(fallback_gp$col) &&
      length(fallback_gp$col) >= 1L
  ) {
    fallback_gp$col[1L]
  } else {
    "black"
  }
  pick <- function(field, default) {
    if (!is.null(leader_gp_list) && !is.null(leader_gp_list[[field]])) {
      leader_gp_list[[field]][1L]
    } else {
      default
    }
  }
  gp <- grid::gpar(
    col = pick("col", fallback_col),
    alpha = pick("alpha", 0.6),
    lwd = pick("lwd", 1),
    lty = pick("lty", 2),
    lex = pick("lex", 1)
  )

  grid::polylineGrob(
    x = c(tx, waypoints_x, end_x),
    y = c(ty, waypoints_y, end_y),
    default.units = "native",
    gp = gp,
    name = name
  )
}

#' Setup grobs for one tag (label + quantity + annotation + leader).
#'
#' Builds the gList via [build_tag_grobs()] and stashes the text / gpar
#' bundle on the resulting `EulerTag` gTree so [makeContent.EulerTags()]
#' can rebuild it at draw time with fresh measurements.
#'
#' @keywords internal
setup_tag <- function(data, labels, quantities, annotations, number) {
  has_label <- !is.null(labels) && !is.na(data$labels_par_id)
  has_quantity <- !is.null(quantities) && !is.na(data$quantities_par_id)
  has_annotation <- !is.null(annotations) &&
    !is.null(data$annotations_par_id) &&
    !is.na(data$annotations_par_id)

  label_text <- if (has_label) data$labels else NA
  quantity_text <- if (has_quantity) data$quantities else NA
  annotation_text <- if (has_annotation) data$annotations else NA
  label_gp <- if (has_label) labels$gp[data$labels_par_id] else NULL
  quantity_gp <- if (has_quantity) quantities$gp[data$quantities_par_id] else NULL
  annotation_gp <- if (has_annotation) {
    annotations$gp[data$annotations_par_id]
  } else {
    NULL
  }
  label_rot <- if (has_label) labels$rot[data$labels_par_id] else 0
  quantity_rot <- if (has_quantity) {
    quantities$rot[data$quantities_par_id]
  } else {
    0
  }
  annotation_rot <- if (has_annotation) {
    annotations$rot[data$annotations_par_id]
  } else {
    0
  }

  padding <- eulerr_options()$padding
  leader_gp_list <- if (!is.null(labels)) labels$leader else NULL

  wp <- data$leader_waypoints
  if (is.list(wp) && length(wp) >= 1L) {
    wp <- wp[[1L]]
  }
  wp_x <- if (!is.null(wp) && !is.null(wp$x)) wp$x else numeric(0)
  wp_y <- if (!is.null(wp) && !is.null(wp$y)) wp$y else numeric(0)

  grobs <- build_tag_grobs(
    ax = data$x,
    ay = data$y,
    kind = data$kind,
    tx = data$tether_x,
    ty = data$tether_y,
    lend_x = data$leader_end_x,
    lend_y = data$leader_end_y,
    label_text = label_text,
    quantity_text = quantity_text,
    annotation_text = annotation_text,
    has_label = has_label,
    has_quantity = has_quantity,
    has_annotation = has_annotation,
    label_gp = label_gp,
    quantity_gp = quantity_gp,
    annotation_gp = annotation_gp,
    label_rot = label_rot,
    quantity_rot = quantity_rot,
    annotation_rot = annotation_rot,
    number = number,
    leader_gp_list = leader_gp_list,
    padding = padding,
    waypoints_x = wp_x,
    waypoints_y = wp_y
  )

  grid::gTree(
    children = grobs,
    combo_key = rownames(data),
    label_text = label_text,
    quantity_text = quantity_text,
    annotation_text = annotation_text,
    has_label = has_label,
    has_quantity = has_quantity,
    has_annotation = has_annotation,
    label_gp = label_gp,
    quantity_gp = quantity_gp,
    annotation_gp = annotation_gp,
    label_rot = label_rot,
    quantity_rot = quantity_rot,
    annotation_rot = annotation_rot,
    leader_gp_list = leader_gp_list,
    padding = padding,
    number = number,
    name = paste("tag", "number", number, sep = "."),
    cl = "EulerTag"
  )
}

#' Setup the complement-count tag, in the same shape as a region tag so
#' it shares one [makeContent.EulerTags()] pass.
#'
#' Acts as a quantity-only tag with `combo_key = ""`. The complement
#' label text comes either from `complement$label` (user override) or
#' from `container_data$quantity_label` (the fitted complement count).
#'
#' @keywords internal
setup_complement_tag <- function(container_data, complement, number) {
  if (is.null(container_data)) {
    return(NULL)
  }
  label_text <- complement$label
  if (is.null(label_text)) {
    label_text <- container_data$quantity_label
  }
  if (
    is.null(label_text) ||
      is.na(label_text) ||
      !is.finite(container_data$label_x) ||
      !is.finite(container_data$label_y)
  ) {
    return(NULL)
  }
  cgp <- complement$gp
  quantity_gp <- grid::gpar(
    col = cgp$col[1L],
    alpha = cgp$alpha[1L],
    fontsize = cgp$fontsize[1L],
    cex = cgp$cex[1L],
    fontfamily = cgp$fontfamily[1L],
    lineheight = cgp$lineheight[1L],
    font = cgp$font[1L]
  )
  leader_gp_list <- complement$leader
  padding <- eulerr_options()$padding

  wp <- container_data$leader_waypoints
  wp_x <- if (!is.null(wp) && !is.null(wp$x)) wp$x else numeric(0)
  wp_y <- if (!is.null(wp) && !is.null(wp$y)) wp$y else numeric(0)

  grobs <- build_tag_grobs(
    ax = container_data$label_x,
    ay = container_data$label_y,
    kind = container_data$kind,
    tx = container_data$tether_x,
    ty = container_data$tether_y,
    lend_x = container_data$leader_end_x,
    lend_y = container_data$leader_end_y,
    label_text = NA,
    quantity_text = label_text,
    annotation_text = NA,
    has_label = FALSE,
    has_quantity = TRUE,
    has_annotation = FALSE,
    label_gp = NULL,
    quantity_gp = quantity_gp,
    annotation_gp = NULL,
    label_rot = 0,
    quantity_rot = 0,
    annotation_rot = 0,
    number = number,
    leader_gp_list = leader_gp_list,
    padding = padding,
    waypoints_x = wp_x,
    waypoints_y = wp_y,
    name_prefix = "complement"
  )

  grid::gTree(
    children = grobs,
    combo_key = "",
    label_text = NA,
    quantity_text = label_text,
    annotation_text = NA,
    has_label = FALSE,
    has_quantity = TRUE,
    has_annotation = FALSE,
    label_gp = NULL,
    quantity_gp = quantity_gp,
    annotation_gp = NULL,
    label_rot = 0,
    quantity_rot = 0,
    annotation_rot = 0,
    leader_gp_list = leader_gp_list,
    padding = padding,
    number = number,
    name_prefix = "complement",
    name = paste("tag", "number", number, "complement", sep = "."),
    cl = "EulerTag"
  )
}

#' Measure one tag's combined AABB in the current viewport's native units.
#' @keywords internal
measure_tag_native <- function(tag, padding_native) {
  label_w <- 0
  label_h <- 0
  if (isTRUE(tag$has_label)) {
    g <- grid::textGrob(tag$label_text, gp = tag$label_gp)
    label_w <- grid::convertWidth(
      grid::grobWidth(g),
      "native",
      valueOnly = TRUE
    )
    label_h <- grid::convertHeight(
      grid::grobHeight(g),
      "native",
      valueOnly = TRUE
    )
  }
  quant_w <- 0
  quant_h <- 0
  if (isTRUE(tag$has_quantity)) {
    g <- grid::textGrob(tag$quantity_text, gp = tag$quantity_gp)
    quant_w <- grid::convertWidth(
      grid::grobWidth(g),
      "native",
      valueOnly = TRUE
    )
    quant_h <- grid::convertHeight(
      grid::grobHeight(g),
      "native",
      valueOnly = TRUE
    )
  }
  annot_w <- 0
  annot_h <- 0
  if (isTRUE(tag$has_annotation)) {
    g <- grid::textGrob(tag$annotation_text, gp = tag$annotation_gp)
    annot_w <- grid::convertWidth(
      grid::grobWidth(g),
      "native",
      valueOnly = TRUE
    )
    annot_h <- grid::convertHeight(
      grid::grobHeight(g),
      "native",
      valueOnly = TRUE
    )
  }
  pad_lq <- if (label_h > 0 && quant_h > 0) padding_native else 0
  pad_qa <- if (quant_h > 0 && annot_h > 0) padding_native else 0
  pad_la <- if (quant_h == 0 && label_h > 0 && annot_h > 0) padding_native else 0
  list(
    w = max(label_w, quant_w, annot_w),
    h = label_h + quant_h + annot_h + pad_lq + pad_qa + pad_la
  )
}

#' Find the `EulerTags` child of an `EulerPanel`, if any.
#' @keywords internal
find_eulertags <- function(panel) {
  for (child in panel$children) {
    if (inherits(child, "EulerTags")) {
      return(child)
    }
  }
  NULL
}

#' Measure every drawable tag inside `tags_grob` against the current
#' viewport. Returns parallel vectors of combo / width / height suitable
#' for handing to [place_euler_labels()], plus the resolved leader gap
#' in native units (so the FFI sees one number per draw pass).
#' @keywords internal
measure_all_tags <- function(tags_grob, padding, gap = NULL) {
  if (is.null(tags_grob) || length(tags_grob$children) == 0L) {
    return(list(
      combos = character(),
      widths = numeric(),
      heights = numeric(),
      gap_native = 0
    ))
  }
  padding_native <- grid::convertHeight(
    padding,
    "native",
    valueOnly = TRUE
  )
  gap_native <- resolve_gap_native(gap, padding_native)
  combos <- character()
  widths <- numeric()
  heights <- numeric()
  for (child in tags_grob$children) {
    sz <- measure_tag_native(child, padding_native)
    if (sz$w > 0 && sz$h > 0 && is.finite(sz$w) && is.finite(sz$h)) {
      combos <- c(combos, if (is.null(child$combo_key)) "" else child$combo_key)
      widths <- c(widths, sz$w)
      heights <- c(heights, sz$h)
    }
  }
  list(
    combos = combos,
    widths = widths,
    heights = heights,
    gap_native = gap_native
  )
}

#' Fixed padding in pt for the panel viewport scale.
#'
#' Geometry / labels that land flush with the bbox would otherwise be
#' clipped at the device edge by stroke width and anti-aliasing — both
#' of which are in device units, not native units, so the padding is in
#' pt rather than a fraction of the coordinate range.
#' @keywords internal
EULER_PANEL_PAD_PT <- 2

#' Pad an axis range by `pt_pad` points, converted to native units
#' against a measurement viewport with the supplied scale. Returns the
#' original range unchanged if the conversion isn't finite (e.g. zero
#' range, no device).
#' @keywords internal
pad_axis_native <- function(
  lim,
  pt_pad,
  axis = c("x", "y"),
  layout_pos_row = NULL,
  layout_pos_col = NULL
) {
  axis <- match.arg(axis)
  if (!all(is.finite(lim)) || diff(range(lim)) <= 0) {
    return(lim)
  }
  meas_vp <- grid::viewport(
    layout.pos.row = layout_pos_row,
    layout.pos.col = layout_pos_col,
    xscale = lim,
    yscale = lim
  )
  grid::pushViewport(meas_vp)
  pad_native <- tryCatch(
    if (axis == "x") {
      grid::convertWidth(grid::unit(pt_pad, "pt"), "native", valueOnly = TRUE)
    } else {
      grid::convertHeight(grid::unit(pt_pad, "pt"), "native", valueOnly = TRUE)
    },
    error = function(e) NA_real_
  )
  grid::popViewport()
  if (!is.finite(pad_native) || pad_native <= 0) {
    return(lim)
  }
  c(lim[1] - pad_native, lim[2] + pad_native)
}

#' Set the panel viewport's `xscale`/`yscale` at draw time.
#'
#' Fires before grid pushes the panel viewport. We can therefore
#' measure labels against the live cell, run eunoia's placement, and
#' compute a viewport bbox that fits both the diagram and the labels.
#' On window resize grid invalidates the gTree and `makeContext`
#' re-runs, so the panel grows or shrinks to track the current device
#' and exterior labels never extend past the viewport edge.
#'
#' Aspect preservation: the new bbox keeps `xrng / yrng` equal to the
#' geometry's natural aspect (set by [setup_geometry()]) so that
#' circles render as circles. The smaller dimension is padded if the
#' label-driven canvas bbox is asymmetric.
#'
#' @export
#' @keywords internal
makeContext.EulerPanel <- function(x) {
  shapes <- x$shapes
  geom_xlim <- x$geom_xlim
  geom_ylim <- x$geom_ylim
  if (
    is.null(geom_xlim) ||
      is.null(geom_ylim) ||
      !all(is.finite(c(geom_xlim, geom_ylim)))
  ) {
    return(x)
  }

  layout_row <- if (is.null(x$vp)) NULL else x$vp$layout.pos.row
  layout_col <- if (is.null(x$vp)) NULL else x$vp$layout.pos.col

  # Always emit a vp so grid has something to push. We update its
  # xscale/yscale below; the layout.pos fields come from
  # `plot.euler.R`. The xscale/yscale gets a small pad in pt on each
  # side so geometry that lands flush with the bbox isn't clipped by
  # stroke width / anti-aliasing at the device edge.
  padded_geom_xlim <- pad_axis_native(
    geom_xlim,
    EULER_PANEL_PAD_PT,
    "x",
    layout_row,
    layout_col
  )
  padded_geom_ylim <- pad_axis_native(
    geom_ylim,
    EULER_PANEL_PAD_PT,
    "y",
    layout_row,
    layout_col
  )
  if (is.null(x$vp)) {
    x$vp <- grid::viewport(
      xscale = padded_geom_xlim,
      yscale = padded_geom_ylim,
      name = x$name %||% "panel.vp"
    )
  } else {
    x$vp$xscale <- padded_geom_xlim
    x$vp$yscale <- padded_geom_ylim
  }

  tags_grob <- find_eulertags(x)
  if (
    is.null(tags_grob) ||
      length(tags_grob$children) == 0L ||
      is.null(shapes) ||
      NROW(shapes) == 0L
  ) {
    return(x)
  }
  shape_type <- shapes$type[1L]

  geom_xrng <- diff(geom_xlim)
  geom_yrng <- diff(geom_ylim)
  if (geom_xrng <= 0 || geom_yrng <= 0) {
    return(x)
  }
  geom_ar <- geom_xrng / geom_yrng

  # The cell's pt aspect is set by `canvas_vp`'s `grid.layout`. We must
  # use that aspect for the new xlim/ylim so circles render as circles
  # AND the panel viewport's pt extent matches what the cell can give
  # us — otherwise exterior label anchors at the edge of `xscale`/
  # `yscale` may map past the cell and clip at the device edge.
  cell_vp <- grid::viewport(
    layout.pos.row = x$vp$layout.pos.row,
    layout.pos.col = x$vp$layout.pos.col
  )
  grid::pushViewport(cell_vp)
  cell_w_pt <- grid::convertWidth(grid::unit(1, "npc"), "pt", valueOnly = TRUE)
  cell_h_pt <- grid::convertHeight(grid::unit(1, "npc"), "pt", valueOnly = TRUE)
  grid::popViewport()
  if (
    !is.finite(cell_w_pt) ||
      !is.finite(cell_h_pt) ||
      cell_w_pt <= 0 ||
      cell_h_pt <= 0
  ) {
    cell_ar <- geom_ar
  } else {
    cell_ar <- cell_w_pt / cell_h_pt
  }

  container <- x$container
  has_container <- !is.null(container)
  placement_opts <- resolve_placement_opts(x$placement_opts)
  precision <- if (
    is.null(x$label_precision) || !is.finite(x$label_precision)
  ) {
    max(geom_xrng, geom_yrng) / 100
  } else {
    x$label_precision
  }

  # Fixed-point loop. Each iteration measures labels at the current
  # candidate xlim/ylim, runs placement, and grows the viewport to
  # include `canvas_bbox`. Label native size scales with `xrng/yrng`
  # (font is in pt; bigger native extent => bigger native label), so
  # growing the viewport between iterations grows the next iteration's
  # labels. The loop converges as long as the labels fit inside the
  # cell at all; we cap at a small iteration count to keep extreme
  # corner cases from spinning. Whether converged or not, the LAST
  # iteration's xlim/ylim is what we pin on the viewport — that's also
  # what `makeContent.EulerTags` will re-measure against, so panel
  # bbox and label positions stay consistent.
  # Cap each axis at 10x the geometry extent. When the label is wider
  # than the cell (label_pt > cell_pt), the fixed-point iteration has
  # no solution and the bbox would otherwise diverge. Clamping at a
  # finite multiple gives a coherent (if zoomed-out) frame — the
  # diagram still draws and the user can see what's happening rather
  # than getting a runaway bbox.
  max_iters <- 30L
  cap_factor <- 20
  cap_xrng <- geom_xrng * cap_factor
  cap_yrng <- geom_yrng * cap_factor
  # Seed the iteration with the same pt-based pad we'd otherwise apply
  # post-loop, so each measurement viewport matches what
  # `makeContent.EulerTags` will measure against at draw time. Without
  # this, post-loop padding inflates the native viewport by ~1-2% and
  # `makeContent` can re-measure labels just large enough that some flip
  # from interior to exterior — but the viewport is already pinned, so
  # the exterior labels overflow the panel and get clipped.
  current_xlim <- pad_axis_native(
    geom_xlim,
    EULER_PANEL_PAD_PT,
    "x",
    layout_row,
    layout_col
  )
  current_ylim <- pad_axis_native(
    geom_ylim,
    EULER_PANEL_PAD_PT,
    "y",
    layout_row,
    layout_col
  )
  for (iter in seq_len(max_iters)) {
    meas_vp <- grid::viewport(
      layout.pos.row = x$vp$layout.pos.row,
      layout.pos.col = x$vp$layout.pos.col,
      xscale = current_xlim,
      yscale = current_ylim
    )
    grid::pushViewport(meas_vp)
    measurements <- tryCatch(
      measure_all_tags(tags_grob, x$padding, placement_opts$gap),
      error = function(e) NULL
    )
    grid::popViewport()
    if (is.null(measurements) || length(measurements$combos) == 0L) {
      break
    }

    placements <- tryCatch(
      place_euler_labels(
        set_names = rownames(shapes),
        shape = shape_type,
        h = shapes$h,
        k = shapes$k,
        a = shapes$a,
        b = shapes$b,
        phi = shapes$phi,
        width = shapes$width,
        height = shapes$height,
        side = shapes$side,
        container_h = if (has_container) container$h else NULL,
        container_k = if (has_container) container$k else NULL,
        container_width = if (has_container) container$width else NULL,
        container_height = if (has_container) container$height else NULL,
        n_vertices = as.integer(x$n_vertices),
        label_combos = measurements$combos,
        label_widths = measurements$widths,
        label_heights = measurements$heights,
        placement = placement_opts$placement,
        placement_margin = placement_opts$margin,
        placement_iterations = placement_opts$iterations,
        placement_min_gap = placement_opts$min_gap,
        placement_tether = placement_opts$tether,
        placement_leader_gap = measurements$gap_native,
        label_precision = precision
      ),
      error = function(e) NULL
    )
    if (
      is.null(placements) ||
        !is.finite(placements$canvas_bbox_h) ||
        !is.finite(placements$canvas_bbox_width) ||
        placements$canvas_bbox_width <= 0 ||
        placements$canvas_bbox_height <= 0
    ) {
      break
    }

    cb_xmin <- placements$canvas_bbox_h -
      placements$canvas_bbox_width / 2
    cb_xmax <- placements$canvas_bbox_h +
      placements$canvas_bbox_width / 2
    cb_ymin <- placements$canvas_bbox_k -
      placements$canvas_bbox_height / 2
    cb_ymax <- placements$canvas_bbox_k +
      placements$canvas_bbox_height / 2

    xmin <- min(geom_xlim[1], cb_xmin)
    xmax <- max(geom_xlim[2], cb_xmax)
    ymin <- min(geom_ylim[1], cb_ymin)
    ymax <- max(geom_ylim[2], cb_ymax)
    xrng <- xmax - xmin
    yrng <- ymax - ymin
    cx <- (xmin + xmax) / 2
    cy <- (ymin + ymax) / 2
    if (xrng / yrng > cell_ar) {
      yrng <- xrng / cell_ar
    } else {
      xrng <- yrng * cell_ar
    }
    new_xlim <- c(cx - xrng / 2, cx + xrng / 2)
    new_ylim <- c(cy - yrng / 2, cy + yrng / 2)

    new_xrng <- diff(new_xlim)
    new_yrng <- diff(new_ylim)
    rel_change <- max(
      abs(new_xrng - diff(current_xlim)) / max(diff(current_xlim), 1e-9),
      abs(new_yrng - diff(current_ylim)) / max(diff(current_ylim), 1e-9)
    )

    # Clamp at cap. Once we hit the cap, stop iterating.
    capped <- FALSE
    if (new_xrng > cap_xrng) {
      mid <- (new_xlim[1] + new_xlim[2]) / 2
      new_xlim <- c(mid - cap_xrng / 2, mid + cap_xrng / 2)
      capped <- TRUE
    }
    if (new_yrng > cap_yrng) {
      mid <- (new_ylim[1] + new_ylim[2]) / 2
      new_ylim <- c(mid - cap_yrng / 2, mid + cap_yrng / 2)
      capped <- TRUE
    }
    current_xlim <- new_xlim
    current_ylim <- new_ylim
    if (rel_change < 0.005 || capped) {
      break
    }
  }

  x$vp$xscale <- current_xlim
  x$vp$yscale <- current_ylim
  x
}

#' Re-place tags at draw time so resizing the device doesn't clip labels.
#'
#' The panel viewport is active when `makeContent` fires, so we can
#' measure each tag's footprint in current native units via
#' `grid::convertWidth(grobWidth(...), "native")` and call eunoia's
#' [place_euler_labels()] again. On resize, grid invalidates the tree
#' and `makeContent` re-runs, so the placement tracks the current
#' device automatically.
#'
#' Tags whose measured size is zero (or whose anchor / kind eunoia
#' can't compute) keep the positions they were built with — typically
#' the setup-time placement stored on `centers$x` / `centers$y`.
#'
#' @export
#' @keywords internal
makeContent.EulerTags <- function(x) {
  n <- length(x$children)
  if (n == 0L) {
    return(x)
  }
  shapes <- x$shapes
  if (is.null(shapes) || NROW(shapes) == 0L) {
    return(x)
  }
  shape_type <- shapes$type[1L]

  padding_native <- grid::convertHeight(
    x$padding,
    "native",
    valueOnly = TRUE
  )

  combos <- character(n)
  widths <- numeric(n)
  heights <- numeric(n)
  for (i in seq_len(n)) {
    child <- x$children[[i]]
    combos[i] <- if (is.null(child$combo_key)) "" else child$combo_key
    sz <- measure_tag_native(child, padding_native)
    widths[i] <- sz$w
    heights[i] <- sz$h
  }

  ok <- widths > 0 & heights > 0 & is.finite(widths) & is.finite(heights)
  if (!any(ok)) {
    return(x)
  }

  container <- x$container
  has_container <- !is.null(container)
  placement_opts <- resolve_placement_opts(x$placement_opts)
  gap_native <- resolve_gap_native(placement_opts$gap, padding_native)

  placements <- place_euler_labels(
    set_names = rownames(shapes),
    shape = shape_type,
    h = shapes$h,
    k = shapes$k,
    a = shapes$a,
    b = shapes$b,
    phi = shapes$phi,
    width = shapes$width,
    height = shapes$height,
    side = shapes$side,
    container_h = if (has_container) container$h else NULL,
    container_k = if (has_container) container$k else NULL,
    container_width = if (has_container) container$width else NULL,
    container_height = if (has_container) container$height else NULL,
    n_vertices = as.integer(x$n_vertices),
    label_combos = combos[ok],
    label_widths = widths[ok],
    label_heights = heights[ok],
    placement = placement_opts$placement,
    placement_margin = placement_opts$margin,
    placement_iterations = placement_opts$iterations,
    placement_min_gap = placement_opts$min_gap,
    placement_tether = placement_opts$tether,
    placement_leader_gap = gap_native,
    label_precision = x$label_precision
  )

  waypoints <- split_waypoints(placements)

  ok_idx <- which(ok)
  for (j in seq_along(ok_idx)) {
    i <- ok_idx[j]
    ax <- placements$anchor_x[j]
    ay <- placements$anchor_y[j]
    if (!is.finite(ax) || !is.finite(ay)) {
      next
    }
    kind <- placements$kind[j]
    tx <- placements$tether_x[j]
    ty <- placements$tether_y[j]
    lend_x <- placements$leader_end_x[j]
    lend_y <- placements$leader_end_y[j]
    wp <- if (!is.null(waypoints)) {
      waypoints[[j]]
    } else {
      list(x = numeric(0), y = numeric(0))
    }
    child <- x$children[[i]]
    new_children <- build_tag_grobs(
      ax = ax,
      ay = ay,
      kind = kind,
      tx = tx,
      ty = ty,
      lend_x = lend_x,
      lend_y = lend_y,
      label_text = child$label_text,
      quantity_text = child$quantity_text,
      annotation_text = child$annotation_text,
      has_label = isTRUE(child$has_label),
      has_quantity = isTRUE(child$has_quantity),
      has_annotation = isTRUE(child$has_annotation),
      label_gp = child$label_gp,
      quantity_gp = child$quantity_gp,
      annotation_gp = child$annotation_gp,
      label_rot = child$label_rot,
      quantity_rot = child$quantity_rot,
      annotation_rot = child$annotation_rot,
      number = child$number,
      leader_gp_list = child$leader_gp_list,
      padding = child$padding,
      waypoints_x = wp$x,
      waypoints_y = wp$y,
      name_prefix = if (is.null(child$name_prefix)) "tag" else child$name_prefix
    )
    # `grid::setChildren()` updates the internal `childrenOrder` index
    # alongside `$children`; direct `$children <-` assignment leaves
    # the order stale and grid silently skips drawing the new grobs.
    x$children[[i]] <- grid::setChildren(child, new_children)
  }

  x
}

#' Default placement options used when the caller doesn't supply any.
#'
#' Mirrors `eunoia::plotting::PlacementStrategy::default()`: raycast
#' exterior solver, POI tether, proportional margin. Per-strategy knobs
#' `iterations` (force-directed) and `min_gap` (elbow) live alongside in
#' the flat shape that the Rust FFI expects; the user-facing sublists in
#' `eulerr_options(labels = list(force_directed = ..., elbow = ...))` are
#' flattened into this shape by `plot.euler()` before placement runs.
#' @keywords internal
default_placement_opts <- function() {
  list(
    placement = "raycast",
    margin = NULL,
    iterations = NULL,
    min_gap = NULL,
    tether = "poi",
    gap = NULL
  )
}

#' Merge user-supplied placement options onto the defaults.
#' @keywords internal
resolve_placement_opts <- function(opts) {
  if (is.null(opts)) {
    return(default_placement_opts())
  }
  defaults <- default_placement_opts()
  for (key in names(defaults)) {
    val <- opts[[key]]
    if (!is.null(val)) {
      defaults[[key]] <- val
    }
  }
  defaults
}

#' Open a temporary grid measurement device + viewport.
#'
#' Returns an idempotent closer thunk that pops the viewport and
#' closes the null PDF device. Used so we can call
#' `grid::convertWidth(grobWidth(...), "native", ...)` at setup time —
#' i.e. before `plot.eulergram()` ever opens a real device — to size
#' the label boxes that drive label placement.
#'
#' Always opens its own off-screen PDF rather than reusing the caller's
#' active device. Pushing a viewport onto the caller's device adds an
#' entry to its display list, which knitr's plot capture
#' treats as visible change and emits as an extra blank plot before
#' the real `plot.eulergram()` draws. The off-screen PDF keeps
#' measurement entirely out of the user's display list.
#' @keywords internal
open_measurement_viewport <- function(xlim, ylim) {
  # Setup-time placement just sizes the *plot region*; draw-time
  # `makeContent.EulerTags` re-places labels against the live device.
  # The measurement device picks a deliberately small reference
  # (3 × 3 in) so the setup-time canvas bbox is conservative enough
  # that exterior-label anchors at typical interactive device sizes
  # still land inside `xlim`/`ylim` after a resize.
  #
  # The measurement viewport must also preserve the diagram's aspect
  # (it's recreated by `plot.eulergram()` via `grid.layout(respect =
  # TRUE)` at draw time). Without aspect preservation the setup-time
  # viewport stretches one axis, so 1 pt in y maps to a smaller native
  # extent than at draw time — measured heights come back too small,
  # the canvas bbox isn't widened enough, and exterior labels overrun
  # `ylim` once the user resizes the device.
  prev_dev <- grDevices::dev.cur()
  grDevices::pdf(NULL, width = 7, height = 7)
  meas_dev <- grDevices::dev.cur()
  xrng <- diff(xlim)
  yrng <- diff(ylim)
  if (!is.finite(xrng) || !is.finite(yrng) || xrng <= 0 || yrng <= 0) {
    width <- grid::unit(1, "snpc")
    height <- grid::unit(1, "snpc")
  } else if (xrng >= yrng) {
    width <- grid::unit(1, "snpc")
    height <- grid::unit(yrng / xrng, "snpc")
  } else {
    width <- grid::unit(xrng / yrng, "snpc")
    height <- grid::unit(1, "snpc")
  }
  grid::pushViewport(grid::viewport(
    width = width,
    height = height,
    xscale = xlim,
    yscale = ylim
  ))
  closed <- FALSE
  function() {
    if (closed) {
      return(invisible(NULL))
    }
    closed <<- TRUE
    grid::popViewport()
    grDevices::dev.off(meas_dev)
    if (prev_dev != 1L) {
      grDevices::dev.set(prev_dev)
    }
  }
}

#' Native-unit AABB of one composite tag (label stacked above quantity,
#' annotation stacked below quantity, separated by `padding`). The
#' geometry matches what `setup_tag()` renders at draw time, so the size
#' handed to eunoia agrees with the actual on-screen footprint.
#' @keywords internal
measure_tag <- function(
  label,
  quantity,
  annotation,
  labels_par_id,
  quantities_par_id,
  annotations_par_id,
  labels_gp,
  quantities_gp,
  annotations_gp,
  padding_native
) {
  # `*_par_id` are only assigned for drawable tags, so they're the
  # authoritative "is there something to draw" signal. Don't call
  # `is.na(label)` on the text — `label` may be an expression (from
  # `str2expression(...)`) and `is.na` warns there.
  do_label <- !is.null(labels_gp) &&
    !is.null(labels_par_id) &&
    !is.na(labels_par_id) &&
    !is.null(label)
  do_quant <- !is.null(quantities_gp) &&
    !is.null(quantities_par_id) &&
    !is.na(quantities_par_id) &&
    !is.null(quantity)
  do_annot <- !is.null(annotations_gp) &&
    !is.null(annotations_par_id) &&
    !is.na(annotations_par_id) &&
    !is.null(annotation)
  if (do_label && !is.expression(label) && is.na(label)) {
    do_label <- FALSE
  }
  if (do_quant && !is.expression(quantity) && is.na(quantity)) {
    do_quant <- FALSE
  }
  if (do_annot && !is.expression(annotation) && is.na(annotation)) {
    do_annot <- FALSE
  }

  if (!do_label && !do_quant && !do_annot) {
    return(list(w = 0, h = 0))
  }

  measure_one <- function(text, gp_row) {
    g <- grid::textGrob(text, gp = gp_row)
    list(
      w = grid::convertWidth(
        grid::grobWidth(g),
        "native",
        valueOnly = TRUE
      ),
      h = grid::convertHeight(
        grid::grobHeight(g),
        "native",
        valueOnly = TRUE
      )
    )
  }

  label_w <- 0
  label_h <- 0
  quant_w <- 0
  quant_h <- 0
  annot_w <- 0
  annot_h <- 0

  if (do_label) {
    m <- measure_one(label, labels_gp[labels_par_id])
    label_w <- m$w
    label_h <- m$h
  }
  if (do_quant) {
    m <- measure_one(quantity, quantities_gp[quantities_par_id])
    quant_w <- m$w
    quant_h <- m$h
  }
  if (do_annot) {
    m <- measure_one(annotation, annotations_gp[annotations_par_id])
    annot_w <- m$w
    annot_h <- m$h
  }

  pad_lq <- if (do_label && do_quant) padding_native else 0
  pad_qa <- if (do_quant && do_annot) padding_native else 0
  pad_la <- if (!do_quant && do_label && do_annot) padding_native else 0
  list(
    w = max(label_w, quant_w, annot_w),
    h = label_h + quant_h + annot_h + pad_lq + pad_qa + pad_la
  )
}

#' Resolve a `gap` option to a numeric in native units inside the current
#' measurement viewport. `NULL` → falls back to `padding_native` so the
#' visible leader-tip gap matches the spacing between label and quantity.
#' A `grid::unit` value converts to native; a bare numeric is interpreted
#' as `lines` (same convention as `eulerr_options()$padding`).
#' @keywords internal
resolve_gap_native <- function(gap, padding_native) {
  if (is.null(gap)) {
    return(padding_native)
  }
  if (inherits(gap, "unit")) {
    return(grid::convertHeight(gap, "native", valueOnly = TRUE))
  }
  grid::convertHeight(grid::unit(gap, "lines"), "native", valueOnly = TRUE)
}

#' Measure all candidate tag sizes (regions + optional complement) inside
#' a fresh measurement viewport scaled to `xlim`/`ylim`.
#' @keywords internal
measure_tag_sizes <- function(
  centers,
  do_complement_label,
  complement_label,
  labels_gp,
  quantities_gp,
  annotations_gp,
  padding,
  gap,
  xlim,
  ylim
) {
  close_vp <- open_measurement_viewport(xlim, ylim)
  on.exit(close_vp(), add = TRUE)

  padding_native <- grid::convertHeight(padding, "native", valueOnly = TRUE)
  gap_native <- resolve_gap_native(gap, padding_native)

  n_rows <- if (is.null(centers)) 0L else NROW(centers)
  widths <- numeric(0)
  heights <- numeric(0)
  combos <- character(0)

  has_annotations_col <- !is.null(centers) &&
    "annotations" %in% names(centers)

  if (n_rows > 0L) {
    widths <- numeric(n_rows)
    heights <- numeric(n_rows)
    combos <- rownames(centers)
    for (i in seq_len(n_rows)) {
      annotation_text <- if (has_annotations_col) centers$annotations[i] else NA
      annotation_par_id <- if (has_annotations_col) {
        centers$annotations_par_id[i]
      } else {
        NA_integer_
      }
      m <- measure_tag(
        label = centers$labels[i],
        quantity = centers$quantities[i],
        annotation = annotation_text,
        labels_par_id = centers$labels_par_id[i],
        quantities_par_id = centers$quantities_par_id[i],
        annotations_par_id = annotation_par_id,
        labels_gp = labels_gp,
        quantities_gp = quantities_gp,
        annotations_gp = annotations_gp,
        padding_native = padding_native
      )
      widths[i] <- m$w
      heights[i] <- m$h
    }
  }

  if (
    do_complement_label &&
      !is.null(complement_label) &&
      !is.na(complement_label)
  ) {
    # The complement label is just text — there's no per-set label or
    # annotation, so measure as if only the quantity is drawn.
    m <- measure_tag(
      label = NA,
      quantity = complement_label,
      annotation = NA,
      labels_par_id = NA_integer_,
      quantities_par_id = 1L,
      annotations_par_id = NA_integer_,
      labels_gp = NULL,
      quantities_gp = quantities_gp,
      annotations_gp = NULL,
      padding_native = padding_native
    )
    widths <- c(widths, m$w)
    heights <- c(heights, m$h)
    combos <- c(combos, "")
  }

  list(
    combos = combos,
    widths = widths,
    heights = heights,
    gap_native = gap_native
  )
}

#' Single placement pass: measure tags, call the Rust FFI, return the
#' placement records and the canvas bbox returned by eunoia.
#' @keywords internal
run_placement_pass <- function(
  centers,
  container_data,
  shapes,
  labels_gp,
  quantities_gp,
  annotations_gp,
  padding,
  placement_opts,
  do_complement_label,
  xlim,
  ylim,
  n_vertices,
  label_precision
) {
  sizes <- measure_tag_sizes(
    centers = centers,
    do_complement_label = do_complement_label,
    complement_label = if (do_complement_label) {
      container_data$quantity_label
    } else {
      NA
    },
    labels_gp = labels_gp,
    quantities_gp = quantities_gp,
    annotations_gp = annotations_gp,
    padding = padding,
    gap = placement_opts$gap,
    xlim = xlim,
    ylim = ylim
  )

  if (length(sizes$combos) == 0L) {
    return(list(sizes = sizes, placements = NULL))
  }

  do_container <- !is.null(container_data)
  shape_type <- if (NROW(shapes) > 0L) shapes$type[1L] else "ellipse"
  placements <- place_euler_labels(
    set_names = rownames(shapes),
    shape = shape_type,
    h = shapes$h,
    k = shapes$k,
    a = shapes$a,
    b = shapes$b,
    phi = shapes$phi,
    width = shapes$width,
    height = shapes$height,
    side = shapes$side,
    container_h = if (do_container) container_data$h else NULL,
    container_k = if (do_container) container_data$k else NULL,
    container_width = if (do_container) container_data$width else NULL,
    container_height = if (do_container) container_data$height else NULL,
    n_vertices = as.integer(n_vertices),
    label_combos = sizes$combos,
    label_widths = sizes$widths,
    label_heights = sizes$heights,
    placement = placement_opts$placement,
    placement_margin = placement_opts$margin,
    placement_iterations = placement_opts$iterations,
    placement_min_gap = placement_opts$min_gap,
    placement_tether = placement_opts$tether,
    placement_leader_gap = sizes$gap_native,
    label_precision = label_precision
  )

  list(sizes = sizes, placements = placements)
}

#' Split a placement's flat `leader_waypoints_x` / `_y` / `_lengths`
#' return triple into a per-label list of `list(x = ..., y = ...)`
#' coordinate pairs. Each list element has length-`lengths[i]` `x`/`y`
#' vectors (often `0` — straight leaders carry no waypoints).
#' @keywords internal
split_waypoints <- function(placements) {
  wx <- placements$leader_waypoints_x
  wy <- placements$leader_waypoints_y
  lens <- placements$leader_waypoints_lengths
  if (is.null(wx) || is.null(wy) || is.null(lens)) {
    return(NULL)
  }
  n <- length(lens)
  out <- vector("list", n)
  cursor <- 0L
  for (i in seq_len(n)) {
    len <- as.integer(lens[i])
    if (len <= 0L) {
      out[[i]] <- list(x = numeric(0), y = numeric(0))
    } else {
      idx <- seq.int(cursor + 1L, cursor + len)
      out[[i]] <- list(x = wx[idx], y = wy[idx])
      cursor <- cursor + len
    }
  }
  out
}

#' Union the current `xlim`/`ylim` with the canvas bbox reported by
#' eunoia. Returns the (possibly widened) limits.
#'
#' `slack` pads the canvas bbox before the union (multiplicative,
#' centered on the bbox center). The setup-time bbox is sized for one
#' reference device; padding gives the draw-time placement headroom
#' when the user resizes the device smaller than that reference and
#' labels grow in native units accordingly. A `slack` of 1.4 absorbs
#' roughly a 30 % linear resize before exterior labels start to fall
#' outside the panel viewport.
#' @keywords internal
expand_limits_with_canvas <- function(limits, placements, slack = 1.4) {
  if (
    is.null(placements) ||
      !is.finite(placements$canvas_bbox_h) ||
      !is.finite(placements$canvas_bbox_k) ||
      !is.finite(placements$canvas_bbox_width) ||
      !is.finite(placements$canvas_bbox_height) ||
      placements$canvas_bbox_width <= 0 ||
      placements$canvas_bbox_height <= 0
  ) {
    return(limits)
  }
  cx_half <- placements$canvas_bbox_width * slack / 2
  cy_half <- placements$canvas_bbox_height * slack / 2
  canvas_xlim <- c(
    placements$canvas_bbox_h - cx_half,
    placements$canvas_bbox_h + cx_half
  )
  canvas_ylim <- c(
    placements$canvas_bbox_k - cy_half,
    placements$canvas_bbox_k + cy_half
  )
  list(
    xlim = range(c(limits$xlim, canvas_xlim)),
    ylim = range(c(limits$ylim, canvas_ylim))
  )
}

#' Run eunoia label placement, expanding limits so exterior labels are
#' not clipped. Drives one initial pass plus one re-measure pass when
#' the limits widened by more than `re_measure_threshold` on the short
#' side. Updates `centers` (and the complement slot on `container_data`)
#' in place with placed `(x, y)` plus `kind`, `tether_x`, `tether_y`,
#' `leader_end_x`, `leader_end_y`.
#'
#' When `placement_opts` is `NULL`, defaults to eunoia's raycast + POI
#' tether.
#'
#' Returns a list with `centers`, `container_data`, and `limits`.
#' @keywords internal
apply_label_placement <- function(
  centers,
  container_data,
  shapes,
  labels,
  quantities,
  annotations = NULL,
  placement_opts = NULL,
  do_complement_label = FALSE,
  limits,
  n_vertices,
  label_precision,
  re_measure_threshold = 0.01
) {
  no_centers <- is.null(centers) || NROW(centers) == 0L
  if (no_centers && !do_complement_label) {
    return(list(
      centers = centers,
      container_data = container_data,
      limits = limits
    ))
  }
  if (NROW(shapes) == 0L) {
    return(list(
      centers = centers,
      container_data = container_data,
      limits = limits
    ))
  }

  placement_opts <- resolve_placement_opts(placement_opts)
  labels_gp <- if (!is.null(labels)) labels$gp else NULL
  quantities_gp <- if (!is.null(quantities)) quantities$gp else NULL
  annotations_gp <- if (!is.null(annotations)) annotations$gp else NULL
  padding <- eulerr_options()$padding

  # First pass at the incoming limits. The aspect-preserving
  # measurement viewport's native-per-pt ratio is set by `xrng/yrng`,
  # so widening the bbox between passes changes the measurement and
  # the iteration can diverge for very large labels relative to the
  # diagram. One re-measure is enough for typical cases and converges
  # for everything we expect to draw correctly at the reference
  # device size.
  pass1 <- run_placement_pass(
    centers = centers,
    container_data = container_data,
    shapes = shapes,
    labels_gp = labels_gp,
    quantities_gp = quantities_gp,
    annotations_gp = annotations_gp,
    padding = padding,
    placement_opts = placement_opts,
    do_complement_label = do_complement_label,
    xlim = limits$xlim,
    ylim = limits$ylim,
    n_vertices = n_vertices,
    label_precision = label_precision
  )
  if (is.null(pass1$placements)) {
    return(list(
      centers = centers,
      container_data = container_data,
      limits = limits
    ))
  }
  new_limits <- expand_limits_with_canvas(limits, pass1$placements)
  final <- pass1

  kinds <- pass1$placements$kind
  has_exterior <- any(nzchar(kinds) & kinds != "interior")
  old_short <- min(diff(limits$xlim), diff(limits$ylim))
  new_short <- min(diff(new_limits$xlim), diff(new_limits$ylim))
  if (has_exterior && is.finite(old_short) && old_short > 0) {
    rel_change <- abs(new_short - old_short) / old_short
    if (rel_change > re_measure_threshold) {
      pass2 <- run_placement_pass(
        centers = centers,
        container_data = container_data,
        shapes = shapes,
        labels_gp = labels_gp,
        quantities_gp = quantities_gp,
        annotations_gp = annotations_gp,
        padding = padding,
        placement_opts = placement_opts,
        do_complement_label = do_complement_label,
        xlim = new_limits$xlim,
        ylim = new_limits$ylim,
        n_vertices = n_vertices,
        label_precision = label_precision
      )
      if (!is.null(pass2$placements)) {
        new_limits <- expand_limits_with_canvas(new_limits, pass2$placements)
        final <- pass2
      }
    }
  }

  # Apply placements back to centers and container_data.
  combos <- final$sizes$combos
  pl <- final$placements
  waypoints <- split_waypoints(pl)

  centers_combos <- if (!no_centers) rownames(centers) else character(0)
  if (!no_centers) {
    centers$kind <- rep("", NROW(centers))
    centers$tether_x <- rep(NA_real_, NROW(centers))
    centers$tether_y <- rep(NA_real_, NROW(centers))
    centers$leader_end_x <- rep(NA_real_, NROW(centers))
    centers$leader_end_y <- rep(NA_real_, NROW(centers))
    empty_wp <- list(x = numeric(0), y = numeric(0))
    centers$leader_waypoints <- replicate(NROW(centers), empty_wp, simplify = FALSE)
    if (length(centers_combos) > 0L) {
      idx <- match(centers_combos, combos)
      ok <- !is.na(idx)
      if (any(ok)) {
        ax <- pl$anchor_x[idx[ok]]
        ay <- pl$anchor_y[idx[ok]]
        kx <- pl$kind[idx[ok]]
        tx <- pl$tether_x[idx[ok]]
        ty <- pl$tether_y[idx[ok]]
        lex <- pl$leader_end_x[idx[ok]]
        ley <- pl$leader_end_y[idx[ok]]
        # Keep the POI fallback when eunoia returned NA / no placement.
        valid <- is.finite(ax) & is.finite(ay)
        rows <- which(ok)[valid]
        centers$x[rows] <- ax[valid]
        centers$y[rows] <- ay[valid]
        centers$kind[rows] <- kx[valid]
        centers$tether_x[rows] <- tx[valid]
        centers$tether_y[rows] <- ty[valid]
        centers$leader_end_x[rows] <- lex[valid]
        centers$leader_end_y[rows] <- ley[valid]
        if (!is.null(waypoints)) {
          wp_idx <- idx[ok][valid]
          for (j in seq_along(rows)) {
            centers$leader_waypoints[[rows[j]]] <- waypoints[[wp_idx[j]]]
          }
        }
      }
    }
  }

  if (do_complement_label && !is.null(container_data)) {
    idx <- match("", combos)
    if (!is.na(idx)) {
      ax <- pl$anchor_x[idx]
      ay <- pl$anchor_y[idx]
      if (is.finite(ax) && is.finite(ay)) {
        container_data$label_x <- ax
        container_data$label_y <- ay
        container_data$kind <- pl$kind[idx]
        container_data$tether_x <- pl$tether_x[idx]
        container_data$tether_y <- pl$tether_y[idx]
        container_data$leader_end_x <- pl$leader_end_x[idx]
        container_data$leader_end_y <- pl$leader_end_y[idx]
        container_data$leader_waypoints <- if (!is.null(waypoints)) {
          waypoints[[idx]]
        } else {
          list(x = numeric(0), y = numeric(0))
        }
      }
    }
  }

  list(
    centers = centers,
    container_data = container_data,
    limits = new_limits
  )
}

#' Fit an Euler diagram using the eunoia Rust library.

# nolint start

#' @keywords internal
fit_euler_diagram <- function(combo_names, combo_values, input, shape, loss, extraopt_threshold, tolerance, max_sets, complement, seed) .Call(wrap__fit_euler_diagram, combo_names, combo_values, input, shape, loss, extraopt_threshold, tolerance, max_sets, complement, seed)

#' Compute polygon geometry and label anchors for plotting a fitted Euler
#' diagram, including the optional complement region inside a fitted
#' container.
#'
#' Inputs are the fitted shape parameters for the **non-empty** sets only,
#' in the order eulerr stores them (`x$shapes` rows after dropping rows
#' with NA). The leading `shape` argument selects which per-set columns are
#' active: `"circle"`/`"ellipse"` consume `(h, k, a, b, phi)`,
#' `"rectangle"` consumes `(h, k, width, height)`, `"square"` consumes
#' `(h, k, side)`. The remaining vectors must still be provided (NaN
#' padding is acceptable). When `container_*` are non-NULL they describe
#' the fitted universe-box rectangle; in that case the result also carries
#' the complement region geometry (the area inside the rectangle outside
#' every shape) and a label anchor for it. Eunoia's `decompose_regions`
#' emits this region under the empty `Combination` whenever the spec
#' carries a complement and a container is supplied.
#'
#' @keywords internal
euler_plot_data <- function(set_names, shape, h, k, a, b, phi, width, height, side, container_h, container_k, container_width, container_height, n_vertices, label_precision) .Call(wrap__euler_plot_data, set_names, shape, h, k, a, b, phi, width, height, side, container_h, container_k, container_width, container_height, n_vertices, label_precision)

#' Place per-region labels using eunoia's `place_labels` API.
#'
#' Inputs mirror `euler_plot_data` for shape geometry and add per-region
#' label sizes plus placement-strategy options. Returns, parallel to
#' `label_combos`:
#'
#' * `anchor_x` / `anchor_y` — placed label anchor (NA on miss);
#' * `kind` — one of `"interior"`, `"exterior_raycast"`,
#'   `"exterior_force_directed"`, `"exterior_elbow"`; `""` if no
#'   placement was produced;
#' * `tether_x` / `tether_y` — tether point for the leader line (NA for
#'   interior placements / misses).
#' * `leader_end_x` / `leader_end_y` — point on the label box AABB where
#'   the leader terminates (NA for interior placements / misses).
#' * `leader_waypoints_x` / `leader_waypoints_y` / `leader_waypoints_lengths`
#'   — concatenated waypoint coordinates and per-label counts. Empty for
#'   straight leaders; carries one knee point per elbow placement.
#'
#' Plus a canvas bbox (`canvas_bbox_h/k/width/height`) from eunoia's
#' `placements_bbox` — NaN when no placements were produced — for the R
#' side to grow `xlim/ylim` so exterior labels are never clipped.
#'
#' The complement region is requested with `""` in `label_combos`; when
#' `container_*` are non-NULL the spec is built with `.complement(1.0)`
#' so eunoia emits the empty `Combination` from `decompose_regions`.
#'
#' @keywords internal
place_euler_labels <- function(set_names, shape, h, k, a, b, phi, width, height, side, container_h, container_k, container_width, container_height, n_vertices, label_combos, label_widths, label_heights, placement, placement_margin, placement_iterations, placement_min_gap, placement_tether, placement_leader_gap, label_precision) .Call(wrap__place_euler_labels, set_names, shape, h, k, a, b, phi, width, height, side, container_h, container_k, container_width, container_height, n_vertices, label_combos, label_widths, label_heights, placement, placement_margin, placement_iterations, placement_min_gap, placement_tether, placement_leader_gap, label_precision)

#' Clip a (possibly multi-polygon) subject path against a single clip
#' polygon. Mirrors the slice of `polyclip::polyclip` behavior eulerr
#' actually uses at the stripe-pattern site.
#'
#' @keywords internal
polygon_clip_rust <- function(subject_x, subject_y, subject_id_lengths, clip_x, clip_y, op) .Call(wrap__polygon_clip_rust, subject_x, subject_y, subject_id_lengths, clip_x, clip_y, op)

#' Default number of sets that `eunoia` accepts before rejecting a spec.
#' Used by the R-side input validator so the cap is not hardcoded.
#' @keywords internal
max_sets_default <- function() .Call(wrap__max_sets_default)

#' Absolute upper bound on the number of sets that `eunoia` can represent
#' in a single diagram. Used by the R-side input validator so the cap is
#' not hardcoded.
#' @keywords internal
max_sets_hard_cap <- function() .Call(wrap__max_sets_hard_cap)


# nolint end

test_that("erroneous input to euler() throw errors", {
  set.seed(1)

  expect_error(euler(c(1, 2, 3)))
  expect_error(euler(c()))
  expect_error(euler(c(A = FALSE, B = TRUE, C = FALSE)))
  expect_error(euler(c(A = 10, A = 5)))
  expect_error(euler(c(A = 10, 4)))
  expect_error(euler(cbind(A = TRUE, "&asdf" = FALSE)))

  dat <- data.frame(
    A = sample(c(TRUE, FALSE), size = 100, replace = TRUE),
    B = sample(c(TRUE, TRUE, FALSE), size = 100, replace = TRUE),
    x = sample(c("Men", "Women"), size = 100, replace = TRUE),
    y = sample(c("Sweden", "Denmark"), size = 100, replace = TRUE),
    z = sample(c("asdf", "qwer"), size = 100, replace = TRUE)
  )

  expect_error(euler(dat, by = list(x, y, z)))
  expect_error(euler(dat, by = dat[1:50, 3]))
  expect_error(euler(dat, by = list(dat[, 2])))
  expect_error(euler(dat, by = list(x, y, z, k)))

  expect_error(euler(cbind(dat, rnorm(nrow(dat)))))

  # impossible configuration
  expect_error(euler(c(A = 10, B = 14, "A&B" = 15), input = "union"))

  # list method
  expect_error(euler(list(c("a", "b"), c("a", "c"))))
  expect_error(euler(list(b = c(1, 2), b = c(1, 3))))
})

test_that("erroneous input to venn() throws errors", {
  expect_error(venn(2))
  expect_error(venn(0))
  expect_error(venn(2, names = "a"))
  expect_error(venn(6, names = letters[1:n]))
})

test_that("erroneous input to print.euler() throw errors", {
  f <- euler(c(A = 10.923, B = 5.4, "A&B" = 0.43))
  expect_error(dont_print(f, round = "hello"))
  expect_error(dont_print(f, round = c(1, 2)))
})

test_that("erroneous input to plot.euler() return errors", {
  f <- euler(c(A = 5, B = 2))
  expect_error(dont_plot(f, n = -1))
})

test_that("erroneous input to error_plot() throws", {
  f <- euler(organisms)

  expect_error(error_plot(f, fills = list(col = "grey")))
  expect_error(error_plot(f, legend = TRUE))
  expect_error(error_plot(f, strips = TRUE))
})

test_that("euler() guards against too many sets", {
  default_cap <- max_sets_default()
  hard_cap <- max_sets_hard_cap()

  # Default cap rejects more than `default_cap` sets (guard fires R-side, no fit)
  many <- stats::setNames(
    rep.int(1, default_cap + 1L),
    paste0("S", seq_len(default_cap + 1L))
  )
  expect_error(euler(many), "too many sets")

  # Lowered cap rejects a small input that would otherwise fit
  small <- c(A = 1, B = 1, C = 1, "A&B" = 0.5)
  expect_error(
    euler(small, control = list(max_sets = 2)),
    "too many sets"
  )

  # Override at exactly n passes the guard
  expect_s3_class(
    euler(small, control = list(max_sets = 3)),
    "euler"
  )

  # Override above the hard cap is rejected R-side
  expect_error(
    euler(small, control = list(max_sets = hard_cap + 1L)),
    "exceeds the hard cap"
  )

  # Bad max_sets values are rejected
  expect_error(euler(small, control = list(max_sets = 0)))
  expect_error(euler(small, control = list(max_sets = -1)))
  expect_error(euler(small, control = list(max_sets = 1.5)))
  expect_error(euler(small, control = list(max_sets = c(10, 20))))
  expect_error(euler(small, control = list(max_sets = "32")))
})

#' Venn diagrams
#'
#' This function fits Venn diagrams using an interface that is
#' almost identical to [euler()]. Strictly speaking,
#' Venn diagrams are Euler diagrams where every intersection is visible,
#' regardless of whether or not it is zero. In almost every incarnation of
#' Venn diagrams, however, the areas in the diagram are also
#' *non-proportional* to the input; this is also the case here.
#'
#' @inheritParams euler
#'
#' @return Returns an object of class `'eulerr_venn', 'venn', 'euler'` with items
#'   \item{shapes}{a data frame of the precomputed ellipse parameters (one
#'     row per set, columns `type, h, k, a, b, phi`). `venn()` always uses
#'     ellipses.}
#'   \item{ellipses}{the legacy 5-column data frame
#'     (`h, k, a, b, phi`) — kept for back-compat alongside the canonical
#'     `shapes` slot.}
#'   \item{original.values}{set relationships in the input}
#'   \item{fitted.values}{set relationships in the solution}
#'
#' @seealso [plot.eulerr_venn()], [print.eulerr_venn()], [euler()]
#'
#' @export
#'
#' @examples
#' # The trivial version
#' f1 <- venn(5, names = letters[1:5])
#' plot(f1)
#'
#' # Using data (a numeric vector)
#' f2 <- venn(c(A = 1, "B&C" = 3, "A&D" = 0.3))
venn <- function(combinations, ...) {
  UseMethod("venn")
}

#' @describeIn venn a named numeric vector, with
#'   combinations separated by an ampersand, for instance `A&B = 10`.
#'   Missing combinations are treated as being 0.
#' @param names a character vector for the names of each set of the same
#'   length as 'combinations'. Must not be `NULL` if `combinations` is a
#'   one-length numeric.
#' @export
venn.default <- function(
  combinations,
  input = c("disjoint", "union"),
  names = letters[length(combinations)],
  ...
) {
  if (is.numeric(combinations) && length(combinations) == 1) {
    if (is.null(names)) {
      stop(
        "'names' must not be NULL when 'combinations' is a length ",
        "one integer."
      )
    }

    if (combinations < 1) {
      stop(
        "'combinations' cannot be less than 1 when 'combinations' is a ",
        "length one integer."
      )
    }

    if (combinations > 5) {
      stop("'venn()' only supports diagrams with up to five sets.")
    }

    if (length(names) != combinations) {
      stop("'names' must be as long as 'combinations'.")
    }

    combo_names <- all_set_combinations(names)
    combinations <- stats::setNames(
      rep.int(1, length(combo_names)),
      combo_names
    )
  } else {
    n_combinations <- length(unique(unlist(strsplit(names(combinations), "&"))))

    if (n_combinations > 5) {
      stop("'venn()' only supports diagrams with up to five sets")
    }
  }

  fit_diagram(
    combinations,
    type = "venn",
    input = input,
    shape = "ellipse",
    control = list(),
    ...
  )
}

#' @describeIn venn A table with `max(dim(x)) < 3`.
#' @export
#'
#' @examples
#'
#' # The table method
#' venn(pain, factor_names = FALSE)
venn.table <- function(combinations, ...) {
  x <- as.data.frame(combinations)
  venn(x[, !(names(x) == "Freq")], weights = x$Freq, ...)
}

#' @describeIn venn a `data.frame` of logicals, binary integers, or
#'   factors.
#' @export
#' @examples
#'
#' # Using grouping via the 'by' argument through the data.frame method
#' venn(fruits, by = list(sex, age))
#'
venn.data.frame <- function(
  combinations,
  weights = NULL,
  by = NULL,
  sep = "_",
  factor_names = TRUE,
  ...
) {
  by <- substitute(by)
  facs <- eval(by, combinations)

  combinations <- parse_dataframe(
    combinations,
    weights,
    by,
    facs,
    sep,
    factor_names
  )

  if (is.list(combinations)) {
    out <- lapply(combinations, venn, ...)
    class(out) <- c("eulerr_venn", "venn", "euler", "list")
    attr(out, "groups") <- attr(combinations, "groups")
  } else {
    out <- venn(combinations, ...)
  }

  out
}

#' @describeIn venn a matrix that can be converted to a data.frame of logicals
#'   (as in the description above) via [base::as.data.frame.matrix()].
#' @export
#' @inheritParams euler
#'
#' @examples
#'
#' # Using the matrix method
#' venn(organisms)
#'
#' # Using weights
#' venn(organisms, weights = c(10, 20, 5, 4, 8, 9, 2))
venn.matrix <- function(combinations, ...) {
  venn(as.data.frame(combinations), ...)
}

#' @describeIn venn a list of vectors, each vector giving the contents of
#'   that set (with no duplicates). Vectors in the list do not need to be named.
#' @export
#' @inheritParams euler
#' @examples
#'
#' # A venn diagram from a list of sample spaces (the list method)
#' venn(plants[c("erigenia", "solanum", "cynodon")])
venn.list <- function(combinations, ...) {
  venn(parse_list(combinations), input = "disjoint", ...)
}

parse_list <- function(combinations) {
  if (is.null(attr(combinations, "names"))) {
    stop(
      "when `combinations` is a list, all vectors in that list must be named"
    )
  }

  if (any(names(combinations) == "")) {
    stop("all elements of `combinations` must be named")
  }

  if (!all(sapply(combinations, anyDuplicated) == 0)) {
    stop("vectors in `combinations` cannot contain duplicates")
  }

  if (any(duplicated(names(combinations)))) {
    stop("names of elements in `combinations` must be unique")
  }

  sets <- names(combinations)

  all_elems <- unlist(lapply(combinations, as.character), use.names = FALSE)
  all_set_idx <- rep.int(seq_along(combinations), lengths(combinations))

  if (length(all_elems) == 0L) {
    counts <- numeric(0)
  } else {
    by_elem <- split(all_set_idx, all_elems)
    patterns <- vapply(
      by_elem,
      function(idx) paste(sets[sort.int(unique(idx))], collapse = "&"),
      character(1)
    )
    tab <- table(patterns)
    counts <- as.numeric(tab)
    names(counts) <- names(tab)
  }

  singletons <- stats::setNames(numeric(length(sets)), sets)
  shared <- intersect(sets, names(counts))
  if (length(shared) > 0L) {
    singletons[shared] <- counts[shared]
  }
  multi <- counts[setdiff(names(counts), sets)]

  c(singletons, multi)
}

parse_dataframe <- function(
  combinations,
  weights = NULL,
  by = NULL,
  facs,
  sep = "_",
  factor_names = TRUE,
  ...
) {
  if (any(grepl("&", colnames(combinations), fixed = TRUE))) {
    stop("names of columns in `combinations` must not contain '$'")
  }

  if (!is.null(facs)) {
    if (is.list(facs)) {
      if (!is.null(names(facs))) {
        nms <- names(facs)
      } else {
        nms <- sapply(by[-1L], deparse)
      }
    } else {
      nms <- deparse(by)
    }

    if (is.list(facs)) {
      stopifnot(length(facs) < 3)
    } else {
      facs <- list(facs)
    }

    dd <- as.data.frame(facs, col.names = nms, stringsAsFactors = TRUE)
    groups <- unique(dd)
    rownames(groups) <- NULL

    out <- g <- vector("list", NROW(groups))

    by_ind <- match(nms, colnames(combinations))

    for (i in seq_len(NROW(groups))) {
      ind <- apply(dd, 1, function(x) all(x == groups[i, ]))
      out[[i]] <- combinations[ind, -by_ind, drop = FALSE]
      names(out)[[i]] <- paste(unlist(groups[i, , drop = TRUE]), collapse = ".")
    }

    attr(out, "groups") <- groups
  } else {
    is_factor <- vapply(
      combinations,
      function(x) is.factor(x) || is.character(x),
      logical(1)
    )

    reals <- vapply(combinations, is_real, FUN.VALUE = logical(1))

    if (any(reals)) {
      stop("you cannot use non-integer numeric variables.")
    }

    if (any(is_factor)) {
      combinations <- dummy_code(
        combinations,
        sep = sep,
        factor_names = factor_names
      )
    }

    if (is.null(weights)) {
      weights <- rep.int(1L, NROW(combinations))
    }

    out <- tally_combinations(combinations, weights)
  }
  out
}

#' Fitted values of euler object
#'
#' @param object object of class `'euler'`
#' @param dense if `TRUE`, return a vector covering every combination of the
#'   non-empty sets (`2^n - 1` entries), filling absent combinations with 0.
#'   The default (`FALSE`) returns the sparse vector stored on the object,
#'   which only contains entries that were either requested as input or fit
#'   to a non-zero area. Use `dense = TRUE` only for diagrams with few sets
#'   --- the full enumeration is exponential in the number of non-empty sets.
#' @param ... ignored
#' @export
#' @return A named numeric vector of fitted areas keyed by combination label
#'   (set names joined by `&`).
fitted.euler <- function(object, dense = FALSE, ...) {
  if (!isTRUE(dense)) {
    return(object$fitted.values)
  }
  expand_dense(object$fitted.values, object$shapes)
}

#' Residuals of euler object
#'
#' @param object object of class `'euler'`
#' @param dense same meaning as in [fitted.euler()]
#' @param ... ignored
#' @export
#' @return A named numeric vector of residuals (input minus fit) keyed by
#'   combination label.
residuals.euler <- function(object, dense = FALSE, ...) {
  if (is.null(object$residuals)) {
    return(NULL)
  }
  if (!isTRUE(dense)) {
    return(object$residuals)
  }
  expand_dense(object$residuals, object$shapes)
}

#' Pad a sparse combination-keyed vector to cover every combination of the
#' non-empty sets in `shapes`, filling absent entries with 0. The lookup is
#' the wide `$shapes` table keyed by row names; empty sets carry NA in `h`
#' so they're filtered out.
#' @keywords internal
#' @noRd
expand_dense <- function(sparse_values, shapes) {
  setnames <- rownames(shapes)
  non_empty <- !is.na(shapes$h)
  all_combos <- all_set_combinations(setnames[non_empty])
  out <- stats::setNames(numeric(length(all_combos)), all_combos)
  present <- intersect(all_combos, names(sparse_values))
  out[present] <- sparse_values[present]
  out
}

#' Return the fitted shape parameters from the euler object
#'
#' Returns the `$shapes` data frame — a tagged uniform schema with one row
#' per set, a `type` column, and shape-specific columns
#' (`h, k, a, b, phi, width, height, side`) populated according to the
#' chosen shape (other columns are `NA`). For circle/ellipse fits the
#' legacy `$ellipses` slot is still populated for back-compat; consumers
#' that need the new shapes (rectangle, square) must read `$shapes`.
#'
#' @param object object of class `'euler'`
#' @param ... ignored
#' @export
#' @keywords internal
#' @return a data frame of the fitted shape parameters
coef.euler <- function(object, ...) {
  if (!is.null(object$shapes)) {
    object$shapes
  } else {
    object$ellipses
  }
}

euler_legend_grob <- function(
  labels,
  gp,
  symbol_size = 1,
  hgap = grid::unit(1, "lines"),
  vgap = grid::unit(0.25, "lines"),
  ncol = 1L,
  nrow = length(labels),
  byrow = FALSE
) {
  n <- length(labels)
  sym_unit <- grid::unit(symbol_size, "char")

  t_pts <- seq(0, 2 * pi, length.out = 64)
  cx <- 0.5 + 0.48 * cos(t_pts)
  cy <- 0.5 + 0.48 * sin(t_pts)

  symbol_grobs <- lapply(seq_len(n), function(i) {
    base <- grid::pathGrob(
      cx,
      cy,
      default.units = "npc",
      gp = grid::gpar(
        fill = gp$fill[i],
        col = gp$col[i],
        lwd = gp$lwd[i],
        lex = gp$lex[i],
        alpha = gp$alpha[i]
      )
    )
    children <- grid::gList(base)

    if (isTRUE(gp$pattern_type[i] == "stripes")) {
      pcol <- gp$pattern_col[i]
      if (is.na(pcol)) {
        pcol <- gp$fill[i]
      }
      clipped <- apply_stripe_pattern(
        fill_data = list(x = cx, y = cy, id.lengths = length(cx)),
        pattern_gp = list(
          type = "stripes",
          angle = gp$pattern_angle[i],
          col = pcol,
          lwd = gp$pattern_lwd[i],
          alpha = gp$pattern_alpha[i]
        ),
        spacing_scale = 3
      )
      if (!is.null(clipped)) {
        stripe_grob <- grid::pathGrob(
          x = unlist(lapply(clipped, "[[", "x"), use.names = FALSE),
          y = unlist(lapply(clipped, "[[", "y"), use.names = FALSE),
          id.lengths = lengths(lapply(clipped, "[[", "x")),
          default.units = "npc",
          gp = grid::gpar(
            fill = pcol,
            col = "transparent",
            alpha = gp$pattern_alpha[i]
          )
        )
        children <- grid::gList(base, stripe_grob)
      }
    }

    # Explicit square viewport so the circle stays round regardless of row height.
    grid::gTree(
      children = children,
      vp = grid::viewport(width = sym_unit, height = sym_unit)
    )
  })

  text_grobs <- lapply(seq_len(n), function(i) {
    grid::textGrob(
      labels[i],
      x = 0,
      y = 0.5,
      just = c("left", "centre"),
      gp = grid::gpar(
        fontsize = gp$fontsize[1L],
        font = gp$font[1L],
        fontfamily = gp$fontfamily[1L],
        cex = gp$cex[1L]
      )
    )
  })

  if (byrow) {
    entry_rows <- ((seq_len(n) - 1L) %/% ncol) + 1L
    entry_cols <- ((seq_len(n) - 1L) %% ncol) + 1L
  } else {
    entry_rows <- ((seq_len(n) - 1L) %% nrow) + 1L
    entry_cols <- ((seq_len(n) - 1L) %/% nrow) + 1L
  }

  # Per legend-column, use the widest label to size the text column.
  text_col_widths <- lapply(seq_len(ncol), function(j) {
    entries <- which(entry_cols == j)
    if (length(entries) == 0L) {
      return(grid::unit(0, "mm"))
    }
    best <- entries[which.max(nchar(labels[entries]))]
    grid::unit(1, "grobwidth", list(text_grobs[[best]]))
  })

  # Frame columns: [sym | hgap | text] per legend-column, with 1-line sep between.
  # Total cols = 4*ncol - 1.
  frame_widths <- vector("list", 4L * ncol - 1L)
  for (j in seq_len(ncol)) {
    base_fc <- (j - 1L) * 4L + 1L
    frame_widths[[base_fc]] <- sym_unit
    frame_widths[[base_fc + 1L]] <- hgap
    frame_widths[[base_fc + 2L]] <- text_col_widths[[j]]
    if (j < ncol) {
      frame_widths[[base_fc + 3L]] <- grid::unit(1, "lines")
    }
  }
  frame_widths <- do.call(grid::unit.c, frame_widths)

  # Row height = max(symbol height, text height).
  # Both are in "char" units (1 char = fontsize pt): symbol = symbol_size char,
  # text = cex char. Grid has no unit.pmax, so compute the max numerically.
  row_height <- grid::unit(max(symbol_size, gp$cex[1L]), "char")

  frame_heights <- vector("list", 2L * nrow - 1L)
  for (r in seq_len(nrow)) {
    frame_heights[[2L * r - 1L]] <- row_height
    if (r < nrow) {
      frame_heights[[2L * r]] <- vgap
    }
  }
  frame_heights <- do.call(grid::unit.c, frame_heights)

  sym_frame_cols <- (seq_len(ncol) - 1L) * 4L + 1L
  text_frame_cols <- (seq_len(ncol) - 1L) * 4L + 3L

  fg <- grid::frameGrob(
    layout = grid::grid.layout(
      nrow = length(frame_heights),
      ncol = length(frame_widths),
      widths = frame_widths,
      heights = frame_heights
    )
  )

  for (i in seq_len(n)) {
    frame_r <- 2L * entry_rows[i] - 1L
    j <- entry_cols[i]
    fg <- grid::placeGrob(
      fg,
      symbol_grobs[[i]],
      row = frame_r,
      col = sym_frame_cols[j]
    )
    fg <- grid::placeGrob(
      fg,
      text_grobs[[i]],
      row = frame_r,
      col = text_frame_cols[j]
    )
  }

  fg
}

#' Plants
#'
#' Data on plants and the states in the US and Canada they occur in.
#'
#' @format A [list] with 33,721 plants, each containing a character vector
#'   listing the states in the US and Canada in which they occur. The names in
#'   the list specify the species or genus of the plant.
#'
#' @source USDA, NRCS. 2008. The PLANTS Database, 31
#'   December 2008). National Plant Data Center, Baton Rouge, LA 70874-4490 USA.
#' @source Dua, D. and Karra Taniskidou, E. (2017). UCI Machine Learning
#' Repository <http://archive.ics.uci.edu/ml/>. Irvine, CA: University of
#' California, School of Information and Computer Science.
"plants"

#' Fruits
#'
#' A synthethic data set of preferences for fruits and their overlaps,
#' generated only to be a showcase for the examples for this package.
#'
#' @format A [data.frame] with 100 observations of 5 variables:
#' \describe{
#'   \item{banana}{whether the person likes bananas, a logical}
#'   \item{apple}{whether the person likes apples, a logical}
#'   \item{orange}{whether the person likes oranges, a logical}
#'   \item{sex}{the sex of the person, a factor with levels 'male' and 'female'}
#'   \item{age}{the age of the person, a factor with levels 'child' and 'adult'}
#' }
"fruits"

#' Organisms
#'
#' Example data from the **VennMaster** package.
#'
#' Note that this data is difficult to fit using an Euler diagram, even
#' if we use ellipses, which is clear if one chooses to study the various
#' overlaps in the resulting diagrams.
#'
#' @format A [matrix] with 7 observations, consisting of various organisms,
#'   and 5 variables: *animal, mammal, plant, sea*, and, *spiny*,
#'   indicating whether the organism belongs to the category or not.
#'
#' @source <https://github.com/sysbio-bioinf/VennMaster/blob/master/data_examples/deploy/example1.list>
"organisms"

#' Pain distribution data
#'
#' Data from a study on pain distribution for patients with
#' persistent neck pain in relation to a whiplash trauma.
#'
#' @section Disclaimer:
#' Note that the maintainer of this package is an author of the source for
#' this data.
#'
#' @format A flat [table] (cross-table) with with sex in columns and
#'   pain distribution in rows and integer counts making up the cells of the
#'   table.
#'
#' @source Westergren H, Larsson J, Freeman M, Carlsson A, Jöud A,
#'   Malmström E-M. Sex-based differences in pain distribution in a cohort of
#'    patients with persistent post-traumatic neck pain. Disability and
#'    Rehabilitation. 2017 Jan 27
"pain"

#' Print a summary of an Euler diagram
#'
#' This function is responsible for printing fits from [euler()] and provides
#' a summary of the fit. Prints a data frame of the original set relationships
#' and the fitted values as well as `diagError` and `stress` statistics.
#'
#' @param x `'euler'` object from [euler()]
#' @param round number of decimal places to round to
#' @param vsep character string to paste in between `euler` objects
#'   when `x` is a nested `euler` object
#' @param ... arguments passed to [base::print.data.frame()]
#'
#' @return Summary statistics of the fitted Euler diagram are printed to
#' screen.
#' @seealso [euler()], [base::print.data.frame()]
#'
#' @export
#' @examples
#' euler(organisms)
print.euler <- function(
  x,
  round = 3,
  vsep = strrep("-", 0.75 * getOption("width")),
  ...
) {
  stopifnot(is.numeric(round), length(round) == 1L, round > 0)

  if (!is.null(attr(x, "groups"))) {
    for (i in seq_along(x)) {
      if (i != 1L && !is.null(vsep)) {
        cat(vsep, "\n")
      }
      cat(names(x)[i], "\n")
      print(x[[i]], round = round, ...)
    }
  } else {
    print(
      round(
        data.frame(
          "original" = x$original.values,
          "fitted" = x$fitted.values,
          "residuals" = x$residuals,
          "regionError" = x$regionError
        ),
        digits = round
      ),
      ...
    )
    cat("\n")
    cat("diagError:", round(x$diagError, digits = round), "\n")
    cat("stress:   ", round(x$stress, digits = round), "\n")
  }
  invisible(x)
}

#' Print a summary of a Venn diagram
#'
#' This function is responsible for printing objects from
#' from [venn()] and provides a simple description of the number of
#' sets and the specifications for the ellipses of the Venn diagram.
#'
#' @param x an object of class `'eulerr_venn'`
#' @param round number of digits to round the ellipse specification to
#' @param vsep character string to paste in between `euler` objects
#'   when `x` is a nested `euler` object
#' @param ... arguments passed to [base::print.data.frame()]
#'
#' @return Summary statistics of the fitted Venn diagram are printed to
#' screen.
#'
#' @seealso [venn()], [base::print.data.frame()]
#'
#' @export
#'
#' @examples
#' venn(organisms)
print.eulerr_venn <- function(
  x,
  round = 3,
  vsep = strrep("-", 0.75 * getOption("width")),
  ...
) {
  stopifnot(is.numeric(round), length(round) == 1L, round > 0)

  if (!is.null(attr(x, "groups"))) {
    for (i in seq_along(x)) {
      if (i != 1L && !is.null(vsep)) {
        cat(vsep, "\n")
      }
      cat(names(x)[i], "\n")
      print(x[[i]], round = round, ...)
    }
  } else {
    table <- x$shapes %||% x$ellipses
    n <- nrow(table)
    cat(n, "set Venn diagram", "\n\n")
    numeric_cols <- vapply(table, is.numeric, logical(1))
    table[numeric_cols] <- lapply(table[numeric_cols], round, digits = round)
    print(table, ...)
  }
  invisible(x)
}

#' @rdname print.eulerr_venn
#' @export
print.venn <- function(...) {
  warning(
    "`print.venn()` is deprecated and will be removed in a future release. ",
    "Use `print()` on objects from `venn()` instead."
  )
  print.eulerr_venn(...)
}

#' Suppress printing
#'
#' @param x object to (not) print
#' @param ... arguments to `x`
#'
#' @return Invisibly returns the output of running print on `x`.
#' @keywords internal
dont_print <- function(x, ...) {
  utils::capture.output(y <- print(x, ...))
  invisible(y)
}

#' Get or set global graphical parameters for eulerr
#'
#' This function provides a means to set default parameters for functions
#' in eulerr. Query [eulerr_options()] (without any
#' argument) to see all the available options and read more about
#' the plot-related ones in [grid::gpar()] and [graphics::par()].
#'
#' Currently, the following items will be considered:
#' \describe{
#'   \item{pointsize}{size in pts to be used as basis for fontsizes and
#'   some margin sizes in the resulting plot}#'
#'   \item{fills}{a list of items `fill` and `alpha`}
#'   \item{patterns}{a list of items `type`, `angle`, `col`, `lwd`, and `alpha`}
#'   \item{edges}{a list of items `col`, `alpha`, `lex`, `lwd`, and `lty`}
#'   \item{labels}{a list of items `rot`,
#'   `col`, `alpha`, `fontsize`, `cex`, `fontfamily`, `fontface`,
#'   `lineheight`, and `font`}
#'   \item{quantities}{a list of items `type`, `template`, `format`, `total`, `rot`,
#'   `col`, `alpha`, `fontsize`, `cex`, `fontfamily`,
#'   `lineheight`, and `font`}
#'   \item{annotations}{a list of items `rot`, `col`, `alpha`, `fontsize`,
#'   `cex`, `fontfamily`, `lineheight`, `font`, and `labels` (a named
#'   character vector keyed by subset name). Used to add a third stacked
#'   text element per region below the quantity.}
#'   \item{strips}{`col`, `alpha`, `fontsize`, `cex`, `fontfamily`,
#'   `lineheight`, and `font`}
#'   \item{legend}{arguments to [grid::legendGrob()] as well as `col`, `alpha`,
#'   `fontsize`, `cex`, `symbol_size` (symbol size multiplier, independent of text
#'   size; defaults to `cex` if `NULL`), `fontfamily`, `lineheight`, and `font`}
#'   \item{main}{arguments to [grid::textGrob()]}
#'   \item{complement}{a list of styling defaults for the container box
#'   and its complement label drawn when [euler()] is called with
#'   `complement =`. Items: `fill`, `alpha`, `col`, `lty`, `lwd`, `lex`,
#'   `fontsize`, `cex`, `font`, `fontfamily`, `lineheight`. The default
#'   `lty = 2` draws the container with a dashed outline.}
#'   \item{padding}{a [grid::unit()] giving the padding between various
#'   elements in plots from [plot.euler()], which you can change
#'   if you, for instance, want to increase spacing between labels,
#'   quantities, and percentages.}
#'   \item{composition}{a list controlling how `eulergram` objects are
#'   arranged when composed via `|` or `/`. Contains a single `spacing`
#'   item (a [grid::unit()]) that sets the gap between adjacent plots.}
#' }
#'
#' @param ... objects to update the global graphical parameters for \pkg{eulerr}
#'   with.
#'
#' @return This function gets or sets updates in the global environment
#'   that are used in [plot.euler()].
#' @export
#' @seealso [plot.euler()], [grid::gpar()], [graphics::par()]
#'
#' @examples
#' eulerr_options(edges = list(col = "blue"), fontsize = 10)
#' eulerr_options(n_threads = 2)
eulerr_options <- function(...) {
  new <- list(...)
  if (is.null(names(new)) && length(new) == 1L && is.list(new[[1L]])) {
    new <- new[[1L]]
  }
  old <- .eulerr_env$options
  if (length(new) == 0L) {
    return(old)
  }
  nm <- names(new)
  if (is.null(nm)) {
    return(old[unlist(new)])
  }
  is_named <- nm != ""
  if (any(!is_named)) {
    nm[!is_named] <- unlist(new[!is_named])
  }
  out <- old[nm]
  names(out) <- nm
  nm <- nm[is_named]
  .eulerr_env$options <- update_list(old, new[nm])
  pointsize <- new$pointsize
  if (!is.null(pointsize)) {
    .eulerr_env$options <- update_list(
      .eulerr_env$options,
      list(
        labels = list(fontsize = pointsize),
        quantities = list(fontsize = pointsize),
        annotations = list(fontsize = pointsize),
        strips = list(fontsize = pointsize),
        legend = list(fontsize = pointsize),
        main = list(fontsize = pointsize),
        pointsize = pointsize
      )
    )
  }
  invisible(out)
}

#' Default options for eulerr
#'
#' @return default options for eulerr
#' @keywords internal
eulerr_default_options <- function() {
  list(
    pointsize = 12,
    fills = list(
      fill = function(n) {
        c(
          "white",
          "grey85",
          "lightblue",
          "lightcoral",
          "lemonchiffon",
          "plum3",
          "aquamarine2",
          "grey55",
          "steelblue2",
          "lightsalmon",
          "lightpink",
          "lightgoldenrod"
        )[seq_len(n)]
      },
      alpha = 1,
      mode = "disjoint"
    ),
    patterns = list(
      type = NA_character_,
      angle = 45,
      col = 1L,
      lwd = 0.5,
      alpha = 1,
      lex = 1,
      mode = "disjoint"
    ),
    edges = list(
      col = 1L,
      alpha = 1,
      lty = 1L,
      lwd = 1,
      lex = 1
    ),
    labels = list(
      rot = 0,
      col = 1L,
      alpha = 1,
      fontsize = 12,
      cex = 1,
      fontfamily = "",
      lineheight = 1.2,
      font = 2,
      placement = "raycast",
      margin = NULL,
      tether = "poi",
      gap = NULL,
      force_directed = list(iterations = NULL),
      elbow = list(min_gap = NULL),
      leader = list(
        col = NULL,
        alpha = 0.6,
        lwd = 1,
        lty = 2,
        lex = 1
      )
    ),
    quantities = list(
      type = "counts",
      template = NULL,
      format = NULL,
      total = NULL,
      rot = 0,
      col = 1L,
      alpha = 1,
      fontsize = 12,
      cex = 1,
      fontfamily = "",
      lineheight = 1.2,
      font = 1
    ),
    annotations = list(
      labels = NULL,
      rot = 0,
      col = 1L,
      alpha = 1,
      fontsize = 12,
      cex = 0.8,
      fontfamily = "",
      lineheight = 1.2,
      font = 1
    ),
    strips = list(
      cex = 1L,
      col = 1L,
      alpha = 1,
      fontsize = 12,
      fontfamily = "",
      lineheight = 1.2,
      rot = 0,
      font = 4
    ),
    legend = list(
      side = "right",
      cex = 1,
      symbol_size = NULL,
      fontsize = 12,
      font = 1,
      fontfamily = "",
      labels = NULL,
      byrow = FALSE,
      do.lines = FALSE,
      lines.first = TRUE,
      hgap = grid::unit(1, "lines"),
      vgap = grid::unit(0.25, "lines"),
      default.units = "lines",
      pch = 21
    ),
    main = list(
      label = NULL,
      x = grid::unit(0.5, "npc"),
      y = grid::unit(0.5, "npc"),
      just = "center",
      hjust = NULL,
      vjust = NULL,
      rot = 0,
      check.overlap = FALSE,
      default.units = "npc",
      cex = 1.5,
      fontsize = 12,
      font = 1,
      fontfamily = "",
      col = 1,
      lineheight = 1.2,
      alpha = 1
    ),
    complement = list(
      fill = "transparent",
      alpha = 1,
      col = 1L,
      lty = 2L,
      lwd = 1,
      lex = 1,
      fontsize = 12,
      cex = 1,
      font = 1,
      fontfamily = "",
      lineheight = 1.2,
      placement = "raycast",
      margin = NULL,
      tether = "poi",
      gap = NULL,
      force_directed = list(iterations = NULL),
      elbow = list(min_gap = NULL),
      leader = list(
        col = NULL,
        alpha = 0.6,
        lwd = 1,
        lty = 2,
        lex = 1
      )
    ),
    padding = grid::unit(0.4, "lines"),
    composition = list(
      spacing = grid::unit(1, "lines")
    )
  )
}

test_that("getting options works", {
  expect_type(eulerr_options(), "list")
})

test_that("setting options works", {
  expect_error(eulerr_options(pointsize = 10), NA)
  expect_silent(eulerr_options(edges = list(col = "blue")))
  expect_silent(eulerr_options(list(fills = list(alpha = 0.5))))
  expect_null(eulerr_options(list("a"))[[1]])
  expect_silent(eulerr_options(eulerr_default_options()))
})

test_that("loading and unloading the package works", {
  expect_error(library(eulerr), NA)
  expect_silent(detach("package:eulerr", character.only = TRUE))
  expect_error(library(eulerr), NA)
})

---
title: "Introducing eulerr"
author: "Johan Larsson"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introducing eulerr}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: eulerr.bib
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

options(digits = 4)
```

## Motivation

**eulerr** generates area-proportional euler diagrams that display set
relationships (intersections, unions, and disjoints) with circles or ellipses.
[Euler diagrams](https://en.wikipedia.org/wiki/Euler_diagram) are Venn
diagrams without the requirement that all set interactions be present (whether
they are empty or not). That is, depending on input, eulerr will sometimes
produce Venn diagrams but sometimes not.

R features several packages that produce Euler diagrams;
some of the more prominent ones (on CRAN) are

* [eVenn](https://cran.r-project.org/package=eVenn),
* [VennDiagram](https://cran.r-project.org/package=VennDiagram)
* [venn](https://cran.r-project.org/package=venn)
* [colorfulVennPlot](https://cran.r-project.org/package=colorfulVennPlot), and
* [venneuler](https://cran.r-project.org/package=venneuler).

The last of these (**venneuler**) was the primary inspiration for this
package, along with the refinements that Fredrickson has presented on his
[blog](http://www.benfrederickson.com/) and made available in his
javascript [venn.js](https://github.com/benfred/venn.js).
[eulerAPE](https://www.eulerdiagrams.org/eulerAPE/), which was the first
program to uses ellipses instead of circles, has also been instrumental in the
design of **eulerr**. The downside to **eulerAPE** is that it only handles
three sets that all need to intersect.

**venneuler**, on the other hand, will take any number of sets (in theory),
yet has been known to produce
[imperfect solutions](http://benfred.github.io/venn.js/tests/venneuler_comparison/)
for set configurations that have perfect such. And unlike **eulerAPE**, it is
restricted to circles (as is **venn.js**).

## Enter eulerr

**eulerr** is based on the improvements to venneuler that Ben Fredrickson
introduced with **venn.js** but has been programmed from scratch, uses different
optimizers, and returns statistics featured in **venneuler** and
**eulerAPE** as well as allows a range of different inputs and conditioning on
additional variables. Moreover, it can model set relationships with ellipses for
any number of sets involved.

## Input 

At the time of writing, it is possible to provide input to **eulerr** as
either

* a named numeric vector with set combinations as disjoint set combinations
or unions (depending on how the argument `type` is set in `euler()`),
* a matrix or data frame of logicals with columns representing sets and rows
the set relationships for each observation,
* a list of sample spaces, or
* a table.

```{r input}
library(eulerr)

# Input in the form of a named numeric vector
fit1 <- euler(c(
  "A" = 25,
  "B" = 5,
  "C" = 5,
  "A&B" = 5,
  "A&C" = 5,
  "B&C" = 3,
  "A&B&C" = 3
))

# Input as a matrix of logicals
set.seed(1)
mat <- cbind(
  A = sample(c(TRUE, TRUE, FALSE), 50, TRUE),
  B = sample(c(TRUE, FALSE), 50, TRUE),
  C = sample(c(TRUE, FALSE, FALSE, FALSE), 50, TRUE)
)
fit2 <- euler(mat)
```

We inspect our results by printing the eulerr object

```{r}
fit2
```

or directly access and plot the residuals.

```{r residual-plot, fig.cap = "Residuals for the fit diagram."}
# Cleveland dot plot of the residuals
dotchart(resid(fit2))
```

We can also use **eulerr**'s built in `error_plot()` function
to diagnose the fit.

```{r error-plot, fig.cap = "A plot from `error_plot()`."}
error_plot(fit2)
```

This shows us that the \code{A\&C} intersection is somewhat overrepresented in
\code{fit2}. Given that these residuals are on the scale of the original
values, however, the residuals are arguably of little concern.

As an alternative, we could plot the circles in another program by retrieving
their coordinates and radii.

```{r coefs}
coef(fit2)
```

### Goodness-of-fit

To tell if we can trust our solution, we use two goodness-of-fit measures: the
stress statistic from **venneuler** [@wilkinson_exact_2012],

$$
\frac{\sum_{i=1}^{n} (y_i - \hat{y}_i)^2}{\sum_{i=1}^{n} y_i ^ 2}
$$

where $\hat{y}_i$ is an ordinary least squares estimate from the regression of
the fitted areas on the original areas that is being explored during
optimization,

and the *diagError* statistic from **eulerAPE** [@Micallef_2014a]:

$$
\max_{i = 1, 2, \dots, n} \left| \frac{y_i}{\sum y_i} -
\frac{\hat{y}_i}{\sum \hat{y}_i} \right|
$$

In our example, the diagError is `r fit2$diag_error` and our
stress is `r fit2$stress`, suggesting that the fit is accurate.

We can now be confident that eulerr provides a reasonable representation of
our input using circles. Were it otherwise, we might try to use ellipses
instead. [@wilkinson_exact_2012] features a difficult combination that it manages
to fit with a reasonably small error; with **eulerr**, however, we can get
rid of that error entirely.

```{r fig.cap = "A difficult combination from Wilkinson 2012."}
wilkinson2012 <- c(
  A = 4,
  B = 6,
  C = 3,
  D = 2,
  E = 7,
  F = 3,
  "A&B" = 2,
  "A&F" = 2,
  "B&C" = 2,
  "B&D" = 1,
  "B&F" = 2,
  "C&D" = 1,
  "D&E" = 1,
  "E&F" = 1,
  "A&B&F" = 1,
  "B&C&D" = 1
)
fit3 <- euler(wilkinson2012, shape = "ellipse")
plot(fit3)
```

If we still lack a good fit after having tried ellipses, we would do best to
stop here and look for another way to visualize our data. (I suggest the
excellent [UpSetR](https://cran.r-project.org/package=UpSetR) package.)

### Visualization

No we get to the fun part: plotting our diagram. This is easy, as well as
highly customizable, with **eulerr**. The default
parameters can easily be adjusted to suit anybody's
needs.

```{r eulerr-plot, fig.cap = "Customizing Euler plots is a breeze in eulerr."}
plot(fit2)

# Remove fills, vary borders, display quantities, and switch font.
plot(
  fit2,
  quantities = TRUE,
  fill = "transparent",
  lty = 1:3,
  labels = list(font = 4)
)
```

Plotting is provided through a custom plotting method built on top of the
excellent facilities made available by the core R package **grid**.
**eulerr**'s default color palette is chosen to be color deficiency-friendly.

## Acknowledgements

**eulerr** would not be possible without Ben Fredrickson's work on
**venn.js** or Leland Wilkinson's **venneuler**.

## References

---
title: "Introducing eulerr"
author: "Johan Larsson"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introducing eulerr}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: eulerr.bib
---

```{r}
#| label: setup
#| include: false

knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

options(digits = 4)
```

## Motivation

**eulerr** generates area-proportional euler diagrams that display set
relationships (intersections, unions, and disjoints) with circles, ellipses,
or axis-aligned rectangles and squares.
[Euler diagrams](https://en.wikipedia.org/wiki/Euler_diagram) are Venn diagrams
without the requirement that all set interactions be present (whether they are
empty or not). That is, depending on input, eulerr will sometimes produce Venn
diagrams but sometimes not.

R features several packages that produce Euler diagrams; some of the more
prominent ones (on CRAN) are

- [eVenn](https://cran.r-project.org/package=eVenn),
- [VennDiagram](https://cran.r-project.org/package=VennDiagram)
- [venn](https://cran.r-project.org/package=venn)
- [colorfulVennPlot](https://cran.r-project.org/package=colorfulVennPlot), and
- [venneuler](https://cran.r-project.org/package=venneuler).

The last of these (**venneuler**) was the primary inspiration for this package,
along with the refinements that Fredrickson has presented on his
[blog](http://www.benfrederickson.com/) and made available in his javascript
[venn.js](https://github.com/benfred/venn.js).
[eulerAPE](https://www.eulerdiagrams.com/eulerAPE/), which was the first program
to uses ellipses instead of circles, has also been instrumental in the design of
**eulerr**. The downside to **eulerAPE** is that it only handles three sets that
all need to intersect.

**venneuler**, on the other hand, will take any number of sets (in theory), yet
has been known to produce [imperfect
solutions](http://benfred.github.io/venn.js/tests/venneuler_comparison/) for set
configurations that have perfect such. And unlike **eulerAPE**, it is restricted
to circles (as is **venn.js**).

## Enter eulerr

**eulerr** is based on the improvements to venneuler that Ben Fredrickson
introduced with **venn.js** but has been programmed from scratch, uses different
optimizers, and returns statistics featured in **venneuler** and **eulerAPE** as
well as allows a range of different inputs and conditioning on additional
variables. Moreover, it can model set relationships with ellipses for any number
of sets involved.

## Input

At the time of writing, it is possible to provide input to **eulerr** as either

- a named numeric vector with set combinations as disjoint set combinations or
  unions (depending on how the argument `type` is set in `euler()`),
- a matrix or data frame of logicals with columns representing sets and rows the
  set relationships for each observation,
- a list of sample spaces, or
- a table.

```{r}
#| label: input

library(eulerr)

# Input in the form of a named numeric vector
fit1 <- euler(c(
  "A" = 25,
  "B" = 5,
  "C" = 5,
  "A&B" = 5,
  "A&C" = 5,
  "B&C" = 3,
  "A&B&C" = 3
))

# Input as a matrix of logicals
set.seed(1)
mat <- cbind(
  A = sample(c(TRUE, TRUE, FALSE), 50, TRUE),
  B = sample(c(TRUE, FALSE), 50, TRUE),
  C = sample(c(TRUE, FALSE, FALSE, FALSE), 50, TRUE)
)
fit2 <- euler(mat)
```

We inspect our results by printing the eulerr object

```{r}
fit2
```

or directly access and plot the residuals.

```{r}
#| label: residual-plot
#| fig-cap: "Residuals for the fit diagram."

# Cleveland dot plot of the residuals
dotchart(resid(fit2))
```

We can also use **eulerr**'s built in `error_plot()` function to diagnose the
fit.

``{r error-plot, fig.cap = "A plot from `error_plot()`."} error_plot(fit2)``

This shows us that the \code{A\&C} intersection is somewhat overrepresented in
\code{fit2}. Given that these residuals are on the scale of the original values,
however, the residuals are arguably of little concern.

As an alternative, we could plot the circles in another program by retrieving
their coordinates and radii.

```{r}
#| label: coefs

coef(fit2)
```

### Goodness-of-fit

To tell if we can trust our solution, we use two goodness-of-fit measures: the
stress statistic from **venneuler** [@wilkinson_exact_2012],

$$
\frac{\sum_{i=1}^{n} (y_i - \hat{y}_i)^2}{\sum_{i=1}^{n} y_i ^ 2}
$$

where $\hat{y}_i$ is an ordinary least squares estimate from the regression of
the fitted areas on the original areas that is being explored during
optimization,

and the *diagError* statistic from **eulerAPE** [@Micallef_2014a]:

$$
\max_{i = 1, 2, \dots, n} \left| \frac{y_i}{\sum y_i} -
\frac{\hat{y}_i}{\sum \hat{y}_i} \right|
$$

In our example, the diagError is `r fit2$diag_error` and our stress is
`r fit2$stress`, suggesting that the fit is accurate.

We can now be confident that eulerr provides a reasonable representation of our
input using circles. Were it otherwise, we might try a different shape via the
`shape` argument — `"ellipse"` is the most expressive choice, but axis-aligned
`"rectangle"` and `"square"` fits are also available.
[@wilkinson_exact_2012] features a difficult combination that ellipses manage
to fit with a reasonably small error; with **eulerr**, however, we can get rid
of that error entirely.

```{r}
#| fig-cap: "A difficult combination from Wilkinson 2012."

wilkinson2012 <- c(
  A = 4,
  B = 6,
  C = 3,
  D = 2,
  E = 7,
  F = 3,
  "A&B" = 2,
  "A&F" = 2,
  "B&C" = 2,
  "B&D" = 1,
  "B&F" = 2,
  "C&D" = 1,
  "D&E" = 1,
  "E&F" = 1,
  "A&B&F" = 1,
  "B&C&D" = 1
)
fit3 <- euler(wilkinson2012, shape = "ellipse")
plot(fit3)
```

If we still lack a good fit after having tried ellipses, we would do best to
stop here and look for another way to visualize our data. (I suggest the
excellent [UpSetR](https://cran.r-project.org/package=UpSetR) package.)

### Visualization

No we get to the fun part: plotting our diagram. This is easy, as well as highly
customizable, with **eulerr**. The default parameters can easily be adjusted to
suit anybody's needs.

```{r}
#| label: eulerr-plot
#| fig-cap: "Customizing Euler plots is a breeze in eulerr."

plot(fit2)

# Switch to squares for the same set combination, customizing borders,
# labels, and quantities. The `shape` argument also accepts `"ellipse"`
# and `"rectangle"`.
plot(
  euler(mat, shape = "square"),
  quantities = TRUE,
  fill = "transparent",
  lty = 1:3,
  labels = list(font = 4)
)
```

Plotting is provided through a custom plotting method built on top of the
excellent facilities made available by the core R package **grid**. **eulerr**'s
default color palette is chosen to be color deficiency-friendly.

## Acknowledgements

**eulerr** would not be possible without Ben Fredrickson's work on **venn.js**
or Leland Wilkinson's **venneuler**.

## References

---
title: "A Gallery of Euler and Venn Diagrams"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{A Gallery of Euler and Venn Diagrams}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| include: false
#| label: setup-options

knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.align = "center",
  fig.height = 4,
  fig.width = 5
)
```

```{r}
#| label: setup

library(eulerr)
```

This vignette serves as a gallery for Euler diagrams and as a showcase of the
various options for customization that are available.

## Euler Diagrams

### Uniform intersections

Here we use squares instead of the default circles.

```{r}
#| label: uniform_intersections

uniform_intersections <- euler(
  c(
    "A" = 10,
    "B" = 10,
    "C" = 10,
    "A&B" = 4,
    "A&C" = 4,
    "B&C" = 4,
    "A&B&C" = 2
  ),
  shape = "square"
)
plot(uniform_intersections)
```

### Disjoint sets

```{r}
#| label: disjoint

disjoint_sets <- euler(c(A = 1, B = 1, C = 1))
plot(
  disjoint_sets,
  labels = c("Tom", "Greg", "Alberta"),
  edges = list(lty = 1:3)
)
```

### A set contained in the intersection of two sets

```{r}
#| label: completely_contained

completely_contained <- euler(c(
  "A" = 15,
  "B" = 15,
  "C" = 0,
  "A&B" = 3,
  "A&C" = 0,
  "B&C" = 0,
  "A&B&C" = 3
))
plot(
  completely_contained,
  labels = list(col = c("white", "black", "black")),
  edges = list(col = "white", lex = 2),
  fills = c("black", "cyan", "orange")
)
```

### Two sets intersecting inside a third

```{r}
#| label: intersecting_inside

intersecting_inside <- euler(c(
  "A" = 15,
  "B" = 0,
  "C" = 0,
  "A&B" = 3,
  "A&C" = 3,
  "B&C" = 0,
  "A&B&C" = 2
))
plot(
  intersecting_inside,
  fills = list(
    fill = c(
      "lavenderblush2",
      "lightblue2",
      "lightsalmon",
      "orange",
      "steelblue4",
      "white",
      "plum2"
    )
  ),
  legend = list(side = "right")
)
```

### Difficult set (for circles!)

```{r}
#| label: one_contained

one_contained <- euler(
  c("A" = 7, B = 6, C = 0, "A&B" = 0, "A&C" = 1, "B&C" = 1, "A&B&C" = 2),
  shape = "ellipse"
)
plot(one_contained, quantities = list(type = "percent"))
```

### Russian doll

Sets intersecting inside other sets.

```{r}
#| label: russian_doll

russian_doll <- euler(c(
  "A" = 15,
  "B" = 0,
  C = 0,
  "A&B" = 10,
  "A&C" = 0,
  "B&C" = 0,
  "A&B&C" = 5
))
plot(russian_doll)
```

### Wilkinson set relationship

This set relationship is taken from Wilkinson et al. It works best with
ellipses.

```{r}
#| label: wilkinson

wilkinson <- euler(
  c(
    A = 4,
    B = 6,
    C = 3,
    D = 2,
    E = 7,
    F = 3,
    "A&B" = 2,
    "A&F" = 2,
    "B&C" = 2,
    "B&D" = 1,
    "B&F" = 2,
    "C&D" = 1,
    "D&E" = 1,
    "E&F" = 1,
    "A&B&F" = 1,
    "B&C&D" = 1
  ),
  shape = "ellipse"
)
plot(
  wilkinson,
  labels = list(fontfamily = "serif"),
  edges = list(lty = 3),
  quantities = list(type = "percent", font = 3)
)
```

### Gene set

```{r}
#| label: gene_set

genes <- euler(c(
  "SE" = 13,
  "Treat" = 28,
  "Anti-CCP" = 101,
  "DAS28" = 91,
  "SE&Treat" = 1,
  "SE&DAS28" = 14,
  "Treat&Anti-CCP" = 6,
  "SE&Anti-CCP&DAS28" = 1
))
plot(genes, quantities = list(type = c("percent", "counts")))
```

### Three sets intersecting inside a fourth

```{r}
#| label: three_inside_fourth

three_inside_fourth <- euler(c(
  "A" = 30,
  "A&B" = 3,
  "A&C" = 3,
  "A&D" = 3,
  "A&B&C" = 0.4,
  "A&B&D" = 0.4,
  "A&C&D" = 0.4,
  "A&B&C&D" = 1
))
plot(three_inside_fourth)
```

### eulerAPE combination

A combination taken from the eulerAPE article.

```{r}
#| label: eulerape

eulerape <- euler(
  c(
    "a" = 3491,
    "b" = 3409,
    "c" = 3503,
    "a&b" = 120,
    "a&c" = 114,
    "b&c" = 132,
    "a&b&c" = 126
  ),
  shape = "ellipse",
  control = list(extraopt = FALSE)
)
plot(eulerape)
```

### Four uniform interactions

This time we fit the diagram using rectangles.

```{r}
#| label: four_uniform

uniform <- euler(
  c(
    "A" = 10,
    "B" = 10,
    "C" = 10,
    "D" = 10,
    "A&B" = 3,
    "A&C" = 3,
    "A&D" = 0,
    "B&C" = 0,
    "B&D" = 3,
    "C&D" = 3,
    "A&B&C" = 1,
    "A&B&D" = 1,
    "A&C&D" = 1,
    "B&C&D" = 1,
    "A&B&C&D" = 1
  ),
  shape = "rectangle"
)
plot(
  uniform,
  labels = list(
    labels = c("Frodo", "Sam", "Merry", "Pippin"),
    font = 1:4,
    col = 1:4,
    cex = seq(1, 1.5, length.out = 4)
  )
)
```

### Two circles intersecting completely

```{r}
#| label: two_overlapping

two_overlapping <- euler(c("A" = 0, "B" = 0, "A&B" = 10))
plot(two_overlapping)
```

---
title: "A Gallery of Euler and Venn Diagrams"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{A Gallery of Euler and Venn Diagrams}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.align = "center",
  fig.height = 4,
  fig.width = 5
)
```

```{r setup}
library(eulerr)
```

This vignette serves as a gallery for Euler diagrams and
as a showcase of the various options for customization that are available.

## Euler Diagrams

### Uniform intersections

```{r}
uniform_intersections <- euler(c("A" = 10, "B" = 10, "C" = 10,
                                 "A&B" = 4, "A&C" = 4, "B&C" = 4,
                                 "A&B&C" = 2))
plot(uniform_intersections)
```

### Disjoint sets

```{r}
disjoint_sets <- euler(c(A = 1, B = 1, C = 1))
plot(disjoint_sets, 
     labels = c("Tom", "Greg", "Alberta"),
     edges = list(lty = 1:3))
```

### A set contained in the intersection of two sets

```{r}
completely_contained <- euler(c("A" = 15, "B" = 15, "C" = 0,
                                "A&B" = 3, "A&C" = 0, "B&C" = 0,
                                "A&B&C" = 3))
plot(completely_contained,
     labels = list(col = c("white", "black", "black")),
     edges = list(col = "white", lex = 2),
     fills = c("black", "cyan", "orange"))
```

### Two sets intersecting inside a third

```{r}
intersecting_inside <- euler(c("A" = 15, "B" = 0, "C" = 0,
                               "A&B" = 3, "A&C" = 3, "B&C" = 0,
                                "A&B&C" = 2))
plot(intersecting_inside,
     fills = list(fill = c("lavenderblush2", 
                           "lightblue2", 
                           "lightsalmon",
                           "", 
                           "", 
                           "",
                           "plum2")),
     legend = list(side = "right"))
```

### Difficult set (for circles!)

```{r}
one_contained <- euler(c("A" = 7, B = 6, C = 0, 
                         "A&B" = 0, "A&C" = 1, "B&C" = 1,
                         "A&B&C" = 2),
                       shape = "ellipse")
plot(one_contained,
     quantities = list(type = "percent"))
```

### Russian doll

Sets intersecting inside other sets. 

```{r}
russian_doll <- euler(c("A" = 15, "B" = 0, C = 0, 
                        "A&B" = 10, "A&C" = 0, "B&C" = 0,
                        "A&B&C" = 5))
plot(russian_doll)
```

### Wilkinson set relationship

This set relationship is taken from Wilkinson et al. It works best
with ellipses.

```{r}
wilkinson <- euler(c(A = 4, B = 6, C = 3, D = 2, E = 7, F = 3,
                     "A&B" = 2, "A&F" = 2, "B&C" = 2, "B&D" = 1,
                     "B&F" = 2, "C&D" = 1, "D&E" = 1, "E&F" = 1,
                     "A&B&F" = 1, "B&C&D" = 1),
                   shape = "ellipse")
plot(wilkinson, 
     labels = list(fontfamily = "serif"),
     edges = list(lty = 3),
     quantities = list(type = "percent", 
                       font = 3))
```

### Gene set

```{r}
genes <- euler(c("SE" = 13, "Treat" = 28, "Anti-CCP" = 101, "DAS28" = 91,
                 "SE&Treat" = 1, "SE&DAS28" = 14, "Treat&Anti-CCP" = 6,
                 "SE&Anti-CCP&DAS28" = 1))
plot(genes,
     quantities = list(type = c("percent", "counts")))
```

### Three sets intersecting inside a fourth

```{r}
three_inside_fourth <- euler(c("A" = 30,
                               "A&B" = 3, "A&C" = 3, "A&D" = 3,
                               "A&B&C" = 2, "A&B&D" = 2, "A&C&D" = 2,
                               "A&B&C&D" = 1))
plot(three_inside_fourth)
```


### eulerAPE combination

A combination taken from the eulerAPE article.

```{r}
eulerape <- euler(c("a" = 3491, "b" = 3409, "c" = 3503,
                    "a&b" = 120, "a&c" = 114, "b&c" = 132,
                    "a&b&c" = 126),
                  shape = "ellipse",
                  control = list(extraopt = FALSE))
plot(eulerape)
```

### Four uniform interactions

```{r}
uniform <- euler(c("A" = 10, "B" = 10, "C" = 10, "D" = 10,
                   "A&B" = 3, "A&C" = 3, "A&D" = 0, "B&C" = 0, "B&D" = 3, "C&D" = 3,
                   "A&B&C" = 1, "A&B&D" = 1, "A&C&D" = 1, "B&C&D" = 1,
                   "A&B&C&D" = 1))
plot(uniform, labels = list(labels = c("Frodo", "Sam", "Merry", "Pippin"),
                            font = 1:4,
                            col = 1:4,
                            cex = seq(1, 1.5, length.out = 4)))
```


### Two circles intersecting completely

```{r}
two_overlapping <- euler(c("A" = 0, "B" = 0, "A&B" = 10))
plot(two_overlapping)
```

---
title: "Comparison with other packages"
author: "Johan Larsson"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Comparison with other packages}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: eulerr.bib
---

```{r}
#| label: setup
#| include: false

knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

library(eulerr)

# The benchmark numbers are precomputed by `data-raw/benchmarks.R` (which needs
# the competitor packages and is not part of the package build) and stored in
# `benchmark_results.rds`. This vignette only reads and displays them, so it
# builds with eulerr, knitr, and lattice alone.
results_path <- "benchmark_results.rds"
have_results <- file.exists(results_path)
if (have_results) {
  results <- readRDS(results_path)
}
```

This vignette compares **eulerr** with the other R packages that genuinely fit
*area-proportional* Euler and Venn diagrams, both quantitatively (accuracy and
speed) and qualitatively (features and scope).

## What counts as a competitor

An area-proportional diagram is one in which each region's *area* is made
proportional to the quantity it represents. Surprisingly few R packages actually
solve this geometric fitting problem. Most Venn/Euler packages draw fixed,
schematic shapes and encode quantities through labels or color instead of area.

On CRAN and Bioconductor, the packages that really fit area-proportional
diagrams are:

- **eulerr** --- circles, ellipses, or axis-aligned rectangles/squares via
  numerical optimization; reports `stress` and `diagError` goodness-of-fit
  statistics.
- **venneuler** --- Wilkinson's circle algorithm [@wilkinson_exact_2012];
  circles only; Java-based (depends on rJava).
- **nVennR** --- the nVenn algorithm [@Perez-Silva_2018]; quasi-proportional,
  n-dimensional diagrams built from irregular polygons. Distributed via GitHub.
- **BioVenn** --- accurate 2--3 circle diagrams from identifier lists
  [@Hulsen_2021].
- **vennplot** --- 2D circles and 3D spheres for 2--3 sets [@Xu_2017]; currently
  dormant on CRAN.
- **VennDiagram** --- a drawing package whose *scaled* mode is genuinely
  area-proportional for two circles only [@Chen_2011].

The [last section](#excluded-packages) lists the many packages that are *not*
area-proportional fitters and explains why they are excluded.

## How accuracy is measured

Comparing fitters fairly is subtle because they optimize different objectives. A
package that minimizes overall squared error will look bad if scored on the
single worst region, and vice versa. We therefore compare **per objective**: for
each competitor we configure eulerr to optimize the *same* objective and score
both packages on the *same* metric.

To make the metric identical across packages, we ignore each package's
self-reported diagnostics and instead recompute the fit from its *realized
geometry*. We turn the fitted shapes into polygons, compute the area of every
disjoint region with `polyclip`, and evaluate eulerr's own statistics on those
areas:

- **stress** --- the normalized residual sum of squares from venneuler,
  $\sum_i (A_i - \beta\omega_i)^2 \big/ \sum_i A_i^2$, with
  $\beta = \sum_i A_i\omega_i \big/ \sum_i \omega_i^2$.
- **diagError** --- the largest absolute difference between a region's realized
  and target *proportion*, $\max_i |A_i/\sum_k A_k - \omega_i/\sum_k \omega_k|$
  (from eulerAPE, [@Micallef_2014]).

Here $\omega_i$ is the input size of region $i$ and $A_i$ its realized area.
Both statistics are scale-invariant, so packages that work in different
coordinate systems remain comparable. We benchmark on a mix of small hand-built
configurations and harder cases derived from eulerr's bundled `fruits` and
`organisms` datasets, plus a five-set example.

The three comparisons are:

  | Comparison | Competitor optimizes         | eulerr setting                 | Scored on |
  | ---------- | ---------------------------- | ------------------------------ | --------- |
  | A          | overall fit (stress)         | `loss = "stress"`, circles     | stress    |
  | B          | region proportionality       | `loss = "diag_error"`, circles | diagError |
  | C          | overlap areas (2--3 circles) | `loss = "stress"`, circles     | stress    |

In each comparison eulerr's default **ellipse** fit is included as a reference
(`eulerr (ellipse)`): ellipses have more degrees of freedom than circles and
typically fit better, but the head-to-head against each circle-based competitor
uses eulerr's circle mode.

```{r}
#| label: helpers
#| include: false

# Reshape a long accuracy/timing slice to a dataset-by-package matrix, dropping
# competitor columns that were never run (all NA) so the scaffolded state (only
# eulerr available) still produces a clean table.
to_matrix <- function(df, value_col) {
  m <- tapply(
    df[[value_col]],
    list(df$dataset, df$package),
    FUN = function(x) x[1]
  )
  m <- m[, colSums(!is.na(m)) > 0, drop = FALSE]
  # Order datasets as first seen, eulerr columns first.
  m <- m[unique(df$dataset), , drop = FALSE]
  pkg_order <- order(!grepl("^eulerr", colnames(m)), colnames(m))
  m[, pkg_order, drop = FALSE]
}

fmt <- function(x) {
  ifelse(is.na(x), "---", formatC(x, format = "g", digits = 2))
}
```

```{r}
#| label: no-results
#| echo: false
#| results: asis

if (!have_results) {
  cat(
    "> **Benchmark results are not available.** Run",
    "`Rscript data-raw/benchmarks.R` to generate `benchmark_results.rds`,",
    "then rebuild this vignette.\n"
  )
}
```

## Accuracy

```{r}
#| label: accuracy-note
#| echo: false
#| results: asis
#| eval: have_results

run <- results$meta$competitors_run
if (length(run) == 0) {
  cat(
    "> The stored results were generated **without any competitor package",
    "installed**, so only eulerr's own numbers are shown below. Install",
    "venneuler, BioVenn, and nVennR and re-run `data-raw/benchmarks.R` for the",
    "full comparison.\n"
  )
} else {
  cat(
    "> Competitors benchmarked:",
    paste(run, collapse = ", "),
    paste0("(generated ", results$meta$generated, ").\n")
  )
}
```

```{r}
#| label: accuracy-tables
#| echo: false
#| results: asis
#| eval: have_results

acc <- results$accuracy
for (cmp in unique(acc$comparison)) {
  sub <- acc[acc$comparison == cmp, ]
  metric <- sub$metric[1]
  m <- to_matrix(sub, "value")
  cat("\n**", cmp, "** --- lower ", metric, " is better.\n\n", sep = "")
  print(knitr::kable(
    apply(m, 2, fmt),
    align = "r",
    caption = NULL
  ))
  cat("\n")
}
```

```{r}
#| label: accuracy-plot
#| echo: false
#| fig-width: 7
#| fig-height: 3
#| fig-cap: "Fit error by dataset and package, faceted by comparison. Lower is better. Each competitor is matched against eulerr configured for the same objective."
#| eval: have_results

library(lattice)

acc <- results$accuracy
acc$package <- factor(acc$package, levels = unique(acc$package))

lattice::barchart(
  value ~ dataset | comparison,
  groups = package,
  data = acc,
  scales = list(x = list(rot = 45), y = list(relation = "free")),
  layout = c(3, 1),
  ylab = "fit error",
  auto.key = list(columns = 1, space = "right"),
  par.settings = lattice::simpleTheme(col = eulerr_options()$fills$fill(8))
)
```

## Speed

Runtimes are the median wall-clock time of repeated fits on the machine that
generated the results (see `sessionInfo` in the stored `meta`). They are
indicative rather than definitive --- venneuler pays a fixed JVM cost, and
absolute numbers depend on hardware --- but they show the broad picture.

```{r}
#| label: timing-table
#| echo: false
#| results: asis
#| eval: have_results

tim <- results$timing
# Average eulerr-circle / competitor timings across datasets for a compact view.
m <- to_matrix(tim, "time_ms")
cat("Median fit time (ms), by dataset and package:\n\n")
knitr::kable(
  apply(m, 2, function(x) formatC(x, format = "f", digits = 1)),
  align = "r"
)
```

```{r}
#| label: timing-plot
#| echo: false
#| fig-width: 7
#| fig-height: 3
#| fig-cap: "Median fit time (ms, log scale) by dataset and package."
#| eval: have_results

tim <- results$timing
tim$package <- factor(tim$package, levels = unique(tim$package))

lattice::barchart(
  time_ms ~ dataset,
  groups = package,
  data = tim[!duplicated(tim[c("dataset", "package")]), ],
  scales = list(x = list(rot = 45), y = list(log = 10)),
  ylab = "median time (ms)",
  auto.key = list(columns = 1, space = "right"),
  par.settings = lattice::simpleTheme(col = eulerr_options()$fills$fill(8))
)
```

## Feature comparison

Accuracy and speed are only part of the story. The table below summarizes how
the packages differ in scope and capabilities.

```{r}
#| label: qualitative-table
#| echo: false

qual <- data.frame(
  Package = c(
    "eulerr",
    "venneuler",
    "nVennR",
    "BioVenn",
    "vennplot",
    "VennDiagram"
  ),
  Shapes = c(
    "circle, ellipse, rectangle, square",
    "circle",
    "irregular polygon",
    "circle",
    "circle (2D), sphere (3D)",
    "circle"
  ),
  `Max sets` = c("many", "many", "many", "2--3", "2--3", "4 (2 scaled)"),
  Proportional = c(
    "approximate",
    "approximate",
    "quasi",
    "accurate (2--3)",
    "approximate",
    "2 sets only"
  ),
  Input = c(
    "vectors, data frames, matrices, tables, lists",
    "named vector, data frame",
    "lists",
    "ID lists",
    "counts, lists",
    "counts"
  ),
  `Fit reported` = c(
    "stress, diagError, regionError",
    "stress",
    "none",
    "none",
    "none",
    "none"
  ),
  `Key dependency` = c(
    "none (Rust)",
    "rJava / Java",
    "C++",
    "none",
    "Rcpp, rgl",
    "none"
  ),
  Source = c(
    "CRAN",
    "CRAN",
    "GitHub",
    "CRAN",
    "CRAN (dormant)",
    "CRAN"
  ),
  check.names = FALSE,
  stringsAsFactors = FALSE
)

knitr::kable(qual, align = "l")
```

A few points worth drawing out:

- **eulerr is the only package that fits ellipses** (and rectangles/squares).
  Ellipses can represent many three-set configurations exactly that are
  impossible with circles, which is why eulerr's ellipse fits score better than
  every circle-based package on the harder datasets above.
- **eulerr and venneuler are the only packages that report a numeric
  goodness-of-fit.** Without one, it is impossible to know how much a diagram
  distorts the data --- and area-proportional diagrams with three or more sets
  almost always distort it to some degree.
- **eulerr has no system dependencies beyond a Rust toolchain at build time**,
  whereas venneuler requires a working Java installation through rJava, which is
  a common source of installation trouble.
- **nVennR achieves closer proportionality only by abandoning smooth shapes**
  for irregular polygons. This is the central trade-off: smooth, interpretable
  shapes (eulerr, venneuler) versus exact topology with jagged regions (nVennR).

## Excluded packages {#excluded-packages}

Many widely cited Venn/Euler packages are *not* area-proportional fitters and
are therefore outside the scope of this comparison.

```{r}
#| label: excluded-table
#| echo: false

excl <- data.frame(
  Package = c(
    "venn, ggVennDiagram, ggvenn, RVenn, gplots",
    "UpSetR",
    "colorfulVennPlot",
    "VennMaster",
    "Vennerable"
  ),
  Reason = c(
    "Draw fixed, schematic shapes; quantity shown via labels or color, not area",
    "Not a Venn/Euler diagram at all (UpSet matrix/bar charts)",
    "Archived from CRAN; only a 2-set helper, never a general fitter",
    "Area-proportional, but a standalone Java application --- not an R package",
    "Area-weighted, but hosted on R-Forge/GitHub, not CRAN or Bioconductor"
  ),
  check.names = FALSE,
  stringsAsFactors = FALSE
)

knitr::kable(excl, align = "l")
```

## Caveats

- **"Area-proportional" is a spectrum.** For three or more sets, exact solutions
  with circles or ellipses are often geometrically impossible, so eulerr and
  venneuler produce *approximate* diagrams; nVennR reaches closer
  proportionality only by giving up smooth shapes. Any honest comparison should
  report fit error, not merely whether a diagram was produced --- which is
  exactly what the accuracy section above does.
- **Self-description bias.** The BioVenn documentation claims it is "the only R
  package that can automatically generate an accurate area-proportional Venn
  diagram," which is not literally true; eulerr, venneuler, and vennplot also
  produce accurate area-proportional circle diagrams.
- **Reproducibility.** The numbers here come from one machine and one set of
  package versions. To regenerate them, install the competitors (`venneuler`,
  `BioVenn`, and `nVennR` from GitHub) and run `data-raw/benchmarks.R`.

## References

---
title: "Visualizing Euler diagrams with eulerr"
author: "Johan Larsson"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Visualizing Euler diagrams with eulerr}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: eulerr.bib
---

```{r}
#| label: setup
#| include: false

knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

library(eulerr)
```

Once we have ascertained that our Euler diagram fits well, we can turn to
visualizing the solution. For this purpose, **eulerr** relies on the **grid**
graphics system [@RCT_2017] and offers intuitive and granular control over the
output.

Plotting the ellipses is straightforward using the parametrization of a rotated
ellipse,

$$
\begin{bmatrix}
x \\ y
\end{bmatrix} =
\begin{bmatrix}
h + a \cos{\theta} \\
k + b \sin{\theta}
\end{bmatrix},
$$

where $\theta \in [0, 2\pi],\quad a,b>0$.

Most users will also prefer to label the ellipses and their intersections with
text and this, however, is considerably more involved.

### Labeling

Labeling the ellipses is complicated since the shapes of the intersections often
are irregular, lacking well-defined centers; we know of no analytical solution
to this problem. Instead, **eulerr** uses the *pole of inaccessibility*
algorithm [@mapbox2018] (originally from *Mapbox*'s **polylabel** C++ library),
implemented in the **eunoia** Rust crate that powers eulerr's geometry layer.

When a label is too large to fit inside its region's polygon, **eulerr** falls
back to one of three exterior-placement strategies, all delegated to **eunoia**.
The strategy is selected via `labels$placement`:

`"raycast"` (the default)
:   labels are placed along the ray from the diagram centroid through the
    region's pole of inaccessibility, padded away from the diagram by a
    per-region margin. Closed-form, fast, deterministic.

`"force_directed"`
:   initial positions come from the raycast geometry, then a damped
    spring-and-repulsion solver relaxes the labels against each other *and*
    against foreign region polygons. Useful when several labels would otherwise
    collide or drift across unrelated regions.

`"elbow"`
:   labels are assigned to a left or right column outside the diagram, stacked
    vertically without overlap and reached by an orthogonal three-segment
    ("elbow") leader. Reminiscent of d3-pie style callouts.

In every case, a leader line is drawn from the source region to the exterior
label; tether and leader styling are controlled via `labels$tether`,
`labels$gap`, and `labels$leader`.

```{r}
#| label: placement-comparison

set.seed(1)
# A chain of four sets with long names and very small overlaps — the
# overlap regions are too narrow to fit their quantity labels inside,
# so all three strategies push the overlap labels to the exterior.
fit <- euler(c(
  Animalia = 8,
  Plantae = 7,
  Fungi = 6,
  Bacteria = 5,
  "Animalia&Plantae" = 0.05,
  "Plantae&Fungi" = 0.05,
  "Fungi&Bacteria" = 0.05,
  "Animalia&Bacteria" = 0.05
))
```

```{r}
#| label: placement-raycast
#| fig-width: 3
#| fig-height: 3
#| fig-cap: The raycast strategy places labels along rays from the diagram
#|   centroid, padded away from the diagram by a per-region margin.

plot(
  fit,
  quantities = TRUE,
  labels = list(placement = "raycast"),
  main = "raycast"
)
```

```{r}
#| label: placement-force-directed
#| fig-width: 3
#| fig-height: 3
#| fig-cap: The force-directed strategy relaxes labels against each other and
#|   against foreign region polygons, which is useful when several labels would
#|   otherwise collide or drift across unrelated regions.

plot(
  fit,
  quantities = TRUE,
  labels = list(placement = "force_directed"),
  main = "force-directed"
)
```

```{r}
#| label: placement-elbow
#| fig-width: 3
#| fig-height: 3
#| fig-cap: The elbow strategy stacks exterior labels in left/right columns
#|   reached by an orthogonal three-segment leader, in the style of d3-pie
#|   callouts.

plot(
  fit,
  quantities = TRUE,
  labels = list(placement = "elbow"),
  main = "elbow"
)
```

Strategy-specific knobs live in their own sublists. The force-directed solver
takes `labels$force_directed = list(iterations = ...)` and the elbow placement
takes `labels$elbow = list(margin = ..., min_gap = ...)`, where `min_gap` sets
the minimum vertical centre-to-centre spacing between stacked label boxes in a
column:

```{r}
#| label: elbow-min-gap
#| fig-cap: "Elbow leaders with extra vertical breathing room between labels"
#| fig-width: 3
#| fig-height: 3

plot(
  fit,
  quantities = TRUE,
  labels = list(
    placement = "elbow",
    elbow = list(min_gap = 1)
  )
)
```

### Aesthetics

Euler diagrams display both quantitative and qualitative data. The quantitative
aspect is the quantities or sizes of the sets depicted in the diagram and is
visualized by the relative sizes, and possibly the labels, of the areas of the
shapes---this is the main focus of this paper. The qualitative aspects,
meanwhile, consist of the mapping of each set to some quality or category, such
as having a certain gene or not. In the diagram, these qualities can be
separated through any of the following aesthetics:

- color,
- border type,
- text labeling,
- transparency,
- patterns,

or a combination of these. The main purpose of these aesthetics is to separate
out the different ellipses so that the audience may interpret the diagram with
ease and clarity.

Among these aesthetics, the best choice (from a viewer perspective) appears to
be color [@Blake_2016], which provides useful information without extraneous
chart junk.

The issue with color, however, is that it cannot be perceived perfectly by all.
Eight percent of men and 0.4% of women in European Caucasian countries, for
instance, suffer the most common form, red--green color deficiency. Moreover,
color is often printed at a premium in scientific publications and adds no
utility to a diagram of two shapes.

For these reasons, **eulerr** defaults to distinguishing ellipses with color
using a manually tuned color palette.

```{r}
#| label: colorexamle
#| fig-cap: "The eight first colors of the default color palette"

set.seed(2)
con <- c(
  A = 1,
  B = 1,
  C = 1,
  D = 1,
  E = 1,
  F = 1,
  G = 1,
  H = 1,
  "A&B" = 0.2,
  "B&C" = 0.2,
  "C&D" = 0.2,
  "D&E" = 0.2,
  "E&F" = 0.2,
  "F&G" = 0.2,
  "G&H" = 0.2
)
plot(euler(con), labels = as.character(1:8))
```

## Composing diagrams

Several Euler diagrams can be arranged into a single figure using operator
syntax inspired by the **patchwork** package. The `|` operator place plots side
by side, while `/` stacks them vertically.

```{r}
#| label: composition-basic
#| fig-cap: "Two diagrams arranged side by side"

p1 <- plot(euler(c(A = 1, B = 8, "A&B" = 1)), main = "First")
p2 <- plot(euler(c(A = 1, C = 1, "A&C" = 1)), main = "Second")

p1 | p2
```

Each operator returns an `eulergram`, so compositions nest freely:

```{r}
#| label: composition-nested
#| fig-cap: "Nested composition: two plots on top, one below"

p3 <- plot(euler(c(X = 3, Y = 2, "X&Y" = 1)), main = "Third")

(p1 | p2) / p3
```

Individual panels can be titled with the `main` argument to `plot()`, as above.
The gap between adjacent plots is controlled by the `composition$spacing` entry
of `eulerr_options()`:

```{r}
#| label: composition-spacing
#| eval: false

eulerr_options(composition = list(spacing = grid::unit(2, "lines")))
p1 | p2
```

## Normalizing dispersed layouts

If there are disjoint clusters of ellipses, the optimizer will often spread
these out more than is necessary, wasting space in our diagram. To tackle this,
we use a SKYLINE-BL rectangle packing algorithm [@Jylaenki_2010] designed
specifically for **eulerr**. In it, we surround each ellipse cluster with a
bounding box, pack these boxes into a bin of appropriate size and aspect ratio,
and adjust the coordinates of the ellipses in the clusters to compact our
diagram. As a bonus, this increases the chance of having similar layouts for
different function calls.

## References

---
title: "Loss Functions"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Loss Functions}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: eulerr.bib
---

```{r}
#| include: false

knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

eulerr features multiple loss functions, which result in different diagrams for
many combinations. In this vignette, we visualize the effect of the loss
function on an example from an issue [posted on the GitHub repository for
eulerr](https://github.com/jolars/eulerr/issues/93).

We list the combinations below, which consists of 5 different sets, *agc*,
*camk*, *cmgc*, and *tk*.

```{r}
combos <- c(
  "agc" = 9,
  "camk" = 17,
  "cmgc" = 16,
  "tk" = 16,
  "tkl" = 23,
  "agc&camk" = 1,
  "camk&tk" = 1,
  "tk&tkl" = 1,
  "camk&cmgc&tkl" = 1,
  "camk&tk&tkl" = 2,
  "agc&camk&tk&tkl" = 1,
  "camk&cmgc&tk&tkl" = 3,
  "agc&camk&cmgc&tk&tkl" = 1
)
```

Notice that the sizes of most of the intersections are small compared to the
size of the sets themselves and that many of the intersections are missing.
Generating an exact Euler diagram that shows these intersections and at the same
time omits the intersections that are here implicitly 0 is an impossible
problem, which means that the best we can do is an approximation.

What kind of approximation we get depends on the loss functions we use. If we
use the default, which in **eulerr** is the sums of squared errors, we will
almost certainly get a design in which the intersections involving many sets are
missing since including them inevitably leads to larger errors from having to
include other intersections that are not present.

```{r}
#| fig-cap: "An Euler diagram fit to the combination given earlier on, showing that only 1-by-1 intersections are present. This fit uses the default loss function, the sum of squared errors."

library(eulerr)

fit <- euler(combos)
plot(fit)
```

If we rather want a diagram that includes these intersections, despite leading
to errors for the zero-intersections, then we need to switch the loss function
we use. In **eulerr**, you can do so via the `loss` argument in `euler()`, which
mirrors the loss functions provided by the underlying `eunoia` Rust crate.

  Loss                       Input                            Definition
  -------------------------  -----------------------------    --------------------------------------------------------
  Sum of squared errors      `sum_squared` (default)          $\sum_i (y_i - \hat y_i)^2$
  Sum of absolute errors     `sum_absolute`                   $\sum_i |y_i - \hat y_i|$
  Sum of squared regErrors   `sum_squared_region_error`       $\sum_i \left(y_i/\sum_k y_k - \hat y_i / \sum_k \hat y_
  Sum of absolute regErrors  `sum_absolute_region_error`      $\sum_i \big|y_i/\sum_k y_k - \hat y_i / \sum_k \hat y_k
  Max squared error          `max_squared`                    $\max_i (y_i - \hat y_i)^2$
  Max absolute error         `max_absolute`                   $\max_i |y_i - \hat y_i|$
  Root-mean-squared error    `root_mean_squared`              $\sqrt{\sum_i (y_i - \hat y_i)^2 / \sum_i y_i^2}$
  Stress                     `stress`                         venneuler-style stress
  diagError                  `diag_error`                     $\max_i \big|y_i/\sum_k y_k - \hat y_i / \sum_k \hat y_k

  : Loss functions in **eulerr**

The pre-1.0 `loss_aggregator` argument is deprecated; combine the behavior you
want into a single `loss` value instead. For example `loss = "square"` together
with `loss_aggregator = "sum"` is now simply `loss = "sum_squared"`, and
`loss = "region"` with `loss_aggregator = "max"` is `loss = "diag_error"`.

To see what these different choices mean for the combination that we have looked
at, we now refit the diagram for each loss.

```{r}
#| fig-cap: "Euler diagrams fit to the combination above, using different loss functions"
#| fig-show: "hold"
#| fig-width: 3
#| fig-height: 3

losses <- c(
  "sum_squared",
  "sum_absolute",
  "sum_absolute_region_error",
  "max_squared",
  "max_absolute",
  "diag_error"
)

for (loss in losses) {
  fit <- euler(combos, loss = loss)
  print(plot(fit, main = loss))
}
```

As you can see, the errors that sum either the absolute or squared errors result
in very similar fits and keep the existing two-set intersections and drop
everything else. The `abs` + `max` and `max` + `square` combos, meanwhile,
produce fits that are much more unpredictable since they only care about the
largest error. Finally, *diagError* results in diagrams that tries to include
many more intersections at the cost of reducing the goodness-of-fit of the
larger intersections.

Feel free to raise a request (or better yet, a pull request) at
<https://github.com/jolars/eulerr/issues> if you know of any other loss function
that you think should be included in the package.

---
title: "Loss Functions"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Loss Functions}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: eulerr.bib
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

eulerr features multiple loss functions, which result in different diagrams
for many combinations. In this vignette, we visualize the effect of the loss
function on an example from an issue [posted on the GitHub repository for
eulerr](https://github.com/jolars/eulerr/issues/93).

We list the combinations below, which consists of 5 different sets,
*agc*, *camk*, *cmgc*, and *tk*.

```{r}
combos <- c(
  "agc"                  = 9,
  "camk"                 = 17,
  "cmgc"                 = 16,
  "tk"                   = 16,
  "tkl"                  = 23,
  "agc&camk"             = 1,
  "camk&tk"              = 1,
  "tk&tkl"               = 1,
  "camk&cmgc&tkl"        = 1,
  "camk&tk&tkl"          = 2,
  "agc&camk&tk&tkl"      = 1,
  "camk&cmgc&tk&tkl"     = 3,
  "agc&camk&cmgc&tk&tkl" = 1
)
```

Notice that the sizes of most of the intersections are small compared to the
size of the sets themselves and that many of the
intersections are missing. Generating an exact Euler diagram that shows these
intersections and at the same time omits the intersections that are here
implicitly 0 is an impossible problem, which means that the best
we can do is an approximation.

What kind of approximation we get depends on the loss functions we use.
If we use the default, which in **eulerr** is the sums of squared errors, we
will almost certainly get a design in which the intersections involving many
sets are missing since including them inevitably leads to larger errors from
having to include other intersections that are not present.

```{r fig.cap = "An Euler diagram fit to the combination given earlier on, showing that only 1-by-1 intersections are present. This fit uses the default loss function, the sum of squared errors."}
library(eulerr)

fit <- euler(combos)
plot(fit)
```

If we rather want a diagram that includes these intersections, despite 
leading to errors for the zero-intersections, then we need to switch the loss
function we use. In **eulerr**, you can do so via the two arguments
`loss` and `loss_aggregator` in `euler()`. We start by listing the alternatives for
the `loss` argument.

Table: Loss functions in **eulerr**

Loss                     Input          Definition
----------------------   ------------   --------------------------------------------------------
Squared errors           `square`       $(y_i - \hat y_i)^2$
Absolute errors          `abs`          $|y_i - \hat y_i|$
RegionErrors             `region`       $\big|y_i/\sum_k y_k - \hat y_i / \sum_k \hat y_k \big|$

How the final loss is computed depends on the value of `loss_aggregator`, which
is the function used to aggregate the values computed for each set intersection
via the function used in `loss`. The two available settings are `"sum"` and
`"max"`, which should be self-explanatory. 

That means that `loss = "square"` and `loss_aggregator = "sum"` leads to the
sum of squared errors. `loss = "region"` uses *regionError*, which is a 
loss metric introduced by [@Micallef_2014a]. Together with
`loss_aggregator = "max"`, `euler()` will use *diagError*
(introduced in the same paper).

To see what these different choices mean for the combination that we have
looked at, we now refit the diagram for each combination.

```{r fig.cap = "Euler diagrams fit to the combination above, using different loss function", fig.show = "hold", fig.width = 3, fig.height = 3}
losses <- c("square", "abs", "region")
aggregators <- c("sum", "max")

for (loss in losses) {
  for (aggregator in aggregators) {
    fit <- euler(combos, loss = loss, loss_aggregator = aggregator)
    print(plot(fit, main = paste(aggregator, loss, sep = ", ")))
  }
}
```

As you can see, the errors that sum either the absolute or squared errors
result in very similar fits and keep the existing two-set intersections and
drop everything else. The `abs` + `max` and `max` + `square` combos, meanwhile,
produce fits that are much more unpredictable since they only care about the
largest error. Finally, *diagError* results in diagrams that tries to include
many more intersections at the cost of reducing the goodness-of-fit of the
larger intersections.

Feel free to raise a request (or better yet, a pull request) at
<https://github.com/jolars/eulerr/issues> if you know of any other loss
function that you think should be included in the package.

---
title: "eulerr under the hood"
author: "Johan Larsson"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{eulerr under the hood}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: eulerr.bib
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  echo = FALSE,
  warning = FALSE,
  message = FALSE,
  fig.height = 3,
  fig.width = 3,
  collapse = TRUE,
  comment = "#>"
)

library(grid)
library(RConics)
library(eulerr)
library(lattice)

pal <- c(
  "#999999",
  "#E69F00",
  "#56B4E9",
  "#009E73",
  "#F0E442",
  "#0072B2",
  "#D55E00",
  "#CC79A7"
)

ellipse_arc <- function(saxes = c(1, 1),
                        loc = c(0, 0),
                        theta = 0,
                        n = 200,
                        rng = c(0, 2 * pi)) {
  b <- min(saxes[1], saxes[2])
  a <- max(saxes[1], saxes[2])
  d2 <- (a - b) * (a + b)
  if (length(rng) == 1) {
    phi <- rng - theta
  } else {
    phi <- seq(rng[1], rng[2], len = n) - theta
  }
  sp <- sin(phi)
  cp <- cos(phi)
  r <- a * b / sqrt((saxes[2] * cp)^2 + (saxes[1] * sp)^2)
  P <- matrix(nrow = n, ncol = 2)
  P[, 1] <- r * cp
  P[, 2] <- r * sp

  if (theta != 0) {
    P <- P %*% matrix(c(cos(theta), sin(theta), -sin(theta), cos(theta)),
      byrow = TRUE, nrow = 2, ncol = 2
    )
  }
  P <- P + matrix(loc[1:2], nrow = nrow(P), ncol = 2, byrow = TRUE)
  P
}

set.seed(1)
```

## Introduction

**eulerr** relies on an extensive machinery to turn user input into a pretty
Euler diagram. Little of this requires any tinkering from the user. To make that
happen, however, **eulerr** needs to make several well-formed decisions about
the design of the diagram on behalf of the user, which is not a trivial task.

This document outlines the implementation of **eulerr** from input to output. It
is designed to be an evolving documentation on the innards of the program.

## Input

Euler diagrams present relationships between sets, wherefore the data must
describe these relationships, either directly or indirectly.  **eulerr** allows
several alternatives for this data, namely,

* intersections and relative complements 
  $$
  A \setminus B = 3 \quad B \setminus A = 2 \quad A \cap B=1,
  $$
* unions and identities
  $$
  A=4 \quad B=3 \quad A \cap B=1,
  $$
* a matrix of binary (or boolean) indices,
  $$
  \begin{bmatrix}
    \mathbf{A} & \mathbf{B}\\
    1          & 0     \\
    1          & 0     \\
    1          & 0     \\
    1          & 1     \\
    0          & 1     \\
    0          & 1,
  \end{bmatrix}
  $$
* a list of sample spaces
  $$
  \begin{array}{l}
      A = \{a,\,b,\,c,\,d\}\\
      B = \{a,\,e,\,f,\}
    \end{array},
  $$
* or a two- or three-way table

                    $A$      $A^\mathsf{c}$
  ----------------  ---     ----------------
          $B$        1             2
   $A^\mathsf{c}$    3             0

As an additional feature for the matrix form, the user may supply a factor
variable with which to split the data set before fitting the diagram, which
sometimes improves diagrams where the set relationships vary across categories.

Whichever type of input is provided, **eulerr** will translate it to the first
and second types, *intersections and relative complements* and *unions and
identities*, which will be used in the steps to come.

The Euler diagram is then fit in two steps: first, an initial layout is formed
with circles using only the sets' pairwise relationships. Second, this layout is
fine-tuned taking all $2^N-1$ intersections into consideration.

## Initial layout

For our initial layout, we adopt a constrained version of multi-dimensional
scaling (MDS) that has been adapted from **venn.js** [@Frederickson_2016], which
in turn is a modification of an algorithm used in **venneuler**
[@wilkinson_exact_2012].  In it, we consider only the pairwise intersections
between sets, attempting to position their respective shapes so as to minimize
the difference between the separation between their centers required to obtain
an optimal overlap and the actual overlap of the shapes in the diagram.

This problem is unfortunately intractable for ellipses, being that there is an
infinite number of ways by which we can position two ellipses to obtain a given
overlap. Thus, we restrict ourselves to circles in our initial layout, for 
which we can use the
circle--circle overlap formula to numerically find the
required distance, $d$, for each pairwise relationship.

$$
\begin{aligned}
O_{ij} = & r_i^2\arccos\left(\frac{d_{ij}^2 + r_i^2 - r_j^2}{2d_{ij}r_i}\right) +
r_j^2\arccos\left(\frac{d_{ij}^2 + r_j^2 - r_i^2}{2d_{ij}r_j}\right) -\\
&\quad \frac{1}{2}\sqrt{(-d_{ij} + r_i + r_j)(d_{ij} + r_i - r_j)(d_{ij} - r_i + r_j)(d_{ij} + r_i + r_j)},
\end{aligned}
$$

where $r_i$ and $r_j$ are the radii of the circles representing the $i$^th^ and
$j$^th^ sets respectively, $O_{ij}$ their overlap, and $d_{ij}$ their separation.

```{r, fig.cap = "The circle--circle overlap is computed as a function of the discs' separation ($d_{ij}$), radii ($r_i,r_j$), and area of overlap ($O_{ij}$)."}
c0 <- ellipseToConicMatrix(c(1, 1), c(0, 0), 0)
c1 <- ellipseToConicMatrix(c(0.7, 0.7), c(1.2, 0), 0)
pp <- intersectConicConic(c0, c1)

theta0 <- atan2(pp[2, 1], pp[1, 1])
theta1 <- atan2(pp[2, 2], pp[1, 2])

phi0 <- atan2(pp[2, 1], pp[1, 1] - 1.2)
phi1 <- atan2(pp[2, 2], pp[1, 2] - 1.2)

seg <- rbind(
  ellipse_arc(c(0.7, 0.7), c(1.2, 0), 0, n = 50, c(-pi, phi0)),
  ellipse_arc(c(1, 1), c(0, 0), 0, n = 100, c(theta0, theta1)),
  ellipse_arc(c(0.7, 0.7), c(1.2, 0), 0, n = 50, c(phi1, pi))
)

ospot <- c(mean(seg[, 1]), mean(seg[, 2]))

xyplot(
  1 ~ 1,
  xlim = c(-1.1, 2), ylim = c(-1.5, 1.1), asp = "iso",
  xlab = NULL, ylab = NULL,
  scales = list(draw = FALSE),
  par.settings = list(axis.line = list(col = "transparent")),
  panel = function() {
    panel.polygon(seg,
      col = "steelblue1", alpha = 0.5,
      border = "transparent"
    )
    grid::grid.circle(0, 0,
      r = 1, default.units = "native",
      gp = gpar(fill = "transparent")
    )
    grid::grid.circle(1.2, 0,
      r = 0.7, default.units = "native",
      gp = gpar(fill = "transparent")
    )
    pBrackets::grid.brackets(1.2, -1.2, 0, -1.2, h = 0.05)
    pBrackets::grid.brackets(-1, 0, 0, 0, h = 0.05)
    pBrackets::grid.brackets(1.2, 0, 1.9, 0, h = 0.05)
    panel.lines(c(0, 0), c(0, -1.2), lty = 2, col = "grey65")
    panel.lines(c(1.2, 1.2), c(0, -1.2), lty = 2, col = "grey65")
    panel.text(0.6, unit(-1.3, "native"),
      labels = expression(italic(d[ij])), pos = 1
    )
    panel.text(ospot[1], ospot[2], labels = expression(italic(O[ij])))
    panel.text(-0.5, 0.1,
      labels = expression(italic(r[i])), pos = 3,
      default.units = "native"
    )
    panel.text(1.55, 0.1,
      labels = expression(italic(r[j])), pos = 3,
      default.units = "native"
    )
    panel.points(c(0, 1.2), c(0, 0),
      pch = 21, col = 1, cex = 1,
      fill = "white"
    )
  }
)
```

Setting $r_i = \sqrt{F_i/\pi}$, where $F_i$ is
the size of the $i$^th^ set, we are able to obtain $d$ numerically using
the squared difference between $O$ and the desired overlap
as loss function,

$$
\mathcal{L}(d_{ij}) = \left(O_{ij} - (F_i \cap F_j)  \right)^2, \quad \text{for } i <
j \leq n,
$$

which we optimize using `optimize()`^[According to the documentation,
`optimize()` consists of a "combination of golden section search and successive
parabolic interpolation." from **stats**.]

For a two-set combination, this is all we need to plot an exact diagram, given
that we now have the two circles' radii and separation and may place the circles
arbitrarily as long as their separation, $d$, remains the same. This is not,
however, the case with more than two sets.

With three or more sets, the circles need to be arranged so that they interfere
minimally with one another. In some cases, the set configuration allows this to
be accomplished flawlessly, but often, compromises must me made. As is often the
case in this context, this turns out to be another optimization problem. It can
be tackled in many ways; **eulerr**'s approach is based on a method developed by
@Frederickson_2015, which the author describes as constrained multi-dimensional
scaling.

The algorithm tries to position the circles so that the separation between each
pair of circles matches the separation required from the separation equation.
If the two sets are disjoint, however, the algorithm is indifferent to the
relative locations of those circles as long as they do not intersect. The
equivalent applies to subset sets: as long as the circle representing the
smaller set remains within the larger circle, their locations are free to vary.
In all other cases, the loss function is the residual sums of squares of the
optimal separation of circles, $d$, that we found in the overlap equation, and
the actual distance in the layout we are currently exploring.

$$
\mathcal{L}(h,k) = \displaystyle \ell{\sum_{1\leq i<j\leq N}}
\begin{cases}
0 & F_i \cap F_j = \emptyset \text{ and } O_{ij} = 0\\
0 & (F_i \subseteq F_j \text{ or } F_i \supseteq F_j) \text{ and } O_{ij}=0\\
\left(\left(h_i-h_j\right)^2+\left(k_i-k_j\right)^2-d_{ij}^2\right)^2  & \text{otherwise} \\
\end{cases}.
$$

The analytical gradient is retrieved as usual by taking the derivative of the
loss function,

$$
\vec{\nabla} f(h_i) = \sum_{j=1}^N
\begin{cases}
\vec{0} & F_i \cap F_j = \emptyset \text{ and } O_{ij} = 0\\
\vec{0} & (F_i \subseteq F_j \text{ or } F_i \supseteq F_j) \text{ and } O_{ij}=0\\
4\left(h_i-h_j\right)\left(\left(h_i-h_j\right)^2+\left(k_i-k_j\right)^2-d_{ij}^2\right) & \text{otherwise}, \\
\end{cases}
$$

where $\vec{\nabla} f(k_i)$ is found as in the gradient with $h_i$ swapped for
$k_i$ (and vice versa).

The Hessian for our loss function is given next. However, because the current
release of R suffers from a bug^[The current development version of R features a
fix for this bug; **eulerr** will be updated to use the analytical Hessian as
soon as it is introduced in a stable version of R.] causing the analytical
Hessian to be updated improperly, the current release of **eulerr** instead
relies on the numerical approximation of the Hessian offered by the optimizer.

$$
\small
\mathbf{H}(h,k) = \ell{\sum_{1\leq i<j\leq N}}
\begin{bmatrix}
4\left(\left(h_i-h_j\right)^2+\left(k_i-k_j\right)^2-d_{ij}^2\right)+8\left(h_i-h_j\right)^2 &
\cdots &
8\left(h_i-h_j\right)\left(k_i-k_j\right)\\
\vdots & \ddots & \vdots \\
8\left(k_i-k_j\right)\left(h_i-h_j\right) &
\cdots &
4\left(\left(h_i-h_j\right)^2+\left(k_i-k_j\right)^2-d_{ij}^2\right)+8\left(k_i-k_j\right)^2
\end{bmatrix}.
$$

Note that the constraints given in loss and gradients still apply to each
element of the Hessian and have been omitted for convenience only.

We optimize the loss function using the nonlinear optimizer `nlm()` from the R
core package **stats**. The underlying code for `nlm` was written by
@Schnabel_1985. It was ported to R by Saikat DebRoy and the R Core team
[@RCT_2017] from a previous FORTRAN to C translation by Richard H. Jones.
`nlm()` consists of a system of Newton-type algorithms and performs well for
difficult problems [@Nash_2014].

The initial layout outlined above will sometimes turn up perfect diagrams, but
only reliably so when the diagram is completely determined by its pairwise
intersections. More pertinently, we have not yet considered the higher-order
intersections in our algorithm and neither have we approached the problem of
using ellipses---as we set out to do.

## Final layout

We now need to account for all the sets' intersections and, consequently, all
the overlaps in the diagram. The goal is to map each area uniquely to a subset
of the data from the input and for this purpose we will use the sets'
intersections and the relative complements of these intersections, for which we
will use the shorthand $\omega$. We introduced this form in the *input* section,
but now define it rigorously.

For a family of *N* sets, $F = F_1, F_2, \dots, F_N$, and their $n=2^N-1$
intersections, we define $\omega$ as the intersections of these sets and their
relative complements, such that

$$
\begin{aligned}
\omega_{1} & = F_1 \setminus \bigcap_{i=2}^N F_i  \\
\omega_{2} & = \bigcap_{i=1}^2 F_i \setminus \bigcap_{i=3}^{N} F_i\\
\vdots    & = \vdots \\
\omega_n & = \bigcap_{i=1}^{N}F_i
\end{aligned}
$$

with

$$
\sum_{i = 1}^n \omega_i =  \bigcup_{j=1}^N F_i.
$$

Analogously to $\omega$, we also introduce the $\&$-operator, such that

$$
F_i \& F_j = (F_i \cap F_j)\setminus (F_i \cap F_j)^\textsf{c}.
$$

The fitted diagram's area-equivalents for $\omega$ will be defined as $A$, so
that an exact diagram requires that $\omega_i = A_i$ for $i=1,2,\dots,2^N-1$,
where $N$ is the number of sets in the input.

In our initial configuration, we restricted ourselves to circles but now extend
ourselves also to ellipses. From now on, we abandon the practice of treating
circles separately---they are only a special case of ellipses, and, hence,
everything that applies to an ellipse does so equally for a circle.

### Intersecting ellipses

We now need the ellipses' points of intersections. **eulerr**'s approach to this
is outlined in [@Richter-Gebert_2011] and based in *projective*, as opposed to
*Euclidean*, geometry.

To collect all the intersection points, we naturally need only to consider two
ellipses at a time. The canonical form of an ellipse is given by

$$
\frac{\left[ (x-h)\cos{\phi}+(y-k)\sin{\phi} \right]^2}{a^2}+
\frac{\left[(x-h) \sin{\phi}-(y-k) \cos{\phi}\right]^2}{b^2} = 1,
$$

where $\phi$ is the counter-clockwise angle from the positive x-axis to the
semi-major axis $a$, $b$ is the semi-minor axis, and $h, k$ are the x- and
y-coordinates, respectively, of ellipse's center.

```{r, fig.cap = "A rotated ellipse with semimajor axis $a$, semiminor axis $b$, rotation $\\phi$, and center $h,k$."}
h <- 0.7
k <- 0.5
a <- 1
b <- 0.6
phi <- pi / 5
n <- 200
ellipse <- RConics::ellipse(c(a, b), c(h, k), phi, n = n)

xyplot(
  1 ~ 1,
  xlim = c(-0.3, 1.7), ylim = c(-0.3, 1.3), asp = "iso",
  xlab = NULL, ylab = NULL,
  par.settings = list(axis.line = list(col = "transparent")),
  scales = list(draw = FALSE),
  panel = function(x, y, ...) {
    # grid
    panel.refline(h = 0)
    panel.refline(v = 0)

    # rotation
    arc <- ellipse_arc(c(0.2, 0.2), c(h, k), rng = c(0, phi))
    theta <- ellipse_arc(c(a, b), c(h, k), phi, n = 1)

    panel.lines(
      x = c(h, ellipse[1, 1]), y = c(k, ellipse[1, 2]), lty = 2,
      col = "grey50"
    )
    panel.lines(
      x = c(h, theta[1, 1]), y = c(k, theta[1, 2]), lty = 2,
      col = "grey50"
    )
    panel.lines(arc, col = "grey50")
    panel.text(
      x = arc[nrow(arc) / 2, 1] + 0.06, y = arc[nrow(arc) / 2, 2] + 0.02,
      labels = expression(italic(phi))
    )

    # semiaxes
    pBrackets::grid.brackets(h, k, ellipse[n / 2, 1], ellipse[n / 2, 2],
      col = "grey50"
    )
    pBrackets::grid.brackets(h, k, ellipse[n / 4, 1], ellipse[n / 4, 2],
      h = 0.04, col = "grey50"
    )

    panel.text(0.37, 0.12, labels = "a", font = 3)
    panel.text(0.43, 0.68, labels = "b", font = 3)

    # center
    panel.text(x = h, y = k + 0.025, pos = 3, label = c("h,k"), font = 3)
    panel.points(x = h, y = k, pch = 21, fill = "white", col = "black")

    # ellipse
    panel.lines(ellipse, col = 1)
  }
)
```

However, because an ellipse is a conic^[The circle, parabola, and hyperbola are
the other types of conics.] it can be represented in quadric form,

$$
Ax^2 + Bxy + Cy^2 + Dx + Ey + F = 0
$$

that in turn can be represented as a matrix,

$$
\begin{bmatrix}
A   & B/2 & D/2 \\
B/2 & C   & E/2 \\
D/2 & E/2 & F
\end{bmatrix},
$$

which is the form we need to intersect our ellipses. We now proceed to form
three degenerate conics from a linear combination of the two ellipses we wish to
intersect, split one of these degenerate conics into two lines, and intersect
one of the ellipses with these lines, yielding 0 to 4 intersection points
points.

```{r intersect-1, fig.cap = "The process used to intersect two ellipses, here yielding four points. This figure was inspired by an example from Richter--Gebert 2011.", fig.width = 2, fig.height = 2, fig.show = "hold"}
C1 <- ellipseToConicMatrix(c(8, 2), c(0, 0), -pi / 3)
C2 <- ellipseToConicMatrix(c(5, 2), c(1, -2), pi / 5)

ellipses <- data.frame(rbind(
  ellipse(c(8, 2), c(0, 0), -pi / 3),
  ellipse(c(5, 2), c(1, -2), pi / 5)
))
colnames(ellipses) <- c("x", "y")
ellipses$fac <- rep(c("A", "B"), each = 201)
pp <- intersectConicConic(C1, C2)
x <- pp[1, ]
y <- pp[2, ]
a <- double(6)
b <- double(6)
k <- 1
for (i in 1:3) {
  for (j in (i + 1):4) {
    b[k] <- (y[j] - y[i]) / (x[j] - x[i])
    a[k] <- b[k] * (-x[i]) + y[i]
    k <- k + 1
  }
}
a <- a[c(1, 6, 3, 4, 2, 5)]
b <- b[c(1, 6, 3, 4, 2, 5)]

xyplot(
  y ~ x,
  data = ellipses, type = "l", groups = fac, asp = 1,
  xlab = NULL, ylab = NULL,
  scales = list(draw = FALSE, axes = FALSE),
  par.settings = list(axis.line = list(col = "transparent")),
  panel = function(x, y, ...) {
    panel.abline(a = a[3], b = b[3], col = "#FEFEFE")
    panel.abline(a = a[4], b = b[4], col = "#FEFEFE")
    panel.xyplot(x, y, ..., col = 1)
  }
)

xyplot(y ~ x,
  data = ellipses, type = "l", groups = fac, asp = 1,
  xlab = NULL, ylab = NULL,
  scales = list(draw = FALSE, axes = FALSE),
  par.settings = list(axis.line = list(col = "transparent")),
  panel = function(x, y, ...) {
    panel.xyplot(x, y, ..., col = 1)
    for (i in seq_along(a)) {
      panel.abline(a = a[i], b = b[i], col = pal[ceiling(i / 2) + 1])
    }
  }
)

xyplot(
  y ~ x,
  data = ellipses, type = "l", groups = fac, asp = 1,
  xlab = NULL, ylab = NULL,
  scales = list(draw = FALSE, axes = FALSE),
  par.settings = list(axis.line = list(col = "transparent")),
  panel = function(x, y, ...) {
    panel.xyplot(x, y, ..., col = c("transparent", "black"))
    panel.abline(a = a[3], b = b[3], col = pal[3])
    panel.abline(a = a[4], b = b[4], col = pal[3])
    panel.points(t(pp[1:2, ]), col = 1, pch = 21, fill = "white")
  }
)
```

### Overlap areas

Using the intersection points of a set of ellipses that we retrieved in, we can
now find the overlap of these ellipses. We are only interested in the points
that are *contained within all of these ellipses*, which together form a
geometric shape consisting of a convex polygon, the sides of which are made up
of straight lines between consecutive points, and a set of elliptical arcs---one
for each pair of points.

```{r polyarea, fig.cap = "The overlap area between three ellipses is the sum of a convex polygon (in grey) and 2--3 ellipse segments (in blue)."}
x <- c(0, -0.3, 0.2)
y <- c(0, 0.1, 0.3)
ra <- a <- c(0.3, 0.5, 0.4)
rb <- b <- c(0.3, 0.3, 0.6)
phi <- c(-pi / 6, 2, -2)

ee <- data.frame(x, y, ra, rb, phi)

tx <- atan2(-b * tan(phi), a)
ty <- atan2(b * tan(pi / 2L - phi), a)

xlim <- range(
  x + a * cos(tx) * cos(phi) - b * sin(tx) * sin(phi),
  x + a * cos(tx + pi) * cos(phi) - b * sin(tx + pi) * sin(phi)
)
ylim <- range(
  y + b * sin(ty) * cos(phi) + a * cos(ty) * sin(phi),
  y + b * sin(ty + pi) * cos(phi) + a * cos(ty + pi) * sin(phi)
)

pp <- matrix(NA, ncol = 4, nrow = 0)

for (i in 1:2) {
  for (j in (i + 1):3) {
    e1 <- ellipseToConicMatrix(c(ra[i], rb[i]), c(x[i], y[i]), phi[i])
    e2 <- ellipseToConicMatrix(c(ra[j], rb[j]), c(x[j], y[j]), phi[j])
    pp <- rbind(pp, cbind(t(intersectConicConic(e1, e2)[1:2, ]), i, j))
  }
}

sel <- logical(nrow(pp))
for (k in seq_len(nrow(pp))) {
  in_which <- ((pp[k, 1] - x) * cos(phi) + (pp[k, 2] - y) * sin(phi))^2 / ra^2 +
    ((pp[k, 1] - x) * sin(phi) - (pp[k, 2] - y) * cos(phi))^2 / rb^2 <= 1 + 0.1
  sel[k] <- all(in_which)
}

pp <- pp[sel, ]
mid <- cbind(mean(pp[, 1]), mean(pp[, 2]))
seglines <- matrix(NA, ncol = 2, nrow = 0)

ang <- atan2(pp[, 1] - mid[1], pp[, 2] - mid[2])
ord <- order(ang)

pp <- pp[ord, ]

j <- nrow(pp)
for (i in seq_len(nrow(pp))) {
  k <- intersect(pp[i, 3:4], pp[j, 3:4])
  start <- atan2(pp[j, 2] - y[k], pp[j, 1] - x[k])
  stop <- atan2(pp[i, 2] - y[k], pp[i, 1] - x[k])
  arc <- ellipse_arc(c(a[k], b[k]), c(x[k], y[k]),
    theta = phi[k],
    rng = c(start, stop)
  )
  seglines <- rbind(seglines, arc)
  j <- i
}

els <- eulerr:::ellipse(x, y, ra, rb, phi)
x0 <- c(lapply(els, "[[", "x"), recursive = TRUE)
y0 <- c(lapply(els, "[[", "y"), recursive = TRUE)

xyplot(
  y ~ x,
  data = ee, asp = "iso",
  xlim = extendrange(xlim, f = 0.01), ylim = extendrange(ylim, f = 0.01),
  scales = list(draw = FALSE), xlab = NULL, ylab = NULL,
  par.settings = list(axis.line = list(col = "transparent")),
  panel = function(x, y, ...) {
    grid.polygon(x0, y0,
      id.lengths = rep(200, 3),
      default.units = "native",
      gp = grid::gpar(fill = "transparent")
    )
    panel.polygon(seglines, col = "slategray2")
    panel.polygon(pp[, 1:2], col = "grey90")
    panel.points(mid, pch = 4, col = 1, cex = 1.5)
    panel.points(pp[, 1:2, drop = FALSE],
      col = 1, pch = 21, fill = "white",
      cex = 1.5
    )
  }
)
```

We continue by ordering the points around their centroid. It is then trivial to
find the area of the polygon section since it is always convex. Now, because
each elliptical segment is formed from the arcs that connect successive points,
we can establish the segments' areas algorithmically [@Eberly_2016].  For each
ellipse and its related pair of points (located at angles $\theta_0$ and
$\theta_1$ from the semimajor axis), we proceed to find its area by

1.  centering the ellipse at $(0, 0)$,
2.  normalizing its rotation, which is not needed to compute the area,
3.  integrating the ellipse over [$0,\theta_0$] and [$0,\theta_1$], producing
    elliptical sectors $F(\theta_0)$ and $F(\theta_1)$,
4.  subtracting the smaller ($F(\theta_0$)) of these sectors from the larger ($F(\theta_0$), and
5.  subtracting the triangle section to finally find the segment area,
    $$
    F(\theta_1) - F(\theta_0) - \frac{1}{2}\left|x_1y_0 - x_0y_1\right|,
    $$
    where
    $$
    F(\theta) = \frac{a}{b}\left[ \theta - \arctan{\left(\frac{(b - a)\sin{2\theta}}{b + a +(b - a )\cos{2\theta}} \right)}\right].
    $$

This procedure is illustrated in the following figure. Note that there are
situations where this algorithm is altered, such that when the sector angle
ranges beyond $\pi$---we refer the interested reader to @Eberly_2016.

```{r ellipsesegment, fig.cap = "The elliptical segment in blue is found by first subtracting the elliptical sector from $(a, 0)$ to $\\theta_0$ from the one from $(a, 0)$ to $\\theta_1$ and then subtracting the triangle part (in grey)."}
ellipse <- ellipse(c(1, 0.6), c(0, 0), 0)
i0 <- 15
i1 <- 45

tri <- rbind(
  cbind(0, 0),
  ellipse[i0, , drop = FALSE],
  ellipse[i1, , drop = FALSE]
)

xyplot(
  1 ~ 1,
  xlim = c(-1.2, 1.2), ylim = c(-0.8, 0.8), asp = "iso",
  xlab = NULL, ylab = NULL,
  par.settings = list(axis.line = list(col = "transparent")),
  scales = list(draw = FALSE),
  panel = function(x, y, ...) {
    panel.grid(x = 0, y = 0, h = 1, v = 1)
    panel.lines(ellipse, col = 1)
    panel.polygon(ellipse[i0:i1, ], col = "steelblue1", alpha = 0.5)
    panel.polygon(tri, col = "grey95")
    panel.points(ellipse[c(i0, i1), ], col = 1, pch = 21, fill = "white")
    panel.text(ellipse[c(i0, i1), ],
      adj = c(-0.5, -0.5),
      labels = c(
        expression(italic(theta[0])),
        expression(italic(theta[1]))
      )
    )
    pBrackets::grid.brackets(1, 0, 0, 0,
      h = 0.05,
      type = 1
    )
    pBrackets::grid.brackets(0, 0, 0, 0.6, h = 0.04)
    panel.text(0.5, unit(-0.05, "npc"), labels = "a", font = 3, pos = 1)
    panel.text(-0.05, unit(0.3, "native"), labels = "b", font = 3, pos = 2)
  }
)
```

Finally, the area of the overlap is then obtained by adding the area of the
polygon and all the elliptical arcs together.

Note that this does not yet give us the areas that we require, namely $A$: the
area-equivalents to the set intersections and relative complements from our
definition of the intersections. For this, we must decompose the overlap areas
so that each area maps uniquely to a subspace of the set configuration. This,
however, is simply a matter of transversing down the hierarchy of overlaps and
subtracting the higher-order overlaps from the lower-order ones. For a three-set
relationship of sets $A$, $B$, and $C$, for instance, this means subtracting the
$A\cap B \cap C$ overlap from the $A \cap B$ one to retrieve the equivalent of
$(A \cap B) \setminus C$.

The exact algorithm may in rare instances^[1 out of approximately 7000 in
our simulations.], break down, the culprit being numerical precision
issues that occur when ellipses are tangent or
completely overlap. In these cases, the algorithm will approximate
the area of the involved overlap by

1.  spreading points across the ellipses using Vogel's
    method.
2.  identifying the points that are inside the intersection via the inequality
    $$
    \begin{aligned}
    &\frac{\left[(x-h)\cos{\phi}+(y-k)\sin{\phi} \right]^2}{a^2} + 
    &\quad \frac{\left[(x-h) \sin{\phi}-(y-k)\cos{\phi}\right]^2}{b^2} < 1,
    \end{aligned}
    $$
    where $x$ and $y$ are the coordinates of the sampled points, and finally
3.  approximating the area by multiplying the proportion of points inside the
    overlap with the area of the ellipse.

With this in place, we are now able to compute the areas of all intersections
and their relative complements, $\omega$, up to numerical precision.

### Final optimization

We feed the initial layout to the optimizer---once again we employ `nlm()` from
**stats** but now also provide the option to use ellipses rather than circles,
allowing the "circles" to rotate and the relation between the semiaxes to vary,
altogether rendering five parameters to optimize per set and ellipse (or three
if we restrict ourselves to circles). For each iteration of the optimizer, the
areas of all intersections are analyzed and a measure of loss returned. The loss
we use is the same as in **venneuler** [@Frederickson_2016], namely the residual
sums of squares.

If the fitted diagram is still inexact after the procedure, we offer a final
step in which we pass the parameters on to a last-ditch optimizer. The weapon of
choice^[We conducted thorough benchmarking, that we opt not to report here, to
decide upon an algorithm for this step.] is *stress* [@wilkinson_exact_2012],
which is also the loss metric we use in our final optimization step and is used
in **venneuler**, as well as *diagError* [@Micallef_2014a], which is used by
**eulerAPE**.

The stress metric is not easily grasped but
can be transformed into a rough analogue of the correlation coefficient
via $r = \sqrt{1-\text{stress}^2}$.

diagError, meanwhile, is given by

$$
{\max_{i = 1, 2, \dots, n}}\left|
\frac{\omega_i}{\sum_{i=1}^n \omega_i} - \frac{A_i}{\sum_{i=1}^n A_i} \right|,
$$

which is the maximum *absolute* difference of the proportion of any
$\omega$ to the respective unique area of the diagram.

## References

---
title: "Loss Functions"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Loss Functions}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: eulerr.bib
---

```{r}
#| include: false

knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

eulerr features multiple loss functions, which result in different diagrams for
many combinations. In this vignette, we visualize the effect of the loss
function on an example from an issue [posted on the GitHub repository for
eulerr](https://github.com/jolars/eulerr/issues/93).

We list the combinations below, which consists of 5 different sets, *agc*,
*camk*, *cmgc*, and *tk*.

```{r}
combos <- c(
  "agc" = 9,
  "camk" = 17,
  "cmgc" = 16,
  "tk" = 16,
  "tkl" = 23,
  "agc&camk" = 1,
  "camk&tk" = 1,
  "tk&tkl" = 1,
  "camk&cmgc&tkl" = 1,
  "camk&tk&tkl" = 2,
  "agc&camk&tk&tkl" = 1,
  "camk&cmgc&tk&tkl" = 3,
  "agc&camk&cmgc&tk&tkl" = 1
)
```

Notice that the sizes of most of the intersections are small compared to the
size of the sets themselves and that many of the intersections are missing.
Generating an exact Euler diagram that shows these intersections and at the same
time omits the intersections that are here implicitly 0 is an impossible
problem, which means that the best we can do is an approximation.

What kind of approximation we get depends on the loss functions we use. If we
use the default, which in **eulerr** is the sums of squared errors, we will
almost certainly get a design in which the intersections involving many sets are
missing since including them inevitably leads to larger errors from having to
include other intersections that are not present.

```{r}
#| fig-cap: "An Euler diagram fit to the combination given earlier on, showing that only 1-by-1 intersections are present. This fit uses the default loss function, the sum of squared errors."

library(eulerr)

fit <- euler(combos)
plot(fit)
```

If we rather want a diagram that includes these intersections, despite leading
to errors for the zero-intersections, then we need to switch the loss function
we use. In **eulerr**, you can do so via the `loss` argument in `euler()`, which
mirrors the loss functions provided by the underlying `eunoia` Rust crate.

  Loss                       Input                            Definition
  -------------------------  -----------------------------    --------------------------------------------------------
  Sum of squared errors      `sum_squared` (default)          $\sum_i (y_i - \hat y_i)^2$
  Sum of absolute errors     `sum_absolute`                   $\sum_i |y_i - \hat y_i|$
  Sum of squared regErrors   `sum_squared_region_error`       $\sum_i \left(y_i/\sum_k y_k - \hat y_i / \sum_k \hat y_k$
  Sum of absolute regErrors  `sum_absolute_region_error`      $\sum_i \big|y_i/\sum_k y_k - \hat y_i / \sum_k \hat y_k$
  Max squared error          `max_squared`                    $\max_i (y_i - \hat y_i)^2$
  Max absolute error         `max_absolute`                   $\max_i |y_i - \hat y_i|$
  Root-mean-squared error    `root_mean_squared`              $\sqrt{\sum_i (y_i - \hat y_i)^2 / \sum_i y_i^2}$
  Stress                     `stress`                         venneuler-style stress
  diagError                  `diag_error`                     $\max_i \big|y_i/\sum_k y_k - \hat y_i / \sum_k \hat y_k$

  : Loss functions in **eulerr**

The pre-1.0 `loss_aggregator` argument is deprecated; combine the behavior you
want into a single `loss` value instead. For example `loss = "square"` together
with `loss_aggregator = "sum"` is now simply `loss = "sum_squared"`, and
`loss = "region"` with `loss_aggregator = "max"` is `loss = "diag_error"`.

To see what these different choices mean for the combination that we have looked
at, we now refit the diagram for each loss.

```{r}
#| fig-cap: "Euler diagrams fit to the combination above, using different loss functions"
#| fig-show: "hold"
#| fig-width: 3
#| fig-height: 3

losses <- c(
  "sum_squared",
  "sum_absolute",
  "sum_absolute_region_error",
  "max_squared",
  "max_absolute",
  "diag_error"
)

for (loss in losses) {
  fit <- euler(combos, loss = loss)
  print(plot(fit, main = loss))
}
```

As you can see, the errors that sum either the absolute or squared errors result
in very similar fits and keep the existing two-set intersections and drop
everything else. The `abs` + `max` and `max` + `square` combos, meanwhile,
produce fits that are much more unpredictable since they only care about the
largest error. Finally, *diagError* results in diagrams that tries to include
many more intersections at the cost of reducing the goodness-of-fit of the
larger intersections.

Feel free to raise a request (or better yet, a pull request) at
<https://github.com/jolars/eulerr/issues> if you know of any other loss function
that you think should be included in the package.

---
title: "eulerr under the hood"
author: "Johan Larsson"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{eulerr under the hood}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: eulerr.bib
---

```{r}
#| label: setup
#| include: false

knitr::opts_chunk$set(
  echo = FALSE,
  warning = FALSE,
  message = FALSE,
  fig.height = 3,
  fig.width = 3,
  collapse = TRUE,
  comment = "#>"
)

library(grid)
library(RConics)
library(eulerr)
library(lattice)

pal <- c(
  "#999999",
  "#E69F00",
  "#56B4E9",
  "#009E73",
  "#F0E442",
  "#0072B2",
  "#D55E00",
  "#CC79A7"
)

ellipse_arc <- function(
  saxes = c(1, 1),
  loc = c(0, 0),
  theta = 0,
  n = 200,
  rng = c(0, 2 * pi)
) {
  b <- min(saxes[1], saxes[2])
  a <- max(saxes[1], saxes[2])
  d2 <- (a - b) * (a + b)
  if (length(rng) == 1) {
    phi <- rng - theta
  } else {
    phi <- seq(rng[1], rng[2], len = n) - theta
  }
  sp <- sin(phi)
  cp <- cos(phi)
  r <- a * b / sqrt((saxes[2] * cp)^2 + (saxes[1] * sp)^2)
  P <- matrix(nrow = n, ncol = 2)
  P[, 1] <- r * cp
  P[, 2] <- r * sp

  if (theta != 0) {
    P <- P %*%
      matrix(
        c(cos(theta), sin(theta), -sin(theta), cos(theta)),
        byrow = TRUE,
        nrow = 2,
        ncol = 2
      )
  }
  P <- P + matrix(loc[1:2], nrow = nrow(P), ncol = 2, byrow = TRUE)
  P
}

set.seed(1)
```

## Introduction

**eulerr** relies on an extensive machinery to turn user input into a pretty
Euler diagram. Little of this requires any tinkering from the user. To make that
happen, however, **eulerr** needs to make several well-formed decisions about
the design of the diagram on behalf of the user, which is not a trivial task.

This document outlines the implementation of **eulerr** from input to output. It
is designed to be an evolving documentation on the innards of the program.

## Input

Euler diagrams present relationships between sets, wherefore the data must
describe these relationships, either directly or indirectly. **eulerr** allows
several alternatives for this data, namely,

- intersections and relative complements
  $$
  A \setminus B = 3 \quad B \setminus A = 2 \quad A \cap B=1,
  $$
- unions and identities
  $$
  A=4 \quad B=3 \quad A \cap B=1,
  $$
- a matrix of binary (or boolean) indices, $$
  \begin{bmatrix}
    \mathbf{A} & \mathbf{B}\\
    1          & 0     \\
    1          & 0     \\
    1          & 0     \\
    1          & 1     \\
    0          & 1     \\
    0          & 1,
  \end{bmatrix}
  $$
- a list of sample spaces $$
  \begin{array}{l}
      A = \{a,\,b,\,c,\,d\}\\
      B = \{a,\,e,\,f,\}
    \end{array},
  $$
- or a two- or three-way table

                    $A$      $A^\mathsf{c}$
  --------------    ---     ----------------
  $B$               1              2
  $A^\mathsf{c}$    3              0

As an additional feature for the matrix form, the user may supply a factor
variable with which to split the data set before fitting the diagram, which
sometimes improves diagrams where the set relationships vary across categories.

Whichever type of input is provided, **eulerr** will translate it to the first
and second types, *intersections and relative complements* and *unions and
identities*, which will be used in the steps to come.

The Euler diagram is then fit in two steps: first, an initial layout is formed
with circles using only the sets' pairwise relationships. Second, this layout is
fine-tuned taking all $2^N-1$ intersections into consideration.

## Initial layout

For our initial layout, we adopt a constrained version of multi-dimensional
scaling (MDS) that has been adapted from **venn.js** [@Frederickson_2016], which
in turn is a modification of an algorithm used in **venneuler**
[@wilkinson_exact_2012]. In it, we consider only the pairwise intersections
between sets, attempting to position their respective shapes so as to minimize
the difference between the separation between their centers required to obtain
an optimal overlap and the actual overlap of the shapes in the diagram.

This problem is unfortunately intractable for ellipses, being that there is an
infinite number of ways by which we can position two ellipses to obtain a given
overlap. Thus, we restrict ourselves to circles in our initial layout, for which
we can use the circle--circle overlap formula to numerically find the required
distance, $d$, for each pairwise relationship.

$$
\begin{aligned}
O_{ij} = & r_i^2\arccos\left(\frac{d_{ij}^2 + r_i^2 - r_j^2}{2d_{ij}r_i}\right) +
r_j^2\arccos\left(\frac{d_{ij}^2 + r_j^2 - r_i^2}{2d_{ij}r_j}\right) -\\
&\quad \frac{1}{2}\sqrt{(-d_{ij} + r_i + r_j)(d_{ij} + r_i - r_j)(d_{ij} - r_i + r_j)(d_{ij} + r_i + r_j)},
\end{aligned}
$$

where $r_i$ and $r_j$ are the radii of the circles representing the $i$^th^ and
$j$^th^ sets respectively, $O_{ij}$ their overlap, and $d_{ij}$ their
separation.

```{r}
#| fig-cap: "The circle--circle overlap is computed as a function of the discs' separation ($d_{ij}$), radii ($r_i,r_j$), and area of overlap ($O_{ij}$)."

c0 <- ellipseToConicMatrix(c(1, 1), c(0, 0), 0)
c1 <- ellipseToConicMatrix(c(0.7, 0.7), c(1.2, 0), 0)
pp <- intersectConicConic(c0, c1)

theta0 <- atan2(pp[2, 1], pp[1, 1])
theta1 <- atan2(pp[2, 2], pp[1, 2])

phi0 <- atan2(pp[2, 1], pp[1, 1] - 1.2)
phi1 <- atan2(pp[2, 2], pp[1, 2] - 1.2)

seg <- rbind(
  ellipse_arc(c(0.7, 0.7), c(1.2, 0), 0, n = 50, c(-pi, phi0)),
  ellipse_arc(c(1, 1), c(0, 0), 0, n = 100, c(theta0, theta1)),
  ellipse_arc(c(0.7, 0.7), c(1.2, 0), 0, n = 50, c(phi1, pi))
)

ospot <- c(mean(seg[, 1]), mean(seg[, 2]))

xyplot(
  1 ~ 1,
  xlim = c(-1.1, 2),
  ylim = c(-1.5, 1.1),
  asp = "iso",
  xlab = NULL,
  ylab = NULL,
  scales = list(draw = FALSE),
  par.settings = list(axis.line = list(col = "transparent")),
  panel = function() {
    panel.polygon(seg, col = "steelblue1", alpha = 0.5, border = "transparent")
    grid::grid.circle(
      0,
      0,
      r = 1,
      default.units = "native",
      gp = gpar(fill = "transparent")
    )
    grid::grid.circle(
      1.2,
      0,
      r = 0.7,
      default.units = "native",
      gp = gpar(fill = "transparent")
    )
    pBrackets::grid.brackets(1.2, -1.2, 0, -1.2, h = 0.05)
    pBrackets::grid.brackets(-1, 0, 0, 0, h = 0.05)
    pBrackets::grid.brackets(1.2, 0, 1.9, 0, h = 0.05)
    panel.lines(c(0, 0), c(0, -1.2), lty = 2, col = "grey65")
    panel.lines(c(1.2, 1.2), c(0, -1.2), lty = 2, col = "grey65")
    panel.text(
      0.6,
      unit(-1.3, "native"),
      labels = expression(italic(d[ij])),
      pos = 1
    )
    panel.text(ospot[1], ospot[2], labels = expression(italic(O[ij])))
    panel.text(
      -0.5,
      0.1,
      labels = expression(italic(r[i])),
      pos = 3,
      default.units = "native"
    )
    panel.text(
      1.55,
      0.1,
      labels = expression(italic(r[j])),
      pos = 3,
      default.units = "native"
    )
    panel.points(c(0, 1.2), c(0, 0), pch = 21, col = 1, cex = 1, fill = "white")
  }
)
```

Setting $r_i = \sqrt{F_i/\pi}$, where $F_i$ is the size of the $i$^th^ set, we
are able to obtain $d$ numerically using the squared difference between $O$ and
the desired overlap as loss function,

$$
\mathcal{L}(d_{ij}) = \left(O_{ij} - (F_i \cap F_j)  \right)^2, \quad \text{for } i <
j \leq n,
$$

which we optimize using `optimize()`^[According to the documentation,
`optimize()` consists of a "combination of golden section search and successive
parabolic interpolation." from **stats**.]

For a two-set combination, this is all we need to plot an exact diagram, given
that we now have the two circles' radii and separation and may place the circles
arbitrarily as long as their separation, $d$, remains the same. This is not,
however, the case with more than two sets.

With three or more sets, the circles need to be arranged so that they interfere
minimally with one another. In some cases, the set configuration allows this to
be accomplished flawlessly, but often, compromises must me made. As is often the
case in this context, this turns out to be another optimization problem. It can
be tackled in many ways; **eulerr**'s approach is based on a method developed by
@Frederickson_2015, which the author describes as constrained multi-dimensional
scaling.

The algorithm tries to position the circles so that the separation between each
pair of circles matches the separation required from the separation equation. If
the two sets are disjoint, however, the algorithm is indifferent to the relative
locations of those circles as long as they do not intersect. The equivalent
applies to subset sets: as long as the circle representing the smaller set
remains within the larger circle, their locations are free to vary. In all other
cases, the loss function is the residual sums of squares of the optimal
separation of circles, $d$, that we found in the overlap equation, and the
actual distance in the layout we are currently exploring.

$$
\mathcal{L}(h,k) = \displaystyle \ell{\sum_{1\leq i<j\leq N}}
\begin{cases}
0 & F_i \cap F_j = \emptyset \text{ and } O_{ij} = 0\\
0 & (F_i \subseteq F_j \text{ or } F_i \supseteq F_j) \text{ and } O_{ij}=0\\
\left(\left(h_i-h_j\right)^2+\left(k_i-k_j\right)^2-d_{ij}^2\right)^2  & \text{otherwise} \\
\end{cases}.
$$

The analytical gradient is retrieved as usual by taking the derivative of the
loss function,

$$
\vec{\nabla} f(h_i) = \sum_{j=1}^N
\begin{cases}
\vec{0} & F_i \cap F_j = \emptyset \text{ and } O_{ij} = 0\\
\vec{0} & (F_i \subseteq F_j \text{ or } F_i \supseteq F_j) \text{ and } O_{ij}=0\\
4\left(h_i-h_j\right)\left(\left(h_i-h_j\right)^2+\left(k_i-k_j\right)^2-d_{ij}^2\right) & \text{otherwise}, \\
\end{cases}
$$

where $\vec{\nabla} f(k_i)$ is found as in the gradient with $h_i$ swapped for
$k_i$ (and vice versa).

The Hessian for our loss function is given next. However, because the current
release of R suffers from a bug^[The current development version of R features a
fix for this bug; **eulerr** will be updated to use the analytical Hessian as
soon as it is introduced in a stable version of R.] causing the analytical
Hessian to be updated improperly, the current release of **eulerr** instead
relies on the numerical approximation of the Hessian offered by the optimizer.

$$
\small
\mathbf{H}(h,k) = \ell{\sum_{1\leq i<j\leq N}}
\begin{bmatrix}
4\left(\left(h_i-h_j\right)^2+\left(k_i-k_j\right)^2-d_{ij}^2\right)+8\left(h_i-h_j\right)^2 &
\cdots &
8\left(h_i-h_j\right)\left(k_i-k_j\right)\\
\vdots & \ddots & \vdots \\
8\left(k_i-k_j\right)\left(h_i-h_j\right) &
\cdots &
4\left(\left(h_i-h_j\right)^2+\left(k_i-k_j\right)^2-d_{ij}^2\right)+8\left(k_i-k_j\right)^2
\end{bmatrix}.
$$

Note that the constraints given in loss and gradients still apply to each
element of the Hessian and have been omitted for convenience only.

We optimize the loss function using the nonlinear optimizer `nlm()` from the R
core package **stats**. The underlying code for `nlm` was written by
@Schnabel_1985. It was ported to R by Saikat DebRoy and the R Core team
[@RCT_2017] from a previous FORTRAN to C translation by Richard H. Jones.
`nlm()` consists of a system of Newton-type algorithms and performs well for
difficult problems [@Nash_2014].

The initial layout outlined above will sometimes turn up perfect diagrams, but
only reliably so when the diagram is completely determined by its pairwise
intersections. More pertinently, we have not yet considered the higher-order
intersections in our algorithm and neither have we approached the problem of
using ellipses---as we set out to do.

## Final layout

We now need to account for all the sets' intersections and, consequently, all
the overlaps in the diagram. The goal is to map each area uniquely to a subset
of the data from the input and for this purpose we will use the sets'
intersections and the relative complements of these intersections, for which we
will use the shorthand $\omega$. We introduced this form in the *input* section,
but now define it rigorously.

For a family of *N* sets, $F = F_1, F_2, \dots, F_N$, and their $n=2^N-1$
intersections, we define $\omega$ as the intersections of these sets and their
relative complements, such that

$$
\begin{aligned}
\omega_{1} & = F_1 \setminus \bigcap_{i=2}^N F_i  \\
\omega_{2} & = \bigcap_{i=1}^2 F_i \setminus \bigcap_{i=3}^{N} F_i\\
\vdots    & = \vdots \\
\omega_n & = \bigcap_{i=1}^{N}F_i
\end{aligned}
$$

with

$$
\sum_{i = 1}^n \omega_i =  \bigcup_{j=1}^N F_i.
$$

Analogously to $\omega$, we also introduce the $\&$-operator, such that

$$
F_i \& F_j = (F_i \cap F_j)\setminus (F_i \cap F_j)^\textsf{c}.
$$

The fitted diagram's area-equivalents for $\omega$ will be defined as $A$, so
that an exact diagram requires that $\omega_i = A_i$ for $i=1,2,\dots,2^N-1$,
where $N$ is the number of sets in the input.

In our initial configuration, we restricted ourselves to circles but now extend
ourselves also to ellipses. From now on, we abandon the practice of treating
circles separately---they are only a special case of ellipses, and, hence,
everything that applies to an ellipse does so equally for a circle.

### Intersecting ellipses

We now need the ellipses' points of intersections. **eulerr**'s approach to this
is outlined in [@Richter-Gebert_2011] and based in *projective*, as opposed to
*Euclidean*, geometry.

To collect all the intersection points, we naturally need only to consider two
ellipses at a time. The canonical form of an ellipse is given by

$$
\frac{\left[ (x-h)\cos{\phi}+(y-k)\sin{\phi} \right]^2}{a^2}+
\frac{\left[(x-h) \sin{\phi}-(y-k) \cos{\phi}\right]^2}{b^2} = 1,
$$

where $\phi$ is the counter-clockwise angle from the positive x-axis to the
semi-major axis $a$, $b$ is the semi-minor axis, and $h, k$ are the x- and
y-coordinates, respectively, of ellipse's center.

```{r}
#| fig-cap: "A rotated ellipse with semimajor axis $a$, semiminor axis $b$, rotation $\\phi$, and center $h,k$."

h <- 0.7
k <- 0.5
a <- 1
b <- 0.6
phi <- pi / 5
n <- 200
ellipse <- RConics::ellipse(c(a, b), c(h, k), phi, n = n)

xyplot(
  1 ~ 1,
  xlim = c(-0.3, 1.7),
  ylim = c(-0.3, 1.3),
  asp = "iso",
  xlab = NULL,
  ylab = NULL,
  par.settings = list(axis.line = list(col = "transparent")),
  scales = list(draw = FALSE),
  panel = function(x, y, ...) {
    # grid
    panel.refline(h = 0)
    panel.refline(v = 0)

    # rotation
    arc <- ellipse_arc(c(0.2, 0.2), c(h, k), rng = c(0, phi))
    theta <- ellipse_arc(c(a, b), c(h, k), phi, n = 1)

    panel.lines(
      x = c(h, ellipse[1, 1]),
      y = c(k, ellipse[1, 2]),
      lty = 2,
      col = "grey50"
    )
    panel.lines(
      x = c(h, theta[1, 1]),
      y = c(k, theta[1, 2]),
      lty = 2,
      col = "grey50"
    )
    panel.lines(arc, col = "grey50")
    panel.text(
      x = arc[nrow(arc) / 2, 1] + 0.06,
      y = arc[nrow(arc) / 2, 2] + 0.02,
      labels = expression(italic(phi))
    )

    # semiaxes
    pBrackets::grid.brackets(
      h,
      k,
      ellipse[n / 2, 1],
      ellipse[n / 2, 2],
      col = "grey50"
    )
    pBrackets::grid.brackets(
      h,
      k,
      ellipse[n / 4, 1],
      ellipse[n / 4, 2],
      h = 0.04,
      col = "grey50"
    )

    panel.text(0.37, 0.12, labels = "a", font = 3)
    panel.text(0.43, 0.68, labels = "b", font = 3)

    # center
    panel.text(x = h, y = k + 0.025, pos = 3, label = c("h,k"), font = 3)
    panel.points(x = h, y = k, pch = 21, fill = "white", col = "black")

    # ellipse
    panel.lines(ellipse, col = 1)
  }
)
```

However, because an ellipse is a conic^[The circle, parabola, and hyperbola are
the other types of conics.] it can be represented in quadric form,

$$
Ax^2 + Bxy + Cy^2 + Dx + Ey + F = 0
$$

that in turn can be represented as a matrix,

$$
\begin{bmatrix}
A   & B/2 & D/2 \\
B/2 & C   & E/2 \\
D/2 & E/2 & F
\end{bmatrix},
$$

which is the form we need to intersect our ellipses. We now proceed to form
three degenerate conics from a linear combination of the two ellipses we wish to
intersect, split one of these degenerate conics into two lines, and intersect
one of the ellipses with these lines, yielding 0 to 4 intersection points
points.

```{r}
#| label: intersect-1
#| fig-cap: "The process used to intersect two ellipses, here yielding four points. This figure was inspired by an example from Richter--Gebert 2011."
#| fig-width: 2
#| fig-height: 2
#| fig-show: "hold"

C1 <- ellipseToConicMatrix(c(8, 2), c(0, 0), -pi / 3)
C2 <- ellipseToConicMatrix(c(5, 2), c(1, -2), pi / 5)

ellipses <- data.frame(rbind(
  ellipse(c(8, 2), c(0, 0), -pi / 3),
  ellipse(c(5, 2), c(1, -2), pi / 5)
))
colnames(ellipses) <- c("x", "y")
ellipses$fac <- rep(c("A", "B"), each = 201)
pp <- intersectConicConic(C1, C2)
x <- pp[1, ]
y <- pp[2, ]
a <- double(6)
b <- double(6)
k <- 1
for (i in 1:3) {
  for (j in (i + 1):4) {
    b[k] <- (y[j] - y[i]) / (x[j] - x[i])
    a[k] <- b[k] * (-x[i]) + y[i]
    k <- k + 1
  }
}
a <- a[c(1, 6, 3, 4, 2, 5)]
b <- b[c(1, 6, 3, 4, 2, 5)]

xyplot(
  y ~ x,
  data = ellipses,
  type = "l",
  groups = fac,
  asp = 1,
  xlab = NULL,
  ylab = NULL,
  scales = list(draw = FALSE, axes = FALSE),
  par.settings = list(axis.line = list(col = "transparent")),
  panel = function(x, y, ...) {
    panel.abline(a = a[3], b = b[3], col = "#FEFEFE")
    panel.abline(a = a[4], b = b[4], col = "#FEFEFE")
    panel.xyplot(x, y, ..., col = 1)
  }
)

xyplot(
  y ~ x,
  data = ellipses,
  type = "l",
  groups = fac,
  asp = 1,
  xlab = NULL,
  ylab = NULL,
  scales = list(draw = FALSE, axes = FALSE),
  par.settings = list(axis.line = list(col = "transparent")),
  panel = function(x, y, ...) {
    panel.xyplot(x, y, ..., col = 1)
    for (i in seq_along(a)) {
      panel.abline(a = a[i], b = b[i], col = pal[ceiling(i / 2) + 1])
    }
  }
)

xyplot(
  y ~ x,
  data = ellipses,
  type = "l",
  groups = fac,
  asp = 1,
  xlab = NULL,
  ylab = NULL,
  scales = list(draw = FALSE, axes = FALSE),
  par.settings = list(axis.line = list(col = "transparent")),
  panel = function(x, y, ...) {
    panel.xyplot(x, y, ..., col = c("transparent", "black"))
    panel.abline(a = a[3], b = b[3], col = pal[3])
    panel.abline(a = a[4], b = b[4], col = pal[3])
    panel.points(t(pp[1:2, ]), col = 1, pch = 21, fill = "white")
  }
)
```

### Overlap areas

Using the intersection points of a set of ellipses that we retrieved in, we can
now find the overlap of these ellipses. We are only interested in the points
that are *contained within all of these ellipses*, which together form a
geometric shape consisting of a convex polygon, the sides of which are made up
of straight lines between consecutive points, and a set of elliptical arcs---one
for each pair of points.

```{r}
#| label: polyarea
#| fig-cap: "The overlap area between three ellipses is the sum of a convex polygon (in grey) and 2--3 ellipse segments (in blue)."

x <- c(0, -0.3, 0.2)
y <- c(0, 0.1, 0.3)
ra <- a <- c(0.3, 0.5, 0.4)
rb <- b <- c(0.3, 0.3, 0.6)
phi <- c(-pi / 6, 2, -2)

ee <- data.frame(x, y, ra, rb, phi)

tx <- atan2(-b * tan(phi), a)
ty <- atan2(b * tan(pi / 2L - phi), a)

xlim <- range(
  x + a * cos(tx) * cos(phi) - b * sin(tx) * sin(phi),
  x + a * cos(tx + pi) * cos(phi) - b * sin(tx + pi) * sin(phi)
)
ylim <- range(
  y + b * sin(ty) * cos(phi) + a * cos(ty) * sin(phi),
  y + b * sin(ty + pi) * cos(phi) + a * cos(ty + pi) * sin(phi)
)

pp <- matrix(NA, ncol = 4, nrow = 0)

for (i in 1:2) {
  for (j in (i + 1):3) {
    e1 <- ellipseToConicMatrix(c(ra[i], rb[i]), c(x[i], y[i]), phi[i])
    e2 <- ellipseToConicMatrix(c(ra[j], rb[j]), c(x[j], y[j]), phi[j])
    pp <- rbind(pp, cbind(t(intersectConicConic(e1, e2)[1:2, ]), i, j))
  }
}

sel <- logical(nrow(pp))
for (k in seq_len(nrow(pp))) {
  in_which <- ((pp[k, 1] - x) * cos(phi) + (pp[k, 2] - y) * sin(phi))^2 /
    ra^2 +
    ((pp[k, 1] - x) * sin(phi) - (pp[k, 2] - y) * cos(phi))^2 / rb^2 <=
    1 + 0.1
  sel[k] <- all(in_which)
}

pp <- pp[sel, ]
mid <- cbind(mean(pp[, 1]), mean(pp[, 2]))
seglines <- matrix(NA, ncol = 2, nrow = 0)

ang <- atan2(pp[, 1] - mid[1], pp[, 2] - mid[2])
ord <- order(ang)

pp <- pp[ord, ]

j <- nrow(pp)
for (i in seq_len(nrow(pp))) {
  k <- intersect(pp[i, 3:4], pp[j, 3:4])
  start <- atan2(pp[j, 2] - y[k], pp[j, 1] - x[k])
  stop <- atan2(pp[i, 2] - y[k], pp[i, 1] - x[k])
  arc <- ellipse_arc(
    c(a[k], b[k]),
    c(x[k], y[k]),
    theta = phi[k],
    rng = c(start, stop)
  )
  seglines <- rbind(seglines, arc)
  j <- i
}

theta <- seq.int(0, 2 * pi, length.out = 200L)
x0 <- as.vector(sapply(seq_along(x), function(i) {
  x[i] + ra[i] * cos(theta) * cos(phi[i]) - rb[i] * sin(theta) * sin(phi[i])
}))
y0 <- as.vector(sapply(seq_along(y), function(i) {
  y[i] + rb[i] * sin(theta) * cos(phi[i]) + ra[i] * cos(theta) * sin(phi[i])
}))

xyplot(
  y ~ x,
  data = ee,
  asp = "iso",
  xlim = extendrange(xlim, f = 0.01),
  ylim = extendrange(ylim, f = 0.01),
  scales = list(draw = FALSE),
  xlab = NULL,
  ylab = NULL,
  par.settings = list(axis.line = list(col = "transparent")),
  panel = function(x, y, ...) {
    grid.polygon(
      x0,
      y0,
      id.lengths = rep(200, 3),
      default.units = "native",
      gp = grid::gpar(fill = "transparent")
    )
    panel.polygon(seglines, col = "slategray2")
    panel.polygon(pp[, 1:2], col = "grey90")
    panel.points(mid, pch = 4, col = 1, cex = 1.5)
    panel.points(
      pp[, 1:2, drop = FALSE],
      col = 1,
      pch = 21,
      fill = "white",
      cex = 1.5
    )
  }
)
```

We continue by ordering the points around their centroid. It is then trivial to
find the area of the polygon section since it is always convex. Now, because
each elliptical segment is formed from the arcs that connect successive points,
we can establish the segments' areas algorithmically [@Eberly_2016]. For each
ellipse and its related pair of points (located at angles $\theta_0$ and
$\theta_1$ from the semimajor axis), we proceed to find its area by

1. centering the ellipse at $(0, 0)$,
2. normalizing its rotation, which is not needed to compute the area,
3. integrating the ellipse over [$0,\theta_0$] and [$0,\theta_1$], producing
   elliptical sectors $F(\theta_0)$ and $F(\theta_1)$,
4. subtracting the smaller ($F(\theta_0$)) of these sectors from the larger
   ($F(\theta_0$), and
5. subtracting the triangle section to finally find the segment area,
   $$
   F(\theta_1) - F(\theta_0) - \frac{1}{2}\left|x_1y_0 - x_0y_1\right|,
   $$
   where
   $$
   F(\theta) = \frac{a}{b}\left[ \theta - \arctan{\left(\frac{(b - a)\sin{2\theta}}{b + a +(b - a )\cos{2\theta}} \right)}\right].
   $$

This procedure is illustrated in the following figure. Note that there are
situations where this algorithm is altered, such that when the sector angle
ranges beyond $\pi$---we refer the interested reader to @Eberly_2016.

```{r}
#| label: ellipsesegment
#| fig-cap: "The elliptical segment in blue is found by first subtracting the elliptical sector from $(a, 0)$ to $\\theta_0$ from the one from $(a, 0)$ to $\\theta_1$ and then subtracting the triangle part (in grey)."

ellipse <- ellipse(c(1, 0.6), c(0, 0), 0)
i0 <- 15
i1 <- 45

tri <- rbind(
  cbind(0, 0),
  ellipse[i0, , drop = FALSE],
  ellipse[i1, , drop = FALSE]
)

xyplot(
  1 ~ 1,
  xlim = c(-1.2, 1.2),
  ylim = c(-0.8, 0.8),
  asp = "iso",
  xlab = NULL,
  ylab = NULL,
  par.settings = list(axis.line = list(col = "transparent")),
  scales = list(draw = FALSE),
  panel = function(x, y, ...) {
    panel.grid(x = 0, y = 0, h = 1, v = 1)
    panel.lines(ellipse, col = 1)
    panel.polygon(ellipse[i0:i1, ], col = "steelblue1", alpha = 0.5)
    panel.polygon(tri, col = "grey95")
    panel.points(ellipse[c(i0, i1), ], col = 1, pch = 21, fill = "white")
    panel.text(
      ellipse[c(i0, i1), ],
      adj = c(-0.5, -0.5),
      labels = c(
        expression(italic(theta[0])),
        expression(italic(theta[1]))
      )
    )
    pBrackets::grid.brackets(1, 0, 0, 0, h = 0.05, type = 1)
    pBrackets::grid.brackets(0, 0, 0, 0.6, h = 0.04)
    panel.text(0.5, unit(-0.05, "npc"), labels = "a", font = 3, pos = 1)
    panel.text(-0.05, unit(0.3, "native"), labels = "b", font = 3, pos = 2)
  }
)
```

Finally, the area of the overlap is then obtained by adding the area of the
polygon and all the elliptical arcs together.

Note that this does not yet give us the areas that we require, namely $A$: the
area-equivalents to the set intersections and relative complements from our
definition of the intersections. For this, we must decompose the overlap areas
so that each area maps uniquely to a subspace of the set configuration. This,
however, is simply a matter of transversing down the hierarchy of overlaps and
subtracting the higher-order overlaps from the lower-order ones. For a three-set
relationship of sets $A$, $B$, and $C$, for instance, this means subtracting the
$A\cap B \cap C$ overlap from the $A \cap B$ one to retrieve the equivalent of
$(A \cap B) \setminus C$.

The exact algorithm may in rare instances^[1 out of approximately 7000 in our
simulations.], break down, the culprit being numerical precision issues that
occur when ellipses are tangent or completely overlap. In these cases, the
algorithm will approximate the area of the involved overlap by

1. spreading points across the ellipses using Vogel's method.
2. identifying the points that are inside the intersection via the inequality $$
    \begin{aligned}
    &\frac{\left[(x-h)\cos{\phi}+(y-k)\sin{\phi} \right]^2}{a^2} + 
    &\quad \frac{\left[(x-h) \sin{\phi}-(y-k)\cos{\phi}\right]^2}{b^2} < 1,
    \end{aligned}
   $$ where $x$ and $y$ are the coordinates of the sampled points, and finally
3. approximating the area by multiplying the proportion of points inside the
   overlap with the area of the ellipse.

With this in place, we are now able to compute the areas of all intersections
and their relative complements, $\omega$, up to numerical precision.

### Final optimization

We feed the initial layout to the optimizer---once again we employ `nlm()` from
**stats** but now also provide the option to use ellipses rather than circles,
allowing the "circles" to rotate and the relation between the semiaxes to vary,
altogether rendering five parameters to optimize per set and ellipse (or three
if we restrict ourselves to circles). For each iteration of the optimizer, the
areas of all intersections are analyzed and a measure of loss returned. The loss
we use is the same as in **venneuler** [@Frederickson_2016], namely the residual
sums of squares.

If the fitted diagram is still inexact after the procedure, we offer a final
step in which we pass the parameters on to a last-ditch optimizer. The weapon of
choice^[We conducted thorough benchmarking, that we opt not to report here, to
decide upon an algorithm for this step.] is *stress* [@wilkinson_exact_2012],
which is also the loss metric we use in our final optimization step and is used
in **venneuler**, as well as *diagError* [@Micallef_2014a], which is used by
**eulerAPE**.

The stress metric is not easily grasped but can be transformed into a rough
analogue of the correlation coefficient via $r = \sqrt{1-\text{stress}^2}$.

diagError, meanwhile, is given by

$$
{\max_{i = 1, 2, \dots, n}}\left|
\frac{\omega_i}{\sum_{i=1}^n \omega_i} - \frac{A_i}{\sum_{i=1}^n A_i} \right|,
$$

which is the maximum *absolute* difference of the proportion of any $\omega$ to
the respective unique area of the diagram.

## References

---
title: "Venn diagrams with eulerr"
author: "Johan Larsson"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Venn diagrams with eulerr}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.show = "hold",
  fig.width = 3.2,
  fig.aling = "center"
)
```

Venn diagrams are specialized Euler diagrams. Unlike Euler diagrams,
they require that all intersections are represented. In most
implementations---including **eulerr**'s---they are also not
area-proportional. 

The first requirement is often prohibitive in terms of interpretability since
we often waste a considerable amount of space on the diagram's canvas with
intersections that just as well might be represented by their absence.
An area-proportional Euler diagram is often much more intuitive and,
for relatively sparse inputs, much easier to interpret. The property
of being area-proportional may sometimes, however, be treacherous, at least
if the viewer isn't advised of the diagram's faults. 

In such instances, it is often better to give up on area-proportionality 
and use a Venn diagram. It might not be as easy on the eye, but at least
will be interpreted correctly.

**eulerr** only supports diagrams for up to 5 sets. Part of the reason is
practical. **eulerr** is built around ellipses and ellipses are only
good for Venn diagrams with at most 5 sets. The other part of the
reason has to do with usability. A five-set diagram is already 
stretching it in terms of what we can reasonably expect the viewer to
be able to decipher. And despite the hilarity of [banana-shaped six-set Venn diagrams](https://www.nature.com/articles/nature11241/figures/4), monstrosities like that are better left in the dark.

## Examples of Venn and Euler diagrams

We will now look at some Venn diagrams and their respective Euler diagrams.

```{r two}
library(eulerr)
set.seed(1)

s2 <- c(A = 1, B = 2)

plot(venn(s2))
plot(euler(s2), quantities = TRUE)
```

```{r three}
plot(venn(fruits[, 1:3]))
plot(euler(fruits[, 1:3], shape = "ellipse"), quantities = TRUE)
```

```{r four}
s4 <- list(a = c(1, 2, 3),
           b = c(1, 2),
           c = c(1, 4),
           e = c(5))
plot(venn(s4))
plot(euler(s4, shape = "ellipse"), quantities = TRUE)
```

```{r five}
plot(venn(organisms))
plot(euler(organisms, shape = "ellipse"), quantities = TRUE)
```

As you can see in the last plot, there are cases where Euler
diagrams can be misleading. Despite the algorithm attempting its best
to make the diagram area-proportional, the constraints imposed by the 
geometry of the ellipses prevent a perfect fit. This is probably
a case where a Venn diagram makes for a good alternative. In the author's
opinion, the opposite is true for the rest.

---
title: "Venn diagrams with eulerr"
author: "Johan Larsson"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Venn diagrams with eulerr}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| label: setup
#| include: false

knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.show = "hold",
  fig.width = 3.2,
  fig.aling = "center"
)
```

Venn diagrams are specialized Euler diagrams. Unlike Euler diagrams, they
require that all intersections are represented. In most
implementations---including **eulerr**'s---they are also not area-proportional.

The first requirement is often prohibitive in terms of interpretability since we
often waste a considerable amount of space on the diagram's canvas with
intersections that just as well might be represented by their absence. An
area-proportional Euler diagram is often much more intuitive and, for relatively
sparse inputs, much easier to interpret. The property of being area-proportional
may sometimes, however, be treacherous, at least if the viewer isn't advised of
the diagram's faults.

In such instances, it is often better to give up on area-proportionality and use
a Venn diagram. It might not be as easy on the eye, but at least will be
interpreted correctly.

**eulerr** only supports diagrams for up to 5 sets. Part of the reason is
practical. **eulerr** is built around ellipses and ellipses are only good for
Venn diagrams with at most 5 sets. The other part of the reason has to do with
usability. A five-set diagram is already stretching it in terms of what we can
reasonably expect the viewer to be able to decipher. And despite the hilarity of
[banana-shaped six-set Venn
diagrams](https://www.nature.com/articles/nature11241/figures/4), monstrosities
like that are better left in the dark.

## Examples of Venn and Euler diagrams

We will now look at some Venn diagrams and their respective Euler diagrams.

```{r}
#| label: two

library(eulerr)
set.seed(1)

s2 <- c(A = 1, B = 2)

plot(venn(s2))
plot(euler(s2), quantities = TRUE)
```

```{r}
#| label: three

plot(venn(fruits[, 1:3]))
plot(euler(fruits[, 1:3], shape = "ellipse"), quantities = TRUE)
```

```{r}
#| label: four

s4 <- list(a = c(1, 2, 3), b = c(1, 2), c = c(1, 4), e = c(5))
plot(venn(s4))
plot(euler(s4, shape = "ellipse"), quantities = TRUE)
```

```{r}
#| label: five

plot(venn(organisms))
plot(euler(organisms, shape = "ellipse"), quantities = TRUE)
```

As you can see in the last plot, there are cases where Euler diagrams can be
misleading. Despite the algorithm attempting its best to make the diagram
area-proportional, the constraints imposed by the geometry of the ellipses
prevent a perfect fit. This is probably a case where a Venn diagram makes for a
good alternative. In the author's opinion, the opposite is true for the rest.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/venn.R
\name{venn}
\alias{venn}
\alias{venn.default}
\alias{venn.table}
\alias{venn.data.frame}
\alias{venn.matrix}
\alias{venn.list}
\title{Venn diagrams}
\usage{
venn(combinations, ...)

\method{venn}{default}(
  combinations,
  input = c("disjoint", "union"),
  names = letters[length(combinations)],
  ...
)

\method{venn}{table}(combinations, ...)

\method{venn}{data.frame}(
  combinations,
  weights = NULL,
  by = NULL,
  sep = "_",
  factor_names = TRUE,
  ...
)

\method{venn}{matrix}(combinations, ...)

\method{venn}{list}(combinations, ...)
}
\arguments{
\item{combinations}{set relationships as a named numeric vector, matrix, or
data.frame (see \strong{methods (by class)})}

\item{...}{arguments passed down to other methods}

\item{input}{type of input: disjoint identities
(\code{'disjoint'}) or unions (\code{'union'}).}

\item{names}{a character vector for the names of each set of the same
length as 'combinations'. Must not be \code{NULL} if \code{combinations} is a
one-length numeric.}

\item{weights}{a numeric vector of weights of the same length as
the number of rows in \code{combinations}.}

\item{by}{a factor or character matrix to be used in \code{\link[base:by]{base::by()}} to
split the data.frame or matrix of set combinations}

\item{sep}{a character to use to separate the dummy-coded factors
if there are factor or character vectors in 'combinations'.}

\item{factor_names}{whether to include factor names when
constructing dummy codes}
}
\value{
Returns an object of class \verb{'eulerr_venn', 'venn', 'euler'} with items
\item{shapes}{a data frame of the precomputed ellipse parameters (one
row per set, columns \verb{type, h, k, a, b, phi}). \code{venn()} always uses
ellipses.}
\item{ellipses}{the legacy 5-column data frame
(\verb{h, k, a, b, phi}) — kept for back-compat alongside the canonical
\code{shapes} slot.}
\item{original.values}{set relationships in the input}
\item{fitted.values}{set relationships in the solution}
}
\description{
This function fits Venn diagrams using an interface that is
almost identical to \code{\link[=euler]{euler()}}. Strictly speaking,
Venn diagrams are Euler diagrams where every intersection is visible,
regardless of whether or not it is zero. In almost every incarnation of
Venn diagrams, however, the areas in the diagram are also
\emph{non-proportional} to the input; this is also the case here.
}
\section{Methods (by class)}{
\itemize{
\item \code{venn(default)}: a named numeric vector, with
combinations separated by an ampersand, for instance \code{A&B = 10}.
Missing combinations are treated as being 0.

\item \code{venn(table)}: A table with \code{max(dim(x)) < 3}.

\item \code{venn(data.frame)}: a \code{data.frame} of logicals, binary integers, or
factors.

\item \code{venn(matrix)}: a matrix that can be converted to a data.frame of logicals
(as in the description above) via \code{\link[base:as.data.frame]{base::as.data.frame.matrix()}}.

\item \code{venn(list)}: a list of vectors, each vector giving the contents of
that set (with no duplicates). Vectors in the list do not need to be named.

}}
\examples{
# The trivial version
f1 <- venn(5, names = letters[1:5])
plot(f1)

# Using data (a numeric vector)
f2 <- venn(c(A = 1, "B&C" = 3, "A&D" = 0.3))

# The table method
venn(pain, factor_names = FALSE)

# Using grouping via the 'by' argument through the data.frame method
venn(fruits, by = list(sex, age))


# Using the matrix method
venn(organisms)

# Using weights
venn(organisms, weights = c(10, 20, 5, 4, 8, 9, 2))

# A venn diagram from a list of sample spaces (the list method)
venn(plants[c("erigenia", "solanum", "cynodon")])
}
\seealso{
\code{\link[=plot.eulerr_venn]{plot.eulerr_venn()}}, \code{\link[=print.eulerr_venn]{print.eulerr_venn()}}, \code{\link[=euler]{euler()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/label_placement.R
\name{resolve_gap_native}
\alias{resolve_gap_native}
\title{Resolve a \code{gap} option to a numeric in native units inside the current
measurement viewport. \code{NULL} → falls back to \code{padding_native} so the
visible leader-tip gap matches the spacing between label and quantity.
A \code{grid::unit} value converts to native; a bare numeric is interpreted
as \code{lines} (same convention as \code{eulerr_options()$padding}).}
\usage{
resolve_gap_native(gap, padding_native)
}
\description{
Resolve a \code{gap} option to a numeric in native units inside the current
measurement viewport. \code{NULL} → falls back to \code{padding_native} so the
visible leader-tip gap matches the spacing between label and quantity.
A \code{grid::unit} value converts to native; a bare numeric is interpreted
as \code{lines} (same convention as \code{eulerr_options()$padding}).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/label_placement.R
\name{run_placement_pass}
\alias{run_placement_pass}
\title{Single placement pass: measure tags, call the Rust FFI, return the
placement records and the canvas bbox returned by eunoia.}
\usage{
run_placement_pass(
  centers,
  container_data,
  shapes,
  labels_gp,
  quantities_gp,
  annotations_gp,
  padding,
  placement_opts,
  do_complement_label,
  xlim,
  ylim,
  n_vertices,
  label_precision
)
}
\description{
Single placement pass: measure tags, call the Rust FFI, return the
placement records and the canvas bbox returned by eunoia.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tag-grobs.R
\docType{data}
\name{EULER_PANEL_PAD_PT}
\alias{EULER_PANEL_PAD_PT}
\title{Fixed padding in pt for the panel viewport scale.}
\format{
An object of class \code{numeric} of length 1.
}
\usage{
EULER_PANEL_PAD_PT
}
\description{
Geometry / labels that land flush with the bbox would otherwise be
clipped at the device edge by stroke width and anti-aliasing — both
of which are in device units, not native units, so the padding is in
pt rather than a fraction of the coordinate range.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/label_placement.R
\name{apply_label_placement}
\alias{apply_label_placement}
\title{Run eunoia label placement, expanding limits so exterior labels are
not clipped. Drives one initial pass plus one re-measure pass when
the limits widened by more than \code{re_measure_threshold} on the short
side. Updates \code{centers} (and the complement slot on \code{container_data})
in place with placed \verb{(x, y)} plus \code{kind}, \code{tether_x}, \code{tether_y},
\code{leader_end_x}, \code{leader_end_y}.}
\usage{
apply_label_placement(
  centers,
  container_data,
  shapes,
  labels,
  quantities,
  annotations = NULL,
  placement_opts = NULL,
  do_complement_label = FALSE,
  limits,
  n_vertices,
  label_precision,
  re_measure_threshold = 0.01
)
}
\description{
When \code{placement_opts} is \code{NULL}, defaults to eunoia's raycast + POI
tether.
}
\details{
Returns a list with \code{centers}, \code{container_data}, and \code{limits}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tag-grobs.R
\name{makeContent.EulerTags}
\alias{makeContent.EulerTags}
\title{Re-place tags at draw time so resizing the device doesn't clip labels.}
\usage{
\method{makeContent}{EulerTags}(x)
}
\description{
The panel viewport is active when \code{makeContent} fires, so we can
measure each tag's footprint in current native units via
\code{grid::convertWidth(grobWidth(...), "native")} and call eunoia's
\code{\link[=place_euler_labels]{place_euler_labels()}} again. On resize, grid invalidates the tree
and \code{makeContent} re-runs, so the placement tracks the current
device automatically.
}
\details{
Tags whose measured size is zero (or whose anchor / kind eunoia
can't compute) keep the positions they were built with — typically
the setup-time placement stored on \code{centers$x} / \code{centers$y}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extendr-wrappers.R
\name{place_euler_labels}
\alias{place_euler_labels}
\title{Place per-region labels using eunoia's \code{place_labels} API.}
\usage{
place_euler_labels(
  set_names,
  shape,
  h,
  k,
  a,
  b,
  phi,
  width,
  height,
  side,
  container_h,
  container_k,
  container_width,
  container_height,
  n_vertices,
  label_combos,
  label_widths,
  label_heights,
  placement,
  placement_margin,
  placement_iterations,
  placement_min_gap,
  placement_tether,
  placement_leader_gap,
  label_precision
)
}
\description{
Inputs mirror \code{euler_plot_data} for shape geometry and add per-region
label sizes plus placement-strategy options. Returns, parallel to
\code{label_combos}:
}
\details{
\itemize{
\item \code{anchor_x} / \code{anchor_y} — placed label anchor (NA on miss);
\item \code{kind} — one of \code{"interior"}, \code{"exterior_raycast"},
\code{"exterior_force_directed"}, \code{"exterior_elbow"}; \code{""} if no
placement was produced;
\item \code{tether_x} / \code{tether_y} — tether point for the leader line (NA for
interior placements / misses).
\item \code{leader_end_x} / \code{leader_end_y} — point on the label box AABB where
the leader terminates (NA for interior placements / misses).
\item \code{leader_waypoints_x} / \code{leader_waypoints_y} / \code{leader_waypoints_lengths}
— concatenated waypoint coordinates and per-label counts. Empty for
straight leaders; carries one knee point per elbow placement.
}

Plus a canvas bbox (\code{canvas_bbox_h/k/width/height}) from eunoia's
\code{placements_bbox} — NaN when no placements were produced — for the R
side to grow \code{xlim/ylim} so exterior labels are never clipped.

The complement region is requested with \code{""} in \code{label_combos}; when
\verb{container_*} are non-NULL the spec is built with \code{.complement(1.0)}
so eunoia emits the empty \code{Combination} from \code{decompose_regions}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tag-grobs.R
\name{build_leader_grob}
\alias{build_leader_grob}
\title{Build the polyline leader for an exterior tag, or \code{\link[grid:grid.null]{grid::nullGrob()}}
for interior / missing-tether placements.}
\usage{
build_leader_grob(
  ax,
  ay,
  kind,
  tx,
  ty,
  lend_x,
  lend_y,
  waypoints_x = numeric(0),
  waypoints_y = numeric(0),
  leader_gp_list,
  fallback_gp,
  name
)
}
\description{
Draws the polyline \verb{tether → waypoints[1..] → leader_end}. For
straight leaders (raycast / force-directed) \verb{waypoints_*} are empty,
so the polyline collapses to the single \verb{tether → leader_end}
segment. For elbow leaders eunoia emits one knee waypoint, producing
the orthogonal \verb{tether → knee → leader_end} bend.
}
\details{
Terminates at \verb{(lend_x, lend_y)} — the point on the label box AABB
edge supplied by eunoia (\code{LabelPlacement::leader_end}). Falls back
to the anchor when the leader endpoint isn't finite so older /
partial placement results still draw something sensible.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.euler.R
\name{print.eulerr_venn}
\alias{print.eulerr_venn}
\alias{print.venn}
\title{Print a summary of a Venn diagram}
\usage{
\method{print}{eulerr_venn}(x, round = 3, vsep = strrep("-", 0.75 * getOption("width")), ...)

\method{print}{venn}(...)
}
\arguments{
\item{x}{an object of class \code{'eulerr_venn'}}

\item{round}{number of digits to round the ellipse specification to}

\item{vsep}{character string to paste in between \code{euler} objects
when \code{x} is a nested \code{euler} object}

\item{...}{arguments passed to \code{\link[base:print.dataframe]{base::print.data.frame()}}}
}
\value{
Summary statistics of the fitted Venn diagram are printed to
screen.
}
\description{
This function is responsible for printing objects from
from \code{\link[=venn]{venn()}} and provides a simple description of the number of
sets and the specifications for the ellipses of the Venn diagram.
}
\examples{
venn(organisms)
}
\seealso{
\code{\link[=venn]{venn()}}, \code{\link[base:print.dataframe]{base::print.data.frame()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/label_placement.R
\name{open_measurement_viewport}
\alias{open_measurement_viewport}
\title{Open a temporary grid measurement device + viewport.}
\usage{
open_measurement_viewport(xlim, ylim)
}
\description{
Returns an idempotent closer thunk that pops the viewport and
closes the null PDF device. Used so we can call
\code{grid::convertWidth(grobWidth(...), "native", ...)} at setup time —
i.e. before \code{plot.eulergram()} ever opens a real device — to size
the label boxes that drive label placement.
}
\details{
Always opens its own off-screen PDF rather than reusing the caller's
active device. Pushing a viewport onto the caller's device adds an
entry to its display list, which knitr's plot capture
treats as visible change and emits as an extra blank plot before
the real \code{plot.eulergram()} draws. The off-screen PDF keeps
measurement entirely out of the user's display list.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tag-grobs.R
\name{setup_complement_tag}
\alias{setup_complement_tag}
\title{Setup the complement-count tag, in the same shape as a region tag so
it shares one \code{\link[=makeContent.EulerTags]{makeContent.EulerTags()}} pass.}
\usage{
setup_complement_tag(container_data, complement, number)
}
\description{
Acts as a quantity-only tag with \code{combo_key = ""}. The complement
label text comes either from \code{complement$label} (user override) or
from \code{container_data$quantity_label} (the fitted complement count).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{rescale}
\alias{rescale}
\title{Rescale values to new range}
\usage{
rescale(x, new_min, new_max)
}
\arguments{
\item{x}{numeric vector}

\item{new_min}{new min}

\item{new_max}{new max}
}
\value{
Rescaled vector
}
\description{
Rescale values to new range
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geometry.R
\name{get_bounding_box}
\alias{get_bounding_box}
\title{Get the bounding box for a row-wise collection of fitted shapes.}
\usage{
get_bounding_box(shapes, k = NULL, a = NULL, b = NULL, phi = NULL)
}
\arguments{
\item{shapes}{the \verb{$shapes} data frame of a fitted euler object (after
dropping NA rows for empty sets), or — for legacy callers — the
numeric \code{h} vector. When \code{h} is numeric the function falls back to
the legacy ellipse signature for back-compat with external code.}

\item{k, a, b, phi}{legacy ellipse parameters; only used when \code{shapes} is
numeric (the legacy signature).}
}
\value{
The bounding box as a list with \code{xlim} and \code{ylim}.
}
\description{
Dispatches on the shape kind. Ellipse/circle use the rotated-ellipse
bounding-box formula; rectangle/square use width/height (or side)
directly since both are axis-aligned in eunoia.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{plants}
\alias{plants}
\title{Plants}
\format{
A \link{list} with 33,721 plants, each containing a character vector
listing the states in the US and Canada in which they occur. The names in
the list specify the species or genus of the plant.
}
\source{
USDA, NRCS. 2008. The PLANTS Database, 31
December 2008). National Plant Data Center, Baton Rouge, LA 70874-4490 USA.

Dua, D. and Karra Taniskidou, E. (2017). UCI Machine Learning
Repository \url{http://archive.ics.uci.edu/ml/}. Irvine, CA: University of
California, School of Information and Computer Science.
}
\usage{
plants
}
\description{
Data on plants and the states in the US and Canada they occur in.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geometry.R
\name{shape_bounding_box}
\alias{shape_bounding_box}
\title{Per-shape bounding box dispatch. Reads the \code{type} tag on \code{shapes} (rows
are assumed to share a tag since a diagram fixes one shape kind), then
picks the appropriate width/height calculation. Falls back to the
rotated-ellipse formula when the type is unknown so external callers
constructing ad-hoc \verb{$shapes} frames still get a sensible box.}
\usage{
shape_bounding_box(shapes)
}
\description{
Per-shape bounding box dispatch. Reads the \code{type} tag on \code{shapes} (rows
are assumed to share a tag since a diagram fixes one shape kind), then
picks the appropriate width/height calculation. Falls back to the
rotated-ellipse formula when the type is unknown so external callers
constructing ad-hoc \verb{$shapes} frames still get a sensible box.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/euler.R
\name{euler}
\alias{euler}
\alias{euler.default}
\alias{euler.data.frame}
\alias{euler.matrix}
\alias{euler.table}
\alias{euler.list}
\title{Area-proportional Euler diagrams}
\usage{
euler(combinations, ...)

\method{euler}{default}(
  combinations,
  input = c("disjoint", "union"),
  shape = c("circle", "ellipse", "rectangle", "square"),
  loss = c("sum_squared", "sum_absolute", "sum_absolute_region_error",
    "sum_squared_region_error", "max_absolute", "max_squared", "root_mean_squared",
    "stress", "diag_error"),
  loss_aggregator = NULL,
  complement = NULL,
  control = list(),
  ...
)

\method{euler}{data.frame}(
  combinations,
  weights = NULL,
  by = NULL,
  sep = "_",
  factor_names = TRUE,
  ...
)

\method{euler}{matrix}(combinations, ...)

\method{euler}{table}(combinations, ...)

\method{euler}{list}(combinations, ...)
}
\arguments{
\item{combinations}{set relationships as a named numeric vector, matrix, or
data.frame (see \strong{methods (by class)})}

\item{...}{arguments passed down to other methods}

\item{input}{type of input: disjoint identities
(\code{'disjoint'}) or unions (\code{'union'}).}

\item{shape}{geometric shape used in the diagram}

\item{loss}{type of loss to minimize over. The default,
\code{"sum_squared"}, minimizes the sum of squared errors. The available
options mirror the loss functions exposed by the \code{eunoia} Rust crate
that powers the optimizer:
\itemize{
\item \code{"sum_squared"} --- normalized sum of squared errors (default).
\item \code{"sum_absolute"} --- normalized sum of absolute errors.
\item \code{"sum_absolute_region_error"} --- normalized sum of absolute
region errors.
\item \code{"sum_squared_region_error"} --- normalized sum of squared region
errors.
\item \code{"max_absolute"} --- normalized maximum absolute error.
\item \code{"max_squared"} --- normalized maximum squared error.
\item \code{"root_mean_squared"} --- normalized root-mean-squared error.
\item \code{"stress"} --- venneuler-style stress.
\item \code{"diag_error"} --- eulerAPE-style \code{diagError}.
}}

\item{loss_aggregator}{deprecated; use \code{loss} directly instead. Pre-1.0
code that combined \code{loss} (\code{"square"}/\code{"abs"}/\code{"region"}) with
\code{loss_aggregator} (\code{"sum"}/\code{"max"}) still works but emits a warning;
the combination is mapped to the equivalent new \code{loss} value.}

\item{complement}{an optional single non-negative number giving the area
of the \emph{complement} — that is, the universe outside every named set.
When supplied, the fitter jointly optimizes a containing rectangle
together with the diagram shapes so that the area of the rectangle
minus the union of (clipped) shapes matches \code{complement}. This is the
classical "everything not in any set" region; see \code{\link[=plot.euler]{plot.euler()}} for
how it is rendered. Defaults to \code{NULL} (no container; classical
shape-only fit). Not supported for \code{\link[=venn]{venn()}}.}

\item{control}{a list of control parameters.
\itemize{
\item \code{extraopt}: should the global-search fallback optimizer (CMA-ES) kick
in when the primary optimizer's \code{diagError} exceeds \code{extraopt_threshold}?
The default is \code{TRUE} for three-set ellipse fits and \code{FALSE} otherwise.
\item \code{extraopt_threshold}: threshold, in terms of \code{diagError}, for when
the CMA-ES fallback kicks in. A value of 0 means it will kick in for
\emph{any} error; a value of 1 means it will never kick in. Default \code{0.001}.
\item \code{tolerance}: convergence tolerance passed to the underlying solver.
Tighter values give more accurate fits at higher cost. Default \code{1e-8}.
\item \code{max_sets}: maximum number of sets the underlying engine will accept.
Defaults to \code{NULL}, which uses the engine's built-in default of 32.
Region masks are stored in a bitset, so values may be raised up to 63
(the absolute hard cap). Going higher is rarely useful in practice
since fully-overlapping diagrams have \code{2^n - 1} regions.
}}

\item{weights}{a numeric vector of weights of the same length as
the number of rows in \code{combinations}.}

\item{by}{a factor or character matrix to be used in \code{\link[base:by]{base::by()}} to
split the data.frame or matrix of set combinations}

\item{sep}{a character to use to separate the dummy-coded factors
if there are factor or character vectors in 'combinations'.}

\item{factor_names}{whether to include factor names when
constructing dummy codes}
}
\value{
A list object of class \code{'euler'} with the following parameters.
\item{shapes}{a data frame of fitted shape parameters. One row per set
with a \code{type} column (one of \code{"circle"}, \code{"ellipse"}, \code{"rectangle"},
\code{"square"}), the center coordinates \code{h} and \code{k}, and the
shape-specific columns: \code{a}, \code{b}, \code{phi} for ellipses/circles; \code{width}
and \code{height} for rectangles; \code{side} (plus mirrored \code{width}/\code{height})
for squares. Columns that don't apply to the chosen shape are \code{NA}.}
\item{ellipses}{for \code{shape = "circle"} and \code{shape = "ellipse"} fits,
the legacy 5-column data frame of \code{h}, \code{k}, \code{a}, \code{b}, \code{phi}. This
slot is deprecated in favour of \code{shapes} and is not populated for
rectangle/square fits.}
\item{original.values}{set relationships in the input}
\item{fitted.values}{set relationships in the solution}
\item{residuals}{residuals}
\item{regionError}{the difference in percentage points between each
disjoint subset in the input and the respective area in the output}
\item{diagError}{the largest \code{regionError}}
\item{stress}{normalized residual sums of squares}
}
\description{
Fit Euler diagrams (a generalization of Venn diagrams) using numerical
optimization to find exact or approximate solutions to a specification of set
relationships. The shape of the diagram may be a circle, an ellipse, an
axis-aligned rectangle, or an axis-aligned square.
}
\details{
If the input is a matrix or data frame and argument \code{by} is specified,
the function returns a list of euler diagrams.

The function minimizes the residual sums of squares,
\deqn{
  \sum_{i=1}^n (A_i - \omega_i)^2,
}{
  \sum (A_i - \omega_i)^2,
}
by default, where \eqn{\omega_i} the size of the ith disjoint subset, and
\eqn{A_i} the corresponding area in the diagram, that is, the unique
contribution to the total area from this overlap. The loss function
can, however, be controlled via the \code{loss} argument.

\code{\link[=euler]{euler()}} also returns \code{stress} (from \pkg{venneuler}), as well as
\code{diagError}, and \code{regionError} from \pkg{eulerAPE}.

The \emph{stress} statistic is computed as

\deqn{
  \frac{\sum_{i=1}^n (A_i - \beta\omega_i)^2}{\sum_{i=1}^n A_i^2},
}{
  \sum (A_i - \beta\omega_i)^2 / \sum A_i^2,
}
where
\deqn{
  \beta = \sum_{i=1}^n A_i\omega_i / \sum_{i=1}^n \omega_i^2.
}{
  \beta = \sum A_i\omega_i / \sum \omega_i^2.
}

\code{regionError} is computed as

\deqn{
  \left| \frac{A_i}{\sum_{i=1}^n A_i} - \frac{\omega_i}{\sum_{i=1}^n \omega_i}\right|.
}{
  max|A_i / \sum A  - \omega_i / \sum \omega|.
}

\code{diagError} is simply the maximum of regionError.
}
\section{Methods (by class)}{
\itemize{
\item \code{euler(default)}: a named numeric vector, with
combinations separated by an ampersand, for instance \code{A&B = 10}.
Missing combinations are treated as being 0.

\item \code{euler(data.frame)}: a \code{data.frame} of logicals, binary integers, or
factors.

\item \code{euler(matrix)}: a matrix that can be converted to a data.frame of logicals
(as in the description above) via \code{\link[base:as.data.frame]{base::as.data.frame.matrix()}}.

\item \code{euler(table)}: A table with \code{max(dim(x)) < 3}.

\item \code{euler(list)}: a list of vectors, each vector giving the contents of
that set (with no duplicates). Vectors in the list must be named.

}}
\examples{
# Fit a diagram with circles
combo <- c(A = 2, B = 2, C = 2, "A&B" = 1, "A&C" = 1, "B&C" = 1)
fit1 <- euler(combo)

# Investigate the fit
fit1

# Refit using ellipses instead
fit2 <- euler(combo, shape = "ellipse")

# Investigate the fit again (which is now exact)
fit2

# Plot it
plot(fit2)

# A set with no perfect solution
euler(c(
  "a" = 3491, "b" = 3409, "c" = 3503,
  "a&b" = 120, "a&c" = 114, "b&c" = 132,
  "a&b&c" = 50
))


# Using grouping via the 'by' argument through the data.frame method
euler(fruits, by = list(sex, age))


# Using the matrix method
euler(organisms)

# Using weights
euler(organisms, weights = c(10, 20, 5, 4, 8, 9, 2))

# The table method
euler(pain, factor_names = FALSE)

# A euler diagram from a list of sample spaces (the list method)
euler(plants[c("erigenia", "solanum", "cynodon")])
}
\references{
Wilkinson L. Exact and Approximate Area-Proportional Circular
Venn and Euler Diagrams. IEEE Transactions on Visualization and Computer
Graphics (Internet). 2012 Feb (cited 2016 Apr 9);18(2):321-31. Available
from: \doi{10.1109/TVCG.2011.56}

Micallef L, Rodgers P. eulerAPE: Drawing Area-Proportional 3-Venn Diagrams
Using Ellipses. PLOS ONE (Internet). 2014 Jul (cited 2016 Dec
10);9(7):e101717. Available from:
\doi{10.1371/journal.pone.0101717}
}
\seealso{
\code{\link[=plot.euler]{plot.euler()}}, \code{\link[=print.euler]{print.euler()}}, \code{\link[=eulerr_options]{eulerr_options()}}, \code{\link[=venn]{venn()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extendr-wrappers.R
\name{polygon_clip_rust}
\alias{polygon_clip_rust}
\title{Clip a (possibly multi-polygon) subject path against a single clip
polygon. Mirrors the slice of \code{polyclip::polyclip} behavior eulerr
actually uses at the stripe-pattern site.}
\usage{
polygon_clip_rust(subject_x, subject_y, subject_id_lengths, clip_x, clip_y, op)
}
\description{
Clip a (possibly multi-polygon) subject path against a single clip
polygon. Mirrors the slice of \code{polyclip::polyclip} behavior eulerr
actually uses at the stripe-pattern site.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{tally_combinations}
\alias{tally_combinations}
\title{Tally set relationships}
\usage{
tally_combinations(sets, weights)
}
\arguments{
\item{sets}{a data.frame with set relationships and weights}

\item{weights}{a numeric vector}
}
\value{
Calls \code{\link[=euler]{euler()}} after the set relationships have been coerced to a
named numeric vector.
}
\description{
Tally set relationships
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{mix_colors}
\alias{mix_colors}
\title{Blend (average) colors}
\usage{
mix_colors(rcol_in)
}
\arguments{
\item{rcol_in}{a vector of R colors}
}
\value{
A single R color
}
\description{
Blend (average) colors
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_diagram.R
\name{new_shape_frame}
\alias{new_shape_frame}
\title{Allocate a fresh \verb{$shapes} data frame for \code{n_all} sets. Rows for empty
sets keep NA in every column so downstream plotting can detect them via
\code{is.na(shapes$h)} regardless of shape kind.}
\usage{
new_shape_frame(shape, n_all, row_names)
}
\description{
Allocate a fresh \verb{$shapes} data frame for \code{n_all} sets. Rows for empty
sets keep NA in every column so downstream plotting can detect them via
\code{is.na(shapes$h)} regardless of shape kind.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extendr-wrappers.R
\name{fit_euler_diagram}
\alias{fit_euler_diagram}
\title{Fit an Euler diagram using the eunoia Rust library.}
\usage{
fit_euler_diagram(
  combo_names,
  combo_values,
  input,
  shape,
  loss,
  extraopt_threshold,
  tolerance,
  max_sets,
  complement,
  seed
)
}
\description{
Fit an Euler diagram using the eunoia Rust library.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tag-grobs.R
\name{find_eulertags}
\alias{find_eulertags}
\title{Find the \code{EulerTags} child of an \code{EulerPanel}, if any.}
\usage{
find_eulertags(panel)
}
\description{
Find the \code{EulerTags} child of an \code{EulerPanel}, if any.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geometry.R
\name{ellipse_bounding_box}
\alias{ellipse_bounding_box}
\title{Bounding box of a vector of rotated ellipses.}
\usage{
ellipse_bounding_box(h, k, a, b, phi)
}
\description{
Bounding box of a vector of rotated ellipses.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{apply_panel_overrides}
\alias{apply_panel_overrides}
\title{Overlay per-panel overrides onto a styling parameter}
\usage{
apply_panel_overrides(param, override)
}
\arguments{
\item{param}{a list with \verb{$gp} (a \code{grid::gpar}) and optionally \verb{$rot} and/or
\verb{$set_gp}.}

\item{override}{a flat named list of fields to overlay.}
}
\value{
The param list with overlaid fields.
}
\description{
Used to apply \code{by_group} overrides to \code{fills}, \code{patterns}, \code{edges}, \code{labels},
and \code{quantities} for a single panel produced by \code{euler(..., by = ...)}. The
override list contains gpar-level fields (and optionally \code{rot}), which are
overlaid onto \code{param$gp} (and \code{param$set_gp} for shape-mode patterns).
Structural fields are not overlaid here — they must be filtered upstream.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tag-grobs.R
\name{pad_axis_native}
\alias{pad_axis_native}
\title{Pad an axis range by \code{pt_pad} points, converted to native units
against a measurement viewport with the supplied scale. Returns the
original range unchanged if the conversion isn't finite (e.g. zero
range, no device).}
\usage{
pad_axis_native(
  lim,
  pt_pad,
  axis = c("x", "y"),
  layout_pos_row = NULL,
  layout_pos_col = NULL
)
}
\description{
Pad an axis range by \code{pt_pad} points, converted to native units
against a measurement viewport with the supplied scale. Returns the
original range unchanged if the conversion isn't finite (e.g. zero
range, no device).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tag-grobs.R
\name{measure_tag_native}
\alias{measure_tag_native}
\title{Measure one tag's combined AABB in the current viewport's native units.}
\usage{
measure_tag_native(tag, padding_native)
}
\description{
Measure one tag's combined AABB in the current viewport's native units.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setup_geometry.R
\name{setup_geometry}
\alias{setup_geometry}
\title{Compute geometries and label locations}
\usage{
setup_geometry(
  x,
  fills,
  edges,
  labels,
  quantities,
  annotations,
  n,
  merged_sets,
  placement_opts = NULL,
  do_complement_label = FALSE
)
}
\arguments{
\item{x}{an object of class 'euler'}

\item{fills}{fills}

\item{edges}{edges}

\item{labels}{labels}

\item{quantities}{quantities}

\item{n}{number of vertices to use to render each ellipse}

\item{merged_sets}{which sets have been merged?}
}
\value{
a list object with slots for the various objects
}
\description{
Compute geometries and label locations
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eulerr_options.R
\name{eulerr_options}
\alias{eulerr_options}
\title{Get or set global graphical parameters for eulerr}
\usage{
eulerr_options(...)
}
\arguments{
\item{...}{objects to update the global graphical parameters for \pkg{eulerr}
with.}
}
\value{
This function gets or sets updates in the global environment
that are used in \code{\link[=plot.euler]{plot.euler()}}.
}
\description{
This function provides a means to set default parameters for functions
in eulerr. Query \code{\link[=eulerr_options]{eulerr_options()}} (without any
argument) to see all the available options and read more about
the plot-related ones in \code{\link[grid:gpar]{grid::gpar()}} and \code{\link[graphics:par]{graphics::par()}}.
}
\details{
Currently, the following items will be considered:
\describe{
\item{pointsize}{size in pts to be used as basis for fontsizes and
some margin sizes in the resulting plot}#'
\item{fills}{a list of items \code{fill} and \code{alpha}}
\item{patterns}{a list of items \code{type}, \code{angle}, \code{col}, \code{lwd}, and \code{alpha}}
\item{edges}{a list of items \code{col}, \code{alpha}, \code{lex}, \code{lwd}, and \code{lty}}
\item{labels}{a list of items \code{rot},
\code{col}, \code{alpha}, \code{fontsize}, \code{cex}, \code{fontfamily}, \code{fontface},
\code{lineheight}, and \code{font}}
\item{quantities}{a list of items \code{type}, \code{template}, \code{format}, \code{total}, \code{rot},
\code{col}, \code{alpha}, \code{fontsize}, \code{cex}, \code{fontfamily},
\code{lineheight}, and \code{font}}
\item{annotations}{a list of items \code{rot}, \code{col}, \code{alpha}, \code{fontsize},
\code{cex}, \code{fontfamily}, \code{lineheight}, \code{font}, and \code{labels} (a named
character vector keyed by subset name). Used to add a third stacked
text element per region below the quantity.}
\item{strips}{\code{col}, \code{alpha}, \code{fontsize}, \code{cex}, \code{fontfamily},
\code{lineheight}, and \code{font}}
\item{legend}{arguments to \code{\link[grid:legendGrob]{grid::legendGrob()}} as well as \code{col}, \code{alpha},
\code{fontsize}, \code{cex}, \code{symbol_size} (symbol size multiplier, independent of text
size; defaults to \code{cex} if \code{NULL}), \code{fontfamily}, \code{lineheight}, and \code{font}}
\item{main}{arguments to \code{\link[grid:grid.text]{grid::textGrob()}}}
\item{complement}{a list of styling defaults for the container box
and its complement label drawn when \code{\link[=euler]{euler()}} is called with
\verb{complement =}. Items: \code{fill}, \code{alpha}, \code{col}, \code{lty}, \code{lwd}, \code{lex},
\code{fontsize}, \code{cex}, \code{font}, \code{fontfamily}, \code{lineheight}. The default
\code{lty = 2} draws the container with a dashed outline.}
\item{padding}{a \code{\link[grid:unit]{grid::unit()}} giving the padding between various
elements in plots from \code{\link[=plot.euler]{plot.euler()}}, which you can change
if you, for instance, want to increase spacing between labels,
quantities, and percentages.}
\item{composition}{a list controlling how \code{eulergram} objects are
arranged when composed via \code{|} or \code{/}. Contains a single \code{spacing}
item (a \code{\link[grid:unit]{grid::unit()}}) that sets the gap between adjacent plots.}
}
}
\examples{
eulerr_options(edges = list(col = "blue"), fontsize = 10)
eulerr_options(n_threads = 2)
}
\seealso{
\code{\link[=plot.euler]{plot.euler()}}, \code{\link[grid:gpar]{grid::gpar()}}, \code{\link[graphics:par]{graphics::par()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tag-grobs.R
\name{makeContext.EulerPanel}
\alias{makeContext.EulerPanel}
\title{Set the panel viewport's \code{xscale}/\code{yscale} at draw time.}
\usage{
\method{makeContext}{EulerPanel}(x)
}
\description{
Fires before grid pushes the panel viewport. We can therefore
measure labels against the live cell, run eunoia's placement, and
compute a viewport bbox that fits both the diagram and the labels.
On window resize grid invalidates the gTree and \code{makeContext}
re-runs, so the panel grows or shrinks to track the current device
and exterior labels never extend past the viewport edge.
}
\details{
Aspect preservation: the new bbox keeps \code{xrng / yrng} equal to the
geometry's natural aspect (set by \code{\link[=setup_geometry]{setup_geometry()}}) so that
circles render as circles. The smaller dimension is padded if the
label-driven canvas bbox is asymmetric.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{organisms}
\alias{organisms}
\title{Organisms}
\format{
A \link{matrix} with 7 observations, consisting of various organisms,
and 5 variables: \emph{animal, mammal, plant, sea}, and, \emph{spiny},
indicating whether the organism belongs to the category or not.
}
\source{
\url{https://github.com/sysbio-bioinf/VennMaster/blob/master/data_examples/deploy/example1.list}
}
\usage{
organisms
}
\description{
Example data from the \strong{VennMaster} package.
}
\details{
Note that this data is difficult to fit using an Euler diagram, even
if we use ellipses, which is clear if one chooses to study the various
overlaps in the resulting diagrams.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/label_placement.R
\name{measure_tag_sizes}
\alias{measure_tag_sizes}
\title{Measure all candidate tag sizes (regions + optional complement) inside
a fresh measurement viewport scaled to \code{xlim}/\code{ylim}.}
\usage{
measure_tag_sizes(
  centers,
  do_complement_label,
  complement_label,
  labels_gp,
  quantities_gp,
  annotations_gp,
  padding,
  gap,
  xlim,
  ylim
)
}
\description{
Measure all candidate tag sizes (regions + optional complement) inside
a fresh measurement viewport scaled to \code{xlim}/\code{ylim}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/euler-methods.R
\name{fitted.euler}
\alias{fitted.euler}
\title{Fitted values of euler object}
\usage{
\method{fitted}{euler}(object, dense = FALSE, ...)
}
\arguments{
\item{object}{object of class \code{'euler'}}

\item{dense}{if \code{TRUE}, return a vector covering every combination of the
non-empty sets (\code{2^n - 1} entries), filling absent combinations with 0.
The default (\code{FALSE}) returns the sparse vector stored on the object,
which only contains entries that were either requested as input or fit
to a non-zero area. Use \code{dense = TRUE} only for diagrams with few sets
--- the full enumeration is exponential in the number of non-empty sets.}

\item{...}{ignored}
}
\value{
A named numeric vector of fitted areas keyed by combination label
(set names joined by \code{&}).
}
\description{
Fitted values of euler object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{pain}
\alias{pain}
\title{Pain distribution data}
\format{
A flat \link{table} (cross-table) with with sex in columns and
pain distribution in rows and integer counts making up the cells of the
table.
}
\source{
Westergren H, Larsson J, Freeman M, Carlsson A, Jöud A,
Malmström E-M. Sex-based differences in pain distribution in a cohort of
patients with persistent post-traumatic neck pain. Disability and
Rehabilitation. 2017 Jan 27
}
\usage{
pain
}
\description{
Data from a study on pain distribution for patients with
persistent neck pain in relation to a whiplash trauma.
}
\section{Disclaimer}{

Note that the maintainer of this package is an author of the source for
this data.
}

\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.euler.R
\name{print.euler}
\alias{print.euler}
\title{Print a summary of an Euler diagram}
\usage{
\method{print}{euler}(x, round = 3, vsep = strrep("-", 0.75 * getOption("width")), ...)
}
\arguments{
\item{x}{\code{'euler'} object from \code{\link[=euler]{euler()}}}

\item{round}{number of decimal places to round to}

\item{vsep}{character string to paste in between \code{euler} objects
when \code{x} is a nested \code{euler} object}

\item{...}{arguments passed to \code{\link[base:print.dataframe]{base::print.data.frame()}}}
}
\value{
Summary statistics of the fitted Euler diagram are printed to
screen.
}
\description{
This function is responsible for printing fits from \code{\link[=euler]{euler()}} and provides
a summary of the fit. Prints a data frame of the original set relationships
and the fitted values as well as \code{diagError} and \code{stress} statistics.
}
\examples{
euler(organisms)
}
\seealso{
\code{\link[=euler]{euler()}}, \code{\link[base:print.dataframe]{base::print.data.frame()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/label_placement.R
\name{measure_tag}
\alias{measure_tag}
\title{Native-unit AABB of one composite tag (label stacked above quantity,
annotation stacked below quantity, separated by \code{padding}). The
geometry matches what \code{setup_tag()} renders at draw time, so the size
handed to eunoia agrees with the actual on-screen footprint.}
\usage{
measure_tag(
  label,
  quantity,
  annotation,
  labels_par_id,
  quantities_par_id,
  annotations_par_id,
  labels_gp,
  quantities_gp,
  annotations_gp,
  padding_native
)
}
\description{
Native-unit AABB of one composite tag (label stacked above quantity,
annotation stacked below quantity, separated by \code{padding}). The
geometry matches what \code{setup_tag()} renders at draw time, so the size
handed to eunoia agrees with the actual on-screen footprint.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setup_grobs.R
\name{setup_grobs}
\alias{setup_grobs}
\title{Grobify Euler objects}
\usage{
setup_grobs(
  x,
  fills,
  patterns,
  edges,
  labels,
  quantities,
  annotations = NULL,
  complement = NULL,
  number,
  merged_sets,
  n_vertices = 200L,
  placement_opts = NULL
)
}
\arguments{
\item{x}{geometry data}

\item{fills}{fills params}

\item{patterns}{pattern params}

\item{edges}{edges params}

\item{labels}{labels params}

\item{quantities}{quantities params}

\item{number}{current diagram number}

\item{merged_sets}{sets that are the same and have been merged}
}
\value{
A \code{\link[grid:grid.grob]{grid::gList()}} is returned.
}
\description{
Grobify Euler objects
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.euler.R
\name{plot.eulergram}
\alias{plot.eulergram}
\alias{print.eulergram}
\title{Print (plot) Euler diagram}
\usage{
\method{plot}{eulergram}(x, newpage = TRUE, ...)

\method{print}{eulergram}(x, ...)
}
\arguments{
\item{x}{an object of class \code{'eulergram'}, usually the output of
\code{\link[=plot.euler]{plot.euler()}}}

\item{newpage}{if \code{TRUE}, opens a new page via \code{\link[=grid.newpage]{grid.newpage()}} to draw on}

\item{...}{ignored}
}
\value{
A plot is drawn on the current device using \code{\link[grid:Grid]{grid::Grid()}} graphics.
}
\description{
This function is responsible for the actual drawing of
\code{'eulergram'} objects created through \code{\link[=plot.euler]{plot.euler()}}. \code{\link[=print.eulergram]{print.eulergram()}}
is an alias for \code{\link[=plot.eulergram]{plot.eulergram()}}, which has been provided so that
\code{\link[=plot.euler]{plot.euler()}} gets called automatically.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/euler-methods.R
\name{coef.euler}
\alias{coef.euler}
\title{Return the fitted shape parameters from the euler object}
\usage{
\method{coef}{euler}(object, ...)
}
\arguments{
\item{object}{object of class \code{'euler'}}

\item{...}{ignored}
}
\value{
a data frame of the fitted shape parameters
}
\description{
Returns the \verb{$shapes} data frame — a tagged uniform schema with one row
per set, a \code{type} column, and shape-specific columns
(\verb{h, k, a, b, phi, width, height, side}) populated according to the
chosen shape (other columns are \code{NA}). For circle/ellipse fits the
legacy \verb{$ellipses} slot is still populated for back-compat; consumers
that need the new shapes (rectangle, square) must read \verb{$shapes}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eulerr_options.R
\name{eulerr_default_options}
\alias{eulerr_default_options}
\title{Default options for eulerr}
\usage{
eulerr_default_options()
}
\value{
default options for eulerr
}
\description{
Default options for eulerr
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/euler-methods.R
\name{residuals.euler}
\alias{residuals.euler}
\title{Residuals of euler object}
\usage{
\method{residuals}{euler}(object, dense = FALSE, ...)
}
\arguments{
\item{object}{object of class \code{'euler'}}

\item{dense}{same meaning as in \code{\link[=fitted.euler]{fitted.euler()}}}

\item{...}{ignored}
}
\value{
A named numeric vector of residuals (input minus fit) keyed by
combination label.
}
\description{
Residuals of euler object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.euler.R
\name{plot.euler}
\alias{plot.euler}
\alias{plot.eulerr_venn}
\alias{plot.venn}
\title{Plot Euler and Venn diagrams}
\usage{
\method{plot}{euler}(
  x,
  fills = TRUE,
  patterns = FALSE,
  edges = TRUE,
  legend = FALSE,
  labels = identical(legend, FALSE),
  quantities = FALSE,
  annotations = NULL,
  strips = NULL,
  bg = FALSE,
  main = NULL,
  complement = TRUE,
  rotate = 0,
  n = 200L,
  adjust_labels = TRUE,
  ...
)

\method{plot}{eulerr_venn}(
  x,
  fills = TRUE,
  patterns = FALSE,
  edges = TRUE,
  legend = FALSE,
  labels = identical(legend, FALSE),
  quantities = TRUE,
  strips = NULL,
  bg = FALSE,
  main = NULL,
  complement = TRUE,
  n = 200L,
  adjust_labels = TRUE,
  ...
)

\method{plot}{venn}(...)
}
\arguments{
\item{x}{an object of class \code{'euler'}, generated from \code{\link[=euler]{euler()}}}

\item{fills}{a logical, vector, or list of graphical parameters for the fills
in the diagram. Vectors are assumed to be colors for the fills.
See \code{\link[grid:grid.path]{grid::grid.path()}}. Named fill vectors can be matched in
\code{fills$mode = "disjoint"} (default) or \code{fills$mode = "union"}.}

\item{patterns}{a logical, vector, or list of graphical parameters for
fill patterns in the diagram. Vectors are assumed to be pattern types
(currently \code{"stripes"} or \code{NA}), where \code{NA} means no pattern.
Supported list items are \code{type}, \code{angle}, \code{col}, \code{lwd}, and \code{alpha}.
Named pattern vectors can be matched in
\code{patterns$mode = "disjoint"} (default) or \code{patterns$mode = "union"}.}

\item{edges}{a logical, vector, or list of graphical parameters for the edges
in the diagram. Vectors are assumed to be colors for the edges.
See \code{\link[grid:grid.lines]{grid::grid.polyline()}}.}

\item{legend}{a logical scalar or list. If a list, the item \code{side} can be
used to set the location of the legend and \code{symbol_size} can be used to scale
the legend symbols independently of the text size. See
\code{\link[grid:legendGrob]{grid::grid.legend()}}.}

\item{labels}{a logical, vector, or list. Vectors are assumed to be
text for the labels. See \code{\link[grid:grid.text]{grid::grid.text()}}. In addition to the
\code{grid::gpar()} fields, the following placement controls are
supported (delegated to the \code{eunoia} Rust crate):
\code{labels$placement} (\code{"raycast"} (default), \code{"force_directed"}, or
\code{"elbow"}) selects the strategy used when a label does not fit
inside its region. \code{"raycast"} and \code{"force_directed"} produce
straight leader lines (the former places the label along the
centroid→POI ray, the latter relaxes labels with a polygon-aware
force solver). \code{"elbow"} produces d3-pie style orthogonal leaders,
stacking exterior labels in left/right columns reached by a
three-segment polyline. \code{labels$margin} (numeric) overrides the
per-region margin between an exterior label and the diagram
(default is half the larger of the label's width and height);
\code{labels$tether} (\code{"poi"} (default) or \code{"boundary"}) chooses where
the leader line attaches on the source region; \code{labels$gap}
controls the visible gap between the leader tip and the label box
edge — a bare numeric is interpreted as \code{lines} (font-relative),
a \code{\link[grid:unit]{grid::unit()}} is honored as given, and the default \code{NULL}
tracks \code{eulerr_options()$padding} so the gap matches the spacing
between label and quantity; \code{labels$leader} is a list (\code{col},
\code{alpha}, \code{lwd}, \code{lty}, \code{lex}) styling the leader line drawn from
the tether to the exterior label. Strategy-specific knobs live in
their own sublists: \code{labels$force_directed = list(iterations = ...)}
sets the iteration cap for the force-directed solver, and
\code{labels$elbow = list(min_gap = ...)} sets the minimum vertical
centre-to-centre spacing between stacked label boxes in an elbow
column.}

\item{quantities}{a logical, vector, or list. Vectors are assumed to be
text for the quantities' labels, which by
default are the original values in the input to \code{\link[=euler]{euler()}}. In addition
to plain vectors, \code{quantities$labels} can also be a named vector keyed by
subset names (e.g., \code{"A"}, \code{"B"}, \code{"A&B"}), which is useful for supplying
custom text for overlap regions. If \code{quantities$labels} is \code{NULL},
\code{quantities$format} can be used to control number formatting as a list with
an item \code{fun} (a function such as \code{\link[=signif]{signif()}} or \code{\link[=round]{round()}}) and optional
extra arguments passed to that function (for example,
\code{list(fun = prettyNum, big.mark = ",")}). \code{quantities$total} can be used to
set an external
denominator for percent/fraction quantities (instead of the plotted total).
to arguments that apply to \code{\link[grid:grid.text]{grid::grid.text()}}, an argument \code{type} may
also be used which should be a combination of \code{"counts"}, \code{"percent"},
and \code{"fraction"}. The first item will be printed first and the second
will be printed thereafter inside brackets. The default is
\code{type = "counts"}.
For finer control over the rendered text, set \code{quantities$template} to
a string with \code{{counts}}, \code{{percent}}, and/or \code{{fraction}} placeholders,
for example \code{"{counts}\\n{percent}"} to put the count and percentage on
separate lines or \code{"n={counts} ({percent})"} for arbitrary layout. When
\code{template} is set it overrides \code{type}; the set of placeholders in the
template determines which values are computed.}

\item{annotations}{free-form per-region text rendered as a third
stacked element below the quantity (or below the label when no
quantity is drawn). Accepts a named character vector keyed by
subset name (e.g. \code{c(A = "n = 12", "A&B" = "n = 3")}) as a
shorthand for \verb{list(labels = <vector>)}, or a list with \code{labels}
plus \code{\link[grid:gpar]{grid::gpar()}} fields (\code{col}, \code{alpha}, \code{fontsize}, \code{cex},
\code{fontfamily}, \code{lineheight}, \code{font}, \code{rot}). Regions absent from
\code{labels} are not annotated. The composite tag bbox grows to
include the annotation, so exterior placement and leader lines
adapt automatically. Defaults to slightly smaller text than
\code{labels} / \code{quantities} (\code{cex = 0.8}).}

\item{strips}{a list, ignored unless the \code{'by'} argument
was used in \code{\link[=euler]{euler()}}. In addition to graphical parameters, this
argument can include \code{labels = list(top = ..., left = ...)} for custom
strip labels. Unnamed labels are interpreted in display order. Named
labels are matched by factor levels and then reordered to display order.}

\item{bg}{a logical, character, or list controlling the background grob.
Character values are interpreted as the background fill color.}

\item{main}{a title for the plot in the form of a
character, expression, list or something that can be
sensibly converted to a label via \code{\link[grDevices:as.graphicsAnnot]{grDevices::as.graphicsAnnot()}}. A
list of length one can be provided, in which case its only element
is used as the label. If a list of longer length is provided, an item
named \code{'label'} must be provided (and will be used for the actual text).}

\item{complement}{a logical, character, or list controlling the
container box and complement region for diagrams fit with
\verb{complement =} in \code{\link[=euler]{euler()}}. \code{TRUE} (default) draws the container
with a dashed outline (\code{lty = 2}), no fill, and the complement count
inside the complement region. \code{FALSE} suppresses the container and
its label entirely. A character value is treated as a fill color
shorthand. A list accepts \code{fill}, \code{alpha}, \code{col}, \code{lty}, \code{lwd},
\code{lex} (outline + label gpar), \code{fontsize}, \code{cex}, \code{font}, \code{fontfamily},
\code{lineheight} (label only), and \code{label} (custom text — defaults to the
complement count). Also accepts the same placement controls as
\code{labels} (\code{placement}, \code{margin}, \code{tether}, \code{gap}, \code{leader},
\code{force_directed}, \code{elbow})
for the complement count label. Has no effect if the diagram was
fit without \verb{complement =}. Defaults can be set via
\code{eulerr_options(complement = ...)}.}

\item{rotate}{a numeric value giving the angle in degrees by which to rotate
the entire diagram layout. Positive values rotate counter-clockwise.
Defaults to \code{0} (no rotation).}

\item{n}{number of vertices for the \code{edges} and \code{fills}}

\item{adjust_labels}{a logical. If \code{TRUE}, adjustment will be made to avoid
overlaps or out-of-limits plotting of labels, quantities, and
percentages.}

\item{...}{parameters to update \code{fills} and \code{edges} with and thereby a shortcut
to set these parameters
\code{\link[grid:grid.text]{grid::grid.text()}}.}
}
\value{
Provides an object of class \code{'eulergram'} , which is a
description of the diagram to be drawn. \code{\link[=plot.eulergram]{plot.eulergram()}} does the actual
drawing of the diagram.
}
\description{
Plot diagrams fit with \code{\link[=euler]{euler()}} and \code{\link[=venn]{venn()}} using \code{\link[grid:Grid]{grid::Grid()}} graphics.
This
function sets up all the necessary plot parameters and computes
the geometry of the diagram. \code{\link[=plot.eulergram]{plot.eulergram()}}, meanwhile,
does the actual plotting of the diagram. Please see the \strong{Details} section
to learn about the individual settings for each argument.
}
\details{
The only difference between \code{\link[=plot.euler]{plot.euler()}} and \code{\link[=plot.venn]{plot.venn()}} is that
\code{quantities} is set to \code{TRUE} by default in the latter and \code{FALSE} in
the former.

Most of the arguments to this function accept either a logical, a vector, or
a list where
\itemize{
\item logical values set the attribute on or off,
\item vectors are shortcuts to commonly used options (see the individual
parameters), and
\item lists enable fine-grained control, including graphical
parameters as described in \code{\link[grid:gpar]{grid::gpar()}} and control
arguments that are specific to each argument.
}

The various \code{\link[grid:gpar]{grid::gpar()}} values that are available for each argument
are:

\tabular{lcccccccc}{
\tab fills \tab edges \tab labels \tab quantities  \tab annotations \tab strips \tab legend \tab main \cr
col        \tab       \tab x     \tab x      \tab x           \tab x           \tab x      \tab x      \tab x    \cr
fill       \tab x     \tab       \tab        \tab             \tab             \tab        \tab        \tab      \cr
alpha      \tab x     \tab x     \tab x      \tab x           \tab x           \tab x      \tab x      \tab x    \cr
lty        \tab       \tab x     \tab        \tab             \tab             \tab        \tab        \tab      \cr
lwd        \tab       \tab x     \tab        \tab             \tab             \tab        \tab        \tab      \cr
lex        \tab       \tab x     \tab        \tab             \tab             \tab        \tab        \tab      \cr
fontsize   \tab       \tab       \tab x      \tab x           \tab x           \tab x      \tab x      \tab x    \cr
cex        \tab       \tab       \tab x      \tab x           \tab x           \tab x      \tab x      \tab x    \cr
fontfamily \tab       \tab       \tab x      \tab x           \tab x           \tab x      \tab x      \tab x    \cr
lineheight \tab       \tab       \tab x      \tab x           \tab x           \tab x      \tab x      \tab x    \cr
font       \tab       \tab       \tab x      \tab x           \tab x           \tab x      \tab x      \tab x    \cr
}

Defaults for these values, as well as other parameters of the plots, can
be set globally using \code{\link[=eulerr_options]{eulerr_options()}}.

If the diagram has been fit using the \code{data.frame} or \code{matrix} methods
and using the \code{by} argument, the plot area will be split into panels for
each combination of the one to two factors. The \code{fills}, \code{patterns}, \code{edges},
\code{labels}, \code{quantities}, and \code{annotations} arguments each accept an optional
\code{by_group} entry: a named list of override lists keyed by panel name (the
names of the fitted object). For multi-\code{by} fits the panel name is the
levels joined by \code{.}, e.g. \code{"Male.German"}. Panels not listed in \code{by_group}
use the top-level settings unchanged. Only graphical fields (and \code{rot} for
\code{labels}, \code{quantities}, and \code{annotations}) may be overridden per panel;
structural fields such as \code{quantities$type}, \code{quantities$format},
\code{annotations$labels}, or named-by-subset \code{fills$fill} must be set at the
top level.

For users who are looking to plot their diagram using another package,
all the necessary parameters can be collected if the result of this
function is assigned to a variable (rather than printed to screen).
}
\examples{
fit <- euler(c("A" = 10, "B" = 5, "A&B" = 3))

# Customize colors, remove borders, bump alpha, color labels white
plot(fit,
     fills = list(fill = c("red", "steelblue4"), alpha = 0.5),
     labels = list(col = "white", font = 4))

# Add quantities to the plot
plot(fit, quantities = TRUE)

# Add free-form per-region annotations below the counts
plot(
  fit,
  quantities = TRUE,
  annotations = c(A = "mean = 35", "A&B" = "mean = 41")
)

# Add a custom legend and retain quantities
plot(fit, quantities = TRUE, legend = list(labels = c("foo", "bar")))

# Plot without fills and distinguish sets with border types instead
plot(fit, fills = "transparent", lty = 1:2)

# Save plot parameters to plot using some other method
diagram_description <- plot(fit)

# Plots using 'by' argument
plot(euler(fruits[, 1:4], by = list(sex)), legend = TRUE)

# Per-panel styling with `by_group`
plot(
  venn(fruits[, 1:4], by = list(sex)),
  quantities = list(
    by_group = list(
      male = list(col = "steelblue"),
      female = list(col = "tomato")
    )
  )
)
}
\seealso{
\code{\link[=euler]{euler()}}, \code{\link[=plot.eulergram]{plot.eulergram()}}, \code{\link[grid:gpar]{grid::gpar()}},
\code{\link[grid:grid.lines]{grid::grid.polyline()}}, \code{\link[grid:grid.path]{grid::grid.path()}},
\code{\link[grid:legendGrob]{grid::grid.legend()}}, \code{\link[grid:grid.text]{grid::grid.text()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extendr-wrappers.R
\name{max_sets_default}
\alias{max_sets_default}
\title{Default number of sets that \code{eunoia} accepts before rejecting a spec.
Used by the R-side input validator so the cap is not hardcoded.}
\usage{
max_sets_default()
}
\description{
Default number of sets that \code{eunoia} accepts before rejecting a spec.
Used by the R-side input validator so the cap is not hardcoded.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_diagram.R
\name{shapes_to_ellipse_frame}
\alias{shapes_to_ellipse_frame}
\title{Project the wide \verb{$shapes} schema back into the legacy 5-column
(h, k, a, b, phi) ellipse data frame for circle/ellipse fits. Preserves
the row order and row names so back-compat consumers see exactly the
shape they used to.}
\usage{
shapes_to_ellipse_frame(shapes)
}
\description{
Project the wide \verb{$shapes} schema back into the legacy 5-column
(h, k, a, b, phi) ellipse data frame for circle/ellipse fits. Preserves
the row order and row names so back-compat consumers see exactly the
shape they used to.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tag-grobs.R
\name{build_tag_grobs}
\alias{build_tag_grobs}
\title{Build the leader / label / quantity / annotation gList for one tag.}
\usage{
build_tag_grobs(
  ax,
  ay,
  kind,
  tx,
  ty,
  lend_x,
  lend_y,
  label_text,
  quantity_text,
  annotation_text,
  has_label,
  has_quantity,
  has_annotation,
  label_gp,
  quantity_gp,
  annotation_gp,
  label_rot,
  quantity_rot,
  annotation_rot,
  number,
  leader_gp_list,
  padding,
  waypoints_x = numeric(0),
  waypoints_y = numeric(0),
  name_prefix = "tag"
)
}
\description{
Shared between \code{\link[=setup_tag]{setup_tag()}} (initial construction in \code{\link[=setup_grobs]{setup_grobs()}})
and \code{\link[=makeContent.EulerTags]{makeContent.EulerTags()}} (draw-time re-placement on resize).
Pure factory — no measurement; takes anchor + tether already in
native units, plus the stashed text / gpar bundle. The label /
quantity / annotation stack is centered vertically on \verb{(ax, ay)}
so the bbox center matches the anchor eunoia placed and leader
endpoints land on the actual bbox edge.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tag-grobs.R
\name{measure_all_tags}
\alias{measure_all_tags}
\title{Measure every drawable tag inside \code{tags_grob} against the current
viewport. Returns parallel vectors of combo / width / height suitable
for handing to \code{\link[=place_euler_labels]{place_euler_labels()}}, plus the resolved leader gap
in native units (so the FFI sees one number per draw pass).}
\usage{
measure_all_tags(tags_grob, padding, gap = NULL)
}
\description{
Measure every drawable tag inside \code{tags_grob} against the current
viewport. Returns parallel vectors of combo / width / height suitable
for handing to \code{\link[=place_euler_labels]{place_euler_labels()}}, plus the resolved leader gap
in native units (so the FFI sees one number per draw pass).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{n_sets}
\alias{n_sets}
\title{Get the number of sets in he input}
\usage{
n_sets(combinations)
}
\arguments{
\item{combinations}{a vector of combinations (see \code{\link[=euler]{euler()}})}
}
\value{
The number of sets in the input
}
\description{
Get the number of sets in he input
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{setup_gpar}
\alias{setup_gpar}
\title{Setup gpars}
\usage{
setup_gpar(default = list(), user = list(), n)
}
\arguments{
\item{default}{default values}

\item{user}{user-inputted values}

\item{n}{required number of items}
}
\value{
a \code{gpar} object
}
\description{
Setup gpars
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tag-grobs.R
\name{setup_tag}
\alias{setup_tag}
\title{Setup grobs for one tag (label + quantity + annotation + leader).}
\usage{
setup_tag(data, labels, quantities, annotations, number)
}
\description{
Builds the gList via \code{\link[=build_tag_grobs]{build_tag_grobs()}} and stashes the text / gpar
bundle on the resulting \code{EulerTag} gTree so \code{\link[=makeContent.EulerTags]{makeContent.EulerTags()}}
can rebuild it at draw time with fresh measurements.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extendr-wrappers.R
\name{euler_plot_data}
\alias{euler_plot_data}
\title{Compute polygon geometry and label anchors for plotting a fitted Euler
diagram, including the optional complement region inside a fitted
container.}
\usage{
euler_plot_data(
  set_names,
  shape,
  h,
  k,
  a,
  b,
  phi,
  width,
  height,
  side,
  container_h,
  container_k,
  container_width,
  container_height,
  n_vertices,
  label_precision
)
}
\description{
Inputs are the fitted shape parameters for the \strong{non-empty} sets only,
in the order eulerr stores them (\code{x$shapes} rows after dropping rows
with NA). The leading \code{shape} argument selects which per-set columns are
active: \code{"circle"}/\code{"ellipse"} consume \verb{(h, k, a, b, phi)},
\code{"rectangle"} consumes \verb{(h, k, width, height)}, \code{"square"} consumes
\verb{(h, k, side)}. The remaining vectors must still be provided (NaN
padding is acceptable). When \verb{container_*} are non-NULL they describe
the fitted universe-box rectangle; in that case the result also carries
the complement region geometry (the area inside the rectangle outside
every shape) and a label anchor for it. Eunoia's \code{decompose_regions}
emits this region under the empty \code{Combination} whenever the spec
carries a complement and a container is supplied.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/error_plot.R
\name{error_plot}
\alias{error_plot}
\title{Error plot for \code{euler} objects}
\usage{
error_plot(
  x,
  type = c("regionError", "residuals"),
  quantities = TRUE,
  pal = NULL,
  ...
)
}
\arguments{
\item{x}{an object of class \code{euler}, typically the result of
a call to \code{\link[=euler]{euler()}}.}

\item{type}{error metric. \code{'regionError'} is the difference in
\emph{percentage points} from the input}

\item{quantities}{whether to draw the error metric on the plot}

\item{pal}{color palette for the fills in the legend}

\item{...}{arguments passed down to \code{\link[=plot.euler]{plot.euler()}}. Currently,
providing \code{fills}, \code{legend}, or \code{strips} are not allowed and
will return a warning.}
}
\value{
Returns an object of class \code{eulergram}, which will be
plotted on the device in the same manner as objects from
\code{\link[=plot.euler]{plot.euler()}}. See \code{\link[=plot.eulergram]{plot.eulergram()}} for details.
}
\description{
This is a diagnostic tool for evaluating the fit from a call
to \code{\link[=euler]{euler()}} visually. A color key is provided by default, which
represents the chosen error metric so that one can easily detect
which areas in the diagram to be skeptical about.
}
\details{
Notice that this function is purely provided for diagnostic reasons
and does not come with the same kind of customization that
\code{\link[=plot.euler]{plot.euler()}} provides: the color legend can only be customized
in regards to its color palette and another key (instead of labels)
is completely turned off.
}
\examples{
error_plot(euler(organisms), quantities = FALSE)
}
\seealso{
\code{\link[=plot.euler]{plot.euler()}}, \code{\link[=euler]{euler()}},
\code{\link[=plot.eulergram]{plot.eulergram()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compose.R
\name{eulergram-compose}
\alias{eulergram-compose}
\alias{|.eulergram}
\alias{/.eulergram}
\title{Compose Euler Diagrams}
\usage{
\method{|}{eulergram}(e1, e2)

\method{/}{eulergram}(e1, e2)
}
\arguments{
\item{e1, e2}{\code{eulergram} objects, typically returned by \code{\link[=plot.euler]{plot.euler()}}.}
}
\value{
An \code{eulergram} containing the composed layout.
}
\description{
Arrange two \code{eulergram} objects side-by-side or stacked, building up
multi-panel layouts with operator syntax. Compositions can be nested
arbitrarily, e.g. \code{(p1 | p2) / p3}.
}
\details{
\code{|} arranges the two plots horizontally; \code{/} stacks them vertically.
The result is itself an \code{eulergram}, so further composition chains
naturally.

The gap between adjacent plots is controlled by the
\code{composition$spacing} entry of \code{\link[=eulerr_options]{eulerr_options()}}, which must be a
\code{\link[grid:unit]{grid::unit()}} and defaults to \code{grid::unit(1, "lines")}.

Because composition is binary and recursive, panels at different
nesting levels are not size-aligned. In \code{(p1 | p2) / p3}, \code{p3} spans
the full bottom row while \code{p1} and \code{p2} split the top row equally.
}
\examples{
p1 <- plot(euler(c(A = 1, B = 8, "A&B" = 1)))
p2 <- plot(euler(c(A = 1, C = 1, "A&C" = 1)))

p1 | p2
p1 / p2

p3 <- plot(euler(c(X = 3, Y = 2, "X&Y" = 1)))
(p1 | p2) / p3

}
\seealso{
\code{\link[=plot.euler]{plot.euler()}}, \code{\link[=eulerr_options]{eulerr_options()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{update_list}
\alias{update_list}
\title{Update list with input}
\usage{
update_list(old, new)
}
\arguments{
\item{old}{a list to be updated}

\item{new}{stuff to update \code{x} with}
}
\value{
Returns an updated list.
}
\description{
A wrapper for \code{\link[utils:modifyList]{utils::modifyList()}} that attempts to coerce non-lists to
lists before updating.
}
\seealso{
\code{\link[utils:modifyList]{utils::modifyList()}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eulerr.R
\docType{package}
\name{eulerr-package}
\alias{eulerr}
\alias{eulerr-package}
\title{eulerr: Area-Proportional Euler and Venn Diagrams with Ellipses}
\description{
Generate area-proportional Euler diagrams using numerical optimization. An Euler diagram is a generalization of a Venn diagram, relaxing the criterion that all interactions need to be represented. Diagrams may be fit with ellipses and circles via a wide range of inputs and can be visualized in numerous ways.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/jolars/eulerr}
  \item \url{https://jolars.github.io/eulerr/}
  \item Report bugs at \url{https://github.com/jolars/eulerr/issues}
}

}
\author{
\strong{Maintainer}: Johan Larsson \email{johanlarsson@outlook.com} (\href{https://orcid.org/0000-0002-4029-5945}{ORCID})

Other contributors:
\itemize{
  \item A. Jonathan R. Godfrey [contributor]
  \item Peter Gustafsson [contributor]
  \item David H. Eberly (geometric algorithms) [contributor]
  \item Emanuel Huber (root solver code) [contributor]
  \item Florian Privé [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{is_real}
\alias{is_real}
\title{Check if vector is a real (numeric non-integer)}
\usage{
is_real(x, tol = .Machine$double.eps^0.5)
}
\arguments{
\item{x}{}
}
\value{
A logical.
}
\description{
Check if vector is a real (numeric non-integer)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/label_placement.R
\name{expand_limits_with_canvas}
\alias{expand_limits_with_canvas}
\title{Union the current \code{xlim}/\code{ylim} with the canvas bbox reported by
eunoia. Returns the (possibly widened) limits.}
\usage{
expand_limits_with_canvas(limits, placements, slack = 1.4)
}
\description{
\code{slack} pads the canvas bbox before the union (multiplicative,
centered on the bbox center). The setup-time bbox is sized for one
reference device; padding gives the draw-time placement headroom
when the user resizes the device smaller than that reference and
labels grow in native units accordingly. A \code{slack} of 1.4 absorbs
roughly a 30 \% linear resize before exterior labels start to fall
outside the panel viewport.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/label_placement.R
\name{resolve_placement_opts}
\alias{resolve_placement_opts}
\title{Merge user-supplied placement options onto the defaults.}
\usage{
resolve_placement_opts(opts)
}
\description{
Merge user-supplied placement options onto the defaults.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{fruits}
\alias{fruits}
\title{Fruits}
\format{
A \link{data.frame} with 100 observations of 5 variables:
\describe{
\item{banana}{whether the person likes bananas, a logical}
\item{apple}{whether the person likes apples, a logical}
\item{orange}{whether the person likes oranges, a logical}
\item{sex}{the sex of the person, a factor with levels 'male' and 'female'}
\item{age}{the age of the person, a factor with levels 'child' and 'adult'}
}
}
\usage{
fruits
}
\description{
A synthethic data set of preferences for fruits and their overlaps,
generated only to be a showcase for the examples for this package.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{dummy_code}
\alias{dummy_code}
\title{Dummy code a data.frame}
\usage{
dummy_code(x, sep = "_", factor_names = TRUE)
}
\arguments{
\item{x}{a data.frame}

\item{sep}{character for separating dummy code factors and their levels
when constructing names}

\item{factor_names}{whether to include factor names when creating new
names for dummy codes}
}
\value{
A dummy-coded version of x.
}
\description{
Dummy code a data.frame
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/label_placement.R
\name{default_placement_opts}
\alias{default_placement_opts}
\title{Default placement options used when the caller doesn't supply any.}
\usage{
default_placement_opts()
}
\description{
Mirrors \verb{eunoia::plotting::PlacementStrategy::default()}: raycast
exterior solver, POI tether, proportional margin. Per-strategy knobs
\code{iterations} (force-directed) and \code{min_gap} (elbow) live alongside in
the flat shape that the Rust FFI expects; the user-facing sublists in
\code{eulerr_options(labels = list(force_directed = ..., elbow = ...))} are
flattened into this shape by \code{plot.euler()} before placement runs.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extendr-wrappers.R
\name{max_sets_hard_cap}
\alias{max_sets_hard_cap}
\title{Absolute upper bound on the number of sets that \code{eunoia} can represent
in a single diagram. Used by the R-side input validator so the cap is
not hardcoded.}
\usage{
max_sets_hard_cap()
}
\description{
Absolute upper bound on the number of sets that \code{eunoia} can represent
in a single diagram. Used by the R-side input validator so the cap is
not hardcoded.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_diagram.R
\name{ellipse_frame_to_shapes}
\alias{ellipse_frame_to_shapes}
\title{Promote a legacy 5-column ellipse data frame (h, k, a, b, phi) into the
wide \verb{$shapes} schema. Used by the \code{venn()} path, where the precomputed
venn-shape lookup is still expressed as ellipses.}
\usage{
ellipse_frame_to_shapes(fpar, shape)
}
\description{
Promote a legacy 5-column ellipse data frame (h, k, a, b, phi) into the
wide \verb{$shapes} schema. Used by the \code{venn()} path, where the precomputed
venn-shape lookup is still expressed as ellipses.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{all_set_combinations}
\alias{all_set_combinations}
\title{Enumerate all 2^n - 1 combination labels for a set of names}
\usage{
all_set_combinations(setnames)
}
\arguments{
\item{setnames}{a character vector of set names}
}
\value{
A character vector of length 2^n - 1.
}
\description{
Generates labels in cardinality-first order: singletons first, then pairs,
then triples, etc. Within each cardinality, set names appear in their
original order. Used only by the Venn path (bounded at n = 5).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{replace_list}
\alias{replace_list}
\title{Replace (refresh) a list}
\usage{
replace_list(old, new)
}
\arguments{
\item{old}{the original list}

\item{new}{the things to update \code{old} with}
}
\value{
A refreshed list.
}
\description{
Unlike \code{update_list}, this function only modifies, and does not add,
items in the list.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/label_placement.R
\name{split_waypoints}
\alias{split_waypoints}
\title{Split a placement's flat \code{leader_waypoints_x} / \verb{_y} / \verb{_lengths}
return triple into a per-label list of \code{list(x = ..., y = ...)}
coordinate pairs. Each list element has length-\code{lengths[i]} \code{x}/\code{y}
vectors (often \code{0} — straight leaders carry no waypoints).}
\usage{
split_waypoints(placements)
}
\description{
Split a placement's flat \code{leader_waypoints_x} / \verb{_y} / \verb{_lengths}
return triple into a per-label list of \code{list(x = ..., y = ...)}
coordinate pairs. Each list element has length-\code{lengths[i]} \code{x}/\code{y}
vectors (often \code{0} — straight leaders carry no waypoints).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{is_integer}
\alias{is_integer}
\title{Check if a vector is an integer}
\usage{
is_integer(x, tol = .Machine$double.eps^0.5)
}
\arguments{
\item{x}{a vector}
}
\value{
TRUE of FALSE.
}
\description{
Check if a vector is an integer
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{is_false}
\alias{is_false}
\title{Check if object is strictly FALSE}
\usage{
is_false(x)
}
\arguments{
\item{x}{object to check}
}
\value{
A logical.
}
\description{
Check if object is strictly FALSE
}
\keyword{internal}
